/*= R&S ESCI EMI Test Receiver ==============================================*/
/* LabWindows/CVI 7.0 Instrument Driver
/* Original Release: July 2004
/* By: Martin Koutny & Jiri Kominek
/*
/* Should you have any technical questions please contact the hotline of
/* Rohde & Schwarz Vertriebs-GmbH Rohde & Schwarz Support Center
/*
/* e-mail: CustomerSupport@rohde-schwarz.com
/*
/* Modification History: see ChangeLog
/*
/*===========================================================================*/

#include <visa.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <float.h>
#include <math.h>
#include "rsesci.h"

/*= DEFINES =================================================================*/

#define RSESCI_REVISION             "Rev 4.73.0, 10/2013, CVI 2009" /* Instrument driver revision */
#define BUFFER_SIZE                 2048L                       /* File I/O buffer size */
#define NUMERIC_PRECISION           1e-12   /* Precision of compare operations */

#define RSESCI_DEFAULT_TIMEOUT      10000   /* Default Timeout value in ms (10 sec) */
#define RSESCI_IEEE_ERROR_BITS      0x3C    /* AND mask for ESR checking */
#define RSESCI_MESSAGE_AVAILABLE    0x10

/*= Useful Macros ===========================================================*/

/* Error checking macro */
#define CHECKERR(fCal) if (rsesci_status = (fCal), rsesci_status < VI_SUCCESS) \
    return rsesci_status; else

/* Error checking macro with clean up (rsesci_init only) */
#define CHECKERR_CU(fCal) if (rsesci_status = (fCal), rsesci_status < VI_SUCCESS) \
    return rsesci_initCleanUp (rmSession, instrSession, rsesci_status); else

/* Check presency of instrument's option (when error checking is enabled) */
#define CHECKOPT(option) \
    if (rsesci_checkOption(instrSession, option) < VI_SUCCESS) \
        return RSESCI_ERROR_INSTRUMENT_OPTION;

/* Check presency of instrument's model (when error checking is enabled) */
#define CHECKMODEL(model) \
    if (rsesci_checkModel(instrSession, model) < VI_SUCCESS) \
        return RSESCI_ERROR_INSTRUMENT_NSUP_MODEL;

/* Is the result of scanning (sscanf) function OK? If not, return error code. */
#define CHECKSCAN(fCal) if ((fCal) < 0) \
    return VI_ERROR_INSTR_INTERPRETING_RESPONSE; else

/* Error checking macro with jump to the ERROR: label */
#define GOCHECKERR(fCal) if (rsesci_status = (fCal), rsesci_status < VI_SUCCESS) \
                            goto ERROR; else

/* Checking window parameter */
#define CHECKWINDOW if (rsesci_invalidViInt32Range (window, RSESCI_SCREEN_A, RSESCI_SCREEN_B)) return VI_ERROR_PARAMETER2;

/* Checking NULL pointer in argument */
#define CHECKNULL(arg, number) if ( !arg ) return VI_ERROR_PARAMETER##number

#define REMAP(par_in, par_out) if (rsesci_status == VI_ERROR_PARAMETER##par_in)\
                                    rsesci_status = VI_ERROR_PARAMETER##par_out;\
                                if (rsesci_status < VI_SUCCESS)\
                                    return rsesci_status; else
#define REMAPINSTR(par_in, par_out) if (rsesci_status == VI_ERROR_PARAMETER##par_in)\
                                    rsesci_status = VI_ERROR_INSTR_PARAMETER##par_out;\
                                if (rsesci_status < VI_SUCCESS)\
                                    return rsesci_status; else
/*- Instrument Versions ----------------------------------------------------*/

#define RSESCI_UNKNOWN     -1
#define RSESCI_3            0
#define RSESCI_7            1
#define RSESU_8             2
#define RSESU_26            3
#define RSESU_40            4

/*- Instrument Options -----------------------------------------------------*/

#define RSESCI_B2           (1 << 0)    /* 0x1 ...   R&S ESPI-B2 Preselector */
#define RSESCI_B3           (1 << 1)    /* 0x2 ...   R&S FSP-B3 Audio Demodulator */
#define RSESCI_B4           (1 << 2)    /* 0x4 ...   R&S FSP-B4 OCXO - reference oscillator */
#define RSESCI_B6           (1 << 3)    /* 0x8 ...   R&S FSP-B6 TV and RF Trigger */
#define RSESCI_B9           (1 << 4)    /* 0x10 ...  R&S FSP-B9 Tracking Generator 3 GHz / can be I/Q-modulated */
#define RSESCI_B10          (1 << 5)    /* 0x20 ...  R&S FSP-B10 External Generator Control */
#define RSESCI_B16          (1 << 6)    /* 0x40 ...  R&S FSP-B16 LAN Interface */
#define RSESCI_K7           (1 << 7)    /* 0x80 ...  R&S FS-K7 FM Demodulator */
#define RSESU_K53           (1 << 8)    /* 0x100 ...  R&S ESU-K53  */
#define RSESCI_B70          (1 << 9)    /* 0x200 ...  R&S FSP-B70  */
#define RSESU_B24           (1 << 10)    /* 0x400 ...  R&S ESU-B24  */

/*- Instrument Models ------------------------------------------------------*/

#define RSESCI3              (1 << 0)    /* 0x1 ... */
#define RSESCI7              (1 << 1)    /* 0x2 ... */
#define RSESU8               (1 << 2)    /* 0x4 ... */
#define RSESU26              (1 << 3)    /* 0x8 ... */
#define RSESU40              (1 << 4)    /* 0x10 ... */
#define RSESPI3              (1 << 5)    /* 0x20 ... */
#define RSESPI7              (1 << 6)    /* 0x40 ... */

/*****************************************************************************/
/*= INSTRUMENT-DEPENDENT COMMAND ARRAYS =====================================*/
/*****************************************************************************/

/* Instrument versions (See Default Instrument Setup for more details) */
static ViString instrList[] =
{
    "ESCI-3","ESCI-7","ESU-8","ESU-26","ESU-40","ESPI-3","ESPI-7",VI_NULL
};

/* Instrument options (See Default Instrument Setup for more details) */
static ViString optionsList[] =
{
    "B2","B3","B4","B6","B9","B10","B16","K7","K53","B70", VI_NULL
};

/*---------------------------------------------------------------------------*/

static ViString unitArr[] = {"DBM","V","A","W","DBPW","DBUV","DBMV","DBUA","DBPT","DBUV/M","DBUA/M","DB", VI_NULL};
static ViString setLimitPowerUnitArr[] = {"DBM","V","A","W","DBPW","DBUV","DBMV","DBUA","DBPT","DBUV_M","DBUA_M",
									    "DB","DBMV_M","DEG","RAD","S","HZ","PCT","DBUV_MHZ","DBMV_MHZ",
									   "DBUA_MHZ","DBPW_MHZ","DBPT_MHZ","DBUV_MMHZ","DBUA_MMHZ","UNIT",VI_NULL};
static ViString getLimitPowerUnitArr[] = {"DBM","V","A","W","DBPW","DBUV","DBMV","DBUA","DBPT","DBUV_M","DBUA_M",
									    "DB","DBMV_M","DEG","RAD","S","HZ","PCT","DBUV/MHz","DBMV/MHz",
									   "DBUA/MHz","DBPW_MHZ","DBPT_MHZ","DBUV_MMHZ","DBUA_MMHZ","UNIT",VI_NULL};
static ViString limitLineUnitArr[] = {"DBM","VOLT","AMP","WATT","DBPW","DBUV","DBMV","DBUA","DBPT","DBUV_M","DBUA_M",
									  "DB","DBMV_M","DEG","RAD","S","HZ","PCT","DBUV_MHZ","DBMV_MHZ",
									  "DBUA_MHZ","DBPW_MHZ","DBPT_MHZ","DBUV_MMHZ","DBUA_MMHZ","UNIT",VI_NULL};
static ViString switchArr[] = {"OFF","ON",VI_NULL};
static ViString steplinkArr[] = {"SPAN","RBW","OFF", VI_NULL};
static ViString freqModeArr[] = {"FIX","SWE","SCAN","TDOM", VI_NULL};
static ViString spacingArr[] = {"LIN","LOG","LDB", VI_NULL};
static ViString absRelArr[] = {"ABS","REL", VI_NULL};
static ViString couplingArr[] = {"AC","DC", VI_NULL};
static ViString impedanceArr[] = {"50","75", VI_NULL};
static ViString filterTypeArr[] = {"NORM","FFT","CFIL","RRC","NOIS","PULS", "MIL", VI_NULL};
static ViString modeArr[] = {"LIN","LOG", VI_NULL};
static ViReal64 sweeppointsArr[] = {125, 251, 501, 1001, 2001, 4001, 8001, VI_NULL};
static ViReal64 sweeppointsESUArr[] = {155, 313, 625, 1251, 1999, 2501, 5001, 10001, 20001, 30001, VI_NULL};
static ViString sweepspacingArr[] = {"LIN","LOG","AUTO","APUL","ACW", VI_NULL};
static ViString triggersourceArr[] = {"IMM","EXT","VID","IFP","RFP","TV","FM","AM","PM","AF","AMR", VI_NULL};
static ViString polarityArr[] = {"POS","NEG", VI_NULL};
static ViString triggerModeArr[] = {"LEV","EDGE", VI_NULL};
static ViString selStatusRegArr[] = {"",":POW",":LIM1",":LMAR1",":SYNC",":ACPL",":FREQ",":LIM2",":LMAR2",":TRAN", VI_NULL};
static ViString statusRegArr[] = {"OPER:EVEN","OPER:COND","QUES:EVEN","QUES:COND",
                                  "QUES:POW:EVEN","QUES:POW:COND","QUES:LIM:EVEN",
                                  "QUES:LIM:COND","QUES:LMAR:EVEN","QUES:LMAR:COND",
                                  "QUES:SYNC:EVEN","QUES:SYNC:COND","QUES:ACPL:EVEN",
                                  "QUES:ACPL:COND","QUES:FREQ:EVEN","QUES:FREQ:COND",
                                  "QUES:TRAN:EVEN","QUES:TRAN:COND","QUE",
                                  "QUES:LIM2:EVEN","QUES:LIM2:COND","QUES:LMAR2:EVEN",
                                  "QUES:LMAR2:COND"};
static ViString fieldArr[] = {"ALL","ODD","EVEN", VI_NULL};
static ViString ccvsSignalArr[] = {"INT","EXT","EAUT", VI_NULL};
static ViString traceModeArr[] = {"WRIT","VIEW","AVER","MAXH","MINH", VI_NULL};
static ViString traceNamesArr[] = {"","TRACE1","TRACE2","TRACE3",VI_NULL};
static ViString traceDetectorArr[] = {"APE","NEG","POS","SAMP","RMS","AVER",
                                        "QPE","CAV","CRMS", VI_NULL};
static ViString transmissionModeArr[] = {"NEV","ALW", VI_NULL};
static ViString mathArr[] = {"(TRACE1-TRACE2)","(TRACE1-TRACE3)", VI_NULL};
static ViString peakSearchArr[] = {"MAX","MAX:NEXT","MAX:RIGH","MAX:LEFT","MIN","MIN:NEXT","MIN:RIGH","MIN:LEFT", VI_NULL};
static ViString sortModeArr[] = {"X","Y", VI_NULL};
static ViString mkrFunctionArr[] = {"CENT","REF","CST", VI_NULL};
static ViString powerMeasurementArr[] = {"AOFF","STAT","PPE","RMS","MEAN","SDEV", VI_NULL};
static ViString resultTypeArr[] = {"RES","AVER:RES","PHOL:RES", VI_NULL};
static ViString standardArr[] = {"NONE","NADC","TETRA","PDC","PHS","CDPD","FWCD",
                                "RWCD","F8CD","R8CD","F19C","R19C","FW3G",
                                "RW3G","D2CD","S2CD","M2CD","FIS95C0","RIS95C0",
                                "FIS95C1","RIS95C1","TCDM","AWLAN","BWLAN", "USER", VI_NULL};
static ViString sizeArr[] = {"SMAL","LARG", VI_NULL};
static ViString channelBandwidthArr[] = {"CHAN","ACH","ALT1","ALT2","ALT3","ALT4","ALT5","ALT6",
										 "ALT7","ALT8","ALT9","ALT10","ALT11","CHAN2","CHAN3","CHAN4",
										 "CHAN5","CHAN6","CHAN7","CHAN8","CHAN9","CHAN10","CHAN11",VI_NULL};
static ViString acpReferenceChannelArr[] = {"MAN 1","MAN 2","MAN 3","MAN 4",
                                        "AUTO MIN","AUTO MAX","AUTO LHIG", VI_NULL};
static ViString finalDetectorArr[] = {"NEG","POS","RMS","AVER",
                                    "QPE","CAV","CRMS", VI_NULL};
static ViString searchMethodArr[] = {"SUBR","PEAK",VI_NULL};
static ViString indicationArr[] = {"OFF","CROS",VI_NULL};
static ViString demodulationArr[] = {"OFF","AM","FM",VI_NULL};
static ViString measurementTypeArr[] = {"TRAN","REFL",VI_NULL};
static ViString acquisitionTypeArr[] = {"THR;*WAI","OPEN;*WAI",VI_NULL};
static ViString powerMeasTypeArr[] = {"OFF","ACP","CPOW","MCAC","OBAN","CN","CN0", VI_NULL};
static ViReal64 counterResolutionArr[] = {0.1,1,10,100,1000,10000,0};
static ViString acpLimitsArr[] = {"","ACH","ALT1","ALT2","ALT3","ALT4","ALT5","ALT6",
								  "ALT7","ALT8","ALT9","ALT10","ALT11", VI_NULL};
static ViString statMeasArr[] = {"AOFF","APD","CCDF",VI_NULL};
static ViString vnetTypeArr[] = {"OFF","TWOP","FOUR","ESH3Z5","ESH2Z5","ENV4200","ENV216",VI_NULL};
static ViString earthArr[] = {"GRO","FLO",VI_NULL};
static ViString phaseArr[] = {"N","L1","L2","L3",VI_NULL};
static ViString extLinkArr[] = {"GPIB","TTL",VI_NULL};
static ViString limitTypeArr[] = {"LOW","UPP","CONT",VI_NULL};
static ViString detectorTypeArr[] = {"POS","NEG","RMS","AVER","QPE","CAV","CRMS",VI_NULL};
static ViString limitDomainArr[] = {"FREQ","TIME",VI_NULL};
static ViString statResultArr[] = {"ALL","MEAN","PEAK","CFAC",VI_NULL};
static ViString displayModeArr[] = {"SING","SPL",VI_NULL};
static ViString couplingTypeArr[] = {"CENT","SPAN","ATT","PROT","GAIN",
                                     "PRES","DEM","BAND",VI_NULL};
static ViString couplingStateArr[] = {"NONE","ALL",VI_NULL};
static ViString instrCouplingArr[] = {"NONE","RLEV","CF_A","CF_B",VI_NULL};
static ViString serialHWHandshakeArr[] = {"OFF","IBF",VI_NULL};
static ViString serialSWHandshakeArr[] = {"NONE","XON",VI_NULL};
static ViInt32  baudArr[] = {110, 300, 600, 1200, 2400, 9600, 19200, VI_NULL};
static ViString parityArr[] = {"NONE","EVEN","ODD",VI_NULL};
static ViString serviceSourceArr[] = {"RF","CAL",VI_NULL};
static ViReal64 pulseRateArr[] = {10E3,100E3,200E3,500E3,1E6,2E6,4E6,8E6,0};
static ViString dataSetArr[] = {"NONE","DEF","ALL","HWS","TRAC","FIN","LIN:ALL",
                                "SCD","TRAN:ALL","HCOP","CLIS",VI_NULL};
static ViString decSeparatorArr[] = {"POIN","COMM",VI_NULL};
static ViString instrModeArr[] = {"SAN","REC","ADEM","IFAN",VI_NULL};
static ViString sweepModeArr[] = {"0","1", VI_NULL};
static ViString resultDisplayArr[] = {"XTIM:FMS","XTIM:FM:TDOM","XTIM:FM:AFSP",
                                      "XTIM:SPEC","XTIM:PMS","XTIM:PM:TDOM",
                                      "XTIM:PM:AFSP","XTIM:AMS","XTIM:AM",
                                      "XTIM:RFP","XTIM:RFP","XTIM:AM:REL", VI_NULL};
static ViString angleUnitsArr[] = {"RAD","DEG", VI_NULL};
static ViString modulationMeasurementArr[] = {"PPE","MPE","MIDD","RMS", VI_NULL};
static ViString demodulationTypeArr[] = {"AM","FM","PM","AM:REL", VI_NULL};
static ViString afspectrumResultTypeArr[] = {"OFF","WRIT","AVER","MAXH","MINH","VIEW", VI_NULL};
static ViString measurementArr[] = {"IMM","AVER", VI_NULL};
static ViString averageTypeArr[] = {"LIN","VID", VI_NULL};
static ViString displaySizeArr[] = {"SMAL","LARG", VI_NULL};
static ViString predefinedColorsArr[] = {"BLACK","BLUE","BROWN","GREEN","CYAN",
                                         "RED","MAG","YELLOW","WHITE","DGRAY",
                                         "LGRAY","LBLUE","LGREEN","LCYAN","LRED",
                                         "LMAG"};
static ViString dataFormatArr[] = {"WMF","GDI","EWMF","BMP", VI_NULL};
static ViString pageOrientationArr[] = {"PORT","LAND", VI_NULL};
static ViString destinationArr[] = {"MMEM","SYST:COMM:PRIN","SYST:COMM:CLIP", VI_NULL};
static ViString compatibilityModeArr[] = {"OFF","FSP", VI_NULL};
static ViString inputTypeArr[] = {"INPUT1","INPUT2", VI_NULL};
static ViString setPLLBandwidthArr[] = {"AUTO","HIGH","MED","LOW", VI_NULL};
static ViString getPLLBandwidthArr[] = {"WIDE","NORMal","NARRow", VI_NULL};
static ViString FFTFilterModeArr[] = {"WIDE","AUTO","NARR", VI_NULL};
static ViString statProbArr[] = {"P0_1","P1","P10", VI_NULL};
static ViString statUnitArr[] = {"ABS","PCT", VI_NULL};
static ViString modeSEArr[] = {"AUTO","LIST", VI_NULL};
static ViString filterListArr[] = {"NORM","CHAN","RRC","P5","NOIS","PULS", VI_NULL};
static ViString addTraceNamesArr[] = {"ABIT","PWCD", VI_NULL};
static ViString phaseWrapTypeArr[] = {"PHAS","UPH", VI_NULL};
static ViString signalDestinationArr[] = {"IHIG","ILOW","QHIG","QLOW", VI_NULL};
static ViString signalVoltageArr[] = {"0","0.1","0.178","0.316","0.562","1.0", VI_NULL};
static ViString pulseFrequencyArr[] = {"10.0e3","62.5e3","80.0e3","100.0e3","102.4e3",
									   "200.0e3","500.0e3","1.0e6","2.0e6","4.0e6", VI_NULL};
static ViString basebandSourceArr[] = {"IQ","GND","CALD","CALP", VI_NULL};
static ViString trepOutputArr[] = {"SCR","REP", VI_NULL};
static ViString trepItemSelectorArr[] = {"HEAD","DIAG","FRES","PAG","SCAN","SRES","TDST","TRAN", VI_NULL};
static ViString trepItemStateArr[] = {"ALW","ONCE","NEV", VI_NULL};
static ViString trepOperationArr[] = {"LOAD","SAVE","DEL", VI_NULL};
static ViString ifShiftArr[] = {"OFF","A","B","AUTO", VI_NULL};
static ViString PLLBandwidthArr[] = {"NORM","WIDE", VI_NULL};
static ViString TOIMarkerArr[] = {"CALC","SEAR",VI_NULL};
static ViString bandArr[] = {"ABAN","BBAN",VI_NULL};
static ViString switchoverFrequencyArr[] = {"BBAN", VI_NULL};

/*****************************************************************************/
/*= INSTRUMENT-DEPENDENT STATUS/RANGE STRUCTURE  ============================*/
/*****************************************************************************/
/* rsesci_stringValPair is used in the rsesci_error_message function         */
/* rsesci_statusDataRanges is used to track session dependent status & ranges*/
/*===========================================================================*/
typedef struct  rsesci_stringValPair
{
   ViStatus stringVal;
   ViString stringName;
}  rsesci_tStringValPair;

struct rsesci_statusDataRanges
{
    ViBoolean   errorChecking;
    ViInt32     instrOption;
    ViInt32     instrModel;
    ViChar      instrDriverRevision[256];
};

typedef struct rsesci_statusDataRanges *rsesci_instrRange;

/*****************************************************************************/
/*= UTILITY ROUTINE DECLARATIONS (Non-Exportable Functions) =================*/
/*****************************************************************************/
void rsesci_delay (ViReal64 numberOfSeconds);
ViStatus rsesci_itemToConst (ViChar **array, ViString stringItem, ViPInt32 itemNo);
ViBoolean rsesci_invalidViBooleanRange (ViBoolean val);
ViBoolean rsesci_invalidViInt32Range (ViInt32 val, ViInt32 min, ViInt32 max);
ViBoolean rsesci_invalidViUInt32Range (ViUInt32 val, ViUInt32 min, ViUInt32 max);
ViBoolean rsesci_invalidViReal64Range (ViReal64 val, ViReal64 min, ViReal64 max);
ViBoolean rsesci_invalidViString (ViString val, ViUInt32 maxLength);
ViStatus rsesci_waitOPC (ViSession instrSession, ViPString command, ViPString instrResp, ViInt32 timeout);
ViStatus rsesci_initCleanUp (ViSession openRMSession, ViPSession openInstrSession, ViStatus currentStatus);
ViStatus rsesci_defaultInstrSetup (ViSession openInstrSession);

/*****************************************************************************/
/*------ INSERT INSTRUMENT-DEPENDENT UTILITY ROUTINE DECLARATIONS HERE ------*/
/*****************************************************************************/

ViStatus rsesci_checkStatus (ViSession instrSession);
ViStatus rsesci_sendCmd (ViSession instrSession, ViString cmd);
ViStatus rsesci_sendCmdWindow (ViSession instrSession, ViString cmd, ViInt32     window);
ViStatus rsesci_setViInt32 (ViSession instrSession,
                           ViString cmd,
                           ViInt32 value,
                           ViInt32 min,
                           ViInt32 max,
                           ViStatus parameterErrorCode);
ViStatus rsesci_getViInt32 (ViSession instrSession,
                           ViString cmd,
                           ViInt32 *value,
                           ViStatus parameterErrorCode);
ViStatus rsesci_setViInt32Window (ViSession instrSession,
                           ViString cmd,
                           ViInt32     window,
                           ViInt32 value,
                           ViInt32 min,
                           ViInt32 max,
                           ViStatus parameterErrorCode);
ViStatus rsesci_getViInt32Window (ViSession instrSession,
                           ViString cmd,
                           ViInt32     window,
                           ViInt32 *value,
                           ViStatus parameterErrorCode);
ViStatus rsesci_setViUInt32 (ViSession instrSession,
                           ViString cmd,
                           ViUInt32 value,
                           ViUInt32 min,
                           ViUInt32 max,
                           ViStatus parameterErrorCode);
ViStatus rsesci_getViUInt32 (ViSession instrSession,
                           ViString cmd,
                           ViUInt32 *value,
                           ViStatus parameterErrorCode);
ViStatus rsesci_setViUInt32Window (ViSession instrSession,
                           ViString cmd,
                           ViInt32     window,
                           ViUInt32 value,
                           ViUInt32 min,
                           ViUInt32 max,
                           ViStatus parameterErrorCode);
ViStatus rsesci_getViUInt32Window (ViSession instrSession,
                           ViString cmd,
                           ViInt32     window,
                           ViUInt32 *value,
                           ViStatus parameterErrorCode);
ViStatus rsesci_setViBoolean (ViSession instrSession,
                           ViString cmd,
                           ViBoolean value,
                           ViStatus parameterErrorCode);
ViStatus rsesci_getViBoolean (ViSession instrSession,
                           ViString cmd,
                           ViBoolean *value,
                           ViStatus parameterErrorCode);
ViStatus rsesci_setViBooleanWindow (ViSession instrSession,
                           ViString cmd,
                           ViInt32     window,
                           ViBoolean value,
                           ViStatus parameterErrorCode);
ViStatus rsesci_getViBooleanWindow (ViSession instrSession,
                           ViString cmd,
                           ViInt32     window,
                           ViBoolean *value,
                           ViStatus parameterErrorCode);
ViStatus rsesci_setViReal64 (ViSession vi,
                            ViString cmd,
                            ViReal64 value,
                            ViReal64 min,
                            ViReal64 max,
                            ViStatus parameterErrorCode);
ViStatus rsesci_getViReal64 (ViSession instrSession,
                            ViString cmd,
                            ViReal64 *value,
                            ViStatus parameterErrorCode);
ViStatus rsesci_setViReal64Window (ViSession instrSession,
                            ViString cmd,
                            ViInt32  window,
                            ViReal64 value,
                            ViReal64 min,
                            ViReal64 max,
                            ViStatus parameterErrorCode);
ViStatus rsesci_getViReal64Window (ViSession instrSession,
                            ViString cmd,
                            ViInt32     window,
                            ViReal64 *value,
                            ViStatus parameterErrorCode);
ViStatus rsesci_setViString (ViSession instrSession,
                            ViString cmd,
                            ViString value,
                            ViStatus parameterErrorValue);
ViStatus rsesci_getViString (ViSession instrSession,
                            ViString cmd,
                            ViString value,
                            ViInt32 buffer_size,
                            ViStatus parameterErrorCode);
ViStatus rsesci_setViStringQuoted (ViSession instrSession,
                            ViString cmd,
                            ViString quotes,
                            ViString value,
                            ViStatus parameterErrorValue);
ViStatus rsesci_getViStringQuoted (ViSession instrSession,
                                  ViString cmd,
                                  ViString quotes,
                                  ViInt32  arraySize,
                                  ViChar   value[],
                                  ViStatus parameterErrorValue);
ViStatus rsesci_setViStringWindow (ViSession instrSession,
                            ViString cmd,
                            ViInt32     window,
                            ViString value,
                            ViStatus parameterErrorValue);
ViStatus rsesci_getViStringWindow (ViSession instrSession,
                                ViString cmd,
                                ViInt32     window,
                                ViString value,
                                ViInt32 buffer_size,
                                ViStatus parameterErrorCode);
ViStatus rsesci_setEnum (ViSession instrSession,
                        ViString cmd,
                        ViInt32 value,
                        ViString list[],
                        ViStatus parameterErrorCode);
ViStatus rsesci_getEnum (ViSession instrSession,
                        ViString cmd,
                        ViInt32 *value,
                        ViString list[],
                        ViStatus parameterErrorCode);
ViStatus rsesci_setEnumWindow (ViSession instrSession,
                        ViString cmd,
                        ViInt32     window,
                        ViInt32 value,
                        ViString list[],
                        ViStatus parameterErrorCode);
ViStatus rsesci_getEnumWindow (ViSession instrSession,
                        ViString cmd,
                        ViInt32     window,
                        ViInt32 *value,
                        ViString list[],
                        ViStatus parameterErrorCode);
ViStatus rsesci_getViReal64Array (ViSession instrSession,
                            ViString cmd,
                            ViReal64 *value,
                            ViInt32  count,
                            ViStatus parameterErrorCode);
ViStatus rsesci_setViReal64Array (ViSession instrSession,
                            ViString cmd,
                            ViReal64 *value,
                            ViInt32  count,
                            ViStatus parameterErrorCode);
ViStatus rsesci_setViReal64ArrayWindow (ViSession instrSession,
                            ViString cmd,
                            ViInt32     window,
                            ViReal64 *value,
                            ViInt32  count,
                            ViStatus parameterErrorCode);
ViStatus rsesci_getViReal64ArrayWindow (ViSession instrSession,
                            ViString cmd,
                            ViInt32     window,
                            ViReal64 *value,
                            ViInt32  count,
                            ViStatus parameterErrorCode);
ViStatus rsesci_getViReal64BinArray (ViSession instrSession,
                            ViString    cmd,
                            ViReal64    *value,
                            ViInt32     maxCount,
                            ViInt32     *retCount,
                            ViStatus    parameterErrorCode);
ViStatus rsesci_checkIndexInList (ViSession instrSession,
                                 ViInt32 index,
                                 ViString list[],
                                 ViStatus statusIfError);
ViStatus rsesci_getIndexFromString (ViSession instrSession,
                                   ViString string,
                                   ViString *list,
                                   ViInt32 *index);
ViStatus rsesci_readDataUnknownLength(ViSession instrSession,
                                     ViChar **outBuffer,
                                     ViUInt32 *count);
ViStatus rsesci_clearBeforeRead (ViSession instrSession);
ViStatus rsesci_checkBeforeRead (ViSession instrSession);
void rsesci_SWAP_32(unsigned char *data, long points);
void rsesci_coerceToNearest (void *table, ViPReal64 value, ViPInt16 index);
ViStatus rsesci_checkOption (ViSession instrSession, ViInt32 option);
ViStatus rsesci_checkModel (ViSession instrSession, ViInt32 model);
ViStatus rsesci_ParseCatalog(ViChar *buffer, ViInt32 size, ViChar *catalogList, ViInt32 *nbValues);

/*****************************************************************************/
/*====== USER-CALLABLE FUNCTIONS (Exportable Functions) =====================*/
/*****************************************************************************/

/*===========================================================================*/
/* Function: Initialize                                                      */
/* Purpose:  This function opens the instrument, queries the instrument      */
/*           for its ID, and initializes the instrument to a known state.    */
/*===========================================================================*/
ViStatus _VI_FUNC rsesci_init (ViRsrc resourceName, ViBoolean IDQuery,
                               ViBoolean resetDevice, ViPSession instrSession)
{
    ViStatus            rsesci_status = VI_SUCCESS;
    ViUInt16            intfc = VI_INTF_GPIB;
    ViSession           rmSession = 0;
    ViChar              buffer[BUFFER_SIZE];
    rsesci_instrRange   instrPtr;

    /*- Check input parameter ranges ----------------------------------------*/
    if (rsesci_invalidViBooleanRange (IDQuery))
        return VI_ERROR_PARAMETER2;
    if (rsesci_invalidViBooleanRange (resetDevice))
        return VI_ERROR_PARAMETER3;

    /*- Open instrument session ---------------------------------------------*/
    CHECKERR( viOpenDefaultRM (&rmSession));

    if ((rsesci_status = viOpen (rmSession, resourceName, VI_LOAD_CONFIG,
                                 VI_NULL, instrSession)) < 0)
        {
        viClose (rmSession);
        return rsesci_status;
        }

    /* Determine if the structure has been initialized for the current VISA  */
    /* Session and malloc if it has not.                                     */
    CHECKERR_CU( viGetAttribute (*instrSession, VI_ATTR_USER_DATA, &instrPtr));

    if (instrPtr == NULL)
        instrPtr = malloc (sizeof (struct rsesci_statusDataRanges));

    CHECKERR_CU( viSetAttribute (*instrSession, VI_ATTR_USER_DATA, (ViAttrState)instrPtr));

    /* --- Interface specific operations --- */
    CHECKERR_CU( viGetAttribute (*instrSession, VI_ATTR_INTF_TYPE, &intfc));

    if (intfc == VI_INTF_ASRL) /* ASRL */
        {
        CHECKERR_CU( viSetAttribute (*instrSession, VI_ATTR_TERMCHAR, '\n'));
        CHECKERR_CU( viSetAttribute (*instrSession, VI_ATTR_TERMCHAR_EN, VI_TRUE));
        CHECKERR_CU( viSetAttribute (*instrSession, VI_ATTR_ASRL_END_IN, VI_ASRL_END_TERMCHAR));
        /* Changed to VI_ASRL_END_NONE due to Agilent VISA */
        CHECKERR_CU( viSetAttribute (*instrSession, VI_ATTR_ASRL_END_OUT, VI_ASRL_END_NONE));
        CHECKERR_CU( viFlush (*instrSession, VI_ASRL_IN_BUF));
        }
    else /* GPIB, RSIB */
        {
        /* VI_ATTR_SEND_END_EN will stay with this attribute set to VI_TRUE */
        CHECKERR_CU( viSetAttribute (*instrSession, VI_ATTR_SEND_END_EN, VI_TRUE));
        }

    /*- Configure VISA Formatted I/O ----------------------------------------*/
    CHECKERR_CU( viSetAttribute (*instrSession, VI_ATTR_TMO_VALUE, RSESCI_DEFAULT_TIMEOUT));
    CHECKERR_CU( viSetBuf (*instrSession, VI_READ_BUF|VI_WRITE_BUF, BUFFER_SIZE));
    CHECKERR_CU( viSetAttribute (*instrSession, VI_ATTR_WR_BUF_OPER_MODE, VI_FLUSH_ON_ACCESS));
    CHECKERR_CU( viSetAttribute (*instrSession, VI_ATTR_RD_BUF_OPER_MODE, VI_FLUSH_ON_ACCESS));

    /* --- Interface specific operations --- */
    switch (intfc)
        {
        case VI_INTF_ASRL: /* Serial Interface. Go to remote mode. */
            CHECKERR_CU( viWrite (*instrSession, (ViBuf) "@REM\n", 5, VI_NULL));
            break;
	  /* not necessarry for LAN    
#ifdef VI_INTF_TCPIP

        case VI_INTF_TCPIP: * RSIB Interface. Go to remote mode. *
            CHECKERR_CU( viSetAttribute (*instrSession, RS_ATTR_RSIB_SRE, 1));
            break;

#endif
	*/		
        }

    /*- Identification Query ------------------------------------------------*/
    if (IDQuery)
        {
        ViInt32 instrModel = 0x0;
        ViInt32 idx = 0;

        CHECKERR_CU( viQueryf (*instrSession, "*IDN?\n", "%*[^,],%[^,],%*[\r\n]",
                               buffer));

        /* Search if instrument is supported or not */
        for ( idx = 0; instrList[idx]; idx++ )
            if (!strncmp (buffer, instrList[idx], strlen(instrList[idx])))
                instrModel |= (1 << idx);

        if (instrModel == 0)
            return rsesci_initCleanUp (rmSession, instrSession, VI_ERROR_FAIL_ID_QUERY);
        }

    /*- Reset instrument ----------------------------------------------------*/
    if (resetDevice)
        CHECKERR_CU( rsesci_reset (*instrSession));
    else  /*- Send Default Instrument Setup ---------------------------------*/
        CHECKERR_CU( rsesci_defaultInstrSetup (*instrSession));

    return rsesci_status;
}

/*****************************************************************************/
/*-------- INSERT USER-CALLABLE INSTRUMENT-DEPENDENT ROUTINES HERE ----------*/
/*****************************************************************************/

/*****************************************************************************
 * Function: Read Spectrum
 * Purpose:  This function is an simple example how to use instrument driver
 *           functions. This function allows to set up start and stop
 *           frequencies, sets the continual or single sweeping and tranfers
 *           trace data from the instrument without reconfiguring it.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_ReadSpectrum (ViSession instrSession,
                ViInt32 measurementMode, ViInt32 frequencyParameter,
                ViReal64 startFrequency, ViReal64 stopFrequency,
                ViPInt32 samplesReturned, ViReal64 _VI_FAR traceData[])
{
    ViStatus rsesci_status  = VI_SUCCESS;
    ViInt32  swePoints      = 501; /* Fixed number of sweep points, do not change! */

    switch (measurementMode)
        {
        case 0: /* Set Up Frequencies */
            if (rsesci_invalidViInt32Range (frequencyParameter, 0, 2))
                return VI_ERROR_PARAMETER3;
            /*
            if (rsesci_invalidViReal64Range (startFrequency, 0.0, 3.0e9))
                return VI_ERROR_PARAMETER4;
            if (rsesci_invalidViReal64Range (startFrequency, 0.0, 3.0e9))
                return VI_ERROR_PARAMETER5;
            */
            CHECKERR (rsesci_SetFrequencyOffset (instrSession, RSESCI_SCREEN_A, 0.0));

            switch (frequencyParameter)
                {
                case 0:
                    CHECKERR (rsesci_SetStartFrequency (instrSession, RSESCI_SCREEN_A, startFrequency));
                    CHECKERR (rsesci_SetStopFrequency (instrSession, RSESCI_SCREEN_A, stopFrequency));
                    break;
                case 1:
                    CHECKERR (rsesci_SetStartFrequency (instrSession, RSESCI_SCREEN_A, startFrequency));
                    break;
                case 2:
                    CHECKERR (rsesci_SetStopFrequency (instrSession, RSESCI_SCREEN_A, stopFrequency));
                    break;
                }

            break;
        case 1: /* Continual Measurement */
            CHECKERR (rsesci_SetSweepPoints (instrSession, RSESCI_SCREEN_A, swePoints));
            CHECKERR (rsesci_SetSweepMode (instrSession, RSESCI_SCREEN_A, RSESCI_CONTINUOUS_SWEEP));
            break;
        case 2: /* Single Measurement */
            CHECKERR (rsesci_SetSweepPoints (instrSession, RSESCI_SCREEN_A, swePoints));
            CHECKERR (rsesci_SetSweepMode (instrSession, RSESCI_SCREEN_A, RSESCI_SINGLE_SWEEP));
            CHECKERR (rsesci_SendTriggerWaitOPC (instrSession, RSESCI_SCREEN_A, RSESCI_DEFAULT_TIMEOUT));
            break;
        case 3: /* Transfer Data Only */
            if (!samplesReturned)
                return VI_ERROR_PARAMETER6;
            if (!traceData)
                return VI_ERROR_PARAMETER7;

            CHECKERR (rsesci_ReadTraceData (instrSession, RSESCI_SCREEN_A, 1, samplesReturned, traceData));

            break;
        default:
            return VI_ERROR_PARAMETER2;
        }

    return rsesci_status;
}

/*****************************************************************************
 * Function: Set Center Frequency
 * Purpose:  This function defines the receiver frequency or the center
 *           frequency in analyzer mode.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetCenterFrequency(
    ViSession   instrSession,
    ViInt32     window,
    ViReal64    centerFrequency
)
{
    return rsesci_setViReal64Window (instrSession,
                                     "SENS%ld:FREQ:CENT",
                                     window,
                                     centerFrequency,
                                     0.0, 0.0,
                                     VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Center Frequency
 * Purpose:  This function returns the receiver frequency or the center
 *           frequency in analyzer mode.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetCenterFrequency(
    ViSession   instrSession,
    ViInt32     window,
    ViReal64*   centerFrequency
)
{
    return rsesci_getViReal64Window (instrSession,
                                     "SENS%ld:FREQ:CENT?",
                                     window,
                                     centerFrequency,
                                     VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set Start Frequency
 * Purpose:  This function defines the start frequency of the overall scan in
 *           receiver mode and defines the start frequency of the sweep in
 *           analyzer mode.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetStartFrequency(
    ViSession   instrSession,
    ViInt32     window,
    ViReal64    startFrequency
)
{
    return rsesci_setViReal64Window (instrSession,
                                     "SENS%ld:FREQ:STAR",
                                     window,
                                     startFrequency,
                                     0.0, 0.0,
                                     VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Start Frequency
 * Purpose:  This function returns the start frequency of the overall scan in
 *           receiver mode and returns the start frequency of the sweep in
 *           analyzer mode.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetStartFrequency(
    ViSession   instrSession,
    ViInt32     window,
    ViReal64*   startFrequency
)
{
    return rsesci_getViReal64Window (instrSession,
                                     "SENS%ld:FREQ:STAR?",
                                     window,
                                     startFrequency,
                                     VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set Stop Frequency
 * Purpose:  This function defines the stop frequency of the overall scan in
 *           receiver mode and defines the stop frequency of the sweep in
 *           analyzer mode.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetStopFrequency(
    ViSession   instrSession,
    ViInt32     window,
    ViReal64    stopFrequency
)
{
    return rsesci_setViReal64Window (instrSession,
                                     "SENS%ld:FREQ:STOP",
                                     window,
                                     stopFrequency,
                                     0.0 , 0.0,
                                     VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Stop Frequency
 * Purpose:  This function returns the stop frequency of the overall scan in
 *           receiver mode and returns the stop frequency of the sweep in
 *           analyzer mode.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetStopFrequency(
    ViSession   instrSession,
    ViInt32     window,
    ViReal64*   stopFrequency
)
{
    return rsesci_getViReal64Window (instrSession,
                                     "SENS%ld:FREQ:STOP?",
                                     window,
                                     stopFrequency,
                                     VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set Frequency Offset
 * Purpose:  This function defines the frequency offset of the instrument.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetFrequencyOffset(
    ViSession   instrSession,
    ViInt32     window,
    ViReal64    frequencyOffset
)
{
    return rsesci_setViReal64Window (instrSession,
                                     "SENS%ld:FREQ:OFFS",
                                     window,
                                     frequencyOffset,
                                     -100.0e9 , 100.0e9,
                                     VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Frequency Offset
 * Purpose:  This function returns the frequency offset of the instrument.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetFrequencyOffset(
    ViSession   instrSession,
    ViInt32     window,
    ViReal64*   frequencyOffset
)
{
    return rsesci_getViReal64Window (instrSession,
                                     "SENS%ld:FREQ:OFFS?",
                                     window,
                                     frequencyOffset,
                                     VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set Center Frequency Step Width
 * Purpose:  This function defines the step width of the center frequency.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetCenterFrequencyStepWidth(
    ViSession   instrSession,
    ViInt32     window,
    ViReal64    stepWidth
)
{
    return rsesci_setViReal64Window (instrSession,
                                     "SENS%ld:FREQ:CENT:STEP",
                                     window,
                                     stepWidth,
                                     0.0 , 40.0e9,
                                     VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Center Frequency Step Width
 * Purpose:  This function returns the step width of the center frequency.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetCenterFrequencyStepWidth(
    ViSession   instrSession,
    ViInt32     window,
    ViReal64*   stepWidth
)
{
    return rsesci_getViReal64Window (instrSession,
                                     "SENS%ld:FREQ:CENT:STEP?",
                                     window,
                                     stepWidth,
                                     VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set Center Frequency Step Link
 * Purpose:  This function couples the step width of the center frequency to
 *           span (span >0) or to the resolution bandwidth (span = 0) or
 *           cancels the couplings.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetCenterFrequencyStepLink(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32 stepLink
)
{
    return rsesci_setEnumWindow (instrSession,
                                 "SENS%ld:FREQ:CENT:STEP:LINK",
                                 window,
                                 stepLink,
                                 steplinkArr,
                                 VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Center Frequency Step Link
 * Purpose:  This function returns the step link.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetCenterFrequencyStepLink(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32*    stepLink
)
{
    return rsesci_getEnumWindow (instrSession,
                                 "SENS%ld:FREQ:CENT:STEP:LINK?",
                                 window,
                                 stepLink,
                                 steplinkArr,
                                 VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set Center Frequency Step Link Factor
 * Purpose:  This function couples the step width of the center frequency with
 *           a factor to the span (span >0) or to the resolution bandwidth
 *           (span = 0).
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetCenterFrequencyStepLinkFactor(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32 stepLinkFactor
)
{
    return rsesci_setViInt32Window (instrSession,
                                    "SENS%ld:FREQ:CENT:STEP:LINK:FACT",
                                    window,
                                    stepLinkFactor,
                                    1 , 100,
                                    VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Center Frequency Step Link Factor
 * Purpose:  This function returns the step likk factor.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetCenterFrequencyStepLinkFactor(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32*    stepLinkFactor
)
{
    return rsesci_getViInt32Window (instrSession,
                                    "SENS%ld:FREQ:CENT:STEP:LINK:FACT?",
                                    window,
                                    stepLinkFactor,
                                    VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set Frequency Mode
 * Purpose:  This function switches between single frequency measurement (CW)
 *           and scan (SCAN), if in EMI receiver operating mode.
 *
 *           In SPECTRUM mode it switches between frequency domain (SWEep) and
 *           time domain (CW | FIXed).
 *
 *           In SCAN mode the frequency setup is performed using the commands
 *           FREQuency:STARt, FREQuency:STOP in the SENSe:SCAN subsystem.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetFrequencyMode(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32 frequencyMode
)
{
	if (frequencyMode == RSESCI_FREQ_TDOM)
	{
		CHECKMODEL(RSESU8 | RSESU26 | RSESU40);
		CHECKOPT(RSESU_K53);
	}
	
    return rsesci_setEnumWindow (instrSession,
                                 "SENS%ld:FREQ:MODE",
                                 window,
                                 frequencyMode,
                                 freqModeArr,
                                 VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Frequency Mode
 * Purpose:  This function returns frequency measurement mode or receiver
 *           operating mode.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetFrequencyMode(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32*    frequencyMode
)
{
    return rsesci_getEnumWindow (instrSession,
                                 "SENS%ld:FREQ:MODE?",
                                 window,
                                 frequencyMode,
                                 freqModeArr,
                                 VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set Signal Tracking
 * Purpose:  This function switches the signal-track function on or off for
 *           the selected measurement window.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetSignalTracking (ViSession instrSession,
                                            ViInt32     window,
                                            ViBoolean signalTracking)
{
    return rsesci_setViBooleanWindow (instrSession,
                                      "CALC%ld:MARK1:FUNC:STR:STAT",
                                      window,
                                      signalTracking,
                                      VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Signal Tracking
 * Purpose:  This function returns the signal-track function state for the
 *           selected measurement window.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetSignalTracking (ViSession instrSession,
                                            ViInt32     window,
                                            ViBoolean *signalTracking)
{
    return rsesci_getViBooleanWindow (instrSession,
                                      "CALC%ld:MARK1:FUNC:STR:STAT?",
                                      window,
                                      signalTracking,
                                      VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set Signal Tracking Bandwidth
 * Purpose:  This function define the bandwidth around the center frequency
 *           within which the largest signal is searched.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetSignalTrackingBandwidth (ViSession instrSession,
                                                     ViInt32     window,
                                                     ViReal64 signalTrackingBandwidth)
{
    return rsesci_setViReal64Window (instrSession,
                                     "CALC%ld:MARK1:FUNC:STR:BAND",
                                     window,
                                     signalTrackingBandwidth,
                                     10.0 , 40.0e9,
                                     VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Signal Tracking Bandwidth
 * Purpose:  This function returns the bandwidth around the center frequency
 *           within which the largest signal is searched.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetSignalTrackingBandwidth (ViSession instrSession,
                                                     ViInt32     window,
                                                     ViReal64 *signalTrackingBandwidth)
{
    return rsesci_getViReal64Window (instrSession,
                                     "CALC%ld:MARK1:FUNC:STR:BAND?",
                                     window,
                                     signalTrackingBandwidth,
                                     VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set Signal Tracking Threshold
 * Purpose:  This function defines the threshold above which the largest
 *           signal is searched for in the selected measurement window.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetSignalTrackingThreshold (ViSession instrSession,
                                                     ViInt32     window,
                                                     ViReal64 signalTrackingThreshold)
{
    return rsesci_setViReal64Window (instrSession,
                                     "CALC%ld:MARK1:FUNC:STR:THR",
                                     window,
                                     signalTrackingThreshold,
                                     -330.0 , 30.0,
                                     VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Signal Tracking Threshold
 * Purpose:  This function returns the threshold above which the largest
 *           signal is searched for in the selected measurement window.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetSignalTrackingThreshold (ViSession instrSession,
                                                     ViInt32     window,
                                                     ViReal64 *signalTrackingThreshold)
{
    return rsesci_getViReal64Window (instrSession,
                                     "CALC%ld:MARK1:FUNC:STR:THR?",
                                     window,
                                     signalTrackingThreshold,
                                     VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set Signal Tracking Trace
 * Purpose:  This function defines the trace on which the largest signal is
 *           searched for in the selected measurement window.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetSignalTrackingTrace (ViSession instrSession,
                                                 ViInt32     window,
                                                 ViInt32 signalTrackingTrace)
{
    return rsesci_setViInt32Window (instrSession,
                                    "CALC%ld:MARK1:FUNC:STR:TRAC",
                                    window,
                                    signalTrackingTrace,
                                    1, 3,
                                    VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Signal Tracking Trace
 * Purpose:  This function returns the trace on which the largest signal is
 *           searched for in the selected measurement window.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetSignalTrackingTrace (ViSession instrSession,
                                                 ViInt32     window,
                                                 ViInt32 *signalTrackingTrace)
{
    return rsesci_getViInt32Window (instrSession,
                                    "CALC%ld:MARK1:FUNC:STR:TRAC?",
                                    window,
                                    signalTrackingTrace,
                                    VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set Frequency Span
 * Purpose:  This function defines the frequency span in the analyzer mode.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetFrequencySpan(
    ViSession   instrSession,
    ViInt32     window,
    ViReal64    frequencySpan
)
{
    return rsesci_setViReal64Window (instrSession,
                                     "SENS%ld:FREQ:SPAN",
                                     window,
                                     frequencySpan,
                                     0.0 , 0.0,
                                     VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Frequency Span
 * Purpose:  This function returns the frequency span in the analyzer mode.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetFrequencySpan(
    ViSession   instrSession,
    ViInt32     window,
    ViReal64*   frequencySpan
)
{
    return rsesci_getViReal64Window (instrSession,
                                     "SENS%ld:FREQ:SPAN?",
                                     window,
                                     frequencySpan,
                                     VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set Frequency Full Span
 * Purpose:  This function sets the frequency span in the analyzer mode to its 
 *           maximum.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetFrequencyFullSpan (ViSession instrSession,
                                               ViInt32 window)
{
	return rsesci_sendCmdWindow (instrSession,
                                 "SENS%ld:FREQ:SPAN:FULL",
                                 window);    
}

/*****************************************************************************
 * Function: Set Reference Level
 * Purpose:  This function defines the reference level in the selected
 *           measurement window.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetReferenceLevel (ViSession instrSession,
                                            ViInt32     window,
                                            ViReal64 referenceLevel)
{
    return rsesci_setViReal64Window (instrSession,
                                     "DISP:WIND%ld:TRAC:Y:SCAL:RLEV",
                                     window,
                                     referenceLevel,
                                     0.0 , 0.0 ,
                                     VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Reference Level
 * Purpose:  This function returns the reference level in the selected
 *           measurement window.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetReferenceLevel (ViSession instrSession,
                                            ViInt32     window,
                                            ViReal64 *referenceLevel)
{
    return rsesci_getViReal64Window (instrSession,
                                     "DISP:WIND%ld:TRAC:Y:SCAL:RLEV?",
                                     window,
                                     referenceLevel,
                                     VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set Reference Level Offset
 * Purpose:  This function defines the offset of the reference level in the
 *           selected measurement window.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetReferenceLevelOffset (ViSession instrSession,
                                                  ViInt32     window,
                                                  ViReal64 referenceLevelOffset)
{
    return rsesci_setViReal64Window (instrSession,
                                     "DISP:WIND%ld:TRAC:Y:SCAL:RLEV:OFFS",
                                     window,
                                     referenceLevelOffset,
                                     -200.0 , 200.0,
                                     VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Reference Level Offset
 * Purpose:  This function returns the offset of the reference level in the
 *           selected measurement window.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetReferenceLevelOffset (ViSession instrSession,
                                                  ViInt32     window,
                                                  ViReal64 *referenceLevelOffset)
{
    return rsesci_getViReal64Window (instrSession,
                                     "DISP:WIND%ld:TRAC:Y:SCAL:RLEV:OFFS?",
                                     window,
                                     referenceLevelOffset,
                                     VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set Reference Level Position
 * Purpose:  This function defines the position of the reference value in the
 *           selected measurement window.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetReferenceLevelPosition (ViSession instrSession,
                                                    ViInt32     window,
                                                    ViReal64 referenceLevelPosition)
{
    return rsesci_setViReal64Window (instrSession,
                                     "DISP:WIND%ld:TRAC:Y:SCAL:RPOS",
                                     window,
                                     referenceLevelPosition,
                                     0.0, 100.0,
                                     VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Reference Level Position
 * Purpose:  This function returns the position of the reference value in the
 *           selected measurement window.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetReferenceLevelPosition (
    ViSession instrSession,
    ViInt32     window,
    ViReal64 *referenceLevelPosition
)
{
    return rsesci_getViReal64Window (instrSession,
                                     "DISP:WIND%ld:TRAC:Y:SCAL:RPOS?",
                                     window,
                                     referenceLevelPosition,
                                     VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set Reference Level Position Value
 * Purpose:  If the tracking generator option or the external generator
 *           control option (R&S FSP-B9/B10) is mounted and the normalization
 *           in the NETWORK mode is activated, this value defines the power
 *           value assigned to the reference position in the selected
 *           measurement window.
*****************************************************************************/
ViStatus _VI_FUNC rsesci_SetReferenceLevelPositionValue (
    ViSession instrSession,
    ViInt32     window,
    ViReal64 referenceLevelPosition
)
{
    CHECKOPT (RSESCI_B9 | RSESCI_B10 | RSESCI_K7);

    return rsesci_setViReal64Window (instrSession,
                                     "DISP:WIND%ld:TRACE:Y:SCAL:RVAL",
                                     window,
                                     referenceLevelPosition,
                                     0 , 100,
                                     VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Reference Level Position Value
 * Purpose:  This function returns the position of the reference value in the
 *           selected measurement window.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetReferenceLevelPositionValue (
    ViSession instrSession,
    ViInt32     window,
    ViReal64 *referenceLevelPosition
)
{
    CHECKOPT (RSESCI_B9 | RSESCI_B10 | RSESCI_K7);

    return rsesci_getViReal64Window (instrSession,
                                     "DISP:WIND%ld:TRACE:Y:SCAL:RVAL?",
                                     window,
                                     referenceLevelPosition,
                                     VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set Y-Axis Scale Division
 * Purpose:  This function defines the scaling of the Y axis in the current unit.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetYAxisScaleDivision (ViSession instrSession,
                                                ViInt32 window,
                                                ViReal64 yScaleDiv)
{
	CHECKOPT (RSESCI_K7);

    return rsesci_setViReal64Window (instrSession,
                                     "DISP:WIND%ld:TRACE:Y:SCAL:PDIV",
                                     window,
                                     yScaleDiv,
                                     0.1, 20.0,
                                     VI_ERROR_PARAMETER3);    
}

/*****************************************************************************
 * Function: Get Y-Axis Scale Division
 * Purpose:  This function returns the scaling of the Y axis in the current unit.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetYAxisScaleDivision (ViSession instrSession,
                                                ViInt32 window,
                                                ViReal64 *yScaleDiv)
{
	CHECKOPT (RSESCI_K7);

    return rsesci_getViReal64Window (instrSession,
                                     "DISP:WIND%ld:TRACE:Y:SCAL:PDIV?",
                                     window,
                                     yScaleDiv,
                                     VI_ERROR_PARAMETER3);    
}

/*****************************************************************************
 * Function: Set Level Axis Range
 * Purpose:  This function defines the display range of the Y axis (level
 *           axis) in the selected measurement window with logarithmic scaling
 *           (DISP:TRAC:Y:SPAC LOG).
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetLevelAxisRange (ViSession instrSession,
                                            ViInt32     window,
                                            ViReal64 levelAxisRange)
{
    return rsesci_setViReal64Window (instrSession,
                                    "DISP:WIND%ld:TRAC:Y:SCAL",
                                    window,
                                    levelAxisRange,
                                    10.0 , 200.0,
                                    VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Level Axis Range
 * Purpose:  This function returns the display range of the Y axis (level
 *           axis) in the selected measurement window with logarithmic scaling
 *           (DISP:TRAC:Y:SPAC LOG).
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetLevelAxisRange (ViSession instrSession,
                                            ViInt32     window,
                                            ViReal64 *levelAxisRange)
{
    return rsesci_getViReal64Window (instrSession,
                                    "DISP:WIND%ld:TRAC:Y:SCAL?",
                                    window,
                                    levelAxisRange,
                                    VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set Level Axis Minimum
 * Purpose:  This function defines the minimum grid level in the current unit
 *           for the scan display in the receiver mode.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetLevelAxisMinimum (ViSession instrSession,
                                              ViInt32     window,
                                              ViReal64 levelAxisMinimum)
{
    return rsesci_setViReal64Window (instrSession,
                                     "DISP:WIND%ld:TRAC:Y:SCAL:BOTT",
                                     window,
                                     levelAxisMinimum,
                                     0.0 , 0.0,
                                     VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Level Axis Minimum
 * Purpose:  This function returns the minimum grid level in the current unit
 *           for the scan display in the receiver mode.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetLevelAxisMinimum (ViSession instrSession,
                                              ViInt32     window,
                                              ViReal64 *levelAxisMinimum)
{
    return rsesci_getViReal64Window (instrSession,
                                     "DISP:WIND%ld:TRAC:Y:SCAL:BOTT?",
                                     window,
                                     levelAxisMinimum,
                                     VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set Level Axis Range Spacing
 * Purpose:  This function toggles between linear and logarithmic display in
 *           the selected measurement window.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetLevelAxisRangeSpacing (ViSession instrSession,
                                                   ViInt32     window,
                                                   ViInt32 levelAxisRangeSpacing)
{
    return rsesci_setEnumWindow (instrSession,
                                 "DISP:WIND%ld:TRAC:Y:SPAC",
                                 window,
                                 levelAxisRangeSpacing,
                                 spacingArr,
                                 VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Level Axis Range Spacing
 * Purpose:  This function returns display type in the selected measurement
 *           window.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetLevelAxisRangeSpacing (ViSession instrSession,
                                                   ViInt32     window,
                                                   ViInt32 *levelAxisRangeSpacing)
{
    return rsesci_getEnumWindow (instrSession,
                                 "DISP:WIND%ld:TRAC:Y:SPAC?",
                                 window,
                                 levelAxisRangeSpacing,
                                 spacingArr,
                                 VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set Level Axis Unit
 * Purpose:  This function selects the unit for power in the selected
 *           measurement window.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetLevelAxisUnit(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32 levelAxisUnit
)
{
	if ((rsesci_invalidViInt32Range (levelAxisUnit, RSESCI_UNIT_DBM, RSESCI_UNIT_DBUA_M)) && (rsesci_invalidViInt32Range (levelAxisUnit, RSESCI_UNIT_DBUV_MHZ, RSESCI_UNIT_DBUA_MHZ)))
        return VI_ERROR_PARAMETER3;
	
	if ((levelAxisUnit >= RSESCI_UNIT_DBUV_MHZ) && (levelAxisUnit <= RSESCI_UNIT_DBUA_MHZ))
		CHECKMODEL (RSESU8 | RSESU26 | RSESU40);
	
    return rsesci_setEnumWindow (instrSession,
                                 "CALC%ld:UNIT:POW",
                                 window,
                                 levelAxisUnit,
                                 setLimitPowerUnitArr,
                                 VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Level Axis Unit
 * Purpose:  This function returns the unit for power in the selected
 *           measurement window.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetLevelAxisUnit(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32*    levelAxisUnit
)
{
    return rsesci_getEnumWindow (instrSession,
                                 "CALC%ld:UNIT:POW?",
                                 window,
                                 levelAxisUnit,
                                 getLimitPowerUnitArr,
                                 VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set Level Axis Scaling
 * Purpose:  This function defines the scale type of the Y axis (absolute or
 *           relative) in the selected measurement window.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetLevelAxisScaling(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32 levelAxisScaling
)
{
    ViChar  buffer[BUFFER_SIZE];

    CHECKWINDOW;

    sprintf (buffer, "DISP:WIND%ld:TRAC:Y:SCAL:MODE", window);

    return rsesci_setEnum (instrSession,
                           buffer,
                           levelAxisScaling,
                           absRelArr,
                           VI_ERROR_PARAMETER4);
}

/*****************************************************************************
 * Function: Get Level Axis Scaling
 * Purpose:  This function returns the scale type of the Y axis (absolute or
 *           relative) in the selected measurement window.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetLevelAxisScaling(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32*    levelAxisScaling
)
{
    ViChar  buffer[BUFFER_SIZE];

    CHECKWINDOW;

    sprintf (buffer, "DISP:WIND%ld:TRAC:Y:SCAL:MODE?", window);

    return rsesci_getEnum (instrSession,
                           buffer,
                           levelAxisScaling,
                           absRelArr,
                           VI_ERROR_PARAMETER4);
}

/*****************************************************************************
 * Function: Set RF Input Coupling
 * Purpose:  This function switches the input coupling of the RF input between
 *           AC and DC.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetRFInputCoupling(
    ViSession   instrSession,
    ViInt32 rfInputCoupling
)
{
    return rsesci_setEnum (instrSession,
                           "INP:COUP",
                           rfInputCoupling,
                           couplingArr,
                           VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get RF Input Coupling
 * Purpose:  This function returns the input coupling of the RF input.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetRFInputCoupling(
    ViSession   instrSession,
    ViInt32*    rfInputCoupling
)
{
    return rsesci_getEnum (instrSession,
                           "INP:COUP?",
                           rfInputCoupling,
                           couplingArr,
                           VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set RF Input Attenuation
 * Purpose:  This function programs the input attenuator.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetRFInputAttenuation (ViSession instrSession,
                                                ViInt32     window,
                                                ViReal64 attenuation)
{
    return rsesci_setViReal64Window (instrSession,
                                     "INP%ld:ATT",
                                     window,
                                     attenuation,
                                     0.0 , 75.0 ,
                                     VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get RF Input Attenuation
 * Purpose:  This function returns the input attenuator.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetRFInputAttenuation (ViSession instrSession,
                                                ViInt32     window,
                                                ViReal64 *attenuation)
{
    return rsesci_getViReal64Window (instrSession,
                                     "INP%ld:ATT?",
                                     window,
                                     attenuation,
                                     VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set RF Input Attenuation Auto
 * Purpose:  In analyzer mode, this function automatically couples the input
 *           attenuation to the reference level (state ON) or switches the
 *           input attenuation to manual entry (state OFF). The minimum input
 *           attenuation set with the coupling switched on is 10 dB.
 *
 *           In receiver mode, this function automatically sets the
 *           attenuation so that a good S/N ratio is obtained without the
 *           receiver stages being overdriven (state ON).
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetRFInputAttenuationAuto(
    ViSession   instrSession,
    ViInt32     window,
    ViBoolean   attenuationAuto
)
{
    return rsesci_setViBooleanWindow (instrSession,
                                      "INP%ld:ATT:AUTO",
                                      window,
                                      attenuationAuto,
                                      VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get RF Input Attenuation Auto
 * Purpose:  This function returns auto attenuation state.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetRFInputAttenuationAuto(
    ViSession   instrSession,
    ViInt32     window,
    ViBoolean*  attenuationAuto
)
{
    return rsesci_getViBooleanWindow (instrSession,
                                      "INP%ld:ATT:AUTO?",
                                      window,
                                      attenuationAuto,
                                      VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set RF Input Attenuation Protection
 * Purpose:  This function defines whether the 0 dB position of the attenuator
 *           is to be used in manual or automatic adjustment.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetRFInputAttenuationProtection(
    ViSession   instrSession,
    ViInt32     window,
    ViBoolean   attenuationProtection
)
{
    return rsesci_setViBooleanWindow (instrSession,
                                      "INP%ld:ATT:PROT:STAT",
                                      window,
                                      attenuationProtection,
                                      VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get RF Input Attenuation Protection
 * Purpose:  This function returns whether the 0 dB position of the attenuator
 *           is to be used in manual or automatic adjustment.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetRFInputAttenuationProtection(
    ViSession   instrSession,
    ViInt32     window,
    ViBoolean*  attenuationProtection
)
{
    return rsesci_getViBooleanWindow (instrSession,
                                      "INP%ld:ATT:PROT:STAT?",
                                      window,
                                      attenuationProtection,
                                      VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Reset RF Input Attenuation Protection
 * Purpose:  This function resets the attenuator into the state that it had 
 *           before the overload condition was detected. It re-connects the RF 
 *           input with the input mixer.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_ResetRFInputAttenuationProtection (ViSession instrSession,
                                                            ViInt32 window)
{
	CHECKMODEL(RSESU8 | RSESU26 | RSESU40);
	
	return rsesci_sendCmdWindow (instrSession,
                                 "INP%ld:ATT:PROT:RES",
                                 window);    
}

/*****************************************************************************
 * Function: Set RF Input Impedance
 * Purpose:  This function sets the nominal input impedance of the instrument.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetRFInputImpedance(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32 rfInputImpedance
)
{
    return rsesci_setEnumWindow (instrSession,
                                 "INP%ld:IMP",
                                 window,
                                 rfInputImpedance,
                                 impedanceArr,
                                 VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get RF Input Impedance
 * Purpose:  This function returns the nominal input impedance of the
 *           instrument.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetRFInputImpedance(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32*    rfInputImpedance
)
{
    return rsesci_getEnumWindow (instrSession,
                                 "INP%ld:IMP?",
                                 window,
                                 rfInputImpedance,
                                 impedanceArr,
                                 VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set RF Input Type
 * Purpose:  This function selects the signal input.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetRFInputType (ViSession instrSession,
                                         ViInt32 window, ViInt32 inputType)
{
	CHECKMODEL(RSESU8 | RSESU26 | RSESU40);
	
	return rsesci_setEnumWindow (instrSession,
                                 "INP%ld:TYPE",
                                 window,
                                 inputType,
                                 inputTypeArr,
                                 VI_ERROR_PARAMETER3);    
}

/*****************************************************************************
 * Function: Get RF Input Type
 * Purpose:  This function returns the signal input.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetRFInputType (ViSession instrSession,
                                         ViInt32 window, ViInt32 *inputType)
{
	CHECKMODEL(RSESU8 | RSESU26 | RSESU40);
	
	return rsesci_getEnumWindow (instrSession,
                                 "INP%ld:TYPE?",
                                 window,
                                 inputType,
                                 inputTypeArr,
                                 VI_ERROR_PARAMETER3);    
}

/// HIFN  This function sets the switchover frequency of the 150 kHz highpass.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/The measurement windows are selected by means of this parameter.
/// HIPAR switchoverFrequency/This control sets the switchover frequency of the 150 kHz highpass.
ViStatus _VI_FUNC rsesci_SetRFInputSwitchoverFrequency(
	ViSession	instrSession,
	ViInt32	window,
	ViInt32	switchoverFrequency
)
{
 	CHECKMODEL(RSESU8 | RSESU26 | RSESU40);
	
	return rsesci_setEnumWindow (instrSession,
                                 "INP%ld:PRES:CUT",
                                 window,
                                 switchoverFrequency,
								 switchoverFrequencyArr,
                                 VI_ERROR_PARAMETER3);   
}


/// HIFN  This function returns the switchover frequency of the 150 kHz 
/// HIFN  highpass.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/The measurement windows are selected by means of this parameter.
/// HIPAR switchoverFrequency/This control returns the switchover frequency of the 150 kHz highpass.
ViStatus _VI_FUNC rsesci_GetRFInputSwitchoverFrequency(
	ViSession	instrSession,
	ViInt32	window,
	ViInt32*	switchoverFrequency
)
{
	CHECKMODEL(RSESU8 | RSESU26 | RSESU40);
	
	return rsesci_getEnumWindow (instrSession,
                                 "INP%ld:PRES:CUT?",
                                 window,
                                 switchoverFrequency,
								 switchoverFrequencyArr,
                                 VI_ERROR_PARAMETER3);    
}

/*****************************************************************************
 * Function: Set Preamplifier Auto
 * Purpose:  This function includes the preamplifier into the autoranging
 *           function of the receiver.
  *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetPreamplifierAuto(
    ViSession   instrSession,
    ViInt32     window,
    ViBoolean   preamplifierAuto
)
{
    return rsesci_setViBooleanWindow (instrSession,
                                      "INP%ld:GAIN:AUTO",
                                      window,
                                      preamplifierAuto,
                                      VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Preamplifier Auto
 * Purpose:  This function returns the preamplifier autoranging function
 *           state.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetPreamplifierAuto(
    ViSession   instrSession,
    ViInt32     window,
    ViBoolean*  preamplifierAuto
)
{
    return rsesci_getViBooleanWindow (instrSession,
                                      "INP%ld:GAIN:AUTO?",
                                      window,
                                      preamplifierAuto,
                                      VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set Preamplifier
 * Purpose:  This function switches on the preamplifier for the instrument.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetPreamplifier(
    ViSession   instrSession,
    ViInt32     window,
    ViBoolean   preamplifier
)
{
    return rsesci_setViBooleanWindow (instrSession,
                                      "INP%ld:GAIN:STAT",
                                      window,
                                      preamplifier,
                                      VI_ERROR_PARAMETER3);

}

/*****************************************************************************
 * Function: Get Preamplifier
 * Purpose:  This function returns the preamplifier state for the instrument.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetPreamplifier(
    ViSession   instrSession,
    ViInt32     window,
    ViBoolean*  preamplifier
)
{
    return rsesci_getViBooleanWindow (instrSession,
                                      "INP%ld:GAIN:STAT?",
                                      window,
                                      preamplifier,
                                      VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set Low Noise Preamplifier
 * Purpose:  This function switches switches the low noise preamplifier (100
 * 			 kHz to upper frequency limit of the instrument) on and off.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetLowNoisePreamplifier (
	ViSession instrSession,
	ViInt32   window,
	ViBoolean preamplifier
)
{
    return rsesci_setViBooleanWindow (instrSession,
                                      "INP%ld:GAIN:LNA:STAT",
                                      window,
                                      preamplifier,
                                      VI_ERROR_PARAMETER3);    
}

/*****************************************************************************
 * Function: Get PreaLow Noise Preamplifier
 * Purpose:  This function returns the state of low noise preamplifier.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetLowNoisePreamplifier (
	ViSession  instrSession,
	ViInt32    window,
	ViBoolean *preamplifier
)
{
    return rsesci_getViBooleanWindow (instrSession,
                                      "INP%ld:GAIN:LNA:STAT?",
                                      window,
                                      preamplifier,
                                      VI_ERROR_PARAMETER3);    
}

/*****************************************************************************
 * Function: Configure Input Mixer Level
 * Purpose:  This function configures input mixer of the analyzer.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_ConfigureInputMixerLevel (ViSession instrSession,
                                                   ViInt32 window,
                                                   ViBoolean automaticSetup,
                                                   ViReal64 power)
{
	ViStatus	rsesci_status = VI_SUCCESS;
	
	CHECKMODEL(RSESU8 | RSESU26 | RSESU40);
					
    CHECKERR (rsesci_SetInputMixerLevelState (instrSession, window, automaticSetup));
					
	rsesci_status = rsesci_SetInputMixerPower (instrSession, window, power);
    REMAP (3,4);
	
	return rsesci_status;
}

/*****************************************************************************
 * Function: Set Input Mixer Level State
 * Purpose:  This function enables/disables the automatic setup of the mixer level.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetInputMixerLevelState (ViSession instrSession,
                                                  ViInt32 window,
                                                  ViBoolean automaticSetup)
{
	CHECKMODEL(RSESU8 | RSESU26 | RSESU40);
	
	return rsesci_setViBooleanWindow (instrSession,
                                      "INP%ld:MIX:AUTO",
                                      window,
                                      automaticSetup,
                                      VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Input Mixer Level State 
 * Purpose:  This function returns the state of the automatic setup of the 
 *           mixer level.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetInputMixerLevelState (ViSession instrSession,
                                                  ViInt32 window,
                                                  ViBoolean *automaticSetup)
{
	CHECKMODEL(RSESU8 | RSESU26 | RSESU40);
	
	return rsesci_getViBooleanWindow (instrSession,
                                      "INP%ld:MIX:AUTO?",
                                      window,
                                      automaticSetup,
                                      VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set Input Mixer Level Power
 * Purpose:  This function defines the desired power at the input mixer of the 
 *           analyzer. On any change to the reference level the RF attenuation 
 *           will be  adjusted in a way that makes the difference between 
 *           reference level and RF attenuation come as close to the desired mixer
 *           level as possible.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetInputMixerPower (ViSession instrSession,
                                             ViInt32 window, ViReal64 power)
{
	CHECKMODEL(RSESU8 | RSESU26 | RSESU40);
	
	return rsesci_setViReal64Window (instrSession,
                                     "INP%ld:MIX:POW",
                                     window,
                                     power,
									 -100.0, 5.0,
                                     VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Input Mixer Level Power
 * Purpose:  This function returns the power at the input mixer of the analyzer.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetInputMixerPower (ViSession instrSession,
                                             ViInt32 window, ViReal64 *power)
{
	CHECKMODEL(RSESU8 | RSESU26 | RSESU40);
	
	return rsesci_getViReal64Window (instrSession,
                                     "INP%ld:MIX:POW?",
                                     window,
                                     power,
                                     VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set Main PLL Bandwidth
 * Purpose:  This function defines the bandwidth of the main PLL of the instrument
 *           synthesizer.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetMainPLLBandwidth (ViSession instrSession,
                                              ViInt32 window,
                                              ViInt32 PLLBandwidth)
{
	CHECKMODEL(RSESU8 | RSESU26 | RSESU40);
	
	return rsesci_setEnumWindow (instrSession,
                                 "SENS%ld:BWID:PLL",
                                 window,
                                 PLLBandwidth,
                                 setPLLBandwidthArr,
                                 VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Main PLL Bandwidth
 * Purpose:  This function returns the bandwidth of the main PLL of the instrument
 *           synthesizer.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetMainPLLBandwidth (ViSession instrSession,
                                              ViInt32 window,
                                              ViInt32 *PLLBandwidth)
{
 	CHECKMODEL(RSESU8 | RSESU26 | RSESU40);
	
	return rsesci_getEnumWindow (instrSession,
                                 "SENS%ld:BWID:PLL?",
                                 window,
                                 PLLBandwidth,
                                 getPLLBandwidthArr,
                                 VI_ERROR_PARAMETER3);   
}

/*****************************************************************************
 * Function: Set PLL Bandwidth Synchronization
 * Purpose:  This function controls the PLL bandwidth used to synchronize
             with the external reference signal and is available only with an
			 external reference frequency of 10 MHz. By default, it is set to
			 NORMal.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetPLLBandwidthSynchronization (ViSession instrSession,
                                                         ViInt32 window,
                                                         ViInt32 PLLBandwidth)
{
	
	 CHECKMODEL(RSESU8 | RSESU26 | RSESU40);
	 
     return rsesci_setEnumWindow (instrSession,
                                 "SENS%ld:ROSC:EXT:PLL",
                                 window,
                                 PLLBandwidth,
                                 PLLBandwidthArr,
                                 VI_ERROR_PARAMETER3);
	
}

/*****************************************************************************
 * Function: Get PLL Bandwidth Synchronization
 * Purpose:  This function queries the PLL bandwidth used to synchronize with
			 the external reference signal and is available only with an
			 external reference frequency of 10 MHz. By default, it is set to
			 NORMal.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetPLLBandwidthSynchronization (ViSession instrSession,
                                                         ViInt32 window,
                                                         ViInt32 *PLLBandwidth)
{
	
    CHECKMODEL(RSESU8 | RSESU26 | RSESU40);
	
	return rsesci_getEnumWindow (instrSession,
                                 "SENS%ld:ROSC:EXT:PLL?",
                                 window,
                                 PLLBandwidth,
                                 PLLBandwidthArr,
                                 VI_ERROR_PARAMETER3);  

}

/*****************************************************************************
 * Function: Set FFT Filter Mode
 * Purpose:  This function specifies the FFT filter.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetFFTFilterMode (ViSession instrSession,
                                           ViInt32 window, ViInt32 FFTFilterMode)
{
	CHECKMODEL(RSESU8 | RSESU26 | RSESU40);
	
	return rsesci_setEnumWindow (instrSession,
                                 "SENS%ld:BWID:FFT",
                                 window,
                                 FFTFilterMode,
                                 FFTFilterModeArr,
                                 VI_ERROR_PARAMETER3);    
}

/*****************************************************************************
 * Function: Get FFT Filter Mode
 * Purpose:  This function returns the FFT filter.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetFFTFilterMode (ViSession instrSession,
                                           ViInt32 window,
                                           ViInt32 *FFTFilterMode)
{
	CHECKMODEL(RSESU8 | RSESU26 | RSESU40);
	
	return rsesci_getEnumWindow (instrSession,
                                 "SENS%ld:BWID:FFT?",
                                 window,
                                 FFTFilterMode,
                                 FFTFilterModeArr,
                                 VI_ERROR_PARAMETER3);    
}

/*****************************************************************************
 * Function: Set IF Bandwidth
 * Purpose:  This function sets the resolution bandwidth of the IF analysis. 
 *           Digitally implemented resolution filters from 10 Hz to 100 kHz 
 *           in steps of 1, 3, and 10 are available (each with 6 dB bandwidths). 
 *           The reliable value range depends on the selected frequency span of
 *           the IF analysis.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetIFBandwidth (ViSession instrSession,
                                         ViInt32 window, ViReal64 IFBandwidth)
{
	CHECKMODEL(RSESU8 | RSESU26 | RSESU40);
	
	return rsesci_setViReal64Window (instrSession,
                                     "SENS%ld:BAND:IF",
                                     window,
                                     IFBandwidth,
                                     10.0, 100.0e3,
                                     VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get IF Bandwidth
 * Purpose:  This function gets the resolution bandwidth of the IF analysis.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetIFBandwidth (ViSession instrSession,
                                         ViInt32 window, ViReal64 *IFBandwidth)
{
	CHECKMODEL(RSESU8 | RSESU26 | RSESU40);
	
	return rsesci_getViReal64Window (instrSession,
                                     "SENS%ld:BAND:IF?",
                                     window,
                                     IFBandwidth,
                                     VI_ERROR_PARAMETER3);    
}

/*****************************************************************************
 * Function: Set Demodulation Bandwidth
 * Purpose:  This function defines the demodulation bandwidth of the instrument 
 *           for analog demodulation. The required sampling rate is automatically 
 *           set depending on the selected demodulation bandwidth. The available
 *           demodulation bandwidths are determined by the existing sampling
 *           rates.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetDemodulationBandwidth (ViSession instrSession,
                                                   ViInt32 window,
                                                   ViReal64 demodulationBandwidth)
{
	CHECKOPT(RSESCI_K7);
	
	return rsesci_setViReal64Window (instrSession,
                                     "SENS%ld:BWID:DEM",
                                     window,
                                     demodulationBandwidth,
                                     0, 0.0,
                                     VI_ERROR_PARAMETER3);    
}

/*****************************************************************************
 * Function: Get Demodulation Bandwidth
 * Purpose:  This function returns the demodulation bandwidth of the instrument 
 *           for analog demodulation.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetDemodulationBandwidth (ViSession instrSession,
                                                   ViInt32 window,
                                                   ViReal64 *demodulationBandwidth)
{
	CHECKOPT(RSESCI_K7);
	
	return rsesci_getViReal64Window (instrSession,
                                     "SENS%ld:BWID:DEM?",
                                     window,
                                     demodulationBandwidth,
                                     VI_ERROR_PARAMETER3);    
}

/*****************************************************************************
 * Function: Set Resolution Bandwidth
 * Purpose:  This function defines the receiver IF bandwidth or, in SPECTRUM
 *           mode, the analyzer's resolution bandwidth.
 *           If the resolution bandwidth is modified in SPECTRUM mode, the
 *           coupling to the span is automatically switched off.
 *           If the resolution bandwidth is modified in FM DEMOD mode, the
 *           coupling to the demodulation bandwidth is automatically switched
 *           off.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetResolutionBandwidth(
    ViSession   instrSession,
    ViInt32     window,
    ViReal64    resolutionBandwidth
)
{
    return rsesci_setViReal64Window (instrSession,
                                     "SENS%ld:BWID:RES",
                                     window,
                                     resolutionBandwidth,
                                     10.0 , 10.0E6 ,
                                     VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Resolution Bandwidth
 * Purpose:  This function returns the receiver IF bandwidth or, in SPECTRUM
 *           mode, the analyzer's resolution bandwidth.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetResolutionBandwidth(
    ViSession   instrSession,
    ViInt32     window,
    ViReal64*   resolutionBandwidth
)
{
    return rsesci_getViReal64Window (instrSession,
                                     "SENS%ld:BWID:RES?",
                                     window,
                                     resolutionBandwidth,
                                     VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set Resolution Bandwidth Auto
 * Purpose:  In RECEIVER mode with activated quasi peak detector, this
 *           function either automatically couples the IF bandwidth of the
 *           receiver to the frequency range or cancels the coupling.
 *           In SPECTRUM mode, this function either automatically couples the
 *           resolution bandwidth of the instrument to the span or cancels the
 *           coupling.
 *           In FM DEMOD mode, this function either automatically couples the
 *           IF bandwidth of the instrument to the demodulation bandwidth or
 *           cancels the coupling (BWRBW = 10 * BWDemod).
 *           In FM DEMOD mode, the coupling refers to the demodulation
 *           bandwidth (BWRBW = 10 * BWDemod).
 *           The automatic coupling adapts the resolution bandwidth to the
 *           currently set frequency span according to the relationship between
 *           frequency span and resolution bandwidth. The 6 dB bandwidths 200
 *           Hz, 9 kHz and 120 kHz and the channel filters available from
 *           Version 1.10 are not set by the automatic coupling.
 *           The ratio resolution bandwidth/span can be modified with the
 *           function rsesci_SetResolutionBandwidthCouplingRatio().
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetResolutionBandwidthAuto(
    ViSession   instrSession,
    ViInt32     window,
    ViBoolean   autoResolutionBandwidth
)
{
    return rsesci_setViBooleanWindow (instrSession,
                                      "SENS%ld:BWID:RES:AUTO",
                                      window,
                                      autoResolutionBandwidth,
                                      VI_ERROR_PARAMETER3);

}

/*****************************************************************************
 * Function: Get Resolution Bandwidth Auto
 * Purpose:  This function returns the setting of auto resolution bandwidth.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetResolutionBandwidthAuto(
    ViSession   instrSession,
    ViInt32     window,
    ViBoolean*  autoResolutionBandwidth
)
{
    return rsesci_getViBooleanWindow (instrSession,
                                      "SENS%ld:BWID:RES:AUTO?",
                                      window,
                                      autoResolutionBandwidth,
                                      VI_ERROR_PARAMETER3);

}

/*****************************************************************************
 * Function: Set Resolution Bandwidth Coupling Ratio
 * Purpose:  This function defines the ratio resolution bandwidth (Hz) / span
 *           (Hz). The ratio to be entered is reciprocal to the ratio span/RBW
 *           used in manual control.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetResolutionBandwidthCouplingRatio(
    ViSession   instrSession,
    ViInt32     window,
    ViReal64    ratio
)
{
    return rsesci_setViReal64Window (instrSession,
                                     "SENS%ld:BWID:RES:RAT",
                                     window,
                                     ratio,
                                     0.0001 , 1 ,
                                     VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Resolution Bandwidth Coupling Ratio
 * Purpose:  This function returns the ratio resolution bandwidth (Hz) / span
 *           (Hz).
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetResolutionBandwidthCouplingRatio(
    ViSession   instrSession,
    ViInt32     window,
    ViReal64*   ratio
)
{
    return rsesci_getViReal64Window (instrSession,
                                     "SENS%ld:BWID:RES:RAT?",
                                     window,
                                     ratio,
                                     VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set Resolution Bandwidth Filter Type
 * Purpose:  This function switches the filter type for the resolution
 *           bandwidth between "normal" analog or FIR filters in 1, 3, 10 steps
 *           and the FFT filtering for bandwidths <100 kHz.
 *           FFT filters are only suitable for periodic signals, and they are
 *           only available for span > 0 Hz.
 *           The filter type 'Pulse' selects the 6 dB bandwidths for EMI
 *           measurements rather than the 3 dB bandwidths for spectrum
 *           analysis, which is selected with type 'Noise'.
 *           Steep-edged channel filters and filters with RRC (Root Raised
 *           Cosine) characteristic are available.
 *           The possible combinations of filter type and filter bandwidth are
 *           listed in the table "List of available
 *           channel filters" in chapter 4, section "Setting Bandwidths and
 *           Sweep Time - Key BW".
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetResolutionBandwidthFilterType(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32 filterType
)
{
    return rsesci_setEnumWindow (instrSession,
                                 "SENS%ld:BWID:RES:TYPE",
                                 window,
                                 filterType,
                                 filterTypeArr,
                                 VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Resolution Bandwidth Filter Type
 * Purpose:  This function returns the filter type for the resolution
 *           bandwidth.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetResolutionBandwidthFilterType(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32*    filterType
)
{
    return rsesci_getEnumWindow (instrSession,
                                 "SENS%ld:BWID:RES:TYPE?",
                                 window,
                                 filterType,
                                 filterTypeArr ,
                                 VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set Video Bandwidth
 * Purpose:  This function defines the instrument's video bandwidth.
 *           Bandwidths from 10 Hz to 10 MHz in 1, 3, 10 steps are available.
 *           The function is not available if FFT filtering is switched on and
 *           the set bandwidth is <= 30 kHz or if the quasi-peak detector is
 *           switched on.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetVideoBandwidth(
    ViSession   instrSession,
    ViInt32     window,
    ViReal64    videoBandwidth
)
{
    return rsesci_setViReal64Window (instrSession,
                                     "SENS%ld:BWID:VID",
                                     window,
                                     videoBandwidth,
                                     1.0 , 10.0E6 ,
                                     VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Video Bandwidth
 * Purpose:  This function returns the instrument's video bandwidth.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetVideoBandwidth(
    ViSession   instrSession,
    ViInt32     window,
    ViReal64*   videoBandwidth
)
{
    return rsesci_getViReal64Window (instrSession,
                                     "SENS%ld:BWID:VID?",
                                     window,
                                     videoBandwidth,
                                     VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set Video Bandwidth Auto
 * Purpose:  This function either automatically couples the instrument's video
 *           bandwidth to the resolution bandwidth or cancels the coupling.
 *           The ratio video bandwidth/resolution bandwidth can be modified
 *           with the function SetVideoBandwidthCouplingRatio().
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetVideoBandwidthAuto(
    ViSession   instrSession,
    ViInt32     window,
    ViBoolean   autoVideoBandwidth
)
{
    return rsesci_setViBooleanWindow (instrSession,
                                      "SENS%ld:BWID:VID:AUTO",
                                      window,
                                      autoVideoBandwidth,
                                      VI_ERROR_PARAMETER3);

}

/*****************************************************************************
 * Function: Get Video Bandwidth Auto
 * Purpose:  This function returns whether automatically couples the
 *           instrument's video bandwidth to the resolution bandwidth or
 *           cancels the coupling.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetVideoBandwidthAuto(
    ViSession   instrSession,
    ViInt32     window,
    ViBoolean*  autoVideoBandwidth
)
{
    return rsesci_getViBooleanWindow (instrSession,
                                      "SENS%ld:BWID:VID:AUTO?",
                                      window,
                                      autoVideoBandwidth,
                                      VI_ERROR_PARAMETER3);

}

/*****************************************************************************
 * Function: Set Video Bandwidth Coupling Ratio
 * Purpose:  This function defines the ratio video bandwidth (Hz) / resolution
 *           bandwidth (Hz). The ratio to be entered is reciprocal to the ratio
 *           RBW/VBW used in manual control.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetVideoBandwidthCouplingRatio(
    ViSession   instrSession,
    ViInt32     window,
    ViReal64    ratio
)
{
    return rsesci_setViReal64Window (instrSession,
                                     "SENS%ld:BWID:VID:RAT",
                                     window,
                                     ratio,
                                     0.01 , 1000 ,
                                     VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Video Bandwidth Coupling Ratio
 * Purpose:  This function returns the value of ratio video bandwidth (Hz) /
 *           resolution bandwidth (Hz).
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetVideoBandwidthCouplingRatio(
    ViSession   instrSession,
    ViInt32     window,
    ViReal64*   ratio
)
{
    return rsesci_getViReal64Window (instrSession,
                                     "SENS%ld:BWID:VID:RAT?",
                                     window,
                                     ratio,
                                     VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set Video Bandwidth Mode
 * Purpose:  This function selects the position of the video filter in the
 *           signal window, provided that the resolution bandwidth is <= 100 kHz.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetVideoBandwidthMode(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32     BandwidthMode
)
{
    return rsesci_setEnumWindow (instrSession,
                                 "SENS%ld:BWID:VID:TYPE",
                                 window,
                                 BandwidthMode,
                                 modeArr,
                                 VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Video Bandwidth Mode
 * Purpose:  This function returns the position of the video filter in the
 *           signal path.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetVideoBandwidthMode(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32*    BandwidthMode
)
{
    return rsesci_getEnumWindow (instrSession,
                                 "SENS%ld:BWID:VID:TYPE?",
                                 window,
                                 BandwidthMode,
                                 modeArr,
                                 VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Continue Single Sweep
 * Purpose:  When a scan is automatically interrupted by a transducer
 *           interrupt point, this function continues the measurement at the
 *           current receiver frequency. The function is useful especially for
 *           trace functions MAXHold, MINHold and AVERage if the previous
 *           results are not to be cleared with Sweep Count > 0 or Average
 *           Count > 0 on restarting the measurement (INIT:IMMediate resets the
 *           previous results on restarting the measurement). The single-sweep
 *           mode is automatically switched on. In the continuous-sweep mode,
 *           synchronization to the sweep end is not possible since the overall
 *           measurement "never" ends.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_ContinueSingleSweep(
    ViSession   instrSession,
    ViInt32     window
)
{
    return rsesci_sendCmdWindow (instrSession,
                                 "INIT%ld:CONM",
                                 window);
}

/*****************************************************************************
 * Function: Set Sweep Mode
 * Purpose:  This function determines whether the trigger system is
 *           continuously initiated (continuous) or performs single
 *           measurements (single).
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetSweepMode(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32     continuousSweep
)
{
    return rsesci_setEnumWindow (instrSession,
                                 "INIT%ld:CONT",
                                 window,
                                 continuousSweep,
                                 sweepModeArr,
                                 VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Sweep Mode
 * Purpose:  This function returns whether the trigger system is continuously
 *           initiated (continuous) or performs single measurements (single).
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetSweepMode(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32*    continuousSweep
)
{
    return rsesci_getEnumWindow (instrSession,
                                 "INIT%ld:CONT?",
                                 window,
                                 continuousSweep,
                                 sweepModeArr,
                                 VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set Sweep Time
 * Purpose:  This function defines the scan or sweep time.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetSweepTime(
    ViSession   instrSession,
    ViInt32     window,
    ViReal64    sweepTime
)
{
    return rsesci_setViReal64Window (instrSession,
                                     "SENS%ld:SWE:TIME",
                                     window,
                                     sweepTime,
                                     1.0E-6 , 16000 ,
                                     VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Sweep Time
 * Purpose:  This function returns the scan or sweep time
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetSweepTime(
    ViSession   instrSession,
    ViInt32     window,
    ViReal64*   sweepTime
)
{
    return rsesci_getViReal64Window (instrSession,
                                     "SENS%ld:SWE:TIME?",
                                     window,
                                     sweepTime,
                                     VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set Sweep Time Auto
 * Purpose:  This function controls the automatic coupling of the sweep time
 *           to the frequency span and bandwidth settings.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetSweepTimeAuto(
    ViSession   instrSession,
    ViInt32     window,
    ViBoolean   autoSweepTime
)
{
    return rsesci_setViBooleanWindow (instrSession,
                                      "SENS%ld:SWE:TIME:AUTO",
                                      window,
                                      autoSweepTime,
                                      VI_ERROR_PARAMETER3);

}

/*****************************************************************************
 * Function: Get Sweep Time Auto
 * Purpose:  This function returns the state of automatic coupling of the
 *           sweep time to the frequency span and bandwidth settings.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetSweepTimeAuto(
    ViSession   instrSession,
    ViInt32     window,
    ViBoolean*  autoSweepTime
)
{
    return rsesci_getViBooleanWindow (instrSession,
                                      "SENS%ld:SWE:TIME:AUTO?",
                                      window,
                                      autoSweepTime,
                                      VI_ERROR_PARAMETER3);

}

/*****************************************************************************
 * Function: Set Sweep Count
 * Purpose:  This function defines the number of scans started with single
 *           scan in receiver mode. In analyzer mode, this function defines the
 *           number of sweeps started with single sweep, which are
 *           used for calculating the average or maximum value. In average
 *           mode, the value 0 defines a continuous averaging of measurement
 *           data over 10 sweeps.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetSweepCount(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32 sweepCount
)
{
    return rsesci_setViInt32Window (instrSession,
                                    "SENS%ld:SWE:COUN",
                                    window,
                                    sweepCount,
                                    0 , 32767 ,
                                    VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Sweep Count
 * Purpose:  This function returns the number of scans started with single
 *           scan in receiver mode. In analyzer mode, this function returns
 *           the number of sweeps started with single sweep, which are
 *           used for calculating the average or maximum value.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetSweepCount(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32*    sweepCount
)
{
    return rsesci_getViInt32Window (instrSession,
                                    "SENS%ld:SWE:COUN?",
                                    window,
                                    sweepCount,
                                    VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Number of Started Sweeps
 * Purpose:  This function queries the number of scans started with single
 *           scan in receiver mode. In analyzer mode, this function queries the
 *           number of sweeps started with single sweep.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetNumberStartedSweeps(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32*    startedScans
)
{
    return rsesci_getViInt32Window (instrSession,
                                    "SENS%ld:SWE:COUN:CURR?",
                                    window,
                                    startedScans,
                                    VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set Sweep Points
 * Purpose:  This function defines the number of measurement points for one
 *           sweep run.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetSweepPoints(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32     sweepPoints
)
{
    ViReal64    sweepPoints_local   = sweepPoints;
    ViInt16     index               = 0;

    // checking for ESU instrument
	if ((rsesci_checkModel(instrSession, RSESU8 | RSESU26 | RSESU40)) == VI_SUCCESS)
	{
		rsesci_coerceToNearest (sweeppointsESUArr, &sweepPoints_local, &index);	
	}
	else
	{
		rsesci_coerceToNearest (sweeppointsArr, &sweepPoints_local, &index);
	}

    return rsesci_setViReal64Window (instrSession,
                                     "SENS%ld:SWE:POIN",
                                     window,
                                     sweepPoints_local,
                                     0.0,0.0,
                                     VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Sweep Points
 * Purpose:  This function returns the number of measurement points for one
 *           sweep run.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetSweepPoints(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32*    sweepPoints
)
{
    return rsesci_getViInt32Window (instrSession,
                                    "SENS%ld:SWE:POIN?",
                                    window,
                                    sweepPoints,
                                    VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set Single Sweep Display
 * Purpose:  This function configures the behavior of the display during a
 *           single sweep.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetSingleSweepDisplay(
    ViSession   instrSession,
    ViBoolean displayState
)
{
    return rsesci_setViBoolean (instrSession,
                                "INIT:DISP",
                                displayState,
                                VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Single Sweep Display
 * Purpose:  This function returns the state of the display during a single
 *           sweep.
  *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetSingleSweepDisplay(
    ViSession   instrSession,
    ViBoolean*    displayState
)
{
    return rsesci_getViBoolean (instrSession,
                                "INIT:DISP?",
                                displayState,
                                VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Sweep Spacing
 * Purpose:  This function switches between linear, logarthmic and
 *           automatically selected linear step size in the receiver. In the
 *           receiver mode, this parameter has no impact on the graphical
 *           representation of the frequency axis.
 *           In the analyzer, the command toggles between linear and
 *           logarithmic sweep. The frequency axis is represented linearly or
 *           logarithmically depending on the sweep.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetSweepSpacing(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32     sweepSpacing
)
{
    if ((sweepSpacing >= RSESCI_APUL) && (sweepSpacing <= RSESCI_ACW))
		CHECKMODEL(RSESU8 | RSESU26 | RSESU40);
	
	return rsesci_setEnumWindow (instrSession,
                                 "SENS%ld:SWE:SPAC",
                                 window,
                                 sweepSpacing,
                                 sweepspacingArr,
                                 VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Sweep Spacing
 * Purpose:  This function returns value of sweep spacing mode.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetSweepSpacing(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32*    sweepSpacing
)
{
    return rsesci_getEnumWindow (instrSession,
                                 "SENS%ld:SWE:SPAC?",
                                 window,
                                 sweepSpacing,
                                 sweepspacingArr,
                                 VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Abort
 * Purpose:  This function aborts a current measurement and resets the trigger
 *           system.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_Abort(
    ViSession   instrSession
)
{
    return rsesci_sendCmd (instrSession,
                           "ABOR");
}

/*****************************************************************************
 * Function: Hold
 * Purpose:  This function interrupts a current scan measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_Hold(
    ViSession   instrSession
)
{
    return rsesci_sendCmd (instrSession,
                           "HOLD");
}

/*****************************************************************************
 * Function: Send Trigger
 * Purpose:  This function initiates a new sweep in the indicated measurement
 *           window.
 *           In receiver mode with 'Single' selected, the R&S ESCI performs a
 *           single scan and stops at the end frequency. With 'Continuous'
 *           selected, the scan is performed continuously until it is
 *           deliberately stopped.
 *           In analyzer mode, with Sweep Count > 0 or Average Count > 0, this
 *           means a restart of the indicated number of measurements. With
 *           trace functions 'Max Hold', 'Min Hold' and 'Average', the previous
 *           results are reset on restarting the measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SendTrigger(
    ViSession   instrSession,
    ViInt32     window
)
{
    return rsesci_sendCmdWindow (instrSession,
                                 "INIT%ld:IMM",
                                 window);
}

/*****************************************************************************
 * Function: Send Trigger and Wait for OPC
 * Purpose:  This function triggers all actions waiting for a trigger event in
 *           the selected window and waits for operation completed (OPC) before
 *           returning the status code.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SendTriggerWaitOPC(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32 timeout
)
{
    ViChar  buffer[BUFFER_SIZE];

    CHECKWINDOW;

    sprintf (buffer, "INIT%ld:IMM;*OPC?\n", window);

    return rsesci_waitOPC (instrSession, buffer, buffer, timeout);
}

/*****************************************************************************
 * Function: Set Trigger Source
 * Purpose:  This function selects the trigger source for the start of a
 *           sweep.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetTriggerSource(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32 triggerSource
)
{
    return rsesci_setEnumWindow (instrSession,
                                 "TRIG%ld:SEQ:SOUR",
                                 window,
                                 triggerSource,
                                 triggersourceArr,
                                 VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Trigger Source
 * Purpose:  This function returns the trigger source for the start of a
 *           sweep.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetTriggerSource(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32*    triggerSource
)
{
    return rsesci_getEnumWindow (instrSession,
                                 "TRIG%ld:SEQ:SOUR?",
                                 window,
                                 triggerSource,
                                 triggersourceArr,
                                 VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set Trigger Level
 * Purpose:  This function sets the trigger level for selected signal.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetTriggerLevel(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32 signalType,
    ViReal64    triggerLevel
)
{
    switch (signalType)
    {
        case RSESCI_TRIG_AM: return rsesci_setViReal64Window (instrSession,
                                                              "TRIG%ld:SEQ:LEV:AM",
                                                              window,
                                                              triggerLevel,
                                                              -100.0  , +30.0 ,
                                                              VI_ERROR_PARAMETER4);
        case RSESCI_TRIG_FM: return rsesci_setViReal64Window (instrSession,
                                                              "TRIG%ld:SEQ:LEV:FM",
                                                              window,
                                                              triggerLevel,
                                                              -10E6   , +10E6 ,
                                                              VI_ERROR_PARAMETER4);
        case RSESCI_TRIG_PM: return rsesci_setViReal64Window (instrSession,
                                                              "TRIG%ld:SEQ:LEV:PM",
                                                              window,
                                                              triggerLevel,
                                                              -1000.0 , +1000.0 ,
                                                              VI_ERROR_PARAMETER4);
        case RSESCI_TRIG_IFP: return rsesci_setViReal64Window (instrSession,
                                                               "TRIG%ld:SEQ:LEV:IFP",
                                                               window,
                                                               triggerLevel,
                                                               -30.0   , -10.0 ,
                                                               VI_ERROR_PARAMETER4);
        case RSESCI_TRIG_RFP: return rsesci_setViReal64Window (instrSession,
                                                               "TRIG%ld:SEQ:LEV:RFP",
                                                               window,
                                                               triggerLevel,
                                                               -50.0   , -10.0 ,
                                                               VI_ERROR_PARAMETER4);
        case RSESCI_TRIG_VID: return rsesci_setViReal64Window (instrSession,
                                                               "TRIG%ld:SEQ:LEV:VID",
                                                               window,
                                                               triggerLevel,
                                                               0.0 , 100.0 ,
                                                               VI_ERROR_PARAMETER4);
		case RSESCI_TRIG_EXT: return rsesci_setViReal64Window (instrSession,
                                                               "TRIG%ld:LEV",
                                                               window,
                                                               triggerLevel,
                                                               0.5, 3.5,
                                                               VI_ERROR_PARAMETER4);
		case RSESCI_TRIG_AM_REL: return rsesci_setViReal64Window (instrSession,
                                                                  "TRIG%ld:SEQ:LEV:AM:REL",
                                                                  window,
                                                                  triggerLevel,
                                                                  0.0, 100.0,
                                                                  VI_ERROR_PARAMETER4);
		default: return VI_ERROR_PARAMETER4;
    }
}

/*****************************************************************************
 * Function: Get Trigger Level
 * Purpose:  This function returns the trigger level for selected signal.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetTriggerLevel(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32 signalType,
    ViReal64*   triggerLevel
)
{
    switch (signalType)
    {
        case RSESCI_TRIG_AM: return rsesci_getViReal64Window (instrSession,
                                                              "TRIG%ld:SEQ:LEV:AM?",
                                                              window,
                                                              triggerLevel,
                                                              VI_ERROR_PARAMETER4);
        case RSESCI_TRIG_FM: return rsesci_getViReal64Window (instrSession,
                                                              "TRIG%ld:SEQ:LEV:FM?",
                                                              window,
                                                              triggerLevel,
                                                              VI_ERROR_PARAMETER4);
        case RSESCI_TRIG_PM: return rsesci_getViReal64Window (instrSession,
                                                              "TRIG%ld:SEQ:LEV:PM?",
                                                              window,
                                                              triggerLevel,
                                                              VI_ERROR_PARAMETER4);
        case RSESCI_TRIG_IFP: return rsesci_getViReal64Window (instrSession,
                                                               "TRIG%ld:SEQ:LEV:IFP?",
                                                               window,
                                                               triggerLevel,
                                                               VI_ERROR_PARAMETER4);
        case RSESCI_TRIG_RFP: return rsesci_getViReal64Window (instrSession,
                                                               "TRIG%ld:SEQ:LEV:RFP?",
                                                               window,
                                                               triggerLevel,
                                                               VI_ERROR_PARAMETER4);
        case RSESCI_TRIG_VID: return rsesci_getViReal64Window (instrSession,
                                                               "TRIG%ld:SEQ:LEV:VID?",
                                                               window,
                                                               triggerLevel,
                                                               VI_ERROR_PARAMETER4);
        case RSESCI_TRIG_EXT: return rsesci_getViReal64Window (instrSession,
                                                               "TRIG%ld:LEV?",
                                                               window,
                                                               triggerLevel,
                                                               VI_ERROR_PARAMETER4);
		case RSESCI_TRIG_AM_REL: return rsesci_getViReal64Window (instrSession,
									                              "TRIG%ld:SEQ:LEV:AM:REL?",
																  window,
																  triggerLevel,
																  VI_ERROR_PARAMETER4);
		default: return VI_ERROR_PARAMETER4;
    }
}

/*****************************************************************************
 * Function: Set Trigger Offset
 * Purpose:  This function defines the length of the trigger delay.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetTriggerOffset(
    ViSession   instrSession,
    ViInt32     window,
    ViReal64    triggerDelay
)
{
    return rsesci_setViReal64Window (instrSession,
                                     "TRIG%ld:SEQ:HOLD",
                                     window,
                                     triggerDelay,
                                     -100.0 , 100.0 ,
                                     VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Trigger Offset
 * Purpose:  This function returns the length of the trigger delay.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetTriggerOffset(
    ViSession   instrSession,
    ViInt32     window,
    ViReal64*   triggerDelay
)
{
    return rsesci_getViReal64Window (instrSession,
                                     "TRIG%ld:SEQ:HOLD?",
                                     window,
                                     triggerDelay,
                                     VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set Trigger Offset Adjust
 * Purpose:  This function defines whether the group delay of the bandwidth 
 *           filters for the external trigger is compensated (ON) or not (OFF). 
 *           If a burst signal is analyzed in the time domain mode, the rising 
 *           edge will remain at the same time position if compensation is on 
 *           and the bandwidth changes.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetTriggerOffsetAdjust (ViSession instrSession,
                                                 ViInt32 window,
                                                 ViBoolean offsetAdjust)
{
	CHECKMODEL(RSESU8 | RSESU26 | RSESU40);
	
	return rsesci_setViBooleanWindow (instrSession,
                                      "TRIG%ld:SEQ:HOLD:ADJ:AUTO",
                                      window,
                                      offsetAdjust,
                                      VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Trigger Offset Adjust
 * Purpose:  This function returns the state of the group delay of the bandwidth 
 *           filters for the external trigger is compensated (ON) or not (OFF). 
 *           If a burst signal is analyzed in the time domain mode, the rising
 *           edge will remain at the same time position if compensation is on
 *           and the bandwidth changes.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetTriggerOffsetAdjust (ViSession instrSession,
                                                 ViInt32 window,
                                                 ViBoolean *offsetAdjust)
{
 	CHECKMODEL(RSESU8 | RSESU26 | RSESU40);
	
	return rsesci_getViBooleanWindow (instrSession,
                                      "TRIG%ld:SEQ:HOLD:ADJ:AUTO?",
                                      window,
                                      offsetAdjust,
                                      VI_ERROR_PARAMETER3);   
}

/*****************************************************************************
 * Function: Set Trigger Source Polarity
 * Purpose:  This function selects the slope of the trigger signal. The
 *           selected trigger slope applies to all trigger signal sources.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetTriggerSourcePolarity(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32 polarity
)
{
    return rsesci_setEnumWindow (instrSession,
                                 "TRIG%ld:SEQ:SLOP",
                                 window,
                                 polarity,
                                 polarityArr,
                                 VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Trigger Source Polarity
 * Purpose:  This function returns the slope of the trigger signal. The
 *           selected trigger slope applies to all trigger signal sources.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetTriggerSourcePolarity(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32*    polarity
)
{
    return rsesci_getEnumWindow (instrSession,
                                 "TRIG%ld:SEQ:SLOP?",
                                 window,
                                 polarity,
                                 polarityArr,
                                 VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set Gated Trigger
 * Purpose:  This function switches on/off the sweep control by an external
 *           gate signal. If the external gate is selected the trigger source
 *           is automatically switched to 'External' as well.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetGatedTrigger(
    ViSession   instrSession,
    ViInt32     window,
    ViBoolean gatedTrigger
)
{
    return rsesci_setViBooleanWindow (instrSession,
                                      "SENS%ld:SWE:EGAT",
                                      window,
                                      gatedTrigger,
                                      VI_ERROR_PARAMETER3);

}

/*****************************************************************************
 * Function: Get Gated Trigger
 * Purpose:  This function returns setting of the sweep control by an external
 *           gate signal.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetGatedTrigger(
    ViSession   instrSession,
    ViInt32     window,
    ViBoolean*  gatedTrigger
)
{
    return rsesci_getViBooleanWindow (instrSession,
                                      "SENS%ld:SWE:EGAT?",
                                      window,
                                      gatedTrigger,
                                      VI_ERROR_PARAMETER3);

}

/*****************************************************************************
 * Function: Set Gated Trigger Source
 * Purpose:  This function toggles between external gate signal and IF power
 *           signal as a signal source for the gate mode. If an IF power signal
 *           is used, the gate is opened as soon as a signal at > -20 dBm is
 *           detected within the IF window bandwidth (10 MHz).
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetGatedTriggerSource(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32     triggerSource
)
{
    return rsesci_setEnumWindow (instrSession,
                                 "SENS%ld:SWE:EGAT:SOUR",
                                 window,
                                 triggerSource,
                                 triggersourceArr,
                                 VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Gated Trigger Source
 * Purpose:  This function returns selected signal source for the gate mode.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetGatedTriggerSource(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32*    triggerSource
)
{
    return rsesci_getEnumWindow (instrSession,
                                 "SENS%ld:SWE:EGAT:SOUR?",
                                 window,
                                 triggerSource,
                                 triggersourceArr,
                                 VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set Gated Trigger Mode
 * Purpose:  This function sets the type of triggering (level or edge) by the
 *           external gate signal.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetGatedTriggerMode(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32 triggerMode
)
{
    return rsesci_setEnumWindow (instrSession,
                                 "SENS%ld:SWE:EGAT:TYPE",
                                 window,
                                 triggerMode,
                                 triggerModeArr,
                                 VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Gated Trigger Mode
 * Purpose:  This function returns the type of triggering (level or edge) by
 *           the external gate signal.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetGatedTriggerMode(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32*    triggerMode
)
{
    return rsesci_getEnumWindow (instrSession,
                                 "SENS%ld:SWE:EGAT:TYPE?",
                                 window,
                                 triggerMode,
                                 triggerModeArr,
                                 VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set Gated Trigger Polarity
 * Purpose:  This function determines the polarity of the external gate
 *           signal. The setting is valid both for the edge of an
 *           edge-triggered signal and the level of a level-triggered signal.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetGatedTriggerPolarity(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32 triggerPolarity
)
{
    return rsesci_setEnumWindow (instrSession,
                                 "SENS%ld:SWE:EGAT:POL",
                                 window,
                                 triggerPolarity,
                                 polarityArr,
                                 VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Gated Trigger Polarity
 * Purpose:  This function returns the polarity of the external gate signal.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetGatedTriggerPolarity(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32*    triggerPolarity
)
{
    return rsesci_getEnumWindow (instrSession,
                                 "SENS%ld:SWE:EGAT:POL?",
                                 window,
                                 triggerPolarity,
                                 polarityArr,
                                 VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set Gated Trigger Delay
 * Purpose:  This function defines the delay time between the external gate
 *           signal and the continuation of the sweep.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetGatedTriggerDelay(
    ViSession   instrSession,
    ViInt32     window,
    ViReal64    triggerDelay
)
{
    return rsesci_setViReal64Window (instrSession,
                                     "SENS%ld:SWE:EGAT:HOLD",
                                     window,
                                     triggerDelay,
                                     125E-9 , 100 ,
                                     VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Gated Trigger Delay
 * Purpose:  This function returns the delay time between the external gate
 *           signal and the continuation of the sweep.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetGatedTriggerDelay(
    ViSession   instrSession,
    ViInt32     window,
    ViReal64*   triggerDelay
)
{
    return rsesci_getViReal64Window (instrSession,
                                     "SENS%ld:SWE:EGAT:HOLD?",
                                     window,
                                     triggerDelay,
                                     VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set Gated Trigger Length
 * Purpose:  In case of edge triggering, this function determines the time
 *           interval during which the instrument sweeps.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetGatedTriggerLength(
    ViSession   instrSession,
    ViInt32     window,
    ViReal64    triggerLength
)
{
    return rsesci_setViReal64Window (instrSession,
                                     "SENS%ld:SWE:EGAT:LENG",
                                     window,
                                     triggerLength,
                                     0 , 100 ,
                                     VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Gated Trigger Length
 * Purpose:  In case of edge triggering, this function returns the time
 *           interval during which the instrument sweeps.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetGatedTriggerLength(
    ViSession   instrSession,
    ViInt32     window,
    ViReal64*   triggerLength
)
{
    return rsesci_getViReal64Window (instrSession,
                                     "SENS%ld:SWE:EGAT:LENG?",
                                     window,
                                     triggerLength,
                                     VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set Gated Trigger State
 * Purpose:  This function enables/disables gate range in gate range table.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetGatedTriggerState (ViSession instrSession,
                                               ViInt32 window, ViInt32 range,
                                               ViInt32 trace,
                                               ViBoolean gateRangeTableState)
{
    ViChar buffer [100];
	
	if (rsesci_invalidViInt32Range (window, 1, 2) == VI_TRUE)
        return VI_ERROR_PARAMETER2;
    if (rsesci_invalidViInt32Range (range, 1, 3) == VI_TRUE)
        return VI_ERROR_PARAMETER3;
    if (rsesci_invalidViInt32Range (trace, 1, 3) == VI_TRUE)
        return VI_ERROR_PARAMETER4;
    
	sprintf (buffer, "SENS%ld:SWE:EGAT:TRAC%ld:STAT%ld"
			, window
			, trace
			, range
			);

	return rsesci_setViBoolean (instrSession
								, buffer
								, gateRangeTableState
								, VI_ERROR_PARAMETER5);
}

/*****************************************************************************
 * Function: Get Gated Trigger State
 * Purpose:  This function queries state of the gate range in gate range
 *           table.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetGatedTriggerState (ViSession instrSession,
                                               ViInt32 window, ViInt32 range,
                                               ViInt32 trace,
                                               ViBoolean *gateRangeTableState)
{
    ViChar buffer [100];
	
	if (rsesci_invalidViInt32Range (window, 1, 2) == VI_TRUE)
        return VI_ERROR_PARAMETER2;
    if (rsesci_invalidViInt32Range (range, 1, 3) == VI_TRUE)
        return VI_ERROR_PARAMETER3;
    if (rsesci_invalidViInt32Range (trace, 1, 3) == VI_TRUE)
        return VI_ERROR_PARAMETER4;
    
	sprintf (buffer, "SENS%ld:SWE:EGAT:TRAC%ld:STAT%ld?"
			, window
			, trace
			, range
			);

	return rsesci_getViBoolean (instrSession
								, buffer
								, gateRangeTableState
								, VI_ERROR_PARAMETER5);
}

/*****************************************************************************
 * Function: Set Gated Trigger Range
 * Purpose:  This function sets the start and stop values of a gate range in
 *           the gate range table.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetGatedTriggerRange (ViSession instrSession,
                                               ViInt32 window, ViInt32 range,
                                               ViInt32 trace, ViReal64 start,
                                               ViReal64 stop)
{
    ViChar buffer [100];
	ViStatus rsesci_status = VI_SUCCESS;
	
	if (rsesci_invalidViInt32Range (window, 1, 2) == VI_TRUE)
        return VI_ERROR_PARAMETER2;
    if (rsesci_invalidViInt32Range (range, 1, 3) == VI_TRUE)
        return VI_ERROR_PARAMETER3;
    if (rsesci_invalidViInt32Range (trace, 1, 3) == VI_TRUE)
        return VI_ERROR_PARAMETER4;
    if (rsesci_invalidViReal64Range (start, 0.0, 70000.0) == VI_TRUE)
        return VI_ERROR_PARAMETER5;
    if (rsesci_invalidViReal64Range (stop, 0.0, 70000.0) == VI_TRUE)
        return VI_ERROR_PARAMETER6;
    
	sprintf (buffer, "SENS%ld:SWE:EGAT:TRAC%ld:STAR%ld"
			, window
			, trace
			, range
			);

	CHECKERR (rsesci_setViReal64 (instrSession
								, buffer
								, start
								, 0.0, 0.0
								, VI_ERROR_PARAMETER5));
	
	sprintf (buffer, "SENS%ld:SWE:EGAT:TRAC%ld:STOP%ld"
			, window
			, trace
			, range
			);
	
	return rsesci_setViReal64 (instrSession
								, buffer
								, stop
								, 0.0, 0.0
								, VI_ERROR_PARAMETER6);
}

/*****************************************************************************
 * Function: Get Gated Trigger Range
 * Purpose:  This function queries the start and stop values of a gate range
 *           in the gate range table.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetGatedTriggerRange (ViSession instrSession,
                                               ViInt32 window, ViInt32 range,
                                               ViInt32 trace, ViReal64 *start,
                                               ViReal64 *stop)
{
    ViChar buffer [100];
	ViStatus rsesci_status = VI_SUCCESS;
	
	if (rsesci_invalidViInt32Range (window, 1, 2) == VI_TRUE)
        return VI_ERROR_PARAMETER2;
    if (rsesci_invalidViInt32Range (range, 1, 3) == VI_TRUE)
        return VI_ERROR_PARAMETER3;
    if (rsesci_invalidViInt32Range (trace, 1, 3) == VI_TRUE)
        return VI_ERROR_PARAMETER4;
    
	sprintf (buffer, "SENS%ld:SWE:EGAT:TRAC%ld:STAR%ld?"
			, window
			, trace
			, range
			);

	CHECKERR (rsesci_getViReal64 (instrSession
								, buffer
								, start
								, VI_ERROR_PARAMETER5));
	
	sprintf (buffer, "SENS%ld:SWE:EGAT:TRAC%ld:STOP%ld?"
			, window
			, trace
			, range
			);
	
	return rsesci_getViReal64 (instrSession
								, buffer
								, stop
								, VI_ERROR_PARAMETER6);
}

/*****************************************************************************
 * Function: Set Gated Trigger Period
 * Purpose:  This function sets the period of the signal for the gate range
 *           table.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetGatedTriggerPeriod (ViSession instrSession,
                                                ViInt32 window, ViInt32 trace,
                                                ViReal64 period)
{
    ViChar buffer [100];
	
	if (rsesci_invalidViInt32Range (window, 1, 2) == VI_TRUE)
        return VI_ERROR_PARAMETER2;
    if (rsesci_invalidViInt32Range (trace, 1, 3) == VI_TRUE)
        return VI_ERROR_PARAMETER3;
    
	sprintf (buffer, "SENS%ld:SWE:EGAT:TRAC%ld:PER"
			, window
			, trace
			);

	return rsesci_setViReal64 (instrSession
								, buffer
								, period
								, 0.0, 70000.0
								, VI_ERROR_PARAMETER4);
}

/*****************************************************************************
 * Function: Get Gated Trigger Period
 * Purpose:  This function sets the period of the signal for the gate range
 *           table.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetGatedTriggerPeriod (ViSession instrSession,
                                                ViInt32 window, ViInt32 trace,
                                                ViReal64 *period)
{
    ViChar buffer [100];
	
	if (rsesci_invalidViInt32Range (window, 1, 2) == VI_TRUE)
        return VI_ERROR_PARAMETER2;
    if (rsesci_invalidViInt32Range (trace, 1, 3) == VI_TRUE)
        return VI_ERROR_PARAMETER3;
    
	sprintf (buffer, "SENS%ld:SWE:EGAT:TRAC%ld:PER?"
			, window
			, trace
			);

	return rsesci_getViReal64 (instrSession
								, buffer
								, period
								, VI_ERROR_PARAMETER4);
}

/*****************************************************************************
 * Function: Set Gated Trigger Comment
 * Purpose:  This function sets the comment string in the gate range table.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetGatedTriggerComment (ViSession instrSession,
                                                 ViInt32 window, ViInt32 trace,
                                                 ViString comment)
{
    ViChar buffer [100];
	
	if (rsesci_invalidViInt32Range (window, 1, 2) == VI_TRUE)
        return VI_ERROR_PARAMETER2;
    if (rsesci_invalidViInt32Range (trace, 1, 3) == VI_TRUE)
        return VI_ERROR_PARAMETER3;
    
	sprintf (buffer, "SENS%ld:SWE:EGAT:TRAC%ld:COMM"
			, window
			, trace
			);

	return rsesci_setViStringQuoted (instrSession
									, buffer
									, "'"
									, comment
									, VI_ERROR_PARAMETER4);
    
}

/*****************************************************************************
 * Function: Get Gated Trigger Comment
 * Purpose:  This function queries the comment string in the gate range
 *           table.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetGatedTriggerComment (ViSession instrSession,
                                                 ViInt32 window, ViInt32 trace,
                                                 ViInt32 arraySize,
                                                 ViChar comment[])
{
    ViChar buffer [100];
	
	if (rsesci_invalidViInt32Range (window, 1, 2) == VI_TRUE)
        return VI_ERROR_PARAMETER2;
    if (rsesci_invalidViInt32Range (trace, 1, 3) == VI_TRUE)
        return VI_ERROR_PARAMETER3;
    
	sprintf (buffer, "SENS%ld:SWE:EGAT:TRAC%ld:COMM?"
			, window
			, trace
			);

	return rsesci_getViStringQuoted (instrSession
									, buffer
									, "'"
									, arraySize
									, comment
									, VI_ERROR_PARAMETER5);
    
}

/*****************************************************************************
 * Function: Set TV Trigger
 * Purpose:  This function switches triggering on TV signals on or off.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetTVTrigger(
    ViSession   instrSession,
    ViInt32     window,
    ViBoolean   tvTrigger
)
{
    CHECKOPT (RSESCI_B6);

    return rsesci_setViBooleanWindow (instrSession,
                                      "SENS%ld:TV:STAT",
                                      window,
                                      tvTrigger,
                                      VI_ERROR_PARAMETER3);

}

/*****************************************************************************
 * Function: Get TV Trigger
 * Purpose:  This function returns the state of triggering on TV signals.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetTVTrigger(
    ViSession   instrSession,
    ViInt32     window,
    ViBoolean*  tvTrigger
)
{
    CHECKOPT (RSESCI_B6);

    return rsesci_getViBooleanWindow (instrSession,
                                      "SENS%ld:TV:STAT?",
                                      window,
                                      tvTrigger,
                                      VI_ERROR_PARAMETER3);

}

/*****************************************************************************
 * Function: Set TV Trigger Vertical Sync
 * Purpose:  With active TV trigger this function activates triggering at the
 *           vertical sync signal. The measurement is triggered on both fields
 *           with selection 'All', on odd fields with selection 'Odd'
 *           and on even fields with selection 'Even'.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetTVTriggerVerticalSync(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32 field
)
{
    CHECKOPT (RSESCI_B6);

    return rsesci_setEnumWindow (instrSession,
                                 "TRIG%ld:SEQ:VID:FIEL:SEL",
                                 window,
                                 field,
                                 fieldArr,
                                 VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get TV Trigger Vertical Sync
 * Purpose:  This function returns selected the field on which the measurement
 *           is triggered.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetTVTriggerVerticalSync(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32*    field
)
{
    CHECKOPT (RSESCI_B6);

    return rsesci_getEnumWindow (instrSession,
                                 "TRIG%ld:SEQ:VID:FIEL:SEL?",
                                 window,
                                 field,
                                 fieldArr,
                                 VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set TV Trigger Horizontal Sync
 * Purpose:  With active TV trigger this function activates triggering at the
 *           horizontal sync signal of the indicated line number.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetTVTriggerHorizontalSync(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32 lineNumber
)
{
    CHECKOPT (RSESCI_B6);

    return rsesci_setViInt32Window (instrSession,
                                    "TRIG%ld:SEQ:VID:LINE:NUMB",
                                    window,
                                    lineNumber,
                                    0 , 0 ,
                                    VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get TV Trigger Horizontal Sync
 * Purpose:  This function returns line number for tirggering at the
 *           horizontal sync signal.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetTVTriggerHorizontalSync(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32*    lineNumber
)
{
    CHECKOPT (RSESCI_B6);

    return rsesci_getViInt32Window (instrSession,
                                    "TRIG%ld:SEQ:VID:LINE:NUMB?",
                                    window,
                                    lineNumber,
                                    VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set TV Trigger Polarity
 * Purpose:  This function selects the slope of the video sync signal.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetTVTriggerPolarity(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32 polarity
)
{
    CHECKOPT (RSESCI_B6);

    return rsesci_setEnumWindow (instrSession,
                                 "TRIG%ld:SEQ:VID:SSIG:POL",
                                 window,
                                 polarity,
                                 polarityArr,
                                 VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get TV Trigger Polarity
 * Purpose:  This function returns the slope of the video sync signal.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetTVTriggerPolarity(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32*    polarity
)
{
    CHECKOPT (RSESCI_B6);

    return rsesci_getEnumWindow (instrSession,
                                 "TRIG%ld:SEQ:VID:SSIG:POL?",
                                 window,
                                 polarity,
                                 polarityArr,
                                 VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set TV Trigger Lines
 * Purpose:  This function defines the line system in use (525 or 625 lines)
 *           with active TV trigger.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetTVTriggerLines(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32     linesSystem
)
{
    CHECKOPT (RSESCI_B6);

    return rsesci_setViInt32Window (instrSession,
                                 "TRIG%ld:SEQ:VID:FORM:LPFR",
                                 window,
                                 linesSystem,
                                 525, 625,
                                 VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get TV Trigger Lines
 * Purpose:  This function returns the line system in use (525 or 625 lines)
 *           with active TV trigger.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetTVTriggerLines(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32*    linesSystem
)
{
    CHECKOPT (RSESCI_B6);

    return rsesci_getViInt32Window (instrSession,
                                 "TRIG%ld:SEQ:VID:FORM:LPFR?",
                                 window,
                                 linesSystem,
                                 VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set TV Trigger CCVS
 * Purpose:  This function selects between an internal and an external CCVS
 *           signal as TV demodulator input
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetTVTriggerCCVS(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32 ccvsSignal
)
{
    CHECKOPT (RSESCI_B6);

    return rsesci_setEnumWindow (instrSession,
                                 "SENS%ld:TV:CCVS",
                                 window,
                                 ccvsSignal,
                                 ccvsSignalArr,
                                 VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get TV Trigger CCVS
 * Purpose:  This function returns selected CCVS signal as TV demodulator
 *           input.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetTVTriggerCCVS(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32*    ccvsSignal
)
{
    CHECKOPT (RSESCI_B6);

    return rsesci_getEnumWindow (instrSession,
                                 "SENS%ld:TV:CCVS?",
                                 window,
                                 ccvsSignal,
                                 ccvsSignalArr,
                                 VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set Trace Mode
 * Purpose:  This function defines the type of display and the evaluation of
 *           the traces in the selected measurement window.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetTraceMode(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32     trace,
    ViInt32     traceMode
)
{
    ViChar  buffer[BUFFER_SIZE];

    CHECKWINDOW;

    if (rsesci_invalidViInt32Range (trace, 1, 3))
        return VI_ERROR_PARAMETER3;

    sprintf (buffer, "DISP:WIND%ld:TRAC%ld:MODE", window, trace);

    return rsesci_setEnum (instrSession,
                         buffer,
                         traceMode,
                         traceModeArr,
                         VI_ERROR_PARAMETER4);
}

/*****************************************************************************
 * Function: Get Trace Mode
 * Purpose:  This function returns the type of display and the evaluation of
 *           the traces in the selected measurement window.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetTraceMode(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32     trace,
    ViInt32*    traceMode
)
{
    ViChar  buffer[BUFFER_SIZE];

    CHECKWINDOW;

    if (rsesci_invalidViInt32Range (trace, 1, 3))
        return VI_ERROR_PARAMETER3;

    sprintf (buffer, "DISP:WIND%ld:TRAC%ld:MODE?", window, trace);

    return rsesci_getEnum (instrSession,
                         buffer,
                         traceMode,
                         traceModeArr,
                         VI_ERROR_PARAMETER4);
}

/*****************************************************************************
 * Function: Set Trace Blanking
 * Purpose:  This function switches on or off the display of the corresponding
 *           trace in the selected measurement window.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetTraceBlanking(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32 trace,
    ViBoolean   traceState
)
{
    ViChar  buffer[BUFFER_SIZE];

    CHECKWINDOW;

    if (rsesci_invalidViInt32Range (trace, 1, 3))
        return VI_ERROR_PARAMETER3;

    sprintf (buffer, "DISP:WIND%ld:TRAC%ld:STAT", window, trace);

    return rsesci_setViBoolean (instrSession,
                                buffer,
                                traceState,
                                VI_ERROR_PARAMETER4);
}

/*****************************************************************************
 * Function: Get Trace Blanking
 * Purpose:  This function returns the state of the display of the
 *           corresponding trace in the selected measurement window.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetTraceBlanking(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32 trace,
    ViBoolean*  traceState
)
{
    ViChar  buffer[BUFFER_SIZE];

    CHECKWINDOW;

    if (rsesci_invalidViInt32Range (trace, 1, 3))
        return VI_ERROR_PARAMETER3;

    sprintf (buffer, "DISP:WIND%ld:TRAC%ld:STAT?", window, trace);

    return rsesci_getViBoolean (instrSession,
                                buffer,
                                traceState,
                                VI_ERROR_PARAMETER4);
}

/*****************************************************************************
 * Function: Set Trace Detector
 * Purpose:  This function switches on the detector for the data acquisition
 *           in the selected trace and the indicated measurement window.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetTraceDetector(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32 trace,
    ViInt32 traceDetector
)
{
    ViChar  buffer[BUFFER_SIZE];

    CHECKWINDOW;

    if (rsesci_invalidViInt32Range (trace, 1, 3))
        return VI_ERROR_PARAMETER3;
    if (rsesci_invalidViInt32Range (traceDetector, 0, 8))
        return VI_ERROR_PARAMETER4;

    sprintf (buffer, "SENS%ld:DET%ld:FUNC", window, trace);

    return rsesci_setEnum (instrSession,
                           buffer,
                           traceDetector,
                           traceDetectorArr,
                           VI_ERROR_PARAMETER4);
}

/*****************************************************************************
 * Function: Get Trace Detector
 * Purpose:  This function reads selected trace detector.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetTraceDetector(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32 trace,
    ViInt32*    traceDetector
)
{
    ViChar  buffer[BUFFER_SIZE];

    CHECKWINDOW;

    if (rsesci_invalidViInt32Range (trace, 1, 3))
        return VI_ERROR_PARAMETER3;

    sprintf (buffer, "SENS%ld:DET%ld:FUNC?", window, trace);

    return rsesci_getEnum (instrSession,
                           buffer,
                           traceDetector,
                           traceDetectorArr,
                           VI_ERROR_PARAMETER4);
}

/*****************************************************************************
 * Function: Set Trace Detector Auto
 * Purpose:  This function either couples the detector in the selected
 *           measurement window to the current trace setting or turns coupling
 *           off.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetTraceDetectorAuto(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32 trace,
    ViBoolean   coupleDetector
)
{
    ViChar  buffer[BUFFER_SIZE];

    CHECKWINDOW;

    if (rsesci_invalidViInt32Range (trace, 1, 3))
        return VI_ERROR_PARAMETER3;

    sprintf (buffer, "SENS%ld:DET%ld:FUNC:AUTO", window, trace);

    return rsesci_setViBoolean (instrSession,
                                buffer,
                                coupleDetector,
                                VI_ERROR_PARAMETER4);
}

/*****************************************************************************
 * Function: Get Trace Detector Auto
 * Purpose:  This function returns detector coupling state.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetTraceDetectorAuto(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32 trace,
    ViBoolean*  coupleDetector
)
{
    ViChar  buffer[BUFFER_SIZE];

    CHECKWINDOW;

    if (rsesci_invalidViInt32Range (trace, 1, 3))
        return VI_ERROR_PARAMETER3;

    sprintf (buffer, "SENS%ld:DET%ld:FUNC:AUTO?", window, trace);

    return rsesci_getViBoolean (instrSession,
                                buffer,
                                coupleDetector,
                                VI_ERROR_PARAMETER4);
}

/*****************************************************************************
 * Function: Set Trace Reset Behavior
 * Purpose:  This function specifies whether or not the traces with peak or 
 *           minimum value detection are reset after specific parameter changes.
 *           Usually the measurement must be restarted after a parameter
 *           change, before an evaluation of the measurement results is
 *           performed (e.g. with a marker). In cases in which a change
 *           causes a compulsory new measurement, the trace is automatically
 *           reset in order to prevent erroneous measurements of previous
 *           measurement results (e.g. when the span changes). For
 *           applications in which this behavior is not desired, this
 *           mechanism can be switched off.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetTraceResetBehavior (ViSession instrSession,
                                                ViInt32 window, ViInt32 trace,
                                                ViBoolean switchOffReset)
{
	ViChar  buffer[BUFFER_SIZE];

    CHECKWINDOW;

    if (rsesci_invalidViInt32Range (trace, 1, 3))
        return VI_ERROR_PARAMETER3;

    sprintf (buffer, "DISP:WIND%ld:TRAC%ld:MODE:HCON", window, trace);

    return rsesci_setViBoolean (instrSession,
                                buffer,
                                switchOffReset,
                                VI_ERROR_PARAMETER4);    
}

/*****************************************************************************
 * Function: Get Trace Reset Behavior
 * Purpose:  This function returns whether or not the traces with peak or 
 *           minimum value detection are reset after specific parameter changes.
 *           Usually the measurement must be restarted after a parameter
 *           change, before an evaluation of the measurement results is
 *           performed (e.g. with a marker). In cases in which a change
 *           causes a compulsory new measurement, the trace is automatically
 *           reset in order to prevent erroneous measurements of previous
 *           measurement results (e.g. when the span changes). For
 *           applications in which this behavior is not desired, this
 *           mechanism can be switched off.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetTraceResetBehavior (ViSession instrSession,
                                                ViInt32 window, ViInt32 trace,
                                                ViBoolean *switchOffReset)
{
	ViChar  buffer[BUFFER_SIZE];

    CHECKWINDOW;

    if (rsesci_invalidViInt32Range (trace, 1, 3))
        return VI_ERROR_PARAMETER3;

    sprintf (buffer, "DISP:WIND%ld:TRAC%ld:MODE:HCON?", window, trace);

    return rsesci_getViBoolean (instrSession,
                                buffer,
                                switchOffReset,
                                VI_ERROR_PARAMETER4);    
}

/*****************************************************************************
 * Function: Set Trace Math
 * Purpose:  This function switches the mathematical relation of traces on or
 *           off.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetTraceMath(
    ViSession   instrSession,
    ViInt32     window,
    ViBoolean   mathRelation
)
{
    return rsesci_setViBooleanWindow (instrSession,
                                      "CALC%ld:MATH:STAT",
                                      window,
                                      mathRelation,
                                      VI_ERROR_PARAMETER3);

}

/*****************************************************************************
 * Function: Get Trace Math
 * Purpose:  This function returns the state of the mathematical relation of
 *           traces.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetTraceMath(
    ViSession   instrSession,
    ViInt32     window,
    ViBoolean*  mathRelation
)
{
    return rsesci_getViBooleanWindow (instrSession,
                                      "CALC%ld:MATH:STAT?",
                                      window,
                                      mathRelation,
                                      VI_ERROR_PARAMETER3);

}

/*****************************************************************************
 * Function: Set Trace Math Mode
 * Purpose:  This function selects linear or logarithmic (= video) calculation
 *           of the mathematical functions related to the traces. The
 *           calculation of the average is one of the affected functions. The
 *           setting is valid for all measurement windows.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetTraceMathMode(
    ViSession   instrSession,
    ViInt32 mathMode
)
{
    return rsesci_setEnum (instrSession,
                           "CALC:MATH:MODE",
                           mathMode,
                           modeArr,
                           VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Trace Math Mode
 * Purpose:  This function returns selected mathematical mode.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetTraceMathMode(
    ViSession   instrSession,
    ViInt32*    mathMode
)
{
    return rsesci_getEnum (instrSession,
                           "CALC:MATH:MODE?",
                           mathMode,
                           modeArr,
                           VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set Trace Math Position
 * Purpose:  This function defines the position of the result of the trace
 *           mathematics in the selected measurement window. The indication is
 *           in % of the screen height, with 100% corresponding to the
 *           upper diagram border.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetTraceMathPosition(
    ViSession   instrSession,
    ViInt32     window,
    ViReal64    position
)
{
    return rsesci_setViReal64Window (instrSession,
                                     "CALC%ld:MATH:POS",
                                     window,
                                     position,
                                     -100 , 200,
                                     VI_ERROR_PARAMETER3);

}

/*****************************************************************************
 * Function: Get Trace Math Position
 * Purpose:  This function returns the position of the result of the trace
 *           mathematics in the selected measurement window.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetTraceMathPosition(
    ViSession   instrSession,
    ViInt32     window,
    ViReal64*   position
)
{
    return rsesci_getViReal64Window (instrSession,
                                     "CALC%ld:MATH:POS?",
                                     window,
                                     position,
                                     VI_ERROR_PARAMETER3);

}

/*****************************************************************************
 * Function: Set Trace Math Expression
 * Purpose:  This function defines the mathematical expression for relating
 *           traces to trace1.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetTraceMathExpression(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32 expression
)
{
    return rsesci_setEnumWindow (instrSession,
                                 "CALC%ld:MATH",
                                 window,
                                 expression,
                                 mathArr,
                                 VI_ERROR_PARAMETER3);

}

/*****************************************************************************
 * Function: Get Trace Math Expression
 * Purpose:  This function returns the mathematical expression for relating
 *           traces to trace1.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetTraceMathExpression(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32*    expression
)
{
    return rsesci_getEnumWindow (instrSession,
                                    "CALC%ld:MATH?",
                                    window,
                                    expression,
                                    mathArr,
                                    VI_ERROR_PARAMETER3);

}

/// HIFN  This function reads the time offset of the trigger in the sample.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/The measurement windows are selected by means of this parameter.
/// HIPAR triggerPosition/This control returns the time offset of the trigger in the sample.
ViStatus _VI_FUNC rsesci_GetIQTraceTriggerPositionInSample(
	ViSession	instrSession,
	ViInt32	window,
	ViReal64*	triggerPosition
)
{
	return rsesci_getViReal64Window (instrSession,
                                     "TRAC%ld:IQ:TPIS?",
                                     window,
                                     triggerPosition,
                                     VI_ERROR_PARAMETER3);    
}

/*****************************************************************************
 * Function: Copy Trace
 * Purpose:  This function copies data from one trace to another.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_CopyTrace(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32 sourceTrace,
    ViInt32 destinationTrace
)
{
    ViChar  buffer[BUFFER_SIZE];

    if (rsesci_invalidViInt32Range (sourceTrace, 1, 3))
        return VI_ERROR_PARAMETER3;

    if (rsesci_invalidViInt32Range (destinationTrace, 1, 3))
        return VI_ERROR_PARAMETER4;

    sprintf (buffer, "TRAC%%ld:COPY TRACE%ld,TRACE%ld", destinationTrace, sourceTrace);

    return rsesci_sendCmdWindow(instrSession, buffer, window);
}

/*****************************************************************************
 * Function: Set Trace Feed Control
 * Purpose:  This function switches block data transmission during a scan on
 *           and off. The block size depends on the scan time, the trace number
 *           is not evaluated.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetTraceFeedControl (ViSession instrSession,
                                              ViInt32 control,
                                              ViInt32 transmissionMode)
{
    ViChar  buffer[BUFFER_SIZE];

    if (rsesci_invalidViInt32Range (control, 1, 4))
        return VI_ERROR_PARAMETER2;

    sprintf (buffer, "TRAC:FEED:CONT%ld", control);

    return rsesci_setEnum (instrSession,
                           buffer,
                           transmissionMode,
                           transmissionModeArr,
                           VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Trace Feed Control
 * Purpose:  This function returns whether block data transmission during a
 *           scan is on or off.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetTraceFeedControl (ViSession instrSession,
                                              ViInt32 control,
                                              ViInt32 *transmissionMode)
{
    ViChar  buffer[BUFFER_SIZE];

    if (rsesci_invalidViInt32Range (control, 1, 4))
        return VI_ERROR_PARAMETER2;

    sprintf (buffer, "TRAC:FEED:CONT%ld?", control);

    return rsesci_getViInt32 (instrSession,
                              buffer,
                              transmissionMode,
                              VI_ERROR_PARAMETER3);
}

/// HIFN  This function clears all active traces.
/// HIFN     
/// HIFN     Clearing all traces is useful, e.g. when you perform a peak search 
/// HIFN  using the current trace buffer before the end of the sweep. The search 
/// HIFN  is not influenced by the part of the trace that has not yet been 
/// HIFN  updated.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/The measurement windows are selected by means of this parameter.
ViStatus _VI_FUNC rsesci_ClearAllActiveTraces(
	ViSession	instrSession,
	ViInt32	window)
{
    return rsesci_sendCmdWindow(instrSession, "DISP:WIND%ld:TRAC:CLE", window);    
}

/*****************************************************************************
 * Function: Read Trace Data
 * Purpose:  This function reads out trace data from the instrument.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_ReadTraceData(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32     trace,
    ViInt32*    noofPoints,
    ViReal64    _VI_FAR traceData[]
)
{
    ViUInt32 retCnt = 0;
    ViChar buffer[BUFFER_SIZE];
    ViChar *exBuf;
    ViReal32 *temp;
    ViInt32 lenLength, i = 0;
    ViStatus rsesci_status = VI_SUCCESS;
    ViUInt16 intfc = VI_INTF_GPIB;
    rsesci_instrRange instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_INTF_TYPE, &intfc));

    if (rsesci_invalidViInt32Range (trace, 1, 3))
        return VI_ERROR_PARAMETER2;

    
    CHECKERR (rsesci_clearBeforeRead(instrSession));
    /* Interface specific operations */
    switch (intfc)
        {
        case VI_INTF_ASRL: /* Serial Interface. */
            sprintf (buffer, ":FORM ASCII;*CLS;:TRAC%ld? %s\n", window, traceNamesArr[trace]);

            CHECKERR (viWrite (instrSession, (ViBuf) buffer, (ViUInt32) strlen (buffer), &retCnt));
            CHECKERR (rsesci_checkBeforeRead (instrSession));
            CHECKERR (viScanf (instrSession, "%,#Lf", noofPoints, traceData));
        break;
        case VI_INTF_GPIB: default:  /* GPIB Interface. Default case */
            sprintf (buffer, ":FORM REAL,32;*CLS;:TRAC%ld? %s\n", window, traceNamesArr[trace]);

            if ((rsesci_status = viWrite (instrSession, (ViBuf) buffer, (ViUInt32) strlen (buffer),
                &retCnt)) < 0)
                return rsesci_status;
            if ((rsesci_status = rsesci_checkBeforeRead (instrSession)) < 0)
            {
                viPrintf (instrSession, "FORM ASCII");
                return rsesci_status;
            }

            if ((rsesci_status = viRead (instrSession, (ViBuf) buffer, 2, &retCnt)) < 0)
            {
                viPrintf (instrSession, "FORM ASCII");
                return rsesci_status;
            }

            if (sscanf (buffer, "#%1d", &lenLength) != 1) {
                viPrintf (instrSession, "FORM ASCII");
                rsesci_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
                return rsesci_status;
            }

            if ((rsesci_status = viRead (instrSession, (ViBuf) buffer, lenLength,
                &retCnt)) < 0)
            {
                viPrintf (instrSession, "FORM ASCII");
                return rsesci_status;
            }

            if (sscanf (buffer, "%ld", noofPoints) != 1) {
                viPrintf (instrSession, "FORM ASCII");
                rsesci_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
                return rsesci_status;
            }

            /* Allocate memory space */
            if ((exBuf = malloc (*noofPoints + 1)) == NULL)
            {
                viPrintf (instrSession, "FORM ASCII");
                return VI_ERROR_ALLOC;
            }
            if ((rsesci_status = viRead (instrSession, (ViBuf) exBuf, *noofPoints + 1,
                &retCnt)) < 0)
                {
                /* If not NULL pointer, then deallocate memory. */
                if (exBuf) free (exBuf);
                viPrintf (instrSession, "FORM ASCII");
                return rsesci_status;
                }

            /* Convert binary data into array of ViReal64 types. */
            *noofPoints = (ViInt32)(*noofPoints/4);
            /* Binary data byte order (MOTOROLA or INTEL). */
            rsesci_SWAP_32((unsigned char *) exBuf, *noofPoints);
            /* Pointer to data */
            temp = (ViReal32*)(exBuf);
            for (i = 0; i < *noofPoints; i++)
                *traceData++ = (ViReal64)*temp++;

            if (exBuf) /* If not NULL pointer, then deallocate memory. */
                free (exBuf);
            if ((rsesci_status = viPrintf (instrSession, "FORM ASCII")) < 0)
                return rsesci_status;

        break;
        }

    if ((rsesci_status = rsesci_checkStatus (instrSession)) < 0)
        return rsesci_status;

    return rsesci_status;
}

/*****************************************************************************
 * Function: Write Trace Data
 * Purpose:  This function writes trace data to the instrument.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_WriteTraceData(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32     trace,
    ViInt32     noofPoints,
    ViReal64    _VI_FAR traceData[]
)
{
    ViStatus    rsesci_status        = VI_SUCCESS;
    ViUInt16    remote_interface    = VI_INTF_GPIB;
    ViChar*     pdynamic_buffer     = VI_NULL;
    ViChar*     pactual_position;
    ViInt32     digits;
    ViReal32    *pinstrument_data   = VI_NULL;
    ViReal64    *pdriver_data       = traceData;
    ViInt32     i = 0;
    rsesci_instrRange instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKERR( viGetAttribute (instrSession, VI_ATTR_INTF_TYPE, &remote_interface));

    if (rsesci_invalidViInt32Range (trace, 1, 3))
        return VI_ERROR_PARAMETER2;
    if (rsesci_invalidViInt32Range (noofPoints, 1, 250000))
        return VI_ERROR_PARAMETER3;

    /* Interface specific operations */
    switch (remote_interface)
        {
        case VI_INTF_ASRL: /* Serial Interface. */

            CHECKERR (viPrintf (instrSession, ":FORM ASCII;:TRAC%ld %s,%,*Lf\n",
                                window,
                                traceNamesArr[trace],
                                noofPoints,
                                traceData));
            break;

        case VI_INTF_GPIB: default: /* GPIB Interface. */

            /*
             * Alloc buffer. Size is calculated as:
             * max buffer for header + number of points * sizeof(ViReal32)
             *          64           +    noofPoints    * sizeof(ViReal32)
             */

            pdynamic_buffer  = (ViChar*)malloc(64 + noofPoints * sizeof(ViReal32));
            pactual_position = pdynamic_buffer;

            digits = ((int)floor(log10(noofPoints*(double)sizeof(ViReal32)))) + 1;

            pactual_position += sprintf (pactual_position,
                                         ":FORM REAL,32;:TRAC%ld %s, #%ld%ld",
                                         window,
                                         traceNamesArr[trace],
                                         digits,
                                         noofPoints*sizeof(ViReal32));

            /* Pointer to begining of data */
            pinstrument_data = (ViReal32*)pactual_position;

            for (i = 0; i < noofPoints; i++)
                *pinstrument_data++ = (ViReal32)*pdriver_data++;
            /* Binary data byte order (MOTOROLA or INTEL). */
            rsesci_SWAP_32((unsigned char*) pactual_position, noofPoints);
            pactual_position = (ViChar*)pinstrument_data;

            if ((rsesci_status = viWrite (instrSession,
                (ViBuf) ":SYST:COMM:GPIB:SELF:RTER EOI\n", 30, VI_NULL)) < 0)
                {
                /* If not NULL pointer, then deallocate memory. */
                free (pdynamic_buffer);
                return rsesci_status;
                }
            if ((rsesci_status = viWrite (instrSession, (ViBuf) pdynamic_buffer,
                                          (ViUInt32) (pactual_position - pdynamic_buffer),
                                          VI_NULL)) < 0)
                {
                /* If not NULL pointer, then deallocate memory. */
                viPrintf (instrSession, "FORM ASC");
                free (pdynamic_buffer);
                return rsesci_status;
                }
            if ((rsesci_status = viWrite (instrSession,
                (ViBuf) "FORM ASCII;:SYST:COMM:GPIB:SELF:RTER LFEOI\n", 42, VI_NULL)) < 0)
                {
                /* If not NULL pointer, then deallocate memory. */
                free (pdynamic_buffer);
                return rsesci_status;
                }

            /* Deallocate memory. */
            free (pdynamic_buffer);

            break;
        }

    CHECKERR( rsesci_checkStatus (instrSession));

    return rsesci_status;
} 

/*****************************************************************************
 * Function: Set IQ Trace
 * Purpose:  This function switches the I/Q data acquisition on or off.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetIQTrace (
    ViSession instrSession,
    ViInt32     window,
    ViBoolean IQTraceState
)
{
    return rsesci_setViBooleanWindow (instrSession,
                                      "TRAC%ld:IQ:STAT",
                                      window,
                                      IQTraceState,
                                      VI_ERROR_PARAMETER3);

}

/*****************************************************************************
 * Function: Get IQ Trace
 * Purpose:  This function returns the state of the I/Q data acquisition.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetIQTrace (
    ViSession instrSession,
    ViInt32     window,
    ViBoolean *IQTraceState
)
{
    return rsesci_getViBooleanWindow (instrSession,
                                      "TRAC%ld:IQ:STAT?",
                                      window,
                                      IQTraceState,
                                      VI_ERROR_PARAMETER3);

}

/*****************************************************************************
 * Function: Set IQ Trace Averaging
 * Purpose:  This function enables averaging of the recorded I/Q data provided
 *           that I/Q data acquisition was previously enabled and the sampling
 *           rate was set to 32 MHz.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetIQTraceAveraging(
    ViSession   instrSession,
    ViInt32     window,
    ViBoolean   iqTraceState
)
{
    return rsesci_setViBooleanWindow (instrSession,
                                      "TRAC%ld:IQ:AVER:STAT",
                                      window,
                                      iqTraceState,
                                      VI_ERROR_PARAMETER3);

}

/*****************************************************************************
 * Function: Get IQ Trace Averaging
 * Purpose:  This function returns the state of averaging of the recorded I/Q
 *           data.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetIQTraceAveraging(
    ViSession   instrSession,
    ViInt32     window,
    ViBoolean*  iqTraceState
)
{
    return rsesci_getViBooleanWindow (instrSession,
                                      "TRAC%ld:IQ:AVER:STAT?",
                                      window,
                                      iqTraceState,
                                      VI_ERROR_PARAMETER3);

}

/*****************************************************************************
 * Function: Set IQ Trace Averaging Count
 * Purpose:  This function defines the number of I/Q data sets that are to
 *           serve as a basis for averaging.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetIQTraceAveragingCount(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32 averageCount
)
{
    return rsesci_setViInt32Window (instrSession,
                                      "TRAC%ld:IQ:AVER:COUN",
                                      window,
                                      averageCount,
                                      0 , 32767,
                                      VI_ERROR_PARAMETER3);

}

/*****************************************************************************
 * Function: Get IQ Trace Averaging Count
 * Purpose:  This function returns the number of I/Q data sets that are to
 *           serve as a basis for averaging.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetIQTraceAveragingCount(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32*    averageCount
)
{
    return rsesci_getViInt32Window (instrSession,
                                      "TRAC%ld:IQ:AVER:COUN?",
                                      window,
                                      averageCount,
                                      VI_ERROR_PARAMETER3);

}

/*****************************************************************************
 * Function: Set IQ Trace Synchronization
 * Purpose:  This function switches on or off the synchronization of the start
 *           of I/Q data acquisition to the trigger event. Prior to this
 *           functions, I/Q data acquisition has to be switched. The sampling
 *           rate must be 32 MHz. The synchronization ensures the correct
 *           calculation of I/Q average values as the data aquisition is always
 *           started with the same phase reference in relation to the trigger
 *           event.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetIQTraceSynchronization(
    ViSession   instrSession,
    ViInt32     window,
    ViBoolean   state
)
{
    return rsesci_setViBooleanWindow (instrSession,
                                      "TRAC%ld:IQ:SYNC:STAT",
                                      window,
                                      state,
                                      VI_ERROR_PARAMETER3);

}

/*****************************************************************************
 * Function: Get IQ Trace Synchronization
 * Purpose:  This function returns the state of the synchronization of the
 *           start of I/Q data acquisition to the trigger event.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetIQTraceSynchronization(
    ViSession   instrSession,
    ViInt32     window,
    ViBoolean*  state
)
{
    return rsesci_getViBooleanWindow (instrSession,
                                      "TRAC%ld:IQ:SYNC:STAT?",
                                      window,
                                      state,
                                      VI_ERROR_PARAMETER3);

}

/*****************************************************************************
 * Function: Set IQ Trace Sampling Rate
 * Purpose:  This function sets the sampling rate for the I/Q data
 *           acquisition. Thus the sample rate can be modified without
 *           affecting the other settings.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetIQTraceSamplingRate(
    ViSession   instrSession,
    ViInt32     window,
    ViReal64    samplingRate
)
{
    return rsesci_setViReal64Window (instrSession,
                                     "TRAC%ld:IQ:SRAT",
                                     window,
                                     samplingRate,
                                     15.625E+3 , 32.0E+6 ,
                                     VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get IQ Trace Sampling Rate
 * Purpose:  This function reads sampling rate for the I/Q data acquisition.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetIQTraceSamplingRate(
    ViSession   instrSession,
    ViInt32     window,
    ViReal64*   samplingRate
)
{
    return rsesci_getViReal64Window (instrSession,
                                     "TRAC%ld:IQ:SRAT?",
                                     window,
                                     samplingRate,
                                     VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set IQ Trace Settings
 * Purpose:  This function defines the settings of the analyzer hardware for
 *           the measurement of I/Q data. This allows setting the bandwidth of
 *           the analog filters in front of the A/D converter as well as
 *           setting the sample rate, trigger conditions and the record length.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetIQTraceSettings(
    ViSession   instrSession,
    ViInt32     window,
    ViReal64    samplingRate,
    ViInt32     triggerMode,
    ViInt32     polarity,
    ViReal64    bandwidth,
    ViInt32     pretriggerSamples,
    ViInt32     noofSamples)
{
    ViChar buffer[BUFFER_SIZE];

    CHECKWINDOW;

    if (rsesci_invalidViReal64Range (samplingRate, 15.625E+3, 32.0E+6))
        return VI_ERROR_PARAMETER3;
    if (rsesci_invalidViInt32Range (triggerMode, 0, 4))
        return VI_ERROR_PARAMETER4;
    if (rsesci_invalidViInt32Range (polarity, 0, 1))
        return VI_ERROR_PARAMETER5;
    if (rsesci_invalidViReal64Range (bandwidth, 0.0, 9.6e6))
        return VI_ERROR_PARAMETER6;
    if (rsesci_invalidViInt32Range (pretriggerSamples, -16744447, 65023))
        return VI_ERROR_PARAMETER7;
    if (rsesci_invalidViInt32Range (noofSamples, 1, 1130560))
        return VI_ERROR_PARAMETER8;

    sprintf (buffer, ":TRAC%ld:IQ:SET NORM,%.12lf,%.12lf,%s,%s,%ld,%ld\n",
            window, bandwidth, samplingRate, triggersourceArr[triggerMode],
            polarityArr[polarity], pretriggerSamples, noofSamples);

    return rsesci_sendCmd (instrSession, buffer);
}

/*****************************************************************************
 * Function: Get IQ Trace Settings
 * Purpose:  This function queries the settings of the analyzer hardware for the
 *           measurement of I/Q data. This allows setting the bandwidth of
 *           the analog filters in front of the A/D converter as well as
 *           setting the sample rate, trigger conditions and the record
 *           length.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetIQTraceSettings(
    ViSession   instrSession,
    ViInt32     window,
    ViPReal64   samplingRate,
    ViPInt32    triggerMode,
    ViPInt32    polarity,
    ViPReal64   bandwidth,
    ViPInt32    pretriggerSamples,
    ViPInt32    noofSamples
)
{
    ViStatus    rsesci_status;
    ViChar      buffer[BUFFER_SIZE] = "";
    ViChar      str_polarity [10],
                str_triggerMode[10];

    CHECKNULL (samplingRate, 3);
    CHECKNULL (triggerMode, 4);
    CHECKNULL (polarity, 5);
    CHECKNULL (bandwidth, 6);
    CHECKNULL (pretriggerSamples, 7);
    CHECKNULL (noofSamples, 8);

    CHECKERR (rsesci_getViStringWindow (instrSession,
                                        ":TRAC%ld:IQ:SET?",
                                        window,
                                        buffer,
                                        BUFFER_SIZE,
                                        VI_ERROR_PARAMETER2));

    sscanf (buffer, "NORM,%le,%le,%[^,],%[^,],%ld,%ld", bandwidth, samplingRate,
                    str_triggerMode, str_polarity, pretriggerSamples, noofSamples);

    CHECKERR (rsesci_getIndexFromString (instrSession, str_triggerMode, triggersourceArr, triggerMode));
    CHECKERR (rsesci_getIndexFromString (instrSession, str_polarity, polarityArr, polarity));

    return rsesci_status;
}

/*****************************************************************************
 * Function: Read IQ Trace Data
 * Purpose:  This function starts a measurement with the settings defined via
 *           rsesci_SetIQTraceSettings() and returns the list of measurement
 *           results immediately after they are corrected in terms of frequency
 *           response. The number of measurement results depends on the
 *           settings defined with rsesci_SetIQTraceSettings().
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_ReadIQTraceData(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32     bufferSize,
    ViInt32*    noofPoints,
    ViReal64    _VI_FAR realParts_I[],
    ViReal64    _VI_FAR imaginaryParts_Q[]
)
{
    ViStatus    rsesci_status = VI_SUCCESS;
    ViUInt32    retCnt = 0;
    ViChar      buffer[BUFFER_SIZE];
    ViChar      *exBuf = 0;
    ViInt32     count = 0;
    ViInt32     lenLength, i = 0, j = 0;
    ViUInt16    intfc = VI_INTF_GPIB;

	window;
    if (bufferSize < 0)
        return VI_ERROR_PARAMETER2;
    if (bufferSize > 0)
    {
        if (realParts_I == VI_NULL)
           return VI_ERROR_PARAMETER4;
        if (imaginaryParts_Q == VI_NULL)
            return VI_ERROR_PARAMETER5;
    }

    CHECKERR( viGetAttribute (instrSession, VI_ATTR_INTF_TYPE, &intfc));

    CHECKERR (rsesci_clearBeforeRead(instrSession));

    sprintf (buffer, ":FORM REAL,32;*CLS;:TRAC:IQ:DATA?\n");

    CHECKERR( viWrite (instrSession, (ViBuf) buffer, (ViUInt32) strlen (buffer), &retCnt));
    if ((rsesci_status = rsesci_checkBeforeRead (instrSession)) < 0)
    {
        viPrintf (instrSession, "FORM ASCII");
        return rsesci_status;
    }

    /* Interface specific operations */
    switch (intfc)
        {
        case VI_INTF_ASRL: /* Serial Interface. */
            /* Switch to Off using of termination character. */
            CHECKERR( viSetAttribute (instrSession, VI_ATTR_ASRL_END_IN, VI_ASRL_END_NONE));
            CHECKERR( viSetAttribute (instrSession, VI_ATTR_TERMCHAR_EN, VI_FALSE));

            /* Parse binary data header. */
            if ((rsesci_status = viRead (instrSession, (ViBuf) buffer, 2, &retCnt)) < 0)
                {
                viPrintf (instrSession, "FORM ASCII");
                viSetAttribute (instrSession, VI_ATTR_ASRL_END_IN, VI_ASRL_END_TERMCHAR);
                viSetAttribute (instrSession, VI_ATTR_TERMCHAR_EN, VI_TRUE);
                return rsesci_status;
                }
            buffer[retCnt] = '\0';
            if (sscanf (buffer, "#%1d", &lenLength) != 1)
                {
                viPrintf (instrSession, "FORM ASCII");
                viSetAttribute (instrSession, VI_ATTR_ASRL_END_IN, VI_ASRL_END_TERMCHAR);
                viSetAttribute (instrSession, VI_ATTR_TERMCHAR_EN, VI_TRUE);
                return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
                }

            /* Get number of points. */
            if ((rsesci_status = viRead (instrSession, (ViBuf) buffer, lenLength, &retCnt)) < 0)
                {
                viPrintf (instrSession, "FORM ASCII");
                viSetAttribute (instrSession, VI_ATTR_ASRL_END_IN, VI_ASRL_END_TERMCHAR);
                viSetAttribute (instrSession, VI_ATTR_TERMCHAR_EN, VI_TRUE);
                return rsesci_status;
                }

            buffer[retCnt] = '\0';
            if (sscanf (buffer, "%ld", &count) != 1)
                {
                viPrintf (instrSession, "FORM ASCII");
                viSetAttribute (instrSession, VI_ATTR_ASRL_END_IN, VI_ASRL_END_TERMCHAR);
                viSetAttribute (instrSession, VI_ATTR_TERMCHAR_EN, VI_TRUE);
                return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
                }

            /* Allocate memory space */
            if ((exBuf = malloc (count + 2)) == NULL)
            {
                viPrintf (instrSession, "FORM ASCII");
                return VI_ERROR_ALLOC;
            }
            /* Read exact number of bytes equal to number of points. */
            if ((rsesci_status = viRead (instrSession, (ViBuf) exBuf, count + 2, &retCnt)) < 0)
                {
                viSetAttribute (instrSession, VI_ATTR_ASRL_END_IN, VI_ASRL_END_TERMCHAR);
                viSetAttribute (instrSession, VI_ATTR_TERMCHAR_EN, VI_TRUE);
                /* If not NULL pointer, then deallocate memory. */
                if (exBuf) free (exBuf);
                viPrintf (instrSession, "FORM ASCII");
                return rsesci_status;
                }
            viSetAttribute (instrSession, VI_ATTR_ASRL_END_IN, VI_ASRL_END_TERMCHAR);
            viSetAttribute (instrSession, VI_ATTR_TERMCHAR_EN, VI_TRUE);
        break;
        case VI_INTF_GPIB: default:  /* GPIB Interface. Default case */
            CHECKERR( viRead (instrSession, (ViBuf) buffer, 2, &retCnt));
            buffer[retCnt] = '\0';

            if (sscanf (buffer, "#%1d", &lenLength) != 1)
            {
                viPrintf (instrSession, "FORM ASCII");
                return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
            }

            CHECKERR( viRead (instrSession, (ViBuf) buffer, lenLength, &retCnt));
            buffer[retCnt] = '\0';

            if (sscanf (buffer, "%ld", &count) != 1)
            {
                viPrintf (instrSession, "FORM ASCII");
                return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
            }

          /* Allocate memory space */
          if ((exBuf = malloc (count + 4)) == NULL)
          {
              viPrintf (instrSession, "FORM ASCII");
              return VI_ERROR_ALLOC;
          }

          if ((rsesci_status = viRead (instrSession, (ViBuf) exBuf, count + 4, &retCnt)) < 0)
              {
              /* If not NULL pointer, then deallocate memory. */
              viPrintf (instrSession, "FORM ASCII");
              if (exBuf) free (exBuf);
              return rsesci_status;
              }
        break;
        }

    if (count <= 4)
    {
        viPrintf (instrSession, "FORM ASCII");
        free (exBuf);
        return RSESCI_ERROR_INSTRUMENT_IQ_OVERLOAD;
    }

    count /= 4; /* number of I and Q samples */
    /* Binary data byte order (MOTOROLA or INTEL). */
    if (bufferSize > 0)
        rsesci_SWAP_32((unsigned char *)exBuf, count);

    if (bufferSize > 0)
    {
        ViInt32 loopCount = count,
                remainder = 0,
                blockSize = 524288; /* I or Q block size (number of values) */

        ViReal64 *p2I = realParts_I,
                 *p2Q = imaginaryParts_Q;
        ViReal32 *tp2I,
                 *tp2Q;
        ViChar   *p2exBuf = exBuf;

        /*
            If number of I/Q samples is > 512k (524288 samples) block tranfer mode applies
                "<I-val 1><I-val 2>..<I-val 524288><Q-val 1><Q-val 2>..<Q-val 524288>..<..>"
            else returned data are splited in half for I and rest for Q
                "<I-val 1><I-val 2>..<I-val count/2><Q-val 1><Q-val 2>..<Q-val count/2>.."
        */

        loopCount = ((bufferSize * 2) < loopCount) ? (bufferSize * 2) : loopCount;

        /* number of I or Q samples */
        if (noofPoints)
            *noofPoints = loopCount / 2;

        /* Number of I/Q data blocks */
        remainder = loopCount % (blockSize * 2);
        loopCount /= (blockSize * 2);

        /* Make sure that loop counter is set to zero! */
        i = 0;

        /* If loopCount is not zero, retrieve all data */
        if (loopCount)
            {
            /* Number of I and Q data blocks */
            for (i = 0; i < loopCount; i++)
                {
                /* Set pointers */
                tp2I = (ViReal32*)(p2exBuf + (blockSize * 4) * (i*2));
                tp2Q = (ViReal32*)(p2exBuf + (blockSize * 4) * (i*2 + 1));
                /* I and Q values */
                for (j = 0; j < blockSize; j++)
                    {
                    *p2I++ = (ViReal64)*tp2I++;
                    *p2Q++ = (ViReal64)*tp2Q++;
                    }
                }
            }

        /* If reminder is not zero, retrieve remaining data */
        if (remainder)
            {
            /* Set pointers */
            if (i == 0)
                {
                tp2I = (ViReal32*)(p2exBuf);
                tp2Q = (ViReal32*)(p2exBuf + (remainder * 2));
                }
            else
                {
                tp2I = (ViReal32*)(p2exBuf + (blockSize * 4) * (2*i));
                tp2Q = (ViReal32*)(p2exBuf + (blockSize * 4) * (2*i) + (remainder*2));
                }
            /* I and Q values */
            for (j = 0; j < remainder; j += 2)
                {
                *p2I++ = (ViReal64)*tp2I++;
                *p2Q++ = (ViReal64)*tp2Q++;
                }
            }
    }

    if (exBuf) /* If not NULL pointer, then deallocate memory. */
        free (exBuf);

    CHECKERR (viPrintf (instrSession, "FORM ASCII"));

    CHECKERR (rsesci_checkStatus (instrSession));

    if ((count / 2) > bufferSize)
        rsesci_status = count;

    return rsesci_status;
}

/*****************************************************************************
 * Function: Read IQ Trace Data Memory
 * Purpose:  This function permits the readout of previously acquired (and
 *           frequency response corrected) I/Q data from the memory, with
 *           indication of the offset related to the start of measurement and
 *           with indication of the number of measurement values. Therefore a
 *           previously acquired data set can be read out in smaller portions.
 *           The result values are scaled linear in unit Volt and correspond to
 *           the voltage at the RF input of the instrument.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_ReadIQTraceDataMemory(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32     offsetSamples,
    ViInt32     noofSamples,
    ViInt32     bufferSize,
    ViInt32*    noofPoints,
    ViReal64    _VI_FAR realParts_I[],
    ViReal64    _VI_FAR imaginaryParts_Q[]
)
{
    ViStatus    rsesci_status = VI_SUCCESS;
    ViUInt32    retCnt = 0;
    ViChar      buffer[BUFFER_SIZE];
    ViChar      *exBuf = 0;
    ViInt32     count = 0;
    ViInt32     lenLength, i = 0, j = 0;
    ViUInt16    intfc = VI_INTF_GPIB;
	window;

    if (bufferSize < 0)
        return VI_ERROR_PARAMETER2;
    if (bufferSize > 0)
    {
    if (realParts_I == VI_NULL)
       return VI_ERROR_PARAMETER4;
    if (imaginaryParts_Q == VI_NULL)
        return VI_ERROR_PARAMETER5;
    }

    if ((rsesci_status = viGetAttribute (instrSession, VI_ATTR_INTF_TYPE, &intfc)) <  VI_SUCCESS)
        return rsesci_status;

    CHECKERR (rsesci_clearBeforeRead(instrSession));

    sprintf (buffer, ":FORM REAL,32;*CLS;:TRAC:IQ:DATA:MEM? %ld,%ld\n",
        offsetSamples, noofSamples);

    if ((rsesci_status = viWrite (instrSession, (ViBuf) buffer, (ViUInt32) strlen (buffer), &retCnt)) < 0)
        return rsesci_status;
    if ((rsesci_status = rsesci_checkBeforeRead (instrSession)) < 0)
    {
        viPrintf (instrSession, "FORM ASCII");
        return rsesci_status;
    }

    /* Interface specific operations */
    switch (intfc)
        {
        case VI_INTF_ASRL: /* Serial Interface. */
            /* Switch to Off using of termination character. */
            if ((rsesci_status = viSetAttribute (instrSession, VI_ATTR_ASRL_END_IN, VI_ASRL_END_NONE)) < 0)
                return rsesci_status;
            if ((rsesci_status =  viSetAttribute (instrSession, VI_ATTR_TERMCHAR_EN, VI_FALSE)) < 0)
                return rsesci_status;

            /* Parse binary data header. */
            if ((rsesci_status = viRead (instrSession, (ViBuf) buffer, 2, &retCnt)) < 0)
                {
                viSetAttribute (instrSession, VI_ATTR_ASRL_END_IN, VI_ASRL_END_TERMCHAR);
                viSetAttribute (instrSession, VI_ATTR_TERMCHAR_EN, VI_TRUE);
                viPrintf (instrSession, "FORM ASCII");
                return rsesci_status;
                }

            if (sscanf (buffer, "#%1d", &lenLength) != 1)
                {
                viSetAttribute (instrSession, VI_ATTR_ASRL_END_IN, VI_ASRL_END_TERMCHAR);
                viSetAttribute (instrSession, VI_ATTR_TERMCHAR_EN, VI_TRUE);
                viPrintf (instrSession, "FORM ASCII");
                return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
                }

            /* Get number of points. */
            if ((rsesci_status = viRead (instrSession, (ViBuf) buffer, lenLength, &retCnt)) < 0)
                {
                viSetAttribute (instrSession, VI_ATTR_ASRL_END_IN, VI_ASRL_END_TERMCHAR);
                viSetAttribute (instrSession, VI_ATTR_TERMCHAR_EN, VI_TRUE);
                viPrintf (instrSession, "FORM ASCII");
                return rsesci_status;
                }

            if (sscanf (buffer, "%ld", &count) != 1)
                {
                viSetAttribute (instrSession, VI_ATTR_ASRL_END_IN, VI_ASRL_END_TERMCHAR);
                viSetAttribute (instrSession, VI_ATTR_TERMCHAR_EN, VI_TRUE);
                viPrintf (instrSession, "FORM ASCII");
                return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
                }

            /* Allocate memory space */
            if ((exBuf = malloc (count + 2)) == NULL)
            {
                viPrintf (instrSession, "FORM ASCII");
                return VI_ERROR_ALLOC;
            }
            /* Read exact number of bytes equal to number of points. */
            if ((rsesci_status = viRead (instrSession, (ViBuf) exBuf, count + 2, &retCnt)) < 0)
                {
                viSetAttribute (instrSession, VI_ATTR_ASRL_END_IN, VI_ASRL_END_TERMCHAR);
                viSetAttribute (instrSession, VI_ATTR_TERMCHAR_EN, VI_TRUE);
                /* If not NULL pointer, then deallocate memory. */
                viPrintf (instrSession, "FORM ASCII");
                if (exBuf) free (exBuf);
                return rsesci_status;
                }
            viSetAttribute (instrSession, VI_ATTR_ASRL_END_IN, VI_ASRL_END_TERMCHAR);
            viSetAttribute (instrSession, VI_ATTR_TERMCHAR_EN, VI_TRUE);
        break;
        case VI_INTF_GPIB: default:  /* GPIB Interface. Default case */
            if ((rsesci_status = viRead (instrSession, (ViBuf) buffer, 2, &retCnt)) < 0)
            {
                viPrintf (instrSession, "FORM ASCII");
                return rsesci_status;
            }
            if (sscanf (buffer, "#%1d", &lenLength) != 1) 
            {
                viPrintf (instrSession, "FORM ASCII");
                return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
            }

            if ((rsesci_status = viRead (instrSession, (ViBuf) buffer, lenLength, &retCnt)) < 0)
            {
                viPrintf (instrSession, "FORM ASCII");
                return rsesci_status;
            }
            if (sscanf (buffer, "%ld", &count) != 1)
            {
                viPrintf (instrSession, "FORM ASCII");
                return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
            }

            /* Allocate memory space */
            if ((exBuf = malloc (count + 4)) == NULL)
            {
                viPrintf (instrSession, "FORM ASCII");
                return VI_ERROR_ALLOC;
            }
            if ((rsesci_status = viRead (instrSession, (ViBuf) exBuf, count + 4, &retCnt)) < 0)
                {
                viPrintf (instrSession, "FORM ASCII");
                /* If not NULL pointer, then deallocate memory. */
                if (exBuf) free (exBuf);
                return rsesci_status;
                }
        break;
        }

    count /= 4; /* number of I and Q samples */
    /* Binary data byte order (MOTOROLA or INTEL). */
    if (bufferSize > 0)
        rsesci_SWAP_32((unsigned char *) exBuf, count);

    if (bufferSize > 0)
    {
        ViInt32 loopCount = count,
                remainder = 0,
                blockSize = 524288; /* I or Q block size (number of values) */

        ViReal64 *p2I = realParts_I,
                 *p2Q = imaginaryParts_Q;
        ViReal32 *tp2I,
                 *tp2Q;
        ViChar   *p2exBuf = exBuf;

        /*
            If number of I/Q samples is > 512k (524288 samples) block tranfer mode applies
                "<I-val 1><I-val 2>..<I-val 524288><Q-val 1><Q-val 2>..<Q-val 524288>..<..>"
            else returned data are splited in half for I and rest for Q
                "<I-val 1><I-val 2>..<I-val count/2><Q-val 1><Q-val 2>..<Q-val count/2>.."
        */

        loopCount = ((bufferSize * 2) < loopCount) ? (bufferSize * 2) : loopCount;

        /* number of I or Q samples */
        if (noofPoints)
            *noofPoints = loopCount / 2;

        /* Number of I/Q data blocks */
        remainder = loopCount % (blockSize * 2);
        loopCount /= (blockSize * 2);

        /* Make sure that loop counter is set to zero! */
        i = 0;

        /* If loopCount is not zero, retrieve all data */
        if (loopCount)
            {
            /* Number of I and Q data blocks */
            for (i = 0; i < loopCount; i++)
                {
                /* Set pointers */
                tp2I = (ViReal32*)(p2exBuf + (blockSize * 4) * (i*2));
                tp2Q = (ViReal32*)(p2exBuf + (blockSize * 4) * (i*2 + 1));
                /* I and Q values */
                for (j = 0; j < blockSize; j++)
                    {
                    *p2I++ = (ViReal64)*tp2I++;
                    *p2Q++ = (ViReal64)*tp2Q++;
                    }
                }
            }

        /* If reminder is not zero, retrieve remaining data */
        if (remainder)
            {
            /* Set pointers */
            if (i == 0)
                {
                tp2I = (ViReal32*)(p2exBuf);
                tp2Q = (ViReal32*)(p2exBuf + (remainder * 2));
                }
            else
                {
                tp2I = (ViReal32*)(p2exBuf + (blockSize * 4) * (2*i));
                tp2Q = (ViReal32*)(p2exBuf + (blockSize * 4) * (2*i) + (remainder*2));
                }
            /* I and Q values */
            for (j = 0; j < remainder; j += 2)
                {
                *p2I++ = (ViReal64)*tp2I++;
                *p2Q++ = (ViReal64)*tp2Q++;
                }
            }
    }

    if (exBuf) /* If not NULL pointer, then deallocate memory. */
        free (exBuf);

    if ((rsesci_status = viPrintf (instrSession, "FORM ASCII")) < 0)
        return rsesci_status;

    if ((rsesci_status = rsesci_checkStatus (instrSession)) < 0)
        return rsesci_status;

    if ((count / 2) > bufferSize)
        rsesci_status = count;

    return rsesci_status;
}

/*****************************************************************************
 * Function: Read IQ Trace Data Memory AIQ
 * Purpose:  This function permits the readout of previously acquired (and frequency
 *           response corrected) I/Q data from the memory, with indication of
 *           the offset related to the start of measurement and with
 *           indication of the number of measurement values. Therefore a
 *           previously acquired data set can be read out in smaller
 *           portions. The result values are scaled linear in unit Volt and
 *           correspond to the voltage at the RF input of the instrument.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_ReadIQTraceDataMemoryAIQ (ViSession instrSession,
                                                   ViInt32 window,
                                                   ViInt32 offsetSamples,
                                                   ViInt32 noofSamples,
                                                   ViInt32 bufferSize,
                                                   ViInt32 *noofPoints,
                                                   ViReal64 realParts_I[],
                                                   ViReal64 imaginaryParts_Q[])
{
	ViStatus    rsesci_status = VI_SUCCESS;
    ViUInt32    retCnt = 0;
    ViChar      buffer[BUFFER_SIZE];
    ViChar      *exBuf = 0;
    ViInt32     count = 0;
    ViInt32     lenLength, i = 0, j = 0;
    ViUInt16    intfc = VI_INTF_GPIB;

	window;

    if (bufferSize < 0)
        return VI_ERROR_PARAMETER2;
    if (bufferSize > 0)
    {
    if (realParts_I == VI_NULL)
       return VI_ERROR_PARAMETER4;
    if (imaginaryParts_Q == VI_NULL)
        return VI_ERROR_PARAMETER5;
    }

    if ((rsesci_status = viGetAttribute (instrSession, VI_ATTR_INTF_TYPE, &intfc)) <  VI_SUCCESS)
        return rsesci_status;

    CHECKERR (rsesci_clearBeforeRead(instrSession));

    sprintf (buffer, ":FORM REAL,32;*CLS;:TRAC:IQ:DATA:MEM:AIQ? %ld,%ld\n",
        offsetSamples, noofSamples);

    if ((rsesci_status = viWrite (instrSession, (ViBuf) buffer, (ViUInt32) strlen (buffer), &retCnt)) < 0)
        return rsesci_status;
    if ((rsesci_status = rsesci_checkBeforeRead (instrSession)) < 0)
    {
        viPrintf (instrSession, "FORM ASCII");
        return rsesci_status;
    }

    /* Interface specific operations */
    switch (intfc)
        {
        case VI_INTF_ASRL: /* Serial Interface. */
            /* Switch to Off using of termination character. */
            if ((rsesci_status = viSetAttribute (instrSession, VI_ATTR_ASRL_END_IN, VI_ASRL_END_NONE)) < 0)
                return rsesci_status;
            if ((rsesci_status =  viSetAttribute (instrSession, VI_ATTR_TERMCHAR_EN, VI_FALSE)) < 0)
                return rsesci_status;

            /* Parse binary data header. */
            if ((rsesci_status = viRead (instrSession, (ViBuf) buffer, 2, &retCnt)) < 0)
                {
                viSetAttribute (instrSession, VI_ATTR_ASRL_END_IN, VI_ASRL_END_TERMCHAR);
                viSetAttribute (instrSession, VI_ATTR_TERMCHAR_EN, VI_TRUE);
                viPrintf (instrSession, "FORM ASCII");
                return rsesci_status;
                }

            if (sscanf (buffer, "#%1d", &lenLength) != 1)
                {
                viSetAttribute (instrSession, VI_ATTR_ASRL_END_IN, VI_ASRL_END_TERMCHAR);
                viSetAttribute (instrSession, VI_ATTR_TERMCHAR_EN, VI_TRUE);
                viPrintf (instrSession, "FORM ASCII");
                return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
                }

            /* Get number of points. */
            if ((rsesci_status = viRead (instrSession, (ViBuf) buffer, lenLength, &retCnt)) < 0)
                {
                viSetAttribute (instrSession, VI_ATTR_ASRL_END_IN, VI_ASRL_END_TERMCHAR);
                viSetAttribute (instrSession, VI_ATTR_TERMCHAR_EN, VI_TRUE);
                viPrintf (instrSession, "FORM ASCII");
                return rsesci_status;
                }

            if (sscanf (buffer, "%ld", &count) != 1)
                {
                viSetAttribute (instrSession, VI_ATTR_ASRL_END_IN, VI_ASRL_END_TERMCHAR);
                viSetAttribute (instrSession, VI_ATTR_TERMCHAR_EN, VI_TRUE);
                viPrintf (instrSession, "FORM ASCII");
                return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
                }

            /* Allocate memory space */
            if ((exBuf = malloc (count + 2)) == NULL)
            {
                viPrintf (instrSession, "FORM ASCII");
                return VI_ERROR_ALLOC;
            }
            /* Read exact number of bytes equal to number of points. */
            if ((rsesci_status = viRead (instrSession, (ViBuf) exBuf, count + 2, &retCnt)) < 0)
                {
                viSetAttribute (instrSession, VI_ATTR_ASRL_END_IN, VI_ASRL_END_TERMCHAR);
                viSetAttribute (instrSession, VI_ATTR_TERMCHAR_EN, VI_TRUE);
                /* If not NULL pointer, then deallocate memory. */
                viPrintf (instrSession, "FORM ASCII");
                if (exBuf) free (exBuf);
                return rsesci_status;
                }
            viSetAttribute (instrSession, VI_ATTR_ASRL_END_IN, VI_ASRL_END_TERMCHAR);
            viSetAttribute (instrSession, VI_ATTR_TERMCHAR_EN, VI_TRUE);
        break;
        case VI_INTF_GPIB: default:  /* GPIB Interface. Default case */
            if ((rsesci_status = viRead (instrSession, (ViBuf) buffer, 2, &retCnt)) < 0)
            {
                viPrintf (instrSession, "FORM ASCII");
                return rsesci_status;
            }
            if (sscanf (buffer, "#%1d", &lenLength) != 1) 
            {
                viPrintf (instrSession, "FORM ASCII");
                return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
            }

            if ((rsesci_status = viRead (instrSession, (ViBuf) buffer, lenLength, &retCnt)) < 0)
            {
                viPrintf (instrSession, "FORM ASCII");
                return rsesci_status;
            }
            if (sscanf (buffer, "%ld", &count) != 1)
            {
                viPrintf (instrSession, "FORM ASCII");
                return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
            }

            /* Allocate memory space */
            if ((exBuf = malloc (count + 4)) == NULL)
            {
                viPrintf (instrSession, "FORM ASCII");
                return VI_ERROR_ALLOC;
            }
            if ((rsesci_status = viRead (instrSession, (ViBuf) exBuf, count + 4, &retCnt)) < 0)
                {
                viPrintf (instrSession, "FORM ASCII");
                /* If not NULL pointer, then deallocate memory. */
                if (exBuf) free (exBuf);
                return rsesci_status;
                }
        break;
        }

    count /= 4; /* number of I and Q samples */
    /* Binary data byte order (MOTOROLA or INTEL). */
    if (bufferSize > 0)
        rsesci_SWAP_32((unsigned char*) exBuf, count);

    if (bufferSize > 0)
    {
        ViInt32 loopCount = count,
                remainder = 0,
                blockSize = 524288; /* I or Q block size (number of values) */

        ViReal64 *p2I = realParts_I,
                 *p2Q = imaginaryParts_Q;
        ViReal32 *tp2I,
                 *tp2Q;
        ViChar   *p2exBuf = exBuf;

        /*
            If number of I/Q samples is > 512k (524288 samples) block tranfer mode applies
                "<I-val 1><I-val 2>..<I-val 524288><Q-val 1><Q-val 2>..<Q-val 524288>..<..>"
            else returned data are splited in half for I and rest for Q
                "<I-val 1><I-val 2>..<I-val count/2><Q-val 1><Q-val 2>..<Q-val count/2>.."
        */

        loopCount = ((bufferSize * 2) < loopCount) ? (bufferSize * 2) : loopCount;

        /* number of I or Q samples */
        if (noofPoints)
            *noofPoints = loopCount / 2;

        /* Number of I/Q data blocks */
        remainder = loopCount % (blockSize * 2);
        loopCount /= (blockSize * 2);

        /* Make sure that loop counter is set to zero! */
        i = 0;

        /* If loopCount is not zero, retrieve all data */
        if (loopCount)
            {
            /* Number of I and Q data blocks */
            for (i = 0; i < loopCount; i++)
                {
                /* Set pointers */
                tp2I = (ViReal32*)(p2exBuf + (blockSize * 4) * (i*2));
                tp2Q = (ViReal32*)(p2exBuf + (blockSize * 4) * (i*2 + 1));
                /* I and Q values */
                for (j = 0; j < blockSize; j++)
                    {
                    *p2I++ = (ViReal64)*tp2I++;
                    *p2Q++ = (ViReal64)*tp2Q++;
                    }
                }
            }

        /* If reminder is not zero, retrieve remaining data */
        if (remainder)
            {
            /* Set pointers */
            if (i == 0)
                {
                tp2I = (ViReal32*)(p2exBuf);
                tp2Q = (ViReal32*)(p2exBuf + (remainder * 2));
                }
            else
                {
                tp2I = (ViReal32*)(p2exBuf + (blockSize * 4) * (2*i));
                tp2Q = (ViReal32*)(p2exBuf + (blockSize * 4) * (2*i) + (remainder*2));
                }
            /* I and Q values */
            for (j = 0; j < remainder; j += 2)
                {
                *p2I++ = (ViReal64)*tp2I++;
                *p2Q++ = (ViReal64)*tp2Q++;
                }
            }
    }

    if (exBuf) /* If not NULL pointer, then deallocate memory. */
        free (exBuf);

    if ((rsesci_status = viPrintf (instrSession, "FORM ASCII")) < 0)
        return rsesci_status;

    if ((rsesci_status = rsesci_checkStatus (instrSession)) < 0)
        return rsesci_status;

    if ((count / 2) > bufferSize)
        rsesci_status = count;

    return rsesci_status;    
}

/*****************************************************************************
 * Function: Calibration Total
 * Purpose:  This function initiates the acquisition of system error
 *           correction data. A "0" is returned if the  acquisition was
 *           successful.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_CalibrationTotal(
    ViSession   instrSession,
    ViInt32*    result
)
{
    return rsesci_getViInt32 (instrSession,
                              "CAL:ALL?",
                              result,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Calibration Abort
 * Purpose:  This function aborts the acquisition of correction data and
 *           restores the last complete correction data set.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_CalibrationAbort(
    ViSession   instrSession
)
{
    return rsesci_sendCmd (instrSession,
                           "CAL:ABOR");
}

/*****************************************************************************
 * Function: Calibration Results
 * Purpose:  This function outputs the results of the correction data
 *           acquisition. The lines of the result table (see manual section
 *           "Recording the correction data of R&S ESCI CAL key") are output
 *           as string data separated by commas.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_CalibrationResults(
    ViSession   instrSession,
    ViChar      _VI_FAR calibrationResult[]
)
{
    return rsesci_getViString (instrSession,
                               "CAL:RES?",
                               calibrationResult,
                               5000,
                               VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Calibration Correction
 * Purpose:  This function determines whether the current calibration data are
 *           taken into account by the instrument (On) or not (Off).
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetCalibrationCorrection(
    ViSession   instrSession,
    ViBoolean   calibrationState
)
{
    return rsesci_setViBoolean (instrSession,
                                "CAL:STAT",
                                calibrationState,
                                VI_ERROR_PARAMETER2);

}

/*****************************************************************************
 * Function: Get Calibration Correction
 * Purpose:  This function reads whether the current calibration data are
 *           taken into account by the instrument (On) or not (Off).
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetCalibrationCorrection(
    ViSession   instrSession,
    ViBoolean*  calibrationState
)
{
    return rsesci_getViBoolean (instrSession,
                                "CAL:STAT?",
                                calibrationState,
                                VI_ERROR_PARAMETER2);

}

/*****************************************************************************
 * Function: Set Scan Low Noise Preamplifier State
 * Purpose:  This function switches the low noise preamplifier on or off for
 *           a receiver subscan.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetScanLowNoisePreamplifierState (ViSession instrSession,
                                                           ViInt32 scanRange,
                                                           ViBoolean lowNoisePreamplifierState)
{
    CHECKOPT (RSESU_B24);

    if (rsesci_invalidViInt32Range (scanRange, 1, 10) == VI_TRUE)
        return VI_ERROR_PARAMETER2;
    
	return rsesci_setViBooleanWindow (instrSession
									, "SCAN%ld:INP:GAIN:LNA"
									, scanRange
									, lowNoisePreamplifierState
									, VI_ERROR_PARAMETER3);
    
}

/*****************************************************************************
 * Function: Get Scan Low Noise Preamplifier State
 * Purpose:  This function queries state of the low noise preamplifier for a
 *           receiver subscan.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetScanLowNoisePreamplifierState (ViSession instrSession,
                                                           ViInt32 scanRange,
                                                           ViBoolean *lowNoisePreamplifierState)
{
    CHECKOPT (RSESU_B24);
    if (rsesci_invalidViInt32Range (scanRange, 1, 10) == VI_TRUE)
        return VI_ERROR_PARAMETER2;
    
	return rsesci_getViBooleanWindow (instrSession
									, "SCAN%ld:INP:GAIN:LNA?"
									, scanRange
									, lowNoisePreamplifierState
									, VI_ERROR_PARAMETER3);
    
}

/*****************************************************************************
 * Function: Set Scan Input
 * Purpose:  This function defines the input for a receiver subscan.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetScanInput (ViSession instrSession,
                                       ViInt32 scanRange, ViInt32 input)
{
    ViChar buffer [100];
	
	CHECKMODEL(RSESU8 | RSESU26 | RSESU40);
    if (rsesci_invalidViInt32Range (scanRange, 1, 10) == VI_TRUE)
        return VI_ERROR_PARAMETER2;
    if (rsesci_invalidViInt32Range (input, 1, 2) == VI_TRUE)
        return VI_ERROR_PARAMETER3;
    
	sprintf (buffer, "SCAN%ld:INP:TYPE INPUT%ld"
			, scanRange
			, input
			);

	return rsesci_sendCmd (instrSession, buffer);
}

/*****************************************************************************
 * Function: Get Scan Input
 * Purpose:  This function queries the input for a receiver subscan.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetScanInput (ViSession instrSession,
                                       ViInt32 scanRange, ViInt32 *input)
{
    ViChar response [20];
	ViStatus rsesci_status = VI_SUCCESS;
	
	CHECKMODEL(RSESU8 | RSESU26 | RSESU40);
    if (rsesci_invalidViInt32Range (scanRange, 1, 10) == VI_TRUE)
        return VI_ERROR_PARAMETER2;
    
	CHECKERR (rsesci_getViStringWindow (instrSession
		                        		, "SCAN%ld:INP:TYPE?"
										, scanRange
										, response
										, 20
										, -1));
	sscanf (response, "INPUT%ld", input);										
										
	return rsesci_status;
}

/*****************************************************************************
 * Function: Set Marker
 * Purpose:  This function switches on or off selected marker in the selected
 *           measurement window. If marker 2, 3 or 4 is selected and used as a
 *           delta marker, it is switched to marker mode.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetMarker(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32     marker,
    ViBoolean   state
)
{
    ViChar  buffer[BUFFER_SIZE];

    CHECKWINDOW;

    if (rsesci_invalidViInt32Range (marker, 1, 4))
        return VI_ERROR_PARAMETER3;

    sprintf (buffer, "CALC%ld:MARK%ld:STAT", window, marker);

    return rsesci_setViBoolean (instrSession,
                                buffer,
                                state,
                                VI_ERROR_PARAMETER4);
}

/*****************************************************************************
 * Function: Get Marker
 * Purpose:  This function  returns the state of selected marker in the
 *           selected measurement window.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetMarker(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32     marker,
    ViBoolean*  state
)
{
    ViChar  buffer[BUFFER_SIZE];

    CHECKWINDOW;

    if (rsesci_invalidViInt32Range (marker, 1, 4))
        return VI_ERROR_PARAMETER3;

    sprintf (buffer, "CALC%ld:MARK%ld:STAT?", window, marker);

    return rsesci_getViBoolean (instrSession,
                                buffer,
                                state,
                                VI_ERROR_PARAMETER4);
}

/*****************************************************************************
 * Function: All Markers Off
 * Purpose:  This function switches off all active markers in the selected
 *           measurement window. All delta markers and active marker/delta
 *           marker measurement functions are switched off.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_AllMarkersOff(
    ViSession   instrSession,
    ViInt32     window
)
{
    return rsesci_sendCmdWindow (instrSession,
                                 "CALC%ld:MARK:AOFF",
                                 window);
}

/*****************************************************************************
 * Function: Set Marker Position
 * Purpose:  This function positions the selected marker to the indicated
 *           frequency (span > 0), time (span = 0) or level (APD measurement or
 *           CCDF measurement ON) in the selected measurement window. If marker
 *           2, 3 or 4 is selected and used as delta marker, it is switched to
 *           marker mode.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetMarkerPosition(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32 marker,
    ViReal64    x
)
{
    ViChar  buffer[BUFFER_SIZE];

    CHECKWINDOW;

    if (rsesci_invalidViInt32Range (marker, 1, 4))
        return VI_ERROR_PARAMETER3;

    sprintf (buffer, "CALC%ld:MARK%ld:X", window, marker);

    return rsesci_setViReal64 (instrSession,
                               buffer,
                               x,
                               0.0 , 0.0,
                               VI_ERROR_PARAMETER4);
}

/*****************************************************************************
 * Function: Get Marker Position
 * Purpose:  This function returns the marker's position.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetMarkerPosition(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32 marker,
    ViReal64*   x
)
{
    ViChar  buffer[BUFFER_SIZE];

    CHECKWINDOW;

    if (rsesci_invalidViInt32Range (marker, 1, 4))
        return VI_ERROR_PARAMETER3;

    sprintf (buffer, "CALC%ld:MARK%ld:X?", window, marker);

    return rsesci_getViReal64 (instrSession,
                               buffer,
                               x,
                               VI_ERROR_PARAMETER4);
}

/*****************************************************************************
 * Function: Get Marker Value
 * Purpose:  This function queries the measured value of the selected marker
 *           in the selected measurement window. The corresponding marker is
 *           activated before or switched to marker mode, if necessary. To
 *           obtain a valid query result, a complete sweep with synchronization
 *           to the sweep end must be performed between the activation of the
 *           marker and the query of the y value. This is only possible in
 *           single sweep mode.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetMarkerValue(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32 marker,
    ViReal64*   y
)
{
    ViChar  buffer[BUFFER_SIZE];

    CHECKWINDOW;

    if (rsesci_invalidViInt32Range (marker, 1, 4))
        return VI_ERROR_PARAMETER3;

    sprintf (buffer, "CALC%ld:MARK%ld:Y?", window, marker);

    return rsesci_getViReal64 (instrSession,
                               buffer,
                               y,
                               VI_ERROR_PARAMETER4);
}

/*****************************************************************************
 * Function: Set Marker Zoom
 * Purpose:  This function defines the range to be zoomed around marker 1 in
 *           the selected measurement window. Marker 1 is activated first, if
 *           necessary.
 *           The subsequent frequency sweep is stopped at the marker position
 *           and the frequency of the signal is counted. This frequency becomes
 *           the new center frequency, and the zoomed span is set. In order to
 *           recognize the end of the operation the synchronization to the
 *           sweep end should be activated. This is only possible in single
 *           sweep mode.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetMarkerZoom(
    ViSession   instrSession,
    ViInt32     window,
    ViReal64    zoom
)
{
    ViChar  buffer[BUFFER_SIZE];

    CHECKWINDOW;

    sprintf (buffer, "CALC%ld:MARKer1:FUNC:ZOOM %.12Lf;*OPC?\n", window, zoom);

	return rsesci_waitOPC (instrSession, buffer, buffer, 10000); 
}

/*****************************************************************************
 * Function: Get Marker Zoom
 * Purpose:  This function returns the range zoomed around marker 1 in the
 *           selected measurement window.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetMarkerZoom(
    ViSession   instrSession,
    ViInt32     window,
    ViReal64*   zoom
)
{
    return rsesci_getViReal64Window (instrSession,
                                     "CALC%ld:MARKer1:FUNC:ZOOM?",
                                     window,
                                     zoom,
                                     VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set Marker Frequency Counter
 * Purpose:  This function switches on or off the frequency counter at the
 *           marker position in the selected measurement window. The count
 *           result is quer? with rsesci_GetMarkerFrequencyCounterValue().
 *
 *           Frequency counting is possible only for one marker at a time for
 *           each measurement window. If it is activated for another marker, it
 *           is automatically de-activated for the previous marker.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetMarkerFrequencyCounter(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32 marker,
    ViBoolean   counterState
)
{
    ViChar  buffer[BUFFER_SIZE];

    CHECKWINDOW;

    if (rsesci_invalidViInt32Range (marker, 1, 4))
        return VI_ERROR_PARAMETER3;

    sprintf (buffer, "CALC%ld:MARK%ld:COUN", window, marker);

    return rsesci_setViBoolean (instrSession,
                                buffer,
                                counterState,
                                VI_ERROR_PARAMETER4);
}

/*****************************************************************************
 * Function: Get Marker Frequency Counter
 * Purpose:  This function returns the state of frequency counter at the
 *           marker position in the selected measurement window.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetMarkerFrequencyCounter(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32 marker,
    ViBoolean*  counterState
)
{
    ViChar  buffer[BUFFER_SIZE];

    CHECKWINDOW;

    if (rsesci_invalidViInt32Range (marker, 1, 4))
        return VI_ERROR_PARAMETER3;

    sprintf (buffer, "CALC%ld:MARK%ld:COUN?", window, marker);

    return rsesci_getViBoolean (instrSession,
                                buffer,
                                counterState,
                                VI_ERROR_PARAMETER4);
}

/*****************************************************************************
 * Function: Set Marker Frequency Counter Resolution
 * Purpose:  This function specifies the resolution of the frequency counter
 *           in the selected measurement window.The setting is independent of
 *           the selected marker.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetMarkerFrequencyCounterResolution(
    ViSession   instrSession,
    ViInt32     window,
    ViReal64    resolution
)
{
    ViReal64    res_local = resolution;
    ViInt16     index;

    rsesci_coerceToNearest (counterResolutionArr, &res_local, &index);

    return rsesci_setViReal64Window (instrSession,
                                     "CALC%ld:MARK:COUN:RES",
                                     window,
                                     res_local,
                                     0.0,0.0,
                                     VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Marker Frequency Counter Resolution
 * Purpose:  This function returns the resolution of the frequency counter in
 *           the selected measurement window.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetMarkerFrequencyCounterResolution(
    ViSession   instrSession,
    ViInt32     window,
    ViReal64*   resolution
)
{
    return rsesci_getViReal64Window (instrSession,
                                     "CALC%ld:MARK:COUN:RES?",
                                     window,
                                     resolution,
                                     VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Marker Frequency Counter Value
 * Purpose:  This function queries the result of the frequency counter for the
 *           indicated marker in the selected measurement window. Before the
 *           command, the frequency counter should be switched on and a
 *           complete measurement performed to obtain a valid count result.
 *           Therefore, a single sweep with synchronization must be performed
 *           between switching on the frequency counter and querying the
 *           count result.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetMarkerFrequencyCounterValue(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32 marker,
    ViReal64*   frequencyCounterValue
)
{
    ViChar  buffer[BUFFER_SIZE];

    CHECKWINDOW;

    if (rsesci_invalidViInt32Range (marker, 1, 4))
        return VI_ERROR_PARAMETER3;

    sprintf (buffer, "CALC%ld:MARK%ld:COUN:FREQ?", window, marker);

    return rsesci_getViReal64 (instrSession,
                               buffer,
                               frequencyCounterValue,
                               VI_ERROR_PARAMETER4);
}

/*****************************************************************************
 * Function: Set Delta Marker
 * Purpose:  This function switches on and off the delta marker when delta
 *           marker 1 is selected. The corresponding marker becomes the delta
 *           marker when delta marker 2 to 4 is selected. If the corresponding
 *           marker is not activated, it will be activated and positioned on
 *           the maximum of the measurement curve.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetDeltaMarker(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32 marker,
    ViBoolean   state
)
{
    ViChar  buffer[BUFFER_SIZE];

    CHECKWINDOW;

    if (rsesci_invalidViInt32Range (marker, 1, 4))
        return VI_ERROR_PARAMETER3;

    sprintf (buffer, "CALC%ld:DELT%ld:STAT", window, marker);

    return rsesci_setViBoolean (instrSession,
                                buffer,
                                state,
                                VI_ERROR_PARAMETER4);
}

/*****************************************************************************
 * Function: Get Delta Marker
 * Purpose:  This function This control returns the state of the delta marker.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetDeltaMarker(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32 marker,
    ViBoolean*  state
)
{
    ViChar  buffer[BUFFER_SIZE];

    CHECKWINDOW;

    if (rsesci_invalidViInt32Range (marker, 1, 4))
        return VI_ERROR_PARAMETER3;

    sprintf (buffer, "CALC%ld:DELT%ld:STAT?", window, marker);

    return rsesci_getViBoolean (instrSession,
                                buffer,
                                state,
                                VI_ERROR_PARAMETER4);
}

/*****************************************************************************
 * Function: All Delta Markers Off
 * Purpose:  This function switches off all active delta markers in the
 *           selected measurement window (screen A or screen B).
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_AllDeltaMarkersOff(
    ViSession   instrSession,
    ViInt32     window
)
{
    return rsesci_sendCmdWindow (instrSession,
                                 "CALC%ld:DELT:AOFF",
                                 window);
}

/*****************************************************************************
 * Function: Set Delta Marker Mode
 * Purpose:  This function switches between relative and absolute frequency
 *           input of the delta marker (or time with span = 0). It affects all
 *           delta markers independent of the measurement window.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetDeltaMarkerMode(
    ViSession   instrSession,
    ViInt32 deltaMarkerMode
)
{
    return rsesci_setEnum (instrSession,
                           "CALC:DELT:MODE",
                           deltaMarkerMode,
                           absRelArr,
                           VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Delta Marker Mode
 * Purpose:  This function returns selected delta marker mode.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetDeltaMarkerMode(
    ViSession   instrSession,
    ViInt32*    deltaMarkerMode
)
{
    return rsesci_getEnum (instrSession,
                           "CALC:DELT:MODE?",
                           deltaMarkerMode,
                           absRelArr,
                           VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set Delta Marker Position
 * Purpose:  This function positions the selected delta marker in the
 *           indicated measurement window to the indicated frequency (span >
 *           0), time (span = 0) or level (APD measurement = ON or
 *           CCDFmeasurement = ON). The input is in absolute values or relative
 *           to marker 1 depending on the setting of
 *           rsesci_SetDeltaMarkerMode(). If Reference Fixed measurement
 *           (rsesci_SetDeltaMarkerReferenceFixed) is active, relative values
 *           refer to the reference position are entered.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetDeltaMarkerPosition(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32     marker,
    ViReal64    position
)
{
    ViChar  buffer[BUFFER_SIZE];

    CHECKWINDOW;

    if (rsesci_invalidViInt32Range (marker, 1, 4))
        return VI_ERROR_PARAMETER3;

    sprintf (buffer, "CALC%ld:DELT%ld:X", window, marker);

    return rsesci_setViReal64 (instrSession,
                               buffer,
                               position,
                               0.0 , 0.0,
                               VI_ERROR_PARAMETER4);
}

/*****************************************************************************
 * Function: Get Delta Marker Position
 * Purpose:  This function queries the position of selected delta marker in
 *           selected window.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetDeltaMarkerPosition(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32     marker,
    ViReal64*   position
)
{
    ViChar  buffer[BUFFER_SIZE];

    CHECKWINDOW;

    if (rsesci_invalidViInt32Range (marker, 1, 4))
        return VI_ERROR_PARAMETER3;

    sprintf (buffer, "CALC%ld:DELT%ld:X?", window, marker);

    return rsesci_getViReal64 (instrSession,
                               buffer,
                               position,
                               VI_ERROR_PARAMETER4);
}

/*****************************************************************************
 * Function: Get Delta Marker Relative Position
 * Purpose:  This function queries the frequency (span > 0) or time (span = 0)
 *           of the selected delta marker relative to marker 1 or to the
 *           reference position ( for rsesci_SetDeltaMarkerReferenceFixed On).
 *           The function activates the corresponding delta marker, if
 *           necessary.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetDeltaMarkerRelativePosition(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32 marker,
    ViReal64*   position
)
{
    ViChar  buffer[BUFFER_SIZE];

    CHECKWINDOW;

    if (rsesci_invalidViInt32Range (marker, 1, 4))
        return VI_ERROR_PARAMETER3;

    sprintf (buffer, "CALC%ld:DELT%ld:X:REL?", window, marker);

    return rsesci_getViReal64 (instrSession,
                               buffer,
                               position,
                               VI_ERROR_PARAMETER4);
}

/*****************************************************************************
 * Function: Get Delta Marker Value
 * Purpose:  This function queries the measured value of the selected delta
 *           marker in the indicated measurement window. The corresponding
 *           delta marker will be activated, if necessary. The output is
 *           always a relative value referred to marker 1 or to the reference
 *           position (reference fixed active). To obtain a valid query result,
 *           a complete sweep with synchronization to the sweep end must be
 *           performed between the activation of the delta marker and the query
 *           of the y value. This is only possible in single sweep mode.
 *           In receiver mode, the markers can be only activated after a scan
 *           is carried out.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetDeltaMarkerValue(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32 marker,
    ViReal64*   y
)
{
    ViChar  buffer[BUFFER_SIZE];

    CHECKWINDOW;

    if (rsesci_invalidViInt32Range (marker, 1, 4))
        return VI_ERROR_PARAMETER3;

    sprintf (buffer, "CALC%ld:DELT%ld:Y?", window, marker);

    return rsesci_getViReal64 (instrSession,
                               buffer,
                               y,
                               VI_ERROR_PARAMETER4);
}

/*****************************************************************************
 * Function: Set Delta Marker Reference Fixed
 * Purpose:  This function switches the relative measurement to a fixed
 *           reference value on or off. Marker 1 will be activated previously
 *           and a peak search will be performed, if necessary. If marker 1 is
 *           activated, its position becomes the reference point for the
 *           measurement. The reference point can then be modified with
 *           functions SetDeltaMarkerReferencePointPosition and
 *           rsesci_SetDeltaMarkerReferencePointLevel independently of the
 *           position of marker 1 and of a trace.
 *
 *           It is valid for all delta markers in the selected measurement
 *           window as long as the function is active.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetDeltaMarkerReferenceFixed(
    ViSession   instrSession,
    ViInt32     window,
    ViBoolean   fixedReferenceValue
)
{
    return rsesci_setViBooleanWindow (instrSession,
                                      "CALC%ld:DELT:FUNC:FIX:STAT",
                                      window,
                                      fixedReferenceValue,
                                      VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Delta Marker Reference Fixed
 * Purpose:  This function returns the setting of the relative measurement to
 *           a fixed reference value.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetDeltaMarkerReferenceFixed(
    ViSession   instrSession,
    ViInt32     window,
    ViBoolean*  fixedReferenceValue
)
{
    return rsesci_getViBooleanWindow (instrSession,
                                      "CALC%ld:DELT:FUNC:FIX:STAT?",
                                      window,
                                      fixedReferenceValue,
                                      VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set Delta Marker Reference Point Level
 * Purpose:  This function defines a new reference point level for all delta
 *           markers in the selected measurement window for a measurement with
 *           fixed reference point.
 *           For phase-noise measurements, the function defines a new
 *           reference point level for delta marker 2 in the selected
 *           measurement window.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetDeltaMarkerReferencePointLevel(
    ViSession   instrSession,
    ViInt32     window,
    ViReal64    referencePointLevel
)
{
    return rsesci_setViReal64Window (instrSession,
                                     "CALC%ld:DELT:FUNC:FIX:RPO:Y",
                                     window,
                                     referencePointLevel,
                                     0.0 , 0.0,
                                     VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Delta Marker Reference Point Level
 * Purpose:  This function returns reference point level for all delta markers
 *           in the selected measurement window for a measurement with fixed
 *           reference point.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetDeltaMarkerReferencePointLevel(
    ViSession   instrSession,
    ViInt32     window,
    ViReal64*   referencePointLevel
)
{
    return rsesci_getViReal64Window (instrSession,
                                     "CALC%ld:DELT:FUNC:FIX:RPO:Y?",
                                     window,
                                     referencePointLevel,
                                     VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set Delta Marker Reference Point To Peak
 * Purpose:  This function sets the reference point level for all delta
 *           markers in the selected measurement window for a measurement with
 *           fixed reference point to the peak of the selected trace.
 *           For phase-noise measurements, the function defines a new
 *           reference point level for delta marker 2 in the selected
 *           measurement window.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetDeltaMarkerReferencePointToPeak(
    ViSession   instrSession,
    ViInt32     window
)
{
    return rsesci_sendCmdWindow (instrSession,
                                 "CALC%ld:DELT:FUNC:FIX:RPO:MAX:PEAK",
                                 window);
}

/*****************************************************************************
 * Function: Set Delta Marker Reference Point Level Offset
 * Purpose:  This function defines an additional level offset for the
 *           measurement with fixed reference value. For this measurement, the
 *           offset is included in the display of all delta markers of the
 *           selected measurement window.
 *           For phase-noise measurements, the function defines an additional
 *           level offset which is included in the display of delta marker 2 in
 *           the selected measurement window.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetDeltaMarkerReferencePointLevelOffset(
    ViSession   instrSession,
    ViInt32     window,
    ViReal64    offset
)
{
    return rsesci_setViReal64Window (instrSession,
                                     "CALC%ld:DELT:FUNC:FIX:RPO:Y:OFFS",
                                     window,
                                     offset,
                                     0.0 , 0.0,
                                     VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Delta Marker Reference Point Level Offset
 * Purpose:  This function queries an additional level offset for the
 *           measurement with fixed reference value.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetDeltaMarkerReferencePointLevelOffset(
    ViSession   instrSession,
    ViInt32     window,
    ViReal64*   offset
)
{
    return rsesci_getViReal64Window (instrSession,
                                     "CALC%ld:DELT:FUNC:FIX:RPO:Y:OFFS?",
                                     window,
                                     offset,
                                     VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set Delta Marker Reference Point Position
 * Purpose:  This function defines a new reference frequency (span > 0) or
 *           time (span = 0) for all delta markers in the selected measurement
 *           window for a measurement with fixed reference value.
 *           For phase-noise measurements, the function defines a new
 *           reference frequency or time for delta marker 2 in the selected
 *           measurement window.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetDeltaMarkerReferencePointPosition(
    ViSession   instrSession,
    ViInt32     window,
    ViReal64    referencePoint
)
{
    return rsesci_setViReal64Window (instrSession,
                                     "CALC%ld:DELT:FUNC:FIX:RPO:X",
                                     window,
                                     referencePoint,
                                     0.0 , 0.0,
                                     VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Delta Marker Reference Point Position
 * Purpose:  This function returns a reference frequency (span > 0) or time
 *           (span = 0) for all delta markers in the selected measurement
 *           window for a measurement with fixed reference value.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetDeltaMarkerReferencePointPosition(
    ViSession   instrSession,
    ViInt32     window,
    ViReal64*   referencePoint
)
{
    return rsesci_getViReal64Window (instrSession,
                                     "CALC%ld:DELT:FUNC:FIX:RPO:X?",
                                     window,
                                     referencePoint,
                                     VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Marker Search
 * Purpose:  This function positions the marker to the desired position in the
 *           trace memory.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_MarkerSearch(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32 marker,
    ViInt32 searchMode
)
{
    ViChar  buffer[BUFFER_SIZE];

    if (rsesci_invalidViInt32Range (marker, 1, 4))
        return VI_ERROR_PARAMETER3;
    if (rsesci_invalidViInt32Range (searchMode, 0, 7))
        return VI_ERROR_PARAMETER4;

    sprintf (buffer, "CALC%%ld:MARK%ld:%s", marker, peakSearchArr[searchMode]);

    return rsesci_sendCmdWindow (instrSession, buffer, window);
}

/*****************************************************************************
 * Function: Set Auto Marker Search
 * Purpose:  This function switches an automatic maximum or minimum peak
 *           search for marker 1 at the end of each particular sweep on and
 *           off. The current marker search limit settings (LEFT LIMIT, RIGHT
 *           LIMIT, THRESHOLD, EXCLUDE LO) are taken into account.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetAutoMarkerSearch (ViSession instrSession,
                                              ViInt32 window,
                                              ViInt32 autoSearchMode,
                                              ViBoolean autoSearchState)
{
    ViStatus rsesci_status = VI_SUCCESS;
	
    if (rsesci_invalidViInt32Range (window, 1, 2))
        return VI_ERROR_PARAMETER2;

	switch (autoSearchMode)
	{
		case RSESCI_MARKER_PMAX:
			rsesci_status = rsesci_setViBooleanWindow (instrSession
				                                     , "CALC%ld:MARK:MAX:AUTO"
													 , window
													 , autoSearchState
													 , VI_ERROR_PARAMETER4);
			break;
		case RSESCI_MARKER_PMIN:
			rsesci_status = rsesci_setViBooleanWindow (instrSession
				                                     , "CALC%ld:MARK:MIN:AUTO"
													 , window
													 , autoSearchState
													 , VI_ERROR_PARAMETER4);
			break;
		default:
        	rsesci_status = VI_ERROR_PARAMETER3;
	}
	
	return rsesci_status;
}

/*****************************************************************************
 * Function: Get Auto Marker Search
 * Purpose:  This function returns state of an automatic maximum or minimum
 *           peak search for marker 1 at the end of each particular sweep on
 *           and off. The current marker search limit settings (LEFT LIMIT,
 *           RIGHT LIMIT, THRESHOLD, EXCLUDE LO) are taken into account.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetAutoMarkerSearch (ViSession instrSession,
                                              ViInt32 window,
                                              ViInt32 autoSearchMode,
                                              ViBoolean *autoSearchState)
{
    ViStatus rsesci_status = VI_SUCCESS;
	
    if (rsesci_invalidViInt32Range (window, 1, 2))
        return VI_ERROR_PARAMETER2;

	switch (autoSearchMode)
	{
		case RSESCI_MARKER_PMAX:
			rsesci_status = rsesci_getViBooleanWindow (instrSession
				                                     , "CALC%ld:MARK:MAX:AUTO?"
													 , window
													 , autoSearchState
													 , VI_ERROR_PARAMETER4);
			break;
		case RSESCI_MARKER_PMIN:
			rsesci_status = rsesci_getViBooleanWindow (instrSession
				                                     , "CALC%ld:MARK:MIN:AUTO?"
													 , window
													 , autoSearchState
													 , VI_ERROR_PARAMETER4);
			break;
		default:
        	rsesci_status = VI_ERROR_PARAMETER3;
	}
	
	return rsesci_status;
}

/*****************************************************************************
 * Function: Set Marker To
 * Purpose:  This function changes instrument parameters via currently active
 *           marker.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetMarkerTo(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32 marker,
    ViInt32 mkrFunction
)
{
    ViChar  buffer[BUFFER_SIZE];

    if (rsesci_invalidViInt32Range (marker, 1, 4))
        return VI_ERROR_PARAMETER3;
    if (rsesci_invalidViInt32Range (mkrFunction, 0, 2))
        return VI_ERROR_PARAMETER4;

    sprintf (buffer, "CALC%%ld:MARK%ld:FUNC:%s", marker, mkrFunctionArr[mkrFunction]);

    return rsesci_sendCmdWindow (instrSession, buffer, window);
}

/*****************************************************************************
 * Function: Set Marker To Trace
 * Purpose:  This function assigns the selected marker (1 to 4) to the
 *           indicated test curve.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetMarkerToTrace(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32 marker,
    ViInt32 trace
)
{
    ViChar  buffer[BUFFER_SIZE];

    CHECKWINDOW;

    if (rsesci_invalidViInt32Range (marker, 1, 4))
        return VI_ERROR_PARAMETER3;

    sprintf (buffer, "CALC%ld:MARK%ld:TRAC", window, marker);

    return rsesci_setViInt32 (instrSession,
                              buffer,
                              trace,
                              1 , 3 ,
                              VI_ERROR_PARAMETER4);
}

/*****************************************************************************
 * Function: Get Marker To Trace
 * Purpose:  This function returns the number of trace associated with marker.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetMarkerToTrace(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32 marker,
    ViInt32*    trace
)
{
    ViChar  buffer[BUFFER_SIZE];

    CHECKWINDOW;

    if (rsesci_invalidViInt32Range (marker, 1, 4))
        return VI_ERROR_PARAMETER3;

    sprintf (buffer, "CALC%ld:MARK%ld:TRAC?", window, marker);

    return rsesci_getViInt32 (instrSession,
                              buffer,
                              trace,
                              VI_ERROR_PARAMETER4);
}

/*****************************************************************************
 * Function: Set Peak Excursion
 * Purpose:  This function defines the peak excursion., i.e. the spacing below
 *           a trace maximum which must be attained before a new maximum is
 *           recognized, or the spacing above a trace minimum which must be
 *           attained before a new minimum is recognized. The set value is
 *           valid for all markers and delta markers. The unit depends on the
 *           selected operating mode.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetPeakExcursion(
    ViSession   instrSession,
    ViInt32     window,
    ViReal64    peakExcursion
)
{
    return rsesci_setViReal64Window (instrSession,
                                     "CALC%ld:MARK:PEXC",
                                     window,
                                     peakExcursion,
                                     0.0 , 0.0 ,
                                     VI_ERROR_PARAMETER4);
}

/*****************************************************************************
 * Function: Get Peak Excursion
 * Purpose:  This function returns the peak excursion value.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetPeakExcursion(
    ViSession   instrSession,
    ViInt32     window,
    ViReal64*   peakExcursion
)
{
    return rsesci_getViReal64Window (instrSession,
                                     "CALC%ld:MARK:PEXC?",
                                     window ,
                                     peakExcursion,
                                     VI_ERROR_PARAMETER4);
}

/*****************************************************************************
 * Function: Set Local Oscillator Suppression
 * Purpose:  This function switches the local oscillator suppression for peak
 *           search on or off. This setting is valid for all markers and delta
 *           markers in all measurement windows.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetLocalOscillatorSuppression(
    ViSession   instrSession,
    ViBoolean   localOscillatorSuppression
)
{
    return rsesci_setViBoolean (instrSession,
                                "CALC:MARK:LOEX",
                                localOscillatorSuppression,
                                VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Local Oscillator Suppression
 * Purpose:  This function returns the setting of the local oscillator
 *           suppression for peak search.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetLocalOscillatorSuppression(
    ViSession   instrSession,
    ViBoolean*  localOscillatorSuppression
)
{
    return rsesci_getViBoolean (instrSession,
                                "CALC:MARK:LOEX?",
                                localOscillatorSuppression,
                                VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Marker Settings Coupled
 * Purpose:  This function switches the coupling of the receiver frequency at
 *           the marker frequency on or off.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetMarkerSettingsCoupled(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32 marker,
    ViBoolean   couplingState
)
{
    ViChar  buffer[BUFFER_SIZE];

    CHECKWINDOW;

    if (rsesci_invalidViInt32Range (marker, 1, 4))
        return VI_ERROR_PARAMETER3;

    sprintf (buffer, "CALC%ld:MARK%ld:SCO:STAT", window, marker);

    return rsesci_setViBoolean (instrSession,
                                buffer,
                                couplingState,
                                VI_ERROR_PARAMETER4);
}

/*****************************************************************************
 * Function: Get Marker Settings Coupled
 * Purpose:  This function returns the state of the coupling of the receiver
 *           frequency at the marker frequency.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetMarkerSettingsCoupled(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32 marker,
    ViBoolean*  couplingState
)
{
    ViChar  buffer[BUFFER_SIZE];

    CHECKWINDOW;

    if (rsesci_invalidViInt32Range (marker, 1, 4))
        return VI_ERROR_PARAMETER3;

    sprintf (buffer, "CALC%ld:MARK%ld:SCO:STAT?", window, marker);

    return rsesci_getViBoolean (instrSession,
                                buffer,
                                couplingState,
                                VI_ERROR_PARAMETER4);
}

/*****************************************************************************
 * Function: Set Marker Coupling
 * Purpose:  This function switches the coupling of the receiver frequency to 
 *           the marker on or off.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetMarkerCoupling (ViSession instrSession,
                                            ViInt32 window,
                                            ViBoolean couplingState)
{
	return rsesci_setViBooleanWindow (instrSession,
                                      "CALC%ld:MARK:COUP:STAT",
									  window,
                                      couplingState,
                                      VI_ERROR_PARAMETER3);    
}

/*****************************************************************************
 * Function: Get Marker Coupling
 * Purpose:  This function returns the state of the coupling of the receiver 
 *           frequency to the marker.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetMarkerCoupling (ViSession instrSession,
                                            ViInt32 window,
                                            ViBoolean *couplingState)
{
	return rsesci_getViBooleanWindow (instrSession,
                                      "CALC%ld:MARK:COUP:STAT?",
									  window,
                                      couplingState,
                                      VI_ERROR_PARAMETER3);    
}

/*****************************************************************************
 * Function: Delta Marker Search
 * Purpose:  This function positions the delta marker to the next smaller
 *           maximum value on the measured curve. The corresponding delta
 *           marker will be activated first, if necessary.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_DeltaMarkerSearch(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32 marker,
    ViInt32 searchMode
)
{
    ViChar  buffer[BUFFER_SIZE];

    if (rsesci_invalidViInt32Range (marker, 1, 4))
        return VI_ERROR_PARAMETER3;
    if (rsesci_invalidViInt32Range (searchMode, 0, 7))
        return VI_ERROR_PARAMETER4;

    sprintf (buffer, "CALC%%ld:DELT%ld:%s", marker, peakSearchArr[searchMode]);

    return rsesci_sendCmdWindow (instrSession, buffer, window);
}

/*****************************************************************************
 * Function: Set Delta Marker To Trace
 * Purpose:  This function assigns the selected delta marker to the indicated
 *           measurement curve in the indicated measurement window. The
 *           selected measurement curve must be active.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetDeltaMarkerToTrace(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32 marker,
    ViInt32 trace
)
{
    ViChar  buffer[BUFFER_SIZE];

    CHECKWINDOW;

    if (rsesci_invalidViInt32Range (marker, 1, 4))
        return VI_ERROR_PARAMETER3;

    sprintf (buffer, "CALC%ld:DELT%ld:TRAC", window, marker);

    return rsesci_setViInt32 (instrSession,
                              buffer,
                              trace,
                              1 , 3 ,
                              VI_ERROR_PARAMETER4);
}

/*****************************************************************************
 * Function: Get Delta Marker To Trace
 * Purpose:  This function returns the number of trace associated with delta
 *           marker.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetDeltaMarkerToTrace(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32 marker,
    ViInt32*    trace
)
{
    ViChar  buffer[BUFFER_SIZE];

    CHECKWINDOW;

    if (rsesci_invalidViInt32Range (marker, 1, 4))
        return VI_ERROR_PARAMETER3;

    sprintf (buffer, "CALC%ld:DELT%ld:TRAC?", window, marker);

    return rsesci_getViInt32 (instrSession,
                              buffer,
                              trace,
                              VI_ERROR_PARAMETER4);
}

/*****************************************************************************
 * Function: Set Link Marker And Delta Marker State
 * Purpose:  This function links delta marker 1 to marker 1. The x-value of 
 *           delta marker 1 is kept fixed at 0. Therefore, delta marker 1 always 
 *           has the same x-value as marker 1. Changing the x-value of marker 1
 *           automatically changes the value of delta marker 1. This function
 *           is useful if marker 1 is positioned on trace 1 and delta marker
 *           2 on trace 2.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetLinkMarkerAndDeltaMarkerState (ViSession instrSession,
                                                           ViInt32 window,
                                                           ViInt32 marker,
                                                           ViBoolean linkingState)
{
	ViChar  buffer[BUFFER_SIZE];

    CHECKWINDOW;

    if (rsesci_invalidViInt32Range (marker, 1, 1))
        return VI_ERROR_PARAMETER3;

    sprintf (buffer, "CALC%ld:DELT%ld:LINK", window, marker);

    return rsesci_setViBoolean (instrSession,
                              buffer,
                              linkingState,
                              VI_ERROR_PARAMETER4);    
}

/*****************************************************************************
 * Function: Get Link Marker And Delta Marker State
 * Purpose:  This function returns the state of the linked delta marker 1 to 
 *           marker 1.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetLinkMarkerAndDeltaMarkerState (ViSession instrSession,
                                                           ViInt32 window,
                                                           ViInt32 marker,
                                                           ViBoolean *linkingState)
{
	ViChar  buffer[BUFFER_SIZE];

    CHECKWINDOW;

    if (rsesci_invalidViInt32Range (marker, 1, 1))
        return VI_ERROR_PARAMETER3;

    sprintf (buffer, "CALC%ld:DELT%ld:LINK?", window, marker);

    return rsesci_getViBoolean (instrSession,
                              buffer,
                              linkingState,
                              VI_ERROR_PARAMETER4);    
}

/*****************************************************************************
 * Function: Set Search Limit Lines
 * Purpose:  This function switches between a limited (ON) and unlimited (OFF)
 *           search range in the selected measurement window. The function is
 *           independent of the selection of a marker.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetSearchLimitLines(
    ViSession   instrSession,
    ViInt32     window,
    ViBoolean   limitedRange
)
{
    return rsesci_setViBooleanWindow (instrSession,
                                      "CALC%ld:MARK:X:SLIM:STAT",
                                      window,
                                      limitedRange,
                                      VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Search Limit Lines
 * Purpose:  This function returns the setting of limited search range in the
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetSearchLimitLines(
    ViSession   instrSession,
    ViInt32     window,
    ViBoolean*  limitedRange
)
{
    return rsesci_getViBooleanWindow (instrSession,
                                      "CALC%ld:MARK:X:SLIM:STAT?",
                                      window,
                                      limitedRange,
                                      VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set Search Left Limit Line
 * Purpose:  This function sets the left limit of the search range for markers
 *           and delta markers in the selected measurement window. Depending on
 *           the x axis domain the indicated value defines a frequency (span >
 *           0) or time (span = 0). The function is independent of the
 *           selection of a marker.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetSearchLeftLimitLine(
    ViSession   instrSession,
    ViInt32     window,
    ViReal64    leftLimit
)
{
    return rsesci_setViReal64Window (instrSession,
                                     "CALC%ld:MARK:X:SLIM:LEFT",
                                     window,
                                     leftLimit,
                                     0.0 , 0.0 ,
                                     VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Search Left Limit Line
 * Purpose:  This function reads the left limit of the search range for
 *           markers and delta markers in the selected measurement window.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetSearchLeftLimitLine(
    ViSession   instrSession,
    ViInt32     window,
    ViReal64*   leftLimit
)
{
    return rsesci_getViReal64Window (instrSession,
                                     "CALC%ld:MARK:X:SLIM:LEFT?",
                                     window,
                                     leftLimit,
                                     VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set Search Right Limit Line
 * Purpose:  This function sets the right limit of the search range for
 *           markers and delta markers in the selected measurement window.
 *           Depending on the x axis domain the indicated value defines a
 *           frequency (span > 0) or time (span = 0). The function is
 *           independent of the selection of a marker.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetSearchRightLimitLine(
    ViSession   instrSession,
    ViInt32     window,
    ViReal64    rightLimit
)
{
    return rsesci_setViReal64Window (instrSession,
                                     "CALC%ld:MARK:X:SLIM:RIGHT",
                                     window,
                                     rightLimit,
                                     0.0 , 0.0 ,
                                     VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Search Right Limit Line
 * Purpose:  This function reads the right limit of the search range for
 *           markers and delta markers in the selected measurement window.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetSearchRightLimitLine(
    ViSession   instrSession,
    ViInt32     window,
    ViReal64*   rightLimit
)
{
    return rsesci_getViReal64Window (instrSession,
                                     "CALC%ld:MARK:X:SLIM:RIGHT?",
                                     window,
                                     rightLimit,
                                     VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set Search Threshold
 * Purpose:  This function switches on or off the threshold line in the
 *           selected measurement window. The unit depends on the setting
 *           performed with CALC:UNIT.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetSearchThreshold(
    ViSession   instrSession,
    ViInt32     window,
    ViBoolean   searchThreshold
)
{
    return rsesci_setViBooleanWindow (instrSession,
                                      "CALC%ld:THR:STAT",
                                      window,
                                      searchThreshold,
                                      VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Search Threshold
 * Purpose:  This function returns the state of threshold line in the selected
 *           measurement window.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetSearchThreshold(
    ViSession   instrSession,
    ViInt32     window,
    ViBoolean*  searchThreshold
)
{
    return rsesci_getViBooleanWindow (instrSession,
                                      "CALC%ld:THR:STAT?",
                                      window,
                                      searchThreshold,
                                      VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set Search Threshold Line
 * Purpose:  This function defines the threshold value for the maximum/minimum
 *           search of markers with marker search functions  in the selected
 *           measurement window. The associated display line is automatically
 *           switched on.
  *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetSearchThresholdLine(
    ViSession   instrSession,
    ViInt32     window,
    ViReal64    threshold
)
{
    return rsesci_setViReal64Window (instrSession,
                                     "CALC%ld:THR",
                                     window,
                                     threshold,
                                     0.0 , 0.0 ,
                                     VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Search Threshold Line
 * Purpose:  This function reads the threshold value for the maximum/minimum
 *           search of markers with marker search functions in the selected
 *           measurement window.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetSearchThresholdLine(
    ViSession   instrSession,
    ViInt32     window,
    ViReal64*   threshold
)
{
    return rsesci_getViReal64Window (instrSession,
                                     "CALC%ld:THR?",
                                     window,
                                     threshold,
                                     VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set Noise Density Measurement
 * Purpose:  This function switches the noise measurement on or off for all
 *           markers of the indicated measurement window. The noise power
 *           density is measured at the position of the markers.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetNoiseDensityMeasurement(
    ViSession   instrSession,
    ViInt32     window,
    ViBoolean   noiseMeasurement
)
{
    return rsesci_setViBooleanWindow (instrSession,
                                      "CALC%ld:MARK:FUNC:NOIS:STAT",
                                      window,
                                      noiseMeasurement,
                                      VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Noise Density Measurement
 * Purpose:  This function reads the state of the noise measurement for all
 *           markers of the indicated measurement window.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetNoiseDensityMeasurement(
    ViSession   instrSession,
    ViInt32     window,
    ViBoolean*  noiseMeasurement
)
{
    return rsesci_getViBooleanWindow (instrSession,
                                      "CALC%ld:MARK:FUNC:NOIS:STAT?",
                                      window,
                                      noiseMeasurement,
                                      VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Noise Density Measurement Value
 * Purpose:  This function queries the result of the noise measurement. A
 *           complete sweep with synchronization to the sweep end must be
 *           performed between switching on the function and querying the
 *           measured value in order to obtain a valid query result. This is
 *           only possible in single sweep mode.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetNoiseDensityMeasurementValue(
    ViSession   instrSession,
    ViInt32     window,
    ViReal64*   noiseMeasurement
)
{
    return rsesci_getViReal64Window (instrSession,
                                     "CALC%ld:MARK:FUNC:NOIS:RES?",
                                     window,
                                     noiseMeasurement,
                                     VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set Phase Noise Measurement
 * Purpose:  This function switches on or off the phase-noise measurement with
 *           all actíve delta markers in the selected measurement window. The
 *           correction values for the bandwidth and the log amplifier are
 *           taken into account in the measurement.
 *           Marker 1 will be activated, if necessary, and a peak search will
 *           be performed. If marker 1 is activated, its position becomes the
 *           reference point for the measurement. The reference point can then
 *           be modified with functions
 *           rsesci_SetDeltaMarkerReferencePointPosition() and
 *           rsesci_SetDeltaMarkerReferencePointLevel() independently of the
 *           position of marker 1 and of a trace (the same functions used for
 *           the measurment with fixed reference point).
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetPhaseNoiseMeasurement(
    ViSession   instrSession,
    ViInt32     window,
    ViBoolean   phaseNoiseMeasurement
)
{
    return rsesci_setViBooleanWindow (instrSession,
                                      "CALC%ld:DELT:FUNC:PNO:STAT",
                                      window,
                                      phaseNoiseMeasurement,
                                      VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Phase Noise Measurement
 * Purpose:  This function queries the state of phase-noise measurement with
 *           all actíve delta markers in the selected measurement window.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetPhaseNoiseMeasurement(
    ViSession   instrSession,
    ViInt32     window,
    ViBoolean*  phaseNoiseMeasurement
)
{
    return rsesci_getViBooleanWindow (instrSession,
                                      "CALC%ld:DELT:FUNC:PNO:STAT?",
                                      window,
                                      phaseNoiseMeasurement,
                                      VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set Phase Noise Measurement Auto
 * Purpose:  This function adds an automatic peak search action for the
 *           reference fixed marker 1 at the end of each particular sweep.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetPhaseNoiseMeasurementAuto (ViSession instrSession,
                                                       ViInt32 window,
                                                       ViBoolean automaticPeakSearch)
{
	CHECKMODEL(RSESU8 | RSESU26 | RSESU40);
    if (rsesci_invalidViInt32Range (window, 1, 2))
        return VI_ERROR_PARAMETER2;

	return rsesci_setViBooleanWindow (instrSession
                                     , "CALC%ld:DELT:FUNC:PNO:AUTO"
									 , window
									 , automaticPeakSearch
									 , VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Phase Noise Measurement Auto
 * Purpose:  This function queries state of the automatic peak search action
 *           for the reference fixed marker 1 at the end of each particular
 *           sweep.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetPhaseNoiseMeasurementAuto (ViSession instrSession,
                                                       ViInt32 window,
                                                       ViBoolean *automaticPeakSearch)
{
	CHECKMODEL(RSESU8 | RSESU26 | RSESU40);
    if (rsesci_invalidViInt32Range (window, 1, 2))
        return VI_ERROR_PARAMETER2;

	return rsesci_getViBooleanWindow (instrSession
                                     , "CALC%ld:DELT:FUNC:PNO:AUTO?"
									 , window
									 , automaticPeakSearch
									 , VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Phase Noise Measurement Value
 * Purpose:  This function queries the result of the phase-noise measurement
 *           in the selected measurement window. The measurement will be
 *           switched on, if necessary.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetPhaseNoiseMeasurementValue(
    ViSession   instrSession,
    ViInt32     window,
    ViReal64*   phaseNoiseMeasurement
)
{
    ViChar  buffer[BUFFER_SIZE];

    CHECKWINDOW;

    sprintf (buffer, "CALC%ld:DELT:FUNC:PNO:RES?", window);

    return rsesci_getViReal64 (instrSession,
                               buffer,
                               phaseNoiseMeasurement,
                               VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set N dB Down
 * Purpose:  This function switches the "N dB Down" function on or off in the
 *           selected measurement window. Marker 1 is activated first, if
 *           necessary.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetNdBDown(
    ViSession   instrSession,
    ViInt32     window,
    ViBoolean   ndBDownState
)
{
    return rsesci_setViBooleanWindow (instrSession,
                                      "CALC%ld:MARK:FUNC:NDBD:STAT",
                                      window,
                                      ndBDownState,
                                      VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get N dB Down
 * Purpose:  This function queries the state of "N dB Down" function in the
 *           selected measurement window.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetNdBDown(
    ViSession   instrSession,
    ViInt32     window,
    ViBoolean*  ndBDownState
)
{
    return rsesci_getViBooleanWindow (instrSession,
                                      "CALC%ld:MARK:FUNC:NDBD:STAT?",
                                      window,
                                      ndBDownState,
                                      VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set N dB Down Level spacing
 * Purpose:  This function defines the level spacing of the two delta markers
 *           to the right and left of marker 1 in the selected measurement
 *           window. Marker 1 is always used as the reference marker. The
 *           temporary markers T1 and T2 are positioned by n dB below the
 *           active reference marker.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetNdBDownLevelSpacing(
    ViSession   instrSession,
    ViInt32     window,
    ViReal64    levelSpacing
)
{
    return rsesci_setViReal64Window (instrSession,
                                     "CALC%ld:MARK:FUNC:NDBD",
                                     window,
                                     levelSpacing,
                                     0.0 , 0.0 ,
                                     VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get N dB Down Level spacing
 * Purpose:  This function queries the level spacing of the two delta markers
 *           to the right and left of marker 1 in the selected measurement
 *           window.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetNdBDownLevelSpacing(
    ViSession   instrSession,
    ViInt32     window,
    ViReal64*   levelSpacing
)
{
    return rsesci_getViReal64Window (instrSession,
                                     "CALC%ld:MARK:FUNC:NDBD?",
                                     window,
                                     levelSpacing,
                                     VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get N dB Down Bandwidth
 * Purpose:  This function queries the frequency spacing (bandwidth) of the
 *           N-dB-down markers in the selected measurement window. A complete
 *           sweep with synchronization to sweep end must be performed between
 *           switching on the function and querying the measured value in order
 *           to obtain a valid query result. This is only possible in single
 *           sweep mode.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetNdBDownBandwidth(
    ViSession   instrSession,
    ViInt32     window,
    ViReal64*   frequencySpacing
)
{
    return rsesci_getViReal64Window (instrSession,
                                     "CALC%ld:MARK:FUNC:NDBD:RES?",
                                     window,
                                     frequencySpacing,
                                     VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get N dB Down Frequencies
 * Purpose:  This function queries the two frequencies of the N-dB-down marker
 *           in the selected measurement window. A complete sweep with
 *           synchronization to sweep end must be performed between switching
 *           on the function and querying the measured value to obtain a valid
 *           query result. This is only possible in single sweep mode.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetNdBDownFrequencies(
    ViSession   instrSession,
    ViInt32     window,
    ViReal64    _VI_FAR frequencies[]
)
{
    return rsesci_getViReal64ArrayWindow (instrSession,
                                          "CALC%ld:MARK:FUNC:NDBD:FREQ?",
                                          window,
                                          frequencies,
                                          2,
                                          VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get N dB Down Times
 * Purpose:  This function queries the two time values of the N-dB-down marker
 *           in the selected measurement window. A complete sweep with
 *           synchronization to sweep end must be performed between switching
 *           on the function and querying the measured value to obtain a valid
 *           query result. This is only possible in single sweep mode.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetNdBDownTimes(
    ViSession   instrSession,
    ViInt32     window,
    ViReal64    _VI_FAR times[]
)
{
    return rsesci_getViReal64ArrayWindow (instrSession,
                                          "CALC%ld:MARK:FUNC:NDBD:TIME?",
                                          window,
                                          times,
                                          2,
                                          VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Search For Peak List
 * Purpose:  This function searches the selected trace for the indicated
 *           number of maxima.
  *****************************************************************************/
ViStatus _VI_FUNC rsesci_SearchForPeakList(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32 numberofmaxima
)
{
    return rsesci_setViInt32Window (instrSession,
                                    "CALC%ld:MARK:FUNC:FPE:IMM",
                                    window,
                                    numberofmaxima,
                                    1 , 50,
                                    VI_ERROR_PARAMETER3);
}

/// HIFN  This function activates or deactivates the peak list.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/The measurement windows are selected by means of this parameter.
/// HIPAR state/This control activates or deactivates the peak list.
ViStatus _VI_FUNC rsesci_SetPeakListState(
	ViSession	instrSession,
	ViInt32	window,
	ViBoolean	state
)
{
	return rsesci_setViBooleanWindow (instrSession,
                                      "CALC%ld:MARK:FUNC:FPE:STAT",
                                      window,
                                      state,
                                      VI_ERROR_PARAMETER3);    
}


/// HIFN  This function returns the peak list state.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/The measurement windows are selected by means of this parameter.
/// HIPAR state/This control returns the peak list state.
ViStatus _VI_FUNC rsesci_GetPeakListState(
	ViSession	instrSession,
	ViInt32	window,
	ViBoolean*	state
)
{
	return rsesci_getViBooleanWindow (instrSession,
                                      "CALC%ld:MARK:FUNC:FPE:STAT?",
                                      window,
                                      state,
                                      VI_ERROR_PARAMETER3);    
}


/// HIFN  This function activates and deactivates automatic peaks search after a 
/// HIFN  sweep is done.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/The measurement windows are selected by means of this parameter.
/// HIPAR state/This control activates and deactivates automatic peaks search after a 
/// HIPAR state/sweep is done.
ViStatus _VI_FUNC rsesci_SetPeakListAutoSearchState(
	ViSession	instrSession,
	ViInt32	window,
	ViBoolean	state
)
{
	return rsesci_setViBooleanWindow (instrSession,
                                      "CALC%ld:MARK:FUNC:FPE:SEAR:AUTO",
                                      window,
                                      state,
                                      VI_ERROR_PARAMETER3);     
}


/// HIFN  This function returns automatic peaks search after a sweep state.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/The measurement windows are selected by means of this parameter.
/// HIPAR state/This control returns automatic peaks search after a sweep state.
ViStatus _VI_FUNC rsesci_GetPeakListAutoSearchState(
	ViSession	instrSession,
	ViInt32	window,
	ViBoolean*	state
)
{
	return rsesci_getViBooleanWindow (instrSession,
                                      "CALC%ld:MARK:FUNC:FPE:SEAR:AUTO?",
                                      window,
                                      state,
                                      VI_ERROR_PARAMETER3);     
}

/*****************************************************************************
 * Function: Set Peak List Sort Mode
 * Purpose:  This function sets the sort mode for the search for maxima.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetPeakListSortMode(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32 sortMode
)
{
    return rsesci_setEnumWindow (instrSession,
                                 "CALC%ld:MARK:FUNC:FPE:SORT",
                                 window,
                                 sortMode,
                                 sortModeArr,
                                 VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Peak List Sort Mode
 * Purpose:  This function queries the sort mode for the search for maxima.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetPeakListSortMode(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32*    sortMode
)
{
    return rsesci_getEnumWindow (instrSession,
                                 "CALC%ld:MARK:FUNC:FPE:SORT?",
                                 window,
                                 sortMode,
                                 sortModeArr,
                                 VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Peak List Count
 * Purpose:  This function reads out the number of maxima found during the
 *           search. If no search for maxima has been performed, 0 is returned.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetPeakListCount(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32*    peakCount
)
{
    return rsesci_getViInt32Window (instrSession,
                                    "CALC%ld:MARK:FUNC:FPE:COUN?",
                                    window,
                                    peakCount,
                                    VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Peak List X Values
 * Purpose:  This function reads out the list of X values of the maxima found.
 *           The number of available values can be queried with
 *           rsesci_GetPeakListCount().
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetPeakListXValues(
    ViSession   instrSession,
    ViInt32     window,
    ViReal64    _VI_FAR xValues[]
)
{
    return rsesci_getViReal64ArrayWindow (instrSession,
                                          "CALC%ld:MARK:FUNC:FPE:X?",
                                          window,
                                          xValues,
                                          50,
                                          VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Peak List Y Values
 * Purpose:  This function reads out the list of Y values of the maxima found.
 *           The number of available values can be queried with
 *           rsesci_GetPeakListCount().
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetPeakListYValues(
    ViSession   instrSession,
    ViInt32     window,
    ViReal64    _VI_FAR yValues[]
)
{
    return rsesci_getViReal64ArrayWindow (instrSession,
                                          "CALC%ld:MARK:FUNC:FPE:Y?",
                                          window,
                                          yValues,
                                          50,
                                          VI_ERROR_PARAMETER3);
}

/// HIFN  This function returns the marker peak list.
/// HIFN     
/// HIFN     Mode(s):
/// HIFN     Spectrum Analysis
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     TRACe<1|2>:DATA? FPE
/// HIFN     :FORM REAL,32
/// HIFN     *CLS
/// HIFN     
/// HIFN     
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or 
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR trace/This control sets the trace.
/// HIPAR noOfValues/This control sets the number of values returned in arrays.
/// HIPAR x/X value. 
/// HIPAR y/Y value.
/// HIPAR returnedValues/Returns number of value sets (frequency, level and delta to limit 
/// HIPAR returnedValues/line) read (available to return) from the instrument. Number of values 
/// HIPAR returnedValues/returned in arrays is limited with the parameter No of Values.
ViStatus _VI_FUNC rsesci_ReadMarkerPeakList (ViSession instrSession,
                                             ViInt32 trace, ViInt32 noOfValues,
                                             ViReal64 x[], ViReal64 y[],
                                             ViInt32 *returnedValues)
{
	ViStatus    rsesci_status = VI_SUCCESS;
    ViChar      cmd [BUFFER_SIZE];
    ViChar      *pbuffer        = VI_NULL;
    ViChar      *p2buf;
    ViUInt32    local_count     = 0;
    ViInt32     cnt             = 0,
                idx             = 0;
    ViReal64    *values;

    CHECKMODEL(RSESU8 | RSESU26 | RSESU40);
	
	if (rsesci_invalidViInt32Range (trace, 1, 2))
        return VI_ERROR_PARAMETER2;
    if (noOfValues < 1)
        return VI_ERROR_PARAMETER3;
    if (!x)
        return VI_ERROR_PARAMETER4;
    if (!y)
        return VI_ERROR_PARAMETER5;
    if (!returnedValues)
        return VI_ERROR_PARAMETER6;

    /* Create common temporary value array */
    values = malloc (2 * noOfValues * sizeof (ViReal64));

    sprintf (cmd, ":FORM ASC;:TRAC%ld? FPE", trace);

    GOCHECKERR(rsesci_clearBeforeRead (instrSession));
    GOCHECKERR (viPrintf(instrSession, "%s\n", cmd));
    GOCHECKERR (rsesci_checkBeforeRead (instrSession));
    GOCHECKERR (rsesci_readDataUnknownLength(instrSession, &pbuffer, &local_count));
    p2buf = pbuffer;

    p2buf = strtok(pbuffer, ",");

    while(p2buf && (cnt < 2*noOfValues))
        {
        values[cnt++] = atof(p2buf);
        p2buf = strtok(NULL, ",");
        };

    /* Distribute all measured values */
    cnt /= 2;
    for (idx = 0; idx < cnt; idx++)
    {
        x[idx]  = values[0 + idx * 2];
        y[idx]      = values[1 + idx * 2];
    }

    *returnedValues = cnt;

    GOCHECKERR (rsesci_checkStatus (instrSession));

ERROR:

    free(pbuffer);
    free(values);

    return rsesci_status;     
}

/*****************************************************************************
 * Function: Set Marker Demodulation
 * Purpose:  This function switches on or off the audio demodulator when the
 *           indicated marker is reached in the selected measurement window. In
 *           the frequency domain (span > 0) the hold time can be defined at
 *           the corresponding marker position with function
 *           rsesci_SetMarkerStopTime.
 *           In the time domain (span = 0) the demodulation is permanently
 *           active.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetMarkerDemodulation(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32 marker,
    ViBoolean   demodulationState
)
{
    ViChar  buffer[BUFFER_SIZE];

    CHECKWINDOW;

    if (rsesci_invalidViInt32Range (marker, 1, 4))
        return VI_ERROR_PARAMETER3;

    sprintf (buffer, "CALC%ld:MARK%ld:FUNC:DEM:STAT", window, marker);

    return rsesci_setViBoolean (instrSession,
                                buffer,
                                demodulationState,
                                VI_ERROR_PARAMETER4);
}

/*****************************************************************************
 * Function: Get Marker Demodulation
 * Purpose:  This function returns the state of the audio demodulator.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetMarkerDemodulation(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32 marker,
    ViBoolean*  demodulationState
)
{
    ViChar  buffer[BUFFER_SIZE];

    CHECKWINDOW;

    if (rsesci_invalidViInt32Range (marker, 1, 4))
        return VI_ERROR_PARAMETER3;

    sprintf (buffer, "CALC%ld:MARK%ld:FUNC:DEM:STAT?", window, marker);

    return rsesci_getViBoolean (instrSession,
                                buffer,
                                demodulationState,
                                VI_ERROR_PARAMETER4);
}

/*****************************************************************************
 * Function: Set Marker Demodulation Type
 * Purpose:  This function selects the demodulation type for the audio
 *           demodulator. The function is independent of the measurement window
 *           and of the selected marker.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetMarkerDemodulationType(
    ViSession   instrSession,
    ViInt32 demodulationType
)
{
    return rsesci_setEnum (instrSession,
                           "CALC:MARK:FUNC:DEM:SEL",
                           demodulationType,
                           demodulationArr,
                           VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Marker Demodulation Type
 * Purpose:  This function queries selected demodulation type for the audio
 *           demodulator.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetMarkerDemodulationType(
    ViSession   instrSession,
    ViInt32*    demodulationType
)
{
    return rsesci_getEnum (instrSession,
                           "CALC:MARK:FUNC:DEM:SEL?",
                           demodulationType,
                           demodulationArr,
                           VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Squelch Analyzer
 * Purpose:  This function enables the input of a level threshold below which 
 *           the audible AF is cut off. The squelch function is associated with 
 *           the internal trigger function (TRIGGER menu), which will be switched
 *           on automatically with the squelch. Squelch level and trigger
 *           level do have the same value.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetSquelchAnalyzer (ViSession instrSession,
                                             ViInt32 window, ViInt32 marker,
                                             ViBoolean squelchState,
                                             ViReal64 squelchLevel)
{
	ViStatus 	rsesci_status = VI_SUCCESS;
	ViChar		buffer[BUFFER_SIZE] = "";
	
	CHECKWINDOW;

    if (rsesci_invalidViInt32Range (marker, 1, 4))
        return VI_ERROR_PARAMETER3;

    sprintf (buffer, "CALC%ld:MARK%ld:FUNC:DEM:SQU:STAT", window, marker);

	rsesci_status = rsesci_setViBoolean (instrSession,
		                                 buffer,
										 squelchState,
										 VI_ERROR_PARAMETER4);
	
	buffer[0] = '\0';
	if (squelchState == VI_TRUE)
	{
		sprintf (buffer, "CALC%ld:MARK%ld:FUNC:DEM:SQU:LEV", window, marker);

		rsesci_status = rsesci_setViReal64 (instrSession,
			                                buffer,
										    squelchLevel,
											0.0, 100.0,
											VI_ERROR_PARAMETER5);
	}
	
	return rsesci_status;
}

/*****************************************************************************
 * Function: Get Squelch Analyzer
 * Purpose:  This function returns the input of a level threshold below which 
 *           the audible AF is cut off.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetSquelchAnalyzer (ViSession instrSession,
                                             ViInt32 window, ViInt32 marker,
                                             ViBoolean *squelchState,
                                             ViReal64 *squelchLevel)
{
	ViStatus 	rsesci_status = VI_SUCCESS;
	ViChar		buffer[BUFFER_SIZE] = "";
	
	CHECKWINDOW;

    if (rsesci_invalidViInt32Range (marker, 1, 4))
        return VI_ERROR_PARAMETER3;

    sprintf (buffer, "CALC%ld:MARK%ld:FUNC:DEM:SQU:STAT?", window, marker);

	rsesci_status = rsesci_getViBoolean (instrSession,
		                                 buffer,
										 squelchState,
										 VI_ERROR_PARAMETER4);
	
	buffer[0] = '\0';
	
	sprintf (buffer, "CALC%ld:MARK%ld:FUNC:DEM:SQU:LEV?", window, marker);

	rsesci_status = rsesci_getViReal64 (instrSession,
		                                buffer,
									    squelchLevel,
										VI_ERROR_PARAMETER5);
	
	
	return rsesci_status;    
}

/*****************************************************************************
 * Function: Set Marker Stop Time
 * Purpose:  This function defines the hold time at the marker position for
 *           the demodulation in the frequency domain (span > 0). The setting
 *           is independent of the measurement window and the selected marker,
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetMarkerStopTime(
    ViSession   instrSession,
    ViReal64    holdoff
)
{
    return rsesci_setViReal64 (instrSession,
                               "CALC:MARK:FUNC:DEM:HOLD",
                               holdoff,
                               10E-3 , 1000 ,
                               VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Marker Stop Time
 * Purpose:  This function returns the hold time at the marker position for
 *           the demodulation in the frequency domain (span > 0).
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetMarkerStopTime(
    ViSession   instrSession,
    ViReal64*   holdoff
)
{
    return rsesci_getViReal64 (instrSession,
                               "CALC:MARK:FUNC:DEM:HOLD?",
                               holdoff,
                               VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Continuous Marker Demodulation
 * Purpose:  This function switches on or off the continuous demodulation in
 *           the frequency domain (span >0) in the selected measurement window.
 *           Thus acoustic monitoring of the signals can be performed in the
 *           frequency domain. The function does not depend on the selected
 *           marker,
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetContinuousMarkerDemodulation(
    ViSession   instrSession,
    ViInt32     window,
    ViBoolean   continuousDemodulation
)
{
    return rsesci_setViBooleanWindow (instrSession,
                                      "CALC%ld:MARK:FUNC:DEM:CONT",
                                      window,
                                      continuousDemodulation,
                                      VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Continuous Marker Demodulation
 * Purpose:  This function returns setting of the continuous demodulation in
 *           the frequency domain (span >0) in the selected measurement window.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetContinuousMarkerDemodulation(
    ViSession   instrSession,
    ViInt32     window,
    ViBoolean*  continuousDemodulation
)
{
    return rsesci_getViBooleanWindow (instrSession,
                                      "CALC%ld:MARK:FUNC:DEM:CONT?",
                                      window,
                                      continuousDemodulation,
                                      VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Initiate Spurious
 * Purpose:  This function starts a new spurious measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_InitiateSpurious (ViSession instrSession,
                                           ViInt32 window)
{
	CHECKMODEL(RSESU8 | RSESU26 | RSESU40);
	
	return rsesci_sendCmdWindow (instrSession,
                                 "INIT%ld:SPUR",
                                 window);    
}

/*****************************************************************************
 * Function: Set Power Unit
 * Purpose:  This function selects the unit for power for the selected 
 *           measurement window.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetPowerUnit (ViSession instrSession,
                                       ViInt32 window, ViInt32 powerUnit)
{
	if ((powerUnit >= RSESCI_UNIT_DBUV_MHZ) && (powerUnit <= RSESCI_UNIT_DBUA_MHZ) || (powerUnit >= RSESCI_UNIT_DBUV_MMHZ) && (powerUnit <= RSESCI_UNIT_UNITLESS))
		CHECKMODEL(RSESU8 | RSESU26 | RSESU40);
	
	return rsesci_setEnumWindow (instrSession,
                                 "UNIT%ld:POW",
								 window,
                                 powerUnit,
                                 setLimitPowerUnitArr,
                                 VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Power Unit
 * Purpose:  This function returns the unit for power for the selected 
 *           measurement window.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetPowerUnit (ViSession instrSession,
                                       ViInt32 window, ViInt32 *powerUnit)
{
	return rsesci_getEnumWindow (instrSession,
                                 "UNIT%ld:POW?",
								 window,
                                 powerUnit,
                                 getLimitPowerUnitArr,
                                 VI_ERROR_PARAMETER3);    
}

/*****************************************************************************
 * Function: Set Time Domain Power
 * Purpose:  This function switches on or off selected measurement in selected
 *           window.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetTimeDomainPower(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32     powerMeasurement,
    ViBoolean   measurementState
)
{
    ViChar  buffer[BUFFER_SIZE];

    CHECKWINDOW;

    if (rsesci_invalidViInt32Range (powerMeasurement, 0, 5))
        return VI_ERROR_PARAMETER3;

    sprintf (buffer, "CALC%ld:MARK:FUNC:SUMM:%s",
             window,
             powerMeasurementArr[powerMeasurement]);

    if (powerMeasurement == RSESCI_POWER_MEAS_AOFF)
        {
        return rsesci_sendCmd (instrSession, buffer);
        }
    else
        {
        return rsesci_setViBoolean (instrSession,
                                    buffer,
                                    measurementState,
                                    VI_ERROR_PARAMETER4);
        }
}

/*****************************************************************************
 * Function: Get Time Domain Power
 * Purpose:  This function queries the state of selected measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetTimeDomainPower(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32     powerMeasurement,
    ViBoolean*  measurementState
)
{
    ViChar  buffer[BUFFER_SIZE];

    CHECKWINDOW;
    if (rsesci_invalidViInt32Range (powerMeasurement, 0, 5))
        return VI_ERROR_PARAMETER3;

    sprintf (buffer, "CALC%ld:MARK:FUNC:SUMM:%s?",
             window,
             powerMeasurementArr[powerMeasurement]);

    return rsesci_getViBoolean (instrSession,
                                buffer,
                                measurementState,
                                VI_ERROR_PARAMETER4);
}

/*****************************************************************************
 * Function: Set Time Domain Power Averaging
 * Purpose:  This function switches on or off averaging for the active time
 *           domain power measurement in the indicated window. The function is
 *           independent of the marker selection. It is only available in the
 *           time domain (span = 0). Averaging is reset by switching it off and
 *           on again. The number of results required for the calculation of
 *           average is defined with [SENSe<1|2>:]AVERage:COUNt. It should be
 *           noted that synchronization to the end of averaging is only
 *           possible in single sweep mode.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetTimeDomainPowerAveraging(
    ViSession   instrSession,
    ViInt32     window,
    ViBoolean   averaging
)
{
    return rsesci_setViBooleanWindow (instrSession,
                                      "CALC%ld:MARK:FUNC:SUMM:AVER",
                                      window,
                                      averaging,
                                      VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Time Domain Power Averaging
 * Purpose:  This function returns the state of averaging for the active time
 *           domain power measurement in the indicated window.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetTimeDomainPowerAveraging(
    ViSession   instrSession,
    ViInt32     window,
    ViBoolean*  averaging
)
{
    return rsesci_getViBooleanWindow (instrSession,
                                      "CALC%ld:MARK:FUNC:SUMM:AVER?",
                                      window,
                                      averaging,
                                      VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set Time Domain Power Peak Hold
 * Purpose:  This function switches on or off the peak-hold function for the
 *           active time domain power measurement in the indicated measurement
 *           window. The function is independent of the marker
 *           selection. It is only available in the time domain (span = 0).
 *           The peak-hold function is reset by switching it off and on again.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetTimeDomainPowerPeakHold(
    ViSession   instrSession,
    ViInt32     window,
    ViBoolean   peakHold
)
{
    return rsesci_setViBooleanWindow (instrSession,
                                      "CALC%ld:MARK:FUNC:SUMM:PHOL",
                                      window,
                                      peakHold,
                                      VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Time Domain Power Peak Hold
 * Purpose:  This function queries the state of the peak-hold function for the
 *           active time domain power measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetTimeDomainPowerPeakHold(
    ViSession   instrSession,
    ViInt32     window,
    ViBoolean*  peakHold
)
{
    return rsesci_getViBooleanWindow (instrSession,
                                      "CALC%ld:MARK:FUNC:SUMM:PHOL?",
                                      window,
                                      peakHold,
                                      VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set Time Domain Power Mode
 * Purpose:  This function selects absolute or relative time domain power
 *           measurement in the indicated measurement window. The function is
 *           independent of the marker selection. It is only available in the
 *           time domain (span = 0).
 *           The reference power for relative measurement is defined with
 *           rsesci_SetTimeDomainPowerReference(). If the reference power is
 *           not defined, the value 0 dBm is used.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetTimeDomainPowerMode(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32 mode
)
{
    return rsesci_setEnumWindow (instrSession,
                                 "CALC%ld:MARK:FUNC:SUMM:MODE",
                                 window,
                                 mode,
                                 absRelArr,
                                 VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Time Domain Power Mode
 * Purpose:  This function queries mode of time domain power measurement in
 *           the indicated measurement window.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetTimeDomainPowerMode(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32*    mode
)
{
    return rsesci_getEnumWindow (instrSession,
                                 "CALC%ld:MARK:FUNC:SUMM:MODE?",
                                 window,
                                 mode,
                                 absRelArr,
                                 VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set Time Domain Power Reference
 * Purpose:  With this function the currently measured average value and RMS
 *           value are declared as reference values for relative measurements
 *           in the indicated measurement window. The function is independent
 *           of the marker selection. It is only available in the time domain
 *           (span = 0). If the measurement of RMS value and average is not
 *           activated, the reference value 0 dBm is used. If the function
 *           rsesci_SetTimeDomainPowerPeakHold is switched on, the current
 *           value is the accumulated measurement value at the time considered.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetTimeDomainPowerReference(
    ViSession   instrSession,
    ViInt32     window
)
{
    return rsesci_sendCmdWindow (instrSession,
                                 "CALC%ld:MARK:FUNC:SUMM:REF:AUTO ONCE",
                                 window);
}

/*****************************************************************************
 * Function: Get Time Domain Power Mean Value
 * Purpose:  This function queries the result of the measurement of the mean
 *           value in the selected measurement window. The function is
 *           independent of the marker selection. It is only available in the
 *           time domain (span = 0). A complete sweep with synchronization to
 *           sweep end must be performed between switching on the function and
 *           querying the measured value to obtain a valid query result. This
 *           is only possible in single sweep mode.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetTimeDomainPowerMeanValue(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32     resultType,
    ViReal64*   result
)
{
    ViChar  buffer[BUFFER_SIZE];

    CHECKWINDOW;
    if (rsesci_invalidViInt32Range (resultType,0,2))
        return VI_ERROR_PARAMETER3;

    sprintf (buffer, "CALC%ld:MARK:FUNC:SUMM:MEAN:%s?", window, resultTypeArr[resultType]);

    return rsesci_getViReal64 (instrSession,
                               buffer,
                               result,
                               VI_ERROR_PARAMETER4);
}

/*****************************************************************************
 * Function: Get Time Domain Power Peak Value
 * Purpose:  This function is used to query the result of the measurement of
 *           the positive peak value in the selected measurement window. The
 *           measurement may have to be switched on previously. The function is
 *           independent of the marker selection. It is only available in the
 *           time domain (span = 0). A complete sweep with synchronization to
 *           sweep end must be performed between switching on the function and
 *           querying the measured value to obtain a valid query result. This
 *           is only possible in single sweep mode.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetTimeDomainPowerPeakValue(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32 resultType,
    ViReal64*   result
)
{
    ViChar  buffer[BUFFER_SIZE];

    CHECKWINDOW;
    if (rsesci_invalidViInt32Range (resultType,0,2))
        return VI_ERROR_PARAMETER3;

    sprintf (buffer, "CALC%ld:MARK:FUNC:SUMM:PPE:%s?", window, resultTypeArr[resultType]);

    return rsesci_getViReal64 (instrSession,
                               buffer,
                               result,
                               VI_ERROR_PARAMETER4);
}

/*****************************************************************************
 * Function: Get Time Domain Power RMS Value
 * Purpose:  This function queries the result of the measurement of the RMS
 *           power value in the selected measurement window. The function is
 *           independent of the marker selection. It is only available in the
 *           time domain (span = 0). A complete sweep with synchronization to
 *           sweep end must be performed between switching on the function and
 *           querying the measured value to obtain a valid query result. This
 *           is only possible in single sweep mode.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetTimeDomainPowerRMSValue(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32 resultType,
    ViReal64*   result
)
{
    ViChar  buffer[BUFFER_SIZE];

    CHECKWINDOW;
    if (rsesci_invalidViInt32Range (resultType,0,2))
        return VI_ERROR_PARAMETER3;

    sprintf (buffer, "CALC%ld:MARK:FUNC:SUMM:RMS:%s?", window, resultTypeArr[resultType]);

    return rsesci_getViReal64 (instrSession,
                               buffer,
                               result,
                               VI_ERROR_PARAMETER4);
}

/*****************************************************************************
 * Function: Get Time Domain Power Standard Deviation Value
 * Purpose:  This function queries the results of the standard deviation
 *           measurement. The function is independent of the marker selection.
 *           It is only available in the time domain (span = 0).
 *           A complete sweep with synchronization to sweep end must be
 *           performed between switching on the function and querying the
 *           measured value to obtain a valid query result. This is only
 *           possible in single sweep mode.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetTimeDomainPowerStandardDeviationValue(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32 resultType,
    ViReal64*   result
)
{
    ViChar  buffer[BUFFER_SIZE];

    CHECKWINDOW;
    if (rsesci_invalidViInt32Range (resultType,0,2))
        return VI_ERROR_PARAMETER3;

    sprintf (buffer, "CALC%ld:MARK:FUNC:SUMM:SDEV:%s?", window, resultTypeArr[resultType]);

    return rsesci_getViReal64 (instrSession,
                               buffer,
                               result,
                               VI_ERROR_PARAMETER4);
}

/*****************************************************************************
 * Function: Get Time Domain Power Signal Pulses Value
 * Purpose:  This function is used to determine the power of a sequence of
 *           signal pulses having the same interval, as are typical for the
 *           slots of a GSM signal, for example. The number of pulses to be
 *           measured as well as the measurement time and the period can be
 *           set. To define the position of the first pulse in the trace, a
 *           suitable offset can be entered. The evaluation is performed on the
 *           measurement data of a previously recorded trace. The data recorded
 *           during the set measurement time is combined to a measured value
 *           for each pulse according to the detector specified and the
 *           indicated number of results is output as a list. TRACE 1 of the
 *           selected screen is always used by the function.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetTimeDomainPowerSignalPulsesValue(
    ViSession   instrSession,
    ViInt32     window,
    ViReal64    timeoffsetoffirstpulse,
    ViReal64    measurementTime,
    ViReal64    period,
    ViInt32     pulses,
    ViReal64*   result
)
{
    ViChar  buffer [BUFFER_SIZE];

    sprintf (buffer, "CALC%%ld:MARK:FUNC:MSUM? %lf,%lf,%lf,%ld",
             timeoffsetoffirstpulse,
             measurementTime,
             period,
             pulses);

    return rsesci_getViReal64Window (instrSession,
                                     buffer,
                                     window,
                                     result,
                                     VI_ERROR_PARAMETER7);
}

/*****************************************************************************
 * Function: Set Power Measurement
 * Purpose:  This function selects type of power measurement in the selected
 *           measurement window. This function is independent of the selected
 *           marker.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetPowerMeasurement(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32     measurement
)
{
    if (measurement == RSESCI_FDOMAIN_MEAS_OFF)
        return rsesci_sendCmdWindow (instrSession,
                                     "CALC%ld:MARK:FUNC:POW OFF",
                                     window);
    else
        return rsesci_setEnumWindow (instrSession,
                                     "CALC%ld:MARK:FUNC:POW:SEL",
                                     window,
                                     measurement,
                                     powerMeasTypeArr,
                                     VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Power Measurement
 * Purpose:  This function queries for selected power measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetPowerMeasurement(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32*    measurement
)
{
    ViStatus    rsesci_status = VI_SUCCESS;

    rsesci_status = rsesci_getEnumWindow (instrSession,
                                          "CALC%ld:MARK:FUNC:POW:SEL?",
                                          window,
                                          measurement,
                                          powerMeasTypeArr,
                                          VI_ERROR_PARAMETER3);

    if (rsesci_status != VI_SUCCESS)
        {
        *measurement = RSESCI_FDOMAIN_MEAS_OFF;
        }

    return rsesci_status;
}

/*****************************************************************************
 * Function: Adjust Power Measurement Settings
 * Purpose:  This function adjusts the frequency span, the measurement
 *           bandwidths and the detector as required for the number of
 *           channels, the channel bandwidths and the channel spacings selected
 *           in the active power measurement. If necessary, adjacent-channel
 *           power measurement is switched on prior to the adjustment. To
 *           obtain valid results, a complete sweep with synchronization to the
 *           end of the sweep must be performed after the adjustment.
 *           Synchronization is possible only in the single-sweep mode.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_AdjustPowerMeasurementSetting(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32 measurement
)
{
    return rsesci_setEnumWindow (instrSession,
                                 "SENS%ld:POW:ACH:PRES",
                                 window,
                                 measurement,
                                 powerMeasTypeArr,
                                 VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Adjust Reference Level
 * Purpose:  This function adapts the reference level to the measured channel
 *           power
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_AdjustReferenceLevel(
    ViSession   instrSession,
    ViInt32     window
)
{
    return rsesci_sendCmdWindow (instrSession,
                                 "SENS%ld:POW:ACH:PRES:RLEV;*WAI",
                                 window);
}

/*****************************************************************************
 * Function: Get Power Measurement Value
 * Purpose:  This function queries the result of the power measurement
 *           performed in the selected window.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetPowerMeasurementValue(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32     measurement,
    ViInt32     arraySize,
    ViReal64    _VI_FAR result[]
)
{
    ViChar  buffer[BUFFER_SIZE];

    if (rsesci_invalidViInt32Range (measurement, 1, 6))
        return VI_ERROR_PARAMETER3;

    sprintf (buffer, "CALC%%ld:MARK:FUNC:POW:RES? %s", powerMeasTypeArr[measurement]);
    return rsesci_getViReal64ArrayWindow(instrSession,
                                         buffer,
                                         window,
                                         result,
                                         arraySize,
                                         VI_ERROR_PARAMETER4);
}

/*****************************************************************************
 * Function: Set Channel Power Hz Measurement
 * Purpose:  This function switches the query response of the power
 *           measurement results in the indicated measurement window between
 *           output of absolute values (OFF) and output referred to the
 *           measurement bandwith (ON).
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetChannelPowerHzMeasurement(
    ViSession   instrSession,
    ViInt32     window,
    ViBoolean   relativeResult
)
{
    return rsesci_setViBooleanWindow (instrSession,
                                      "CALC%ld:MARK:FUNC:POW:RES:PHZ",
                                      window,
                                      relativeResult,
                                      VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Channel Power Hz Measurement
 * Purpose:  This function returns the state of the query response of the
 *           power measurement results in the indicated measurement window.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetChannelPowerHzMeasurement(
    ViSession   instrSession,
    ViInt32     window,
    ViBoolean*  relativeResult
)
{
    return rsesci_getViBooleanWindow (instrSession,
                                      "CALC%ld:MARK:FUNC:POW:RES:PHZ?",
                                      window,
                                      relativeResult,
                                      VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set Power Mode
 * Purpose:  This function selects the Clear Write or Maxhold for Channel Power 
 *           values. This function is independent of the selected marker.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetPowerMode (ViSession instrSession,
                                       ViInt32 window, ViInt32 powerMode)
{
	if (powerMode != 0)
		if (powerMode != 3)
			return VI_ERROR_PARAMETER3;
	
	return rsesci_setEnumWindow (instrSession,
                                 "CALC%ld:MARK:FUNC:POW:MODE",
                                 window,
                                 powerMode,
                                 traceModeArr,
                                 VI_ERROR_PARAMETER3);    
}

/*****************************************************************************
 * Function: Get Power Mode
 * Purpose:  This function returns the Clear Write or Maxhold for Channel Power 
 *           values. This function is independent of the selected marker.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetPowerMode (ViSession instrSession,
                                       ViInt32 window, ViInt32 *powerMode)
{
	return rsesci_getEnumWindow (instrSession,
                                 "CALC%ld:MARK:FUNC:POW:MODE?",
                                 window,
                                 powerMode,
                                 traceModeArr,
                                 VI_ERROR_PARAMETER3);    
}

/*****************************************************************************
 * Function: Set CP ACP Standard
 * Purpose:  This function selects the power measurement setting for a
 *           standard in the indicated measurement window and previously
 *           switches on the corresponding measurement, if required. The
 *           function is independent of the marker selection. The configuration
 *           for a standard comprises of the parameters weighting filter,
 *           channel bandwidth and spacing, resolution and video bandwidth, as
 *           well as detector and sweep time.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetCPACPStandard(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32     standard
)
{
    return rsesci_setEnumWindow (instrSession,
                                 "CALC%ld:MARK:FUNC:POW:PRES",
                                 window,
                                 standard,
                                 standardArr,
                                 VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get CP ACP Standard
 * Purpose:  This function queries selected standard for which power
 *           measurement setting are set.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetCPACPStandard(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32*    standard
)
{
    return rsesci_getEnumWindow (instrSession,
                                 "CALC%ld:MARK:FUNC:POW:PRES?",
                                 window,
                                 standard,
                                 standardArr,
                                 VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get ACP Standard Catalog
 * Purpose:  This function returns all predefine ACP standards and all user
             standards.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetACPStandardCatalog (ViSession instrSession,
                                                ViInt32 window,
                                                ViInt32 arraySize,
                                                ViChar catalog[])
{
   
   return rsesci_getViStringWindow (instrSession,
		                       "CALC%ld:MARK:FUNC:POW:STAN:CAT?",
							   window,
							   catalog,
							   arraySize,
							   VI_ERROR_PARAMETER4);
	
}

/*****************************************************************************
 * Function: Set CP ACP User Standard
 * Purpose:  This function selects user defined power measurement setting for a
 *           standard in the indicated measurement window and previously
 *           switches on the corresponding measurement, if required. The
 *           function is independent of the marker selection. The configuration
 *           for a standard comprises of the parameters weighting filter,
 *           channel bandwidth and spacing, resolution and video bandwidth, as
 *           well as detector and sweep time.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetCPACPUserStandard (ViSession instrSession,
                                               ViInt32 window,
                                               ViString userStandardName)
{
    ViChar	buffer[BUFFER_SIZE] = "";
    
    sprintf (buffer, "CALC%%ld:MARK:FUNC:POW:PRES '%s'", userStandardName);
    return rsesci_sendCmdWindow (instrSession,
                                 buffer,
                                 window);
}

/*****************************************************************************
 * Function: Delete ACP User Standard
 * Purpose:  This function removes the ACP user standard <name>. It is not
             possible to remove predefine ACP standards.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_DeleteACPUserStandard (ViSession instrSession,
                                            ViInt32 window,
                                            ViString standardName)
{
    ViChar	buffer[BUFFER_SIZE] = "";

    sprintf (buffer, "CALC%ld:MARK:FUNC:POW:STAN:DEL", window);
	
	return rsesci_setViStringQuoted (instrSession, 
                                     buffer,
                                     "'",
                                     standardName,
                                     VI_ERROR_PARAMETER3);
	
}

/*****************************************************************************
 * Function: Save ACP User Standard
 * Purpose:  This command saves the current ACP settings as an ACP user
             standard.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SaveACPUserStandard (ViSession instrSession,
                                          ViInt32 window, ViString standardName)
{
	
   ViChar	buffer[BUFFER_SIZE] = "";

    sprintf (buffer, "CALC%ld:MARK:FUNC:POW:STAN:SAVE", window);
	
	return rsesci_setViStringQuoted (instrSession, 
                                     buffer,
                                     "'",
                                     standardName,
                                     VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set ACP Reference Level Offset
 * Purpose:  This function allows to modify the ADJUST REF LEVEL function for
			 user standards. It defines an additional level offset to be
			 taken into account for the analyzer's reference level setting.
			 It is only available if Fast ACP is not active and no predefine
			 standard is selected.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetACPReferenceLevelOffset (ViSession instrSession,
                                                     ViInt32 window,
                                                     ViReal64 referenceLevelOffset)
{
    return rsesci_setViReal64Window (instrSession,
                                     "SENS%ld:POW:ACH:PRES:RLEV:OFFS",
                                     window,
                                     referenceLevelOffset,
                                     0.0 , 100.0 ,
                                     VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get ACP Reference Level Offset
 * Purpose:  This function queries the modification of the ADJUST REF LEVEL
			 function for user standards. It defines an additional level
			 offset to be taken into account for the analyzer's reference
			 level setting. It is only available if Fast ACP is not active
    		 and no predefine standard is selected.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetACPReferenceLevelOffset (ViSession instrSession,
                                                     ViInt32 window,
                                                     ViReal64 *referenceLevelOffset)
{
    return rsesci_getViReal64Window (instrSession,
                                     "SENS%ld:POW:ACH:PRES:RLEV:OFFS?",
                                     window,
                                     referenceLevelOffset,
                                     VI_ERROR_PARAMETER3);
	
}

/*****************************************************************************
 * Function: Set CP Reference
 * Purpose:  This function sets the reference value for the relative
 *           measurement to the currently measured channel power.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetCPReference(
    ViSession   instrSession,
    ViInt32     window
)
{
    return rsesci_sendCmdWindow (instrSession,
                                 "SENS%ld:POW:ACH:REF:AUTO ONCE",
                                 window);
}

/*****************************************************************************
 * Function: Set Noise Correction
 * Purpose:  This function switches on or off the correction of the
 *           instrument inherent noise for channel power measurement. On
 *           activating this function, a reference measurement of the
 *           instrument inherent noise is performed. The measured noise power
 *           is then subtracted from the power in the examined channel. The
 *           instrument inherent noise is then re-determined after any change
 *           of the center frequency, resolution bandwidth, sweep time and
 *           level setting by repeating the reference measurement in the
 *           new instrument setting.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetNoiseCorrection(
    ViSession   instrSession,
    ViInt32     window,
    ViBoolean   correction
)
{
    return rsesci_setViBooleanWindow (instrSession,
                                      "SENS%ld:POW:NCOR",
                                      window,
                                      correction,
                                      VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Noise Correction
 * Purpose:  This function queries the state of the correction of the
 *           instrument inherent noise for channel power measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetNoiseCorrection(
    ViSession   instrSession,
    ViInt32     window,
    ViBoolean*  correction
)
{
    return rsesci_getViBooleanWindow (instrSession,
                                      "SENS%ld:POW:NCOR?",
                                      window,
                                      correction,
                                      VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set Fast ACP
 * Purpose:  This function switches on or off the high-speed channel/adjacent
 *           channel power measurement. The measurement itself is performed in
 *           the time domain on the center frequencies of the individual
 *           channels. The command automatically switches to the time domain
 *           and back. Depending on the selected mobile radio standard,
 *           weighting filters with SQRT(cos) characteristic or very
 *           steep-sided channel filters are used for band limitation. The
 *           command is only available in the frequency domain (span > 0).
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetFastACP(
    ViSession   instrSession,
    ViInt32     window,
    ViBoolean   fastACP
)
{
    return rsesci_setViBooleanWindow (instrSession,
                                      "SENS%ld:POW:HSP",
                                      window,
                                      fastACP,
                                      VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Fast ACP
 * Purpose:  This function queries the state of the high-speed
 *           channel/adjacent channel power measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetFastACP(
    ViSession   instrSession,
    ViInt32     window,
    ViBoolean*  fastACP
)
{
    return rsesci_getViBooleanWindow (instrSession,
                                      "SENS%ld:POW:HSP?",
                                      window,
                                      fastACP,
                                      VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set Full Size Diagram
 * Purpose:  This function switches the measurement window for channel and
 *           adjacent-channel power measurements or for active FM demodulator
 *           (FS-K7) to full screen or half screen.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetFullSizeDiagram(
    ViSession   instrSession,
    ViInt32 size
)
{
    return rsesci_setEnum (instrSession,
                           "DISP:WIND:SIZE",
                            size,
                            sizeArr,
                            VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Full Size Diagram
 * Purpose:  This function queries the state of the measurement window for
 *           channel and adjacent-channel power measurements or for active FM
 *           demodulator (FS-K7) to full screen or half screen.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetFullSizeDiagram(
    ViSession   instrSession,
    ViInt32*    size
)
{
    return rsesci_getEnum (instrSession,
                           "DISP:WIND:SIZE?",
                           size,
                           sizeArr,
                           VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set Number of Adjacent Channels
 * Purpose:  This function sets the number of adjacent channels (upper and
 *           lower channel in pairs).The figure 0 stands for pure channel power
 *           measurement. The function is only available in the frequency
 *           domain (span > 0).
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetNumberAdjacentChannels(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32 numberofAdjacentChannels
)
{
    return rsesci_setViInt32Window (instrSession,
                                  "SENS%ld:POW:ACH:ACP",
                                  window,
                                  numberofAdjacentChannels,
                                  0, 12,
                                  VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Number of Adjacent Channels
 * Purpose:  This function queries the number of adjacent channels (upper and
 *           lower channel in pairs).The figure 0 stands for pure channel power
 *           measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetNumberAdjacentChannels(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32*    numberofAdjacentChannels
)
{
    return rsesci_getViInt32Window (instrSession,
                                  "SENS%ld:POW:ACH:ACP?",
                                  window,
                                  numberofAdjacentChannels,
                                  VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set Number of TX Channels
 * Purpose:  This function selects the number of carrier signals. The function
 *           is available only for multicarrier channel and adjacent-channel
 *           power measurements in the frequency domain (span > 0).
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetNumberTXChannels(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32 numberofCarriers
)
{
    return rsesci_setViInt32Window (instrSession,
                                  "SENS%ld:POW:ACH:TXCH:COUN",
                                  window,
                                  numberofCarriers,
                                  1, 12,
                                  VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Number of TX Channels
 * Purpose:  This function queries the number of carrier signals. The function
 *           is available only for multicarrier channel and adjacent-channel
 *           power measurements in the frequency domain (span > 0).
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetNumberTXChannels(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32*    numberofCarriers
)
{
    return rsesci_getViInt32Window (instrSession,
                                  "SENS%ld:POW:ACH:TXCH:COUN?",
                                  window,
                                  numberofCarriers,
                                  VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set TX ACP Channel Bandwidth
 * Purpose:  This function sets the selected channel bandwidth value.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetTXACPChannelBandwidth(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32 channelBandwidth,
    ViReal64    channelBandwidthValue
)
{
    ViChar  buffer[BUFFER_SIZE];

    CHECKWINDOW;
    if (rsesci_invalidViInt32Range (channelBandwidth, 0, 12))
        return VI_ERROR_PARAMETER3;

    sprintf (buffer, "SENS%ld:POW:ACH:BAND:%s", window, channelBandwidthArr[channelBandwidth]);

    return rsesci_setViReal64 (instrSession,
                               buffer,
                               channelBandwidthValue,
                               100.0 , 1.0E9,
                               VI_ERROR_PARAMETER4);
}

/*****************************************************************************
 * Function: Get TX ACP Channel Bandwidth
 * Purpose:  This function queries the channel bandwidth value.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetTXACPChannelBandwidth(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32 channelBandwidth,
    ViReal64*   channelBandwidthValue
)
{
    ViChar  buffer[BUFFER_SIZE];

    CHECKWINDOW;
    if (rsesci_invalidViInt32Range (channelBandwidth, 0, 12))
        return VI_ERROR_PARAMETER3;

    sprintf (buffer, "SENS%ld:POW:ACH:BAND:%s?", window, channelBandwidthArr[channelBandwidth]);

    return rsesci_getViReal64 (instrSession,
                               buffer,
                               channelBandwidthValue,
                               VI_ERROR_PARAMETER4);
}

/*****************************************************************************
 * Function: Set ACP Channel Spacing
 * Purpose:  This function defines the channel spacing for selected channel.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetACPChannelSpacing(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32 channelSpacing,
    ViReal64    channelSpacingValue
)
{
    ViChar  buffer[BUFFER_SIZE];

    CHECKWINDOW;
    if (rsesci_invalidViInt32Range (channelSpacing, 0, 22))
        return VI_ERROR_PARAMETER3;

    sprintf (buffer, "SENS%ld:POW:ACH:SPAC:%s", window, channelBandwidthArr[channelSpacing]);

    return rsesci_setViReal64 (instrSession,
                               buffer,
                               channelSpacingValue,
                               100.0 , 2.0E9,
                               VI_ERROR_PARAMETER4);
}

/*****************************************************************************
 * Function: Get ACP Channel Spacing
 * Purpose:  This function queries the channel spacing for selected channel.
*****************************************************************************/
ViStatus _VI_FUNC rsesci_GetACPChannelSpacing(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32 channelSpacing,
    ViReal64*   channelSpacingValue
)
{
    ViChar  buffer[BUFFER_SIZE];

    CHECKWINDOW;
    if (rsesci_invalidViInt32Range (channelSpacing, 0, 22))
        return VI_ERROR_PARAMETER3;

    sprintf (buffer, "SENS%ld:POW:ACH:SPAC:%s?", window, channelBandwidthArr[channelSpacing]);

    return rsesci_getViReal64 (instrSession,
                               buffer,
                               channelSpacingValue,
                               VI_ERROR_PARAMETER4);
}

/*****************************************************************************
 * Function: Set ACP Reference Channel
 * Purpose:  This function selects a transmission channel to be used as a
 *           reference channel in relative adjacent-channel power measurements.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetACPReferenceChannel(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32 acpReferenceChannel
)
{
    ViChar  buffer[BUFFER_SIZE];

    sprintf (buffer,"SENS%%ld:POW:ACH:REF:TXCH:%s",acpReferenceChannelArr[acpReferenceChannel]);

    return rsesci_sendCmdWindow (instrSession,
                                 buffer,
                                 window);
}

/*****************************************************************************
 * Function: Set CP ACP Power Measurement Mode
 * Purpose:  This function toggles between absolute and relative adjacent
 *           channel measurement. For the relative measurement the reference
 *           value is set to the currently measured channel power by function
 *           rsesci_SetCPReference().
  *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetCPACPPowerMeasurementMode(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32 mode
)
{
    return rsesci_setEnumWindow (instrSession,
                                 "SENS%ld:POW:ACH:MODE",
                                 window,
                                 mode,
                                 absRelArr,
                                 VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get CP ACP Power Measurement Mode
 * Purpose:  This function returns if absolute and relative adjacent channel
 *           measurement is selected.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetCPACPPowerMeasurementMode(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32*    mode
)
{
    return rsesci_getEnumWindow (instrSession,
                                 "SENS%ld:POW:ACH:MODE?",
                                 window,
                                 mode,
                                 absRelArr,
                                 VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set CP ACP Measurement Trace
 * Purpose:  This function assigns the channel/adjacent channel power
 *           measurement to the indicated trace in the selected measurement
 *           window. The corresponding trace must be active, i.e. its state
 *           must be different from blank.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetCPACPMeasurementTrace(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32     trace
)
{
    return rsesci_setViInt32Window (instrSession,
                                    "SENS%ld:POW:TRAC",
                                    window,
                                    trace,
                                    1, 3,
                                    VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get CP ACP Measurement Trace
 * Purpose:  This function returns to which channel/adjacent channel power
 *           measurement is the trace assigned in the selected measurement
 *           window.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetCPACPMeasurementTrace(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32*    trace
)
{
    return rsesci_getViInt32Window (instrSession,
                                    "SENS%ld:POW:TRAC?",
                                    window,
                                    trace,
                                    VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set All ACP Weigthing Filter State
 * Purpose:  This function switches the weigthing filter state for all the
             channels on on or off.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetAllACPWeigthingFilterState (ViSession instrSession,
                                                        ViInt32 window,
                                                        ViBoolean weightingFilterState)
{
    
	return rsesci_setViBooleanWindow (instrSession,
                                      "SENS%ld:POW:ACH:FILT:STAT:ALL",
                                      window,
                                      weightingFilterState,
                                      VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set All ACP Weighting Filter Factor
 * Purpose:  This function sets the rolloff factor of the RRC weighting
             filter for all the channels.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetAllACPWeightingFilterFactor (ViSession instrSession,
                                                         ViInt32 window,
                                                         ViReal64 rolloffFactor)
{
	return rsesci_setViReal64Window (instrSession,
                                     "SENS%ld:POW:ACH:FILT:ALPH:ALL",
                                     window,
                                     rolloffFactor,
                                     0.0 , 1.0 ,
                                     VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set ACP Weighting Filter State
 * Purpose:  This function switches the weigthing filter state for the TX
             channel the adjacent channel and the alternate channel on on or
			 off.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetACPWeightingFilterState (ViSession instrSession,
                                                     ViInt32 window,
                                                     ViInt32 channel,
                                                     ViBoolean weightingFilterState)
{
	ViChar  buffer[BUFFER_SIZE];
	
    if (rsesci_invalidViInt32Range (channel, 0, 12))
        return VI_ERROR_PARAMETER3;
	
	switch (channel)
	{
		
		case RSESCI_BWID_TX:
		 sprintf (buffer, "SENS%%ld:POW:ACH:FILT:STAT:CHAN");
            break;

		case RSESCI_BWID_ADJ:
            sprintf (buffer, "SENS%%ld:POW:ACH:FILT:STAT:ACH");
            break;
			
		default:
            sprintf (buffer, "SENS%%ld:POW:ACH:FILT:STAT:ALT%ld", channel-1);
            break;
	  
	 }

    return rsesci_setViBooleanWindow (instrSession,
                                     buffer,
                                     window,
                                     weightingFilterState,
                                     VI_ERROR_PARAMETER4);
}

/*****************************************************************************
 * Function: Get ACP Weighting Filter State
 * Purpose:  This function queries the weigthing filter state for the TX
             channel adjacent channel and the alternate channel on on or off.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetACPWeightingFilterState (ViSession instrSession,
                                                     ViInt32 window,
                                                     ViInt32 channel,
                                                     ViBoolean *weightingFilterState)
{
    ViChar  buffer[BUFFER_SIZE];
	
    if (rsesci_invalidViInt32Range (channel, 0, 12))
        return VI_ERROR_PARAMETER3;
	
	switch (channel)
	{
		
		case RSESCI_BWID_TX:
		 sprintf (buffer, "SENS%%ld:POW:ACH:FILT:STAT:CHAN?");
            break;

		case RSESCI_BWID_ADJ:
            sprintf (buffer, "SENS%%ld:POW:ACH:FILT:STAT:ACH?");
            break;
			
		default:
            sprintf (buffer, "SENS%%ld:POW:ACH:FILT:STAT:ALT%ld?", channel-1);
            break;
	  
	 }

    return rsesci_getViBooleanWindow (instrSession,
                                     buffer,
                                     window,
                                     weightingFilterState,
                                     VI_ERROR_PARAMETER4);
}

/*****************************************************************************
 * Function: Set ACP Weighting Filter Factor
 * Purpose:  This function sets the rolloff factor of the RRC weighting
             filter for the TX channel adjacent channel and the alternate
             channel.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetACPWeightingFilterFactor (ViSession instrSession,
                                                      ViInt32 window,
                                                      ViInt32 channel,
                                                      ViReal64 rolloffFactor)
{
   
	ViChar  buffer[BUFFER_SIZE];
	
    if (rsesci_invalidViInt32Range (channel, 0, 12))
        return VI_ERROR_PARAMETER3;
	
	switch (channel)
	{
		
		case RSESCI_BWID_TX:
		 sprintf (buffer, "SENS%%ld:POW:ACH:FILT:ALPH:CHAN");
            break;		   

		case RSESCI_BWID_ADJ:
            sprintf (buffer, "SENS%%ld:POW:ACH:FILT:ALPH:ACH");
            break;
			
		default:
            sprintf (buffer, "SENS%%ld:POW:ACH:FILT:ALPH:ALT%ld", channel-1);
            break;
	  
	 }
	
    return rsesci_setViReal64Window (instrSession,
                                     buffer,
                                     window,
                                     rolloffFactor,
									 0.0, 1.0 ,
                                     VI_ERROR_PARAMETER4);
}

/*****************************************************************************
 * Function: Get ACP Weighting Filter Factor
 * Purpose:  This function queries the rolloff factor of the RRC weighting
             filter for the TX channel adjacent channel and the alternate
             channel.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetACPWeightingFilterFactor (ViSession instrSession,
                                                      ViInt32 window,
                                                      ViInt32 channel,
                                                      ViReal64 *rolloffFactor)
{
   ViChar  buffer[BUFFER_SIZE];
	
    if (rsesci_invalidViInt32Range (channel, 0, 12))
        return VI_ERROR_PARAMETER3;
	
	switch (channel)
	{
		
		case RSESCI_BWID_TX:
		 sprintf (buffer, "SENS%%ld:POW:ACH:FILT:ALPH:CHAN?");
            break;		   

		case RSESCI_BWID_ADJ:
            sprintf (buffer, "SENS%%ld:POW:ACH:FILT:ALPH:ACH?");
            break;
			
		default:
            sprintf (buffer, "SENS%%ld:POW:ACH:FILT:ALPH:ALT%ld?", channel-1);
            break;
	  
	 }
	
    return rsesci_getViReal64Window (instrSession,
                                     buffer,
                                     window,
                                     rolloffFactor,
                                     VI_ERROR_PARAMETER4);
}

/*****************************************************************************
 * Function: Set ACP Limit Checking
 * Purpose:  This function switches on and off the limit check for adjacent
 *           channel power measurements in the selected measurement window. The
 *           limit line number is irrelevant for this function.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetACPLimitChecking(
    ViSession   instrSession,
    ViInt32     window,
    ViBoolean   limitChecking
)
{
    return rsesci_setViBooleanWindow (instrSession,
                                      "CALC%ld:LIM:ACP:STAT",
                                      window,
                                      limitChecking,
                                      VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get ACP Limit Checking
 * Purpose:  This function queries the state of the limit check for adjacent
 *           channel power measurements in the selected measurement window.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetACPLimitChecking(
    ViSession   instrSession,
    ViInt32     window,
    ViBoolean*  limitChecking
)
{
    return rsesci_getViBooleanWindow (instrSession,
                                      "CALC%ld:LIM:ACP:STAT?",
                                      window,
                                      limitChecking,
                                      VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set ACP Limits
 * Purpose:  This function sets the state and value of selected adjacent
 *           channel measurement limit.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetACPLimits(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32     limitType,
    ViInt32     mode,
    ViBoolean   state,
    ViReal64    limitValue
)
{
    ViStatus    rsesci_status;
    ViChar      buffer[BUFFER_SIZE];
    ViChar      cmd[BUFFER_SIZE];
    ViChar      *p2buf;
    ViReal64    min = 0.0, max = 0.0;

    if (rsesci_invalidViInt32Range (limitType, 1, 12))
        return VI_ERROR_PARAMETER3;
    if (rsesci_invalidViInt32Range (mode, 0, 1))
        return VI_ERROR_PARAMETER4;

    p2buf = buffer + sprintf (buffer, "CALC%ld:LIM:ACP:%s:%s",
                              window,
                              acpLimitsArr[limitType],
                              absRelArr[mode]);

    sprintf (p2buf, ":STAT");

    CHECKERR( rsesci_setViBoolean (instrSession,
                                   buffer,
                                   state,
                                   VI_ERROR_PARAMETER5));
    if (state)
    {
        *p2buf = '\0'; /* cut :STAT from the command */
        switch (mode)
        {
            case RSESCI_ABS:
                min = -200.0;
                max = 200.0;
                break;

            case RSESCI_REL:
            default:
                min = 0.0;
                max = 100.0;
                break;
        }

        if (rsesci_invalidViReal64Range (limitValue, min, max))
            return VI_ERROR_PARAMETER6;

        sprintf (cmd, "%s %lf,%lf", buffer, limitValue, limitValue);

        rsesci_status = rsesci_sendCmd(instrSession, cmd);
    }

    return rsesci_status;
}

/*****************************************************************************
 * Function: Get ACP Limits
 * Purpose:  This function queries the limit check state and limit value for
 *           selected adjacent channel limit.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetACPLimits(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32     mode,
    ViInt32     limitType,
    ViBoolean*  state,
    ViReal64*   limitValue
)
{
    ViStatus    rsesci_status;
    ViChar      buffer[BUFFER_SIZE];
    ViChar      cmd[BUFFER_SIZE];

    if (rsesci_invalidViInt32Range (limitType, 1, 12))
        return VI_ERROR_PARAMETER4;
    if (rsesci_invalidViInt32Range (mode,0,1))
        return VI_ERROR_PARAMETER3;

    sprintf (buffer, "CALC%%ld:LIM:ACP:%s:%s:STAT?",
             acpLimitsArr[limitType],
             absRelArr[mode]);

    CHECKERR( rsesci_getViBooleanWindow (instrSession,
                                         buffer,
                                         window,
                                         state,
                                         VI_ERROR_PARAMETER5));
    if (state)
    {
        sprintf (cmd, "CALC%ld:LIM:ACP:%s:%s?",
                 window,
                 acpLimitsArr[limitType],
                 absRelArr[mode]);

        rsesci_status = rsesci_getViString (instrSession,
                                            cmd,
                                            buffer,
                                            BUFFER_SIZE,
                                            VI_ERROR_PARAMETER6);

        sscanf (buffer, "%le", limitValue);
    }

    return rsesci_status;
}

/*****************************************************************************
 * Function: Get ACP Limit Checking Result
 * Purpose:  This function queries the result of the limit check for selected
 *           limit.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetACPLimitCheckingResult(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32     limitType,
    ViBoolean*  result
)
{
    ViStatus    rsesci_status;
    ViChar      buffer[BUFFER_SIZE];
    ViChar      cmd[BUFFER_SIZE];

    if (rsesci_invalidViInt32Range (limitType, 1, 12))
        return VI_ERROR_PARAMETER3;

    sprintf (cmd, "CALC%ld:LIM:ACP:%s:RES?", window, acpLimitsArr[limitType]);

    rsesci_status = rsesci_getViString (instrSession,
                                        cmd,
                                        buffer,
                                        BUFFER_SIZE,
                                        VI_ERROR_PARAMETER4);

    *result = ((strncmp (buffer, "PASSED", 6)) == 0) ? VI_TRUE : VI_FALSE;

    return rsesci_status;
}

/*****************************************************************************
 * Function: Set Power Bandwidth Percentage
 * Purpose:  This function defines the percentage of the power with respect to
 *           the total power. This value is the basis for the occupied
 *           bandwidth measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetPowerBandwidthPercentage(
    ViSession   instrSession,
    ViInt32     window,
    ViReal64    percentage
)
{
    return rsesci_setViReal64Window (instrSession,
                                     "SENS%ld:POW:BAND",
                                     window,
                                     percentage,
                                     10.0 , 99.9 ,
                                     VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Power Bandwidth Percentage
 * Purpose:  This function returns the percentage of the power with respect to
 *           the total power. This value is the basis for the occupied
 *           bandwidth measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetPowerBandwidthPercentage(
    ViSession   instrSession,
    ViInt32     window,
    ViReal64*   percentage
)
{
    return rsesci_getViReal64Window (instrSession,
                                     "SENS%ld:POW:BAND?",
                                     window,
                                     percentage,
                                     VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set Channel Bandwidth
 * Purpose:  This function sets the channel bandwidth of the radio
 *           communication system. The bandwidths of adjacent channels are not
 *           influenced by this modification (in contrast to the FSE family).
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetChannelBandwidth(
    ViSession   instrSession,
    ViInt32     window,
    ViReal64    channelBandwidth
)
{
    return rsesci_setViReal64Window (instrSession,
                                     "SENS%ld:POW:ACH:BAND",
                                     window,
                                     channelBandwidth,
                                     100.0 , 1.0E+9 ,
                                     VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Channel Bandwidth
 * Purpose:  This function queries the channel bandwidth of the radio
 *           communication system.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetChannelBandwidth(
    ViSession   instrSession,
    ViInt32     window,
    ViReal64*   channelBandwidth
)
{
    return rsesci_getViReal64Window (instrSession,
                                     "SENS%ld:POW:ACH:BAND?",
                                     window,
                                     channelBandwidth,
                                     VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set Signal Amplitude Statistics
 * Purpose:  This function selects the statistical measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetSignalAmplitudeStatistics(
    ViSession   instrSession,
    ViInt32 measurement
)
{
    ViChar  buffer[BUFFER_SIZE];

    if (rsesci_invalidViInt32Range (measurement, 0, 2))
        return VI_ERROR_PARAMETER2;

    if (measurement)
        sprintf (buffer, "CALC:STAT:%s ON",statMeasArr[measurement]);
    else
        sprintf (buffer, "CALC:STAT:APD OFF;:CALC:STAT:CCDF OFF");

    return rsesci_sendCmd (instrSession, buffer);
}

/*****************************************************************************
 * Function: Get Signal Amplitude Statistics
 * Purpose:  This function queries selected statistical measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetSignalAmplitudeStatistics(
    ViSession   instrSession,
    ViInt32*    measurement
)
{
    ViStatus    rsesci_status;
    ViBoolean   state_apd, state_ccdf;

    CHECKERR (rsesci_getViBoolean (instrSession,
                                   "CALC:STAT:APD?",
                                   &state_apd,
                                   VI_ERROR_PARAMETER1));
    CHECKERR (rsesci_getViBoolean (instrSession,
                                   "CALC:STAT:CCDF?",
                                   &state_ccdf,
                                   VI_ERROR_PARAMETER1));
    if (state_apd || state_ccdf)
    {
        if (state_apd)
            *measurement = RSESCI_STAT_APD;
        else
            *measurement = RSESCI_STAT_CCDF;
    }
    else
        *measurement = RSESCI_STAT_AOFF;

    return rsesci_status;
}

/*****************************************************************************
 * Function: Set CCDF Percent Marker
 * Purpose:  This function positions the selected marker in the selected
 *           window to the given probability. If marker 2, 3 or 4 is selected
 *           and used as a delta marker, it is switched to marker mode.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetCCDFPercentMarker(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32 marker,
    ViReal64    position
)
{
    ViChar  buffer[BUFFER_SIZE];

    if (rsesci_invalidViInt32Range (marker, 1, 4))
        return VI_ERROR_PARAMETER3;

    sprintf (buffer, "CALC%%ld:MARK%ld:Y:PERC", marker);

    return rsesci_setViReal64Window (instrSession,
                                     buffer,
                                     window,
                                     position,
                                     0.0, 100.0,
                                     VI_ERROR_PARAMETER4);
}

/*****************************************************************************
 * Function: Get CCDF Percent Marker
 * Purpose:  This function queries the position of selected marker in the
 *           selected window.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetCCDFPercentMarker(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32 marker,
    ViReal64*   position
)
{
    ViChar  buffer[BUFFER_SIZE];

    if (rsesci_invalidViInt32Range (marker, 1, 4))
        return VI_ERROR_PARAMETER3;

    sprintf (buffer, "CALC%%ld:MARK%ld:Y:PERC?", marker);

    return rsesci_getViReal64Window (instrSession,
                                     buffer,
                                     window,
                                     position,
                                     VI_ERROR_PARAMETER4);
}

/*****************************************************************************
 * Function: Set Statistics Number of Samples
 * Purpose:  This function sets the number of measurement points to be
 *           acquired for the statistical measurement functions.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetStatisticsNumberSamples(
    ViSession   instrSession,
    ViReal64    numberOfSamples
)
{
    return rsesci_setViReal64 (instrSession,
                               "CALC:STAT:NSAM",
                               numberOfSamples,
                               100.0 , 1.0E9 ,
                               VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Statistics Number of Samples
 * Purpose:  This function queries the number of measurement points to be
 *           acquired for the statistical measurement functions.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetStatisticsNumberSamples(
    ViSession   instrSession,
    ViReal64*   numberOfSamples
)
{
    return rsesci_getViReal64 (instrSession,
                               "CALC:STAT:NSAM?",
                               numberOfSamples,
                               VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Statistics Adjust Settings
 * Purpose:  This function optimizes the level setting of the instrument
 *           depending on the measured peak power, in order to obtain maximum
 *           instrument sensitivity. To obtain maximum resolution, the level
 *           range is set as a function of the measured spacing between peak
 *           power and the minimum power for the APD measurement and of the
 *           spacing between peak power and mean power for the CCDF
 *           measurement. In addition, the probability scale for the number of
 *           test points is adapted.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_StatisticsAdjustSettings(
    ViSession   instrSession
)
{
    return rsesci_sendCmd (instrSession,
                           "CALC:STAT:SCAL:AUTO ONCE;*WAI");
}

/*****************************************************************************
 * Function: Statistics Default Settings
 * Purpose:  This function resets the scaling of the X and Y axes in a
 *           statistical measurement. The following values are set:
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_StatisticsDefaultSettings(
    ViSession   instrSession
)
{
    return rsesci_sendCmd (instrSession,
                           "CALC:STAT:PRES");
}

/*****************************************************************************
 * Function: Get Statistical Measurements Results
 * Purpose:  This function reads out the results of statistical measurements
 *           of a recorded trace.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetStatisticalMeasurementsResults(
    ViSession   instrSession,
    ViInt32     trace,
    ViInt32     resultType,
    ViReal64    _VI_FAR result[]
)
{
    ViStatus    rsesci_status;
    ViChar      buffer[BUFFER_SIZE];

    if (rsesci_invalidViInt32Range (trace, 1, 3))
        return VI_ERROR_PARAMETER2;

    if (rsesci_invalidViInt32Range (resultType, 0, 3))
        return VI_ERROR_PARAMETER3;

    sprintf (buffer, "CALC:STAT:RES%ld? %s", trace, statResultArr[resultType]);

    if (resultType == RSESCI_STAT_RESULT_ALL)
    {
        rsesci_status = rsesci_getViReal64Array (instrSession,
                                                 buffer,
                                                 result,
                                                 3,
                                                 VI_ERROR_PARAMETER4);
    }
    else
    {
        rsesci_status = rsesci_getViReal64 (instrSession,
                                            buffer,
                                            result,
                                            VI_ERROR_PARAMETER4);
    }

    return rsesci_status;
}

/*****************************************************************************
 * Function: Query Signal Statistic CCDF Level
 * Purpose:  This function reads out the level values for the probabilities 
 *           0.1%, 1% and 10%.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_QuerySignalStatisticCCDFLevel (ViSession instrSession,
                                                        ViInt32 trace,
                                                        ViInt32 probability,
                                                        ViReal64 *result)
{
	ViChar	buffer[BUFFER_SIZE] = "";
	
	if (rsesci_invalidViInt32Range (trace, 1, 3) == VI_TRUE)
        return VI_ERROR_PARAMETER2;
    if (rsesci_invalidViInt32Range (probability, 0, 3) == VI_TRUE)
        return VI_ERROR_PARAMETER3; 
    if (result == NULL)
        return VI_ERROR_PARAMETER4; 
	
	sprintf (buffer, "CALC:STAT:CCDF:X%ld? %s", trace, statProbArr[probability]);	
    
    return rsesci_getViReal64 (instrSession, 
		                       buffer, 
							   result,
							   VI_ERROR_PARAMETER4);
        
}

/*****************************************************************************
 * Function: Set Statistics X-Axis Reference Level
 * Purpose:  This function defines the reference level for the X axis of the
 *           measurement diagram. With the reference level offset <> 0 the
 *           indicated value range of the reference level is modified by the
 *           offset. The unit depends on the setting performed with
 *           rsesci_SetLevelAxisUnit.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetStatisticsXAxisReferenceLevel(
    ViSession   instrSession,
    ViReal64    referenceLevel
)
{
    return rsesci_setViReal64 (instrSession,
                               "CALC:STAT:SCAL:X:RLEV",
                               referenceLevel,
                               -130 , 30 ,
                               VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Statistics X-Axis Reference Level
 * Purpose:  This function queries the reference level for the X axis of the
 *           measurement diagram.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetStatisticsXAxisReferenceLevel(
    ViSession   instrSession,
    ViReal64*   referenceLevel
)
{
    return rsesci_getViReal64 (instrSession,
                               "CALC:STAT:SCAL:X:RLEV?",
                               referenceLevel,
                               VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Statistics X-Axis Range
 * Purpose:  This function defines the level range for the X axis of the
 *           measurement diagram.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetStatisticsXAxisRange(
    ViSession   instrSession,
    ViReal64    range
)
{
    return rsesci_setViReal64 (instrSession,
                               "CALC:STAT:SCAL:X:RANG",
                               range,
                               10.0 , 200.0 ,
                               VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Statistics X-Axis Range
 * Purpose:  This function queries the level range for the X axis of the
 *           measurement diagram.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetStatisticsXAxisRange(
    ViSession   instrSession,
    ViReal64*   range
)
{
    return rsesci_getViReal64 (instrSession,
                               "CALC:STAT:SCAL:X:RANG?",
                               range,
                               VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Statistics Y-Axis Min Value
 * Purpose:  This function defines the lower limit for th Y axis of the
 *           diagram in statistical measurements.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetStatisticsYAxisMinValue(
    ViSession   instrSession,
    ViReal64    minValue
)
{
    return rsesci_setViReal64 (instrSession,
                               "CALC:STAT:SCAL:Y:LOW",
                               minValue,
                               1E-9 , 0.1 ,
                               VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Statistics Y-Axis Min Value
 * Purpose:  This function queries the lower limit for th Y axis of the
 *           diagram in statistical measurements.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetStatisticsYAxisMinValue(
    ViSession   instrSession,
    ViReal64*   minValue
)
{
    return rsesci_getViReal64 (instrSession,
                               "CALC:STAT:SCAL:Y:LOW?",
                               minValue,
                               VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Statistics Y-Axis Max Value
 * Purpose:  This function defines the upper limit for th Y axis of the
 *           diagram in statistical measurements.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetStatisticsYAxisMaxValue(
    ViSession   instrSession,
    ViReal64    maxValue
)
{
    return rsesci_setViReal64 (instrSession,
                               "CALC:STAT:SCAL:Y:UPP",
                               maxValue,
                               1E-8 , 1.0 ,
                               VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Statistics Y-Axis Max Value
 * Purpose:  This function queries the upper limit for th Y axis of the
 *           diagram in statistical measurements.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetStatisticsYAxisMaxValue(
    ViSession   instrSession,
    ViReal64*   maxValue
)
{
    return rsesci_getViReal64 (instrSession,
                               "CALC:STAT:SCAL:Y:UPP?",
                               maxValue,
                               VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Statistics Y-Axis Unit
 * Purpose:  This function toggles the scaling of Y axis between percentage 
 *           and absolute.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetStatisticsYAxisUnit (ViSession instrSession,
                                                 ViInt32 units)
{
	return rsesci_setEnum (instrSession,
	                       "CALC:STAT:SCAL:Y:UNIT",
						   units,
						   statUnitArr,
						   VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Statistics Y-Axis Unit
 * Purpose:  This function returns the scaling unit of Y axis.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetStatisticsYAxisUnit (ViSession instrSession,
                                                 ViInt32 *units)
{
 	return rsesci_getEnum (instrSession,
	                       "CALC:STAT:SCAL:Y:UNIT?",
						   units,
						   statUnitArr,
						   VI_ERROR_PARAMETER2);   
}

/*****************************************************************************
 * Function: Set Modulation Depth Measurement
 * Purpose:  This function switches on the measurement of the AM modulation
 *           depth. An AM-modulated carrier is required on the screen for
 *           correct operation. If necessary, marker 1 is previously activated
 *           and set to the largest signal available. The level value of marker
 *           1 is regarded as the carrier level. On activating the function,
 *           marker 2 and marker 3 are automatically set as delta markers
 *           symetrically to the carrier to the adjacent maxima of the trace.
 *           If the position of delta marker 2 is changed, delta marker 3 is
 *           moved symetrically with respect to the reference marker (marker
 *           1). If the position of delta marker 3 is changed, fine adjustment
 *           can be performed independently of delta marker 2. The R&S ESCI
 *           calculates the power at the marker positions from the measured
 *           levels. The AM modulation depth is calculated from the ratio of
 *           power values at the reference marker and the delta markers. If the
 *           two AM sidebands differ in power, the average value of the two
 *           power values is used for calculating the AM modulation depth.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetModulationDepthMeasurement(
    ViSession   instrSession,
    ViInt32     window,
    ViBoolean   modulationDepthMeasurement
)
{
    return rsesci_setViBooleanWindow (instrSession,
                                      "CALC%ld:MARK:FUNC:MDEP",
                                      window,
                                      modulationDepthMeasurement,
                                      VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Modulation Depth Measurement
 * Purpose:  This function queries the state of the measurement of the AM
 *           modulation depth.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetModulationDepthMeasurement(
    ViSession   instrSession,
    ViInt32     window,
    ViBoolean*  modulationDepthMeasurement
)
{
    return rsesci_getViBooleanWindow (instrSession,
                                      "CALC%ld:MARK:FUNC:MDEP?",
                                      window,
                                      modulationDepthMeasurement,
                                      VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Modulation Depth Measurement Value
 * Purpose:  This function queries the AM modulation depth in the indicated
 *           measurement window. A complete sweep with synchronization to sweep
 *           end must be performed between switching on the function and
 *           querying the measured value to obtain a valid query result. This
 *           is only possible in single sweep mode.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetModulationDepthMeasurementValue(
    ViSession   instrSession,
    ViInt32     window,
    ViReal64*   result
)
{
    return rsesci_getViReal64Window (instrSession,
                                     "CALC%ld:MARK:FUNC:MDEP:RES?",
                                     window,
                                     result,
                                     VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set Third Order Intercept Function
 * Purpose:  This function controls the positioning of the intercept markers
             3 and 4 for TOI measurement, based on the two carrier signals.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetThirdOrderInterceptFunction (ViSession instrSession,
                                                         ViInt32 window,
                                                         ViInt32 TOIMarker)
{
   return rsesci_setEnumWindow (instrSession,
                                 "CALC%ld:MARK:FUNC:TOI:MARK",
                                 window,
                                 TOIMarker,
                                 TOIMarkerArr,
                                 VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Third Order Intercept Function
 * Purpose:  This function queries the positioning of the intercept markers 3
             and 4 for TOI measurement, based on the two carrier signals.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetThirdOrderInterceptFunction (ViSession instrSession,
                                                         ViInt32 window,
                                                         ViInt32 *TOIMarker)
{
   return rsesci_getEnumWindow (instrSession,
                                 "CALC%ld:MARK:FUNC:TOI:MARK?",
                                 window,
                                 TOIMarker,
								 TOIMarkerArr,
                                 VI_ERROR_PARAMETER3); 
}

/*****************************************************************************
 * Function: Set Third Order Intercept Measurement
 * Purpose:  This function initiates the measurement of the third-order
 *           intercept point. A two-tone signal with equal carrier levels is
 *           expected at the RF input of the instrument. Marker 1 and marker 2
 *           (both normal markers) are set to the maximum of the two signals.
 *           Delta marker 3 and delta marker 4 are positioned to the
 *           intermodulation products. The third-order intercept is calculated
 *           from the level spacing between the normal markers and the delta
 *           markers.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetThirdOrderInterceptMeasurement(
    ViSession   instrSession,
    ViInt32     window,
    ViBoolean   state
)
{
    return rsesci_setViBooleanWindow (instrSession,
                                      "CALC%ld:MARK:FUNC:TOI",
                                      window,
                                      state,
                                      VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Third Order Intercept Measurement
 * Purpose:  This function queries the state of the third-order intercept
 *           point measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetThirdOrderInterceptMeasurement(
    ViSession   instrSession,
    ViInt32     window,
    ViBoolean*  state
)
{
    return rsesci_getViBooleanWindow (instrSession,
                                      "CALC%ld:MARK:FUNC:TOI?",
                                      window,
                                      state,
                                      VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Third Order Intercept Measurement Value
 * Purpose:  This function queries the third-order intercept point measurement
 *           in the indicated measurement window.
 *           A complete sweep with synchronization to sweep end must be
 *           performed between switching on the function and querying the
 *           measured value to obtain a valid query result. This is only
 *           possible in single sweep mode.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetThirdOrderInterceptMeasurementValue(
    ViSession   instrSession,
    ViInt32     window,
    ViReal64*   toiResult
)
{
    return rsesci_getViReal64Window (instrSession,
                                     "CALC%ld:MARK:FUNC:TOI:RES?",
                                     window,
                                     toiResult,
                                     VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set Multiple Power Measurement Parameters
 * Purpose:  This function defines the constant settings for the list during
 *           multiple power measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetMultiplePowerMeasurementParameters(
    ViSession   instrSession,
    ViInt32     window,
    ViBoolean   peakDetector,
    ViBoolean   rmsDetector,
    ViBoolean   averageDetector,
    ViInt32     triggerMode,
    ViInt32     triggerSlope,
    ViReal64    triggerOffset,
    ViReal64    gateLength
)
{
    ViChar  buffer[BUFFER_SIZE];

    if (rsesci_invalidViBooleanRange (peakDetector))
        return VI_ERROR_PARAMETER2;
    if (rsesci_invalidViBooleanRange (rmsDetector))
        return VI_ERROR_PARAMETER3;
    if (rsesci_invalidViBooleanRange (averageDetector))
        return VI_ERROR_PARAMETER4;
    if (rsesci_invalidViInt32Range (triggerMode, 0, 3))
        return VI_ERROR_PARAMETER5;
    if (rsesci_invalidViInt32Range (triggerSlope, 0, 1))
        return VI_ERROR_PARAMETER6;
    if ((triggerOffset != 0.0) && rsesci_invalidViReal64Range (triggerOffset, 125.0e-9,100.0))
        return VI_ERROR_PARAMETER7;
    if ((gateLength != 0.0) && rsesci_invalidViReal64Range (gateLength, 125.0e-9,100.0))
        return VI_ERROR_PARAMETER8;

    sprintf (buffer,"SENS%%ld:LIST:POW:SET %s,%s,%s,%s,%s,%.12Lf,%.12Lf", switchArr[peakDetector],
                                                                    switchArr[rmsDetector],
                                                                    switchArr[averageDetector],
                                                                    triggersourceArr[triggerMode],
                                                                    polarityArr[triggerSlope],
                                                                    triggerOffset,
                                                                    gateLength);
    return rsesci_sendCmdWindow (instrSession, buffer, window);
}

/*****************************************************************************
 * Function: Get Multiple Power Measurement Parameters
 * Purpose:  This function returns defined settings of the multiple power
 *           measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetMultiplePowerMeasurementParameters(
    ViSession   instrSession,
    ViInt32     window,
    ViBoolean*  peakDetector,
    ViBoolean*  rmsDetector,
    ViBoolean*  averageDetector,
    ViInt32*    triggerMode,
    ViInt32*    triggerSlope,
    ViReal64*   triggerOffset,
    ViReal64*   gateLength
)
{
    ViStatus    rsesci_status;
    ViChar      buffer[BUFFER_SIZE];

    ViChar      peakDetectorTmp[32],
                rmsDetectorTmp[32],
                averageDetectorTmp[32],
                triggerModeTmp[32],
                triggerSlopeTmp[32];

    peakDetectorTmp[0]      = '\0';
    rmsDetectorTmp[0]       = '\0';
    averageDetectorTmp[0]   = '\0';
    triggerModeTmp[0]       = '\0';
    triggerSlopeTmp[0]      = '\0';

    if (!peakDetector)
        return VI_ERROR_PARAMETER2;
    if (!rmsDetector)
        return VI_ERROR_PARAMETER3;
    if (!averageDetector)
        return VI_ERROR_PARAMETER4;
    if (!triggerMode)
        return VI_ERROR_PARAMETER5;
    if (!triggerSlope)
        return VI_ERROR_PARAMETER6;
    if (!triggerOffset)
        return VI_ERROR_PARAMETER7;
    if (!gateLength)
        return VI_ERROR_PARAMETER8;

    CHECKERR (rsesci_getViStringWindow (instrSession,
                                        "SENS%ld:LIST:POW:SET?",
                                        window,
                                        buffer,
                                        BUFFER_SIZE,
                                        VI_ERROR_PARAMETER1))

    sscanf (buffer, "%[^,],%[^,],%[^,],%[^,],%[^,],%le,%le", peakDetectorTmp,
                                                             rmsDetectorTmp,
                                                             averageDetectorTmp,
                                                             triggerModeTmp,
                                                             triggerSlopeTmp,
                                                             triggerOffset,
                                                             gateLength);

    *peakDetector = ((strncmp (peakDetectorTmp, "ON", 2)) == 0) ? VI_TRUE : VI_FALSE;
    *rmsDetector = ((strncmp (rmsDetectorTmp, "ON", 2)) == 0) ? VI_TRUE : VI_FALSE;
    *averageDetector = ((strncmp (averageDetectorTmp, "ON", 2)) == 0) ? VI_TRUE : VI_FALSE;

    CHECKERR (rsesci_getIndexFromString (instrSession,
                                         triggerModeTmp,
                                         triggersourceArr,
                                         triggerMode));
    CHECKERR (rsesci_getIndexFromString (instrSession,
                                         triggerSlopeTmp,
                                         polarityArr,
                                         triggerSlope));
    return rsesci_status;
}

/*****************************************************************************
 * Function: Multiple Power Measurement Sequence
 * Purpose:  This function configures the list of settings (max. 100 entries)
 *           for the multiple power measurement and starts a measurement
 *           sequence. When synchronizing the command with *OPC, a service
 *           request is generated as soon as all frequency points are processed
 *           and the defined number of individual measurements is reached.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_MultiplePowerMeasurementSequence(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32     numberofEntries,
    ViReal64    _VI_FAR analyzerFrequency[],
    ViReal64    _VI_FAR referenceLevel[],
    ViReal64    _VI_FAR rfInputAttenuation[],
    ViInt32     _VI_FAR filterType[],
    ViReal64    _VI_FAR resolutionBandwidth[],
    ViReal64    _VI_FAR videoBandwidth[],
    ViReal64    _VI_FAR measurementTime[],
    ViReal64    _VI_FAR triggerLevel[]
)
{
    ViStatus    rsesci_status;
    ViInt32     i;
    ViChar      *pbuffer;
    ViChar      *p2buf;

    pbuffer = (ViChar*) malloc (150 * numberofEntries + 30);

    p2buf = pbuffer + sprintf (pbuffer, "SENS%%ld:LIST:POW ");

    for (i = 0; i < numberofEntries; i++)
    {
        p2buf += sprintf (p2buf, "%.12lf,%lf,%lf,OFF,%s,%.12lf,%.12lf,%.12lf,%lf,", analyzerFrequency[i],
                                                                        referenceLevel[i],
                                                                        rfInputAttenuation[i],
                                                                        filterTypeArr[filterType[i]],
                                                                        resolutionBandwidth[i],
                                                                        videoBandwidth[i],
                                                                        measurementTime[i],
                                                                        triggerLevel[i]);
    }
    *--p2buf = '\0';
    rsesci_status = rsesci_sendCmdWindow (instrSession, pbuffer, window);

    free (pbuffer);

    return rsesci_status;
}

/*****************************************************************************
 * Function: Multiple Power Measurement Results
 * Purpose:  This function queries the result of a previous list measurement
 *           as configured and initiated with SENSe:LIST:POWer[:SEQuence].
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_MultiplePowerMeasurementResults(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32*    numberofResults,
    ViReal64    _VI_FAR powerMeasurementResults[]
)
{
    ViStatus    rsesci_status;
    ViInt32     i = 0;
    ViChar      buffer[BUFFER_SIZE];
    ViChar      *p2buf;

    CHECKERR (rsesci_getViStringWindow (instrSession,
                                        "SENS%ld:LIST:POW:RES?",
                                        window,
                                        buffer,
                                        BUFFER_SIZE,
                                        VI_ERROR_PARAMETER1));

    p2buf = strtok (buffer, ",");

    while (p2buf && (i < 100)) /* Max 100 elements */
    {
        powerMeasurementResults[i] = atof (p2buf);
        i++;
        p2buf = strtok (VI_NULL, ",");
    }

    *numberofResults = i;

    return rsesci_status;
}

/*****************************************************************************
 * Function: Set Multiple Power Measurement Averaging
 * Purpose:  This function sets the the average type of the sense list 
 *           measurement either to linear (default) or logarithmic. In linear 
 *           mode voltages are averaged in logarithmic mode levels.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetMultiplePowerMeasurementAveraging (ViSession instrSession,
                                                               ViInt32 window,
                                                               ViInt32 averaging)
{
	return rsesci_setEnumWindow (instrSession,
                                 "SENS%ld:LIST:POW:SET:AVER:TYPE",
                                  window,
                                  averaging,
								  modeArr,
                                  VI_ERROR_PARAMETER3);    
}

/*****************************************************************************
 * Function: Get Multiple Power Measurement Averaging
 * Purpose:  This function gets the the average type of the sense list 
 *           measurement either to linear (default) or logarithmic. In linear 
 *           mode voltages are averaged in logarithmic mode levels.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetMultiplePowerMeasurementAveraging (ViSession instrSession,
                                                               ViInt32 window,
                                                               ViInt32 *averaging)
{
	return rsesci_getEnumWindow (instrSession,
                                 "SENS%ld:LIST:POW:SET:AVER:TYPE?",
                                  window,
                                  averaging,
								  modeArr,
                                  VI_ERROR_PARAMETER3);    
}

/*****************************************************************************
 * Function: Deactivate Multiple Power Measurement
 * Purpose:  This function deactivates the multiple power (list) measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_DeactivateMultiplePowerMeasurement(
    ViSession   instrSession,
    ViInt32     window
)
{
    return rsesci_sendCmdWindow (instrSession, "SENS%ld:LIST:POW:STAT OFF", window);
}

/*****************************************************************************
 * Function: Multiple Burst Power Measurement Sequence
 * Purpose:  This function configures the instrument setup for multiple burst
 *           power measurement and starts a measurement sequence. When
 *           synchronizing this function with *OPC, a service request is
 *           generated as soon as the defined number of individual measurements
 *           (# of meas) is reached.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_MultipleBurstPowerMeasurementSequence(
    ViSession   instrSession,
    ViInt32     window,
    ViReal64    analyzerFrequency,
    ViReal64    resolutionBandwidth,
    ViReal64    measurementTime,
    ViInt32     triggerSource,
    ViReal64    triggerLevel,
    ViReal64    triggerOffset,
    ViInt32     measurementType,
    ViInt32     noofMeasurements
)
{
    ViChar      buffer[BUFFER_SIZE];
    ViChar      tmpBuffer[32];

    if (rsesci_invalidViReal64Range (resolutionBandwidth, 10.0, 10.0e6))
        return VI_ERROR_PARAMETER4;
    if (rsesci_invalidViReal64Range (measurementTime, 1.0e-6, 30.0))
        return VI_ERROR_PARAMETER5;
    if (rsesci_invalidViInt32Range (triggerSource, 1, 2))
        return VI_ERROR_PARAMETER6;
    if (rsesci_invalidViReal64Range (triggerLevel, 0.0, 100.0))
        return VI_ERROR_PARAMETER7;
    if (rsesci_invalidViReal64Range (triggerOffset, 125.0e-9,100.0))
        return VI_ERROR_PARAMETER8;
    if (rsesci_invalidViInt32Range (measurementType, 2, 3))
        return VI_ERROR_INSTR_PARAMETER9;
    if (rsesci_invalidViInt32Range (noofMeasurements, 1, 501))
        return VI_ERROR_INSTR_PARAMETER10;

    if (measurementType == RSESCI_POWER_MEAS_PEAK)
        {
        sprintf (tmpBuffer, "PEAK");
        }
    else /* RSESCI_POWER_MEAS_RMS */
        {
        sprintf (tmpBuffer, "MEAN");
        }

    sprintf (buffer, "SENS%%ld:MPOW %.12lf,%.12lf,%le,%s,%le,%.12lf,%s,%ld", analyzerFrequency,
                                                                    resolutionBandwidth,
                                                                    measurementTime,
                                                                    triggersourceArr[triggerSource],
                                                                    triggerLevel,
                                                                    triggerOffset,
                                                                    tmpBuffer,
                                                                    measurementType);

    return rsesci_sendCmdWindow (instrSession, buffer, window);
}

/*****************************************************************************
 * Function: Multiple Burst Power Measurement Result
 * Purpose:  This function queries the results of a multiple burst power
 *           measurement as configured and initiated with
 *           SENSe:MPOWer[:SEQuence].
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_MultipleBurstPowerMeasurementResult(
    ViSession   instrSession,
    ViInt32     window,
    ViReal64    _VI_FAR powerMeasurementResults[],
    ViInt32*    numberofResults
)
{
    ViStatus    rsesci_status;
    ViInt32     i = 0;
    ViChar      buffer[BUFFER_SIZE];
    ViChar      *p2buf;

    CHECKERR (rsesci_getViStringWindow (instrSession,
                                        "SENS%ld:MPOW:RES?",
                                        window,
                                        buffer,
                                        BUFFER_SIZE,
                                        VI_ERROR_PARAMETER1));

    p2buf = strtok (buffer, ",");

    while (p2buf && (i < 501)) /* Max elements 501 */
    {
        powerMeasurementResults[i] = atof (p2buf);
        i++;
        p2buf = strtok (VI_NULL, ",");
    }

    *numberofResults = i;

    return rsesci_status;
}

/*****************************************************************************
 * Function: Multiple Burst Power Measurement Result Minimum
 * Purpose:  This function queries the minimum power value in a multiple burst
 *           power measurement as configured and initiated with
 *           SENSe:MPOWer[:SEQuence].
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_MultipleBurstPowerMeasurementResultMinimum(
    ViSession   instrSession,
    ViInt32     window,
    ViReal64*   powerMeasurementResult
)
{
    return rsesci_getViReal64Window (instrSession,
                                     "SENS%ld:MPOW:RES:MIN?",
                                     window,
                                     powerMeasurementResult,
                                     VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set YIG Correction State
 * Purpose:  This function switches on or off the automatic correction of the 
 *           YIG filter frequency drift.
 *           When correction is switched on, it is checked once per minute
 *           whether the temperature on the YIG filter has changed by more
 *           than 5K relative to the last instance of correction. If this is
 *           the case, the frequency of the YIG filter is  at the end of the
 *           next measurement  adjusted as required for the new temperature.
 *           For time-critical applications, the correction function can be
 *           switched off after an operating period of >= 30 minutes.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetYIGCorrectionState (ViSession instrSession,
                                                ViInt32 window, ViBoolean state)
{
	return rsesci_setViBooleanWindow (instrSession,
                                      "SENS%ld:CORR:YIG:TEMP:AUTO",
                                      window,
                                      state,
                                      VI_ERROR_PARAMETER3);    
}

/*****************************************************************************
 * Function: Get YIG Correction State
 * Purpose:  This function returns the state of the automatic correction of the 
 *           YIG filter frequency drift.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetYIGCorrectionState (ViSession instrSession,
                                                ViInt32 window, ViBoolean *state)
{
	return rsesci_getViBooleanWindow (instrSession,
                                      "SENS%ld:CORR:YIG:TEMP:AUTO?",
                                      window,
                                      state,
                                      VI_ERROR_PARAMETER3);    
}

/*****************************************************************************
 * Function: Run Scan
 * Purpose:  This function initiates a new sweep.
 *           In receiver mode with 'Single' selected, the R&S ESCI performs a
 *           single scan and stops at the end frequency. With 'Continuous'
 *           selected, the scan is performed continuously until it is
 *           deliberately stopped.
 *           In analyzer mode, with Sweep Count > 0 or Average Count > 0, this
 *           means a restart of the indicated number of measurements. With
 *           trace functions 'Max Hold', 'Min Hold' and 'Average', the previous
 *           results are reset on restarting the measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_RunScan(
    ViSession   instrSession
)
{
    return rsesci_sendCmd (instrSession,
                           "INIT2");
}

/*****************************************************************************
 * Function: Hold Scan
 * Purpose:  This function interrupts a current scan measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_HoldScan(
    ViSession   instrSession
)
{
    return rsesci_sendCmd (instrSession,
                           "HOLD");
}

/*****************************************************************************
 * Function: Continue Scan
 * Purpose:  When a scan is automatically interrupted by a transducer
 *           interrupt point, this function continues the measurement at the
 *           current receiver frequency. The function is useful especially for
 *           trace functions MAXHold, MINHold and AVERage if the previous
 *           results are not to be cleared with Sweep Count > 0 or Average
 *           Count > 0 on restarting the measurement (INIT:IMMediate resets the
 *           previous results on restarting the measurement). The single-sweep
 *           mode is automatically switched on. In the continuous-sweep mode,
 *           synchronization to the sweep end is not possible since the overall
 *           measurement "never" ends.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_ContinueScan(
    ViSession   instrSession
)
{
    return rsesci_sendCmd (instrSession,
                           "INIT2:CONM");
}

/*****************************************************************************
 * Function: Stop Scan
 * Purpose:  This function aborts a current measurement and resets the trigger
 *           system.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_StopScan(
    ViSession   instrSession
)
{
    return rsesci_sendCmd (instrSession,
                           "ABOR");
}

/*****************************************************************************
 * Function: Set Scan Mode
 * Purpose:  This function determines whether the trigger system is
 *           continuously initiated (continuous) or performs single
 *           measurements (single).
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetScanMode(
    ViSession   instrSession,
    ViInt32 scanMode
)
{
    return rsesci_setViBoolean (instrSession,
                                "INIT2:CONT",
                                (ViBoolean) scanMode,
                                VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Scan Mode
 * Purpose:  This function queries whether the trigger system is continuously
 *           initiated (continuous) or performs single measurements (single).
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetScanMode(
    ViSession   instrSession,
    ViInt32*    scanMode
)
{
    ViStatus    rsesci_status;
    ViBoolean   state;

    if (!scanMode)
        return VI_ERROR_PARAMETER2;

    CHECKERR (rsesci_getViBoolean (instrSession,
                                  "INIT2:CONT?",
                                   &state,
                                   VI_ERROR_PARAMETER2));
    *scanMode = state;

    return rsesci_status;
}

/*****************************************************************************
 * Function: Set Scan Points
 * Purpose:  This function defines the maximum number of measurement points
 *           which are transferred in one block after the query command
 *           TRACE? SCAN.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetScanPoints (ViSession instrSession,
                                        ViInt32 number_ofPoints)
{
    return rsesci_setViInt32 (instrSession
                            , "TRAC:POIN LIM,"
                            , number_ofPoints
                            , 1, 10000
                            , VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Scan Start
 * Purpose:  This function defines the start frequency of the selected
 *           receiver scan range.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetScanStart(
    ViSession   instrSession,
    ViInt32     scanRange,
    ViReal64    startFrequency
)
{
    ViChar  buffer[BUFFER_SIZE];

    if (rsesci_invalidViInt32Range (scanRange, 1, 10))
        return VI_ERROR_PARAMETER2;

    sprintf (buffer, "SENS:SCAN%ld:STAR", scanRange);

    return rsesci_setViReal64 (instrSession,
                               buffer,
                               startFrequency,
                               0.0 , 0.0,
                               VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Scan Start
 * Purpose:  This function queries the start frequency of the selected
 *           receiver scan range.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetScanStart(
    ViSession   instrSession,
    ViInt32     scanRange,
    ViReal64*   startFrequency
)
{
    ViChar  buffer[BUFFER_SIZE];

    if (rsesci_invalidViInt32Range (scanRange, 1, 10))
        return VI_ERROR_PARAMETER2;

    sprintf (buffer, "SENS:SCAN%ld:STAR?", scanRange);

    return rsesci_getViReal64 (instrSession,
                               buffer,
                               startFrequency,
                               VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set Scan Stop
 * Purpose:  This function defines the stop frequency of the selected receiver
 *           scan range.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetScanStop(
    ViSession   instrSession,
    ViInt32     scanRange,
    ViReal64    stopFrequency
)
{
    ViChar  buffer[BUFFER_SIZE];

    if (rsesci_invalidViInt32Range (scanRange, 1, 10))
        return VI_ERROR_PARAMETER2;

    sprintf (buffer, "SENS:SCAN%ld:STOP", scanRange);

    return rsesci_setViReal64 (instrSession,
                               buffer,
                               stopFrequency,
                               0.0 , 0.0,
                               VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Scan Stop
 * Purpose:  This function queries the stop frequency of the selected receiver
 *           scan range.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetScanStop(
    ViSession   instrSession,
    ViInt32     scanRange,
    ViReal64*   stopFrequency
)
{
    ViChar  buffer[BUFFER_SIZE];

    if (rsesci_invalidViInt32Range (scanRange, 1, 10))
        return VI_ERROR_PARAMETER2;

    sprintf (buffer, "SENS:SCAN%ld:STOP?", scanRange);

    return rsesci_getViReal64 (instrSession,
                               buffer,
                               stopFrequency,
                               VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set Scan Step Size
 * Purpose:  This function defines the step size for the frequency of the
 *           selected receiver scan range.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetScanStepSize(
    ViSession   instrSession,
    ViInt32     scanRange,
    ViReal64    stepSize
)
{
    ViChar  buffer[BUFFER_SIZE];

    if (rsesci_invalidViInt32Range (scanRange, 1, 10))
        return VI_ERROR_PARAMETER2;

    sprintf (buffer, "SENS:SCAN%ld:STEP", scanRange);

    return rsesci_setViReal64 (instrSession,
                               buffer,
                               stepSize,
                               0.0 , 0.0,
                               VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Scan Step Size
 * Purpose:  This function queries the step size for the frequency of the
 *           selected receiver scan range.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetScanStepSize(
    ViSession   instrSession,
    ViInt32     scanRange,
    ViReal64*   stepSize
)
{
    ViChar  buffer[BUFFER_SIZE];

    if (rsesci_invalidViInt32Range (scanRange, 1, 10))
        return VI_ERROR_PARAMETER2;

    sprintf (buffer, "SENS:SCAN%ld:STEP?", scanRange);

    return rsesci_getViReal64 (instrSession,
                               buffer,
                               stepSize,
                               VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set Scan Bandwidth
 * Purpose:  This function defines the bandwidth for the selected receiver
 *           scan range.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetScanBandwidth(
    ViSession   instrSession,
    ViInt32     scanRange,
    ViReal64    bandwidth
)
{
    ViChar  buffer[BUFFER_SIZE];

    if (rsesci_invalidViInt32Range (scanRange, 1, 10))
        return VI_ERROR_PARAMETER2;

    sprintf (buffer, "SENS:SCAN%ld:BAND:RES", scanRange);

    return rsesci_setViReal64 (instrSession,
                               buffer,
                               bandwidth,
                               0.0, 0.0,
                               VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Scan Bandwidth
 * Purpose:  This function queries the bandwidth for the selected receiver
 *           scan range.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetScanBandwidth(
    ViSession   instrSession,
    ViInt32     scanRange,
    ViReal64*   bandwidth
)
{
    ViChar  buffer[BUFFER_SIZE];

    if (rsesci_invalidViInt32Range (scanRange, 1, 10))
        return VI_ERROR_PARAMETER2;

    sprintf (buffer, "SENS:SCAN%ld:BAND:RES?", scanRange);

    return rsesci_getViReal64 (instrSession,
                               buffer,
                               bandwidth,
                               VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set Scan Measurement Time
 * Purpose:  This function defines the measurement time of the receiver
 *           subscan.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetScanMeasurementTime(
    ViSession   instrSession,
    ViInt32     scanRange,
    ViReal64    measurementTime
)
{
    ViChar  buffer[BUFFER_SIZE];

    if (rsesci_invalidViInt32Range (scanRange, 1, 10))
        return VI_ERROR_PARAMETER2;

    sprintf (buffer, "SENS:SCAN%ld:TIME", scanRange);

    return rsesci_setViReal64 (instrSession,
                               buffer,
                               measurementTime,
                               100.0E-6 , 100.0,
                               VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Scan Measurement Time
 * Purpose:  This function queries the measurement time of the receiver
 *           subscan.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetScanMeasurementTime(
    ViSession   instrSession,
    ViInt32     scanRange,
    ViReal64*   measurementTime
)
{
    ViChar  buffer[BUFFER_SIZE];

    if (rsesci_invalidViInt32Range (scanRange, 1, 10))
        return VI_ERROR_PARAMETER2;

    sprintf (buffer, "SENS:SCAN%ld:TIME?", scanRange);

    return rsesci_getViReal64 (instrSession,
                               buffer,
                               measurementTime,
                               VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set Scan Auto Ranging
 * Purpose:  This function switches on or off the autoranging function in the
 *           selected receiver scan range.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetScanAutoRanging(
    ViSession   instrSession,
    ViInt32     scanRange,
    ViBoolean   autoRanging
)
{
    ViChar  buffer[BUFFER_SIZE];

    if (rsesci_invalidViInt32Range (scanRange, 1, 10))
        return VI_ERROR_PARAMETER2;

    sprintf (buffer, "SENS:SCAN%ld:INP:ATT:AUTO", scanRange);

    return rsesci_setViBoolean (instrSession,
                                buffer,
                                autoRanging,
                                VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Scan Auto Ranging
 * Purpose:  This function queries the state of the autoranging function in
 *           the selected receiver scan range.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetScanAutoRanging(
    ViSession   instrSession,
    ViInt32     scanRange,
    ViBoolean*  autoRanging
)
{
    ViChar  buffer[BUFFER_SIZE];

    if (rsesci_invalidViInt32Range (scanRange, 1, 10))
        return VI_ERROR_PARAMETER2;

    sprintf (buffer, "SENS:SCAN%ld:INP:ATT:AUTO?", scanRange);

    return rsesci_getViBoolean (instrSession,
                                buffer,
                                autoRanging,
                                VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set Scan RF Attenuation
 * Purpose:  This function defines the RF attenuation for the selected
 *           receiver scan range.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetScanRFAttenuation(
    ViSession   instrSession,
    ViInt32     scanRange,
    ViReal64    attenuation
)
{
    ViChar  buffer[BUFFER_SIZE];

    if (rsesci_invalidViInt32Range (scanRange, 1, 10))
        return VI_ERROR_PARAMETER2;

    sprintf (buffer, "SENS:SCAN%ld:INP:ATT", scanRange);

    return rsesci_setViReal64 (instrSession,
                               buffer,
                               attenuation,
                               0.0, 0.0,
                               VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Scan RF Attenuation
 * Purpose:  This function queries the RF attenuation for the selected
 *           receiver scan range.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetScanRFAttenuation(
    ViSession   instrSession,
    ViInt32     scanRange,
    ViReal64*   attenuation
)
{
    ViChar  buffer[BUFFER_SIZE];

    if (rsesci_invalidViInt32Range (scanRange, 1, 10))
        return VI_ERROR_PARAMETER2;

    sprintf (buffer, "SENS:SCAN%ld:INP:ATT?", scanRange);

    return rsesci_getViReal64 (instrSession,
                               buffer,
                               attenuation,
                               VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set Scan Preamplifier
 * Purpose:  This function switches on or off the preamplifier in the selected
 *           receiver scan range.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetScanPreamplifier(
    ViSession   instrSession,
    ViInt32     scanRange,
    ViBoolean   preamplifierState
)
{
    ViChar  buffer[BUFFER_SIZE];

    if (rsesci_invalidViInt32Range (scanRange, 1, 10))
        return VI_ERROR_PARAMETER2;

    sprintf (buffer, "SENS:SCAN%ld:INP:GAIN:STATE", scanRange);

    return rsesci_setViBoolean (instrSession,
                                buffer,
                                preamplifierState,
                                VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Scan Preamplifier
 * Purpose:  This function queries the state of the preamplifier in the
 *           selected receiver scan range.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetScanPreamplifier(
    ViSession   instrSession,
    ViInt32     scanRange,
    ViBoolean*  preamplifierState
)
{
    ViChar  buffer[BUFFER_SIZE];

    if (rsesci_invalidViInt32Range (scanRange, 1, 10))
        return VI_ERROR_PARAMETER2;

    sprintf (buffer, "SENS:SCAN%ld:INP:GAIN:STATE?", scanRange);

    return rsesci_getViBoolean (instrSession,
                                buffer,
                                preamplifierState,
                                VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set Scan Auto Preamplifier
 * Purpose:  This function enables or disables the preamplifier in the
 *           autoranging function of the selected receiver scan range.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetScanAutoPreamplifier(
    ViSession   instrSession,
    ViInt32     scanRange,
    ViBoolean   preamplifierInAutoranging
)
{
    ViChar  buffer[BUFFER_SIZE];

    if (rsesci_invalidViInt32Range (scanRange, 1, 10))
        return VI_ERROR_PARAMETER2;

    sprintf (buffer, "SENS:SCAN%ld:INP:GAIN:AUTO", scanRange);

    return rsesci_setViBoolean (instrSession,
                                buffer,
                                preamplifierInAutoranging,
                                VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Scan Auto Preamplifier
 * Purpose:  This function queries the state of the preamplifier in the
 *           autoranging function of the selected receiver scan range.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetScanAutoPreamplifier(
    ViSession   instrSession,
    ViInt32     scanRange,
    ViBoolean*  preamplifierInAutoranging
)
{
    ViChar  buffer[BUFFER_SIZE];

    if (rsesci_invalidViInt32Range (scanRange, 1, 10))
        return VI_ERROR_PARAMETER2;

    sprintf (buffer, "SENS:SCAN%ld:INP:GAIN:AUTO?", scanRange);

    return rsesci_getViBoolean (instrSession,
                                buffer,
                                preamplifierInAutoranging,
                                VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set Scan Ranges
 * Purpose:  This function determines the number of ranges. With setting 0,
 *           the scan is performed using the current receiver settings and not
 *           the settings defined with commands SENSe:SCAN:....
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetScanRanges(
    ViSession   instrSession,
    ViInt32     numberofranges
)
{
    return rsesci_setViInt32 (instrSession,
                                "SENS:SCAN:RANG:COUN",
                                numberofranges,
                                1 , 10 ,
                                VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Scan Ranges
 * Purpose:  This function queries the number of ranges.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetScanRanges(
    ViSession   instrSession,
    ViInt32*    numberofranges
)
{
    return rsesci_getViInt32 (instrSession,
                                "SENS:SCAN:RANG:COUN?",
                                numberofranges,
                                VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Scan Time Domain
 * Purpose:  This function sets the overall measurement time for time domain
 *           analysis.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetScanTimeDomain(
    ViSession   instrSession,
    ViReal64    measurementTime
)
{
    return rsesci_setViReal64 (instrSession,
                                    "SENS:SCAN:TDOM",
                                    measurementTime,
                                    100.0E-6 , 10000.0 ,
                                    VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Scan Time Domain
 * Purpose:  This function queries the overall measurement time for time
 *           domain analysis.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetScanTimeDomain(
    ViSession   instrSession,
    ViReal64*   measurementTime
)
{
    return rsesci_getViReal64 (instrSession,
                                    "SENS:SCAN:TDOM?",
                                    measurementTime,
                                    VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Scan Display Spacing
 * Purpose:  This function selects linear or logarithmic display in the
 *           selected measurement window.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetScanDisplaySpacing(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32     frequencyAxisRangeSpacing
)
{
    return rsesci_setEnumWindow (instrSession,
                                 "DISP:WIND%ld:TRACE:X:SPAC",
                                 window,
                                 frequencyAxisRangeSpacing,
                                 modeArr,
                                 VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Scan Display Spacing
 * Purpose:  This function queries setting of display in receiver mode.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetScanDisplaySpacing(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32*    frequencyAxisRangeSpacing
)
{
    return rsesci_getEnumWindow (instrSession,
                                 "DISP:WIND%ld:TRACE:X:SPAC?",
                                 window,
                                 frequencyAxisRangeSpacing,
                                 modeArr,
                                 VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set Hardcopy Output
 * Purpose:  This function selects the type of output that is used to document 
 *           the measurements.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetHardcopyOutput (ViSession instrSession,
                                            ViInt32 output)
{
	CHECKMODEL(RSESU8 | RSESU26 | RSESU40);
	
	return rsesci_setEnum (instrSession,
                           "HCOP:MODE",
                           output,
                           trepOutputArr,
                           VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Hardcopy Output
 * Purpose:  This function returns the type of output that is used to document 
 *           the measurements.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetHardcopyOutput (ViSession instrSession,
                                            ViInt32 *output)
{
	CHECKMODEL(RSESU8 | RSESU26 | RSESU40);
	
	return rsesci_getEnum (instrSession,
                           "HCOP:MODE?",
                           output,
                           trepOutputArr,
                           VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Create New Test Report
 * Purpose:  This function generates a new test report in a new document in 
 *           accordance with the test report configuration settings. An existing 
 *           report document is deleted. This report document is initially stored
 *           internally until it is printed using the HCOPy:IMMediate command
 *           or exported to a portable file format.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_CreateNewTestReport (ViSession instrSession)
{
	CHECKMODEL(RSESU8 | RSESU26 | RSESU40);
	
	return rsesci_sendCmd (instrSession, "HCOP:TREP:NEW");    
}

/*****************************************************************************
 * Function: Append Test Report
 * Purpose:  This function adds a new test report to an existing document in 
 *           accordance with the test report configuration settings. This report
 *           document is initially stored internally until it is printed
 *           using the HCOPy:IMMediate command or exported to a portable file
 *           format.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_AppendTestReport (ViSession instrSession)
{
	CHECKMODEL(RSESU8 | RSESU26 | RSESU40);
	
	return rsesci_sendCmd (instrSession, "HCOP:TREP:APP");    
}

/*****************************************************************************
 * Function: Set Test Report Default Settings
 * Purpose:  This function sets the test report configuration to a defined 
 *           default status. A newly generated test report will then contain 
 *           the following:
 *           Header line
 *           Diagram
 *           Scan table
 *           Final measured values
 *           Time of day and date
 *           Page number
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetTestReportDefaultSettings (ViSession instrSession)
{
	CHECKMODEL(RSESU8 | RSESU26 | RSESU40);
	
	return rsesci_sendCmd (instrSession, "HCOP:TREP:ITEM:DEF");    
}

/*****************************************************************************
 * Function: Configure Test Report Page Settings
 * Purpose:  This function configures the page settings of the test report.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_ConfigureTestReportPageSettings (ViSession instrSession,
                                                          ViInt32 itemSelector,
                                                          ViBoolean state)
{
	ViChar	buffer[BUFFER_SIZE] = "";
	
	CHECKMODEL(RSESU8 | RSESU26 | RSESU40);
	
	sprintf (buffer, "HCOP:TREP:ITEM:%s:STAT", trepItemSelectorArr[itemSelector]);
	
	return rsesci_setViBoolean (instrSession,
                                buffer,
                                state,
                                VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Query Test Report Page Settings
 * Purpose:  This function returns the page settings of the test report.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_QueryTestReportPageSettings (ViSession instrSession,
                                                      ViInt32 itemSelector,
                                                      ViBoolean *state)
{
	ViChar	buffer[BUFFER_SIZE] = "";
	
	CHECKMODEL(RSESU8 | RSESU26 | RSESU40);
	
	sprintf (buffer, "HCOP:TREP:ITEM:%s:STAT?", trepItemSelectorArr[itemSelector]);
	
	return rsesci_getViBoolean (instrSession,
                                buffer,
                                state,
                                VI_ERROR_PARAMETER3);    
}

/*****************************************************************************
 * Function: Configure Test Report Header
 * Purpose:  This function configures the test report header and logo.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_ConfigureTestReportHeader (ViSession instrSession,
                                                    ViInt32 itemSelector,
                                                    ViInt32 lineNumber,
                                                    ViInt32 state,
                                                    ViString lineTextTitle)
{
	ViStatus 	rsesci_status = VI_SUCCESS;
	ViChar		buffer[BUFFER_SIZE] = "";
	
	CHECKMODEL(RSESU8 | RSESU26 | RSESU40);
	
	switch (itemSelector)
	{
		case RSESCI_TREP_LOGO:
			 
			rsesci_setEnum (instrSession,
				            "HCOP:TREP:ITEM:LOGO:CONT",
							state,
							trepItemStateArr,
							VI_ERROR_PARAMETER4);
	
		break;
		case RSESCI_TREP_LINE:
			
			if (rsesci_invalidViInt32Range (lineNumber, 1, 7) == VI_TRUE)
        		return VI_ERROR_PARAMETER3; 
			
			rsesci_setEnumWindow (instrSession,
				                  "HCOP:TREP:ITEM:HEAD:LINE%ld:CONT",
								  lineNumber,
							      state,
							      trepItemStateArr,
							      VI_ERROR_PARAMETER4);
		break;
		case RSESCI_TREP_LTEXT:
			 
			if (rsesci_invalidViInt32Range (lineNumber, 1, 7) == VI_TRUE)
        		return VI_ERROR_PARAMETER3; 
			
			sprintf (buffer, "HCOP:TREP:ITEM:HEAD:LINE%ld:TEXT", lineNumber);
	
			rsesci_setViStringQuoted (instrSession,
				                      buffer,
									  "'",
									  lineTextTitle,
									  VI_ERROR_PARAMETER5);
		break;
		case RSESCI_TREP_LTITLE:
			 
			if (rsesci_invalidViInt32Range (lineNumber, 1, 7) == VI_TRUE)
        		return VI_ERROR_PARAMETER3; 
			
			sprintf (buffer, "HCOP:TREP:ITEM:HEAD:LINE%ld:TITL", lineNumber);
	
			rsesci_setViStringQuoted (instrSession,
				                      buffer,
									  "'",
									  lineTextTitle,
									  VI_ERROR_PARAMETER5);
		break;
		default:
			VI_ERROR_PARAMETER2;
		
	}
	
	return rsesci_status;    
}

/*****************************************************************************
 * Function: Query Test Report Header
 * Purpose:  This function queries the test report header and logo.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_QueryTestReportHeader (ViSession instrSession,
                                                ViInt32 itemSelector,
                                                ViInt32 lineNumber,
                                                ViInt32 *state,
                                                ViChar _VI_FAR lineTextTitle[])
{
	ViStatus 	rsesci_status = VI_SUCCESS;
	ViChar		buffer[BUFFER_SIZE] = "";
	
	CHECKMODEL(RSESU8 | RSESU26 | RSESU40);
	
	switch (itemSelector)
	{
		case RSESCI_TREP_LOGO:
			 
			rsesci_getEnum (instrSession,
				            "HCOP:TREP:ITEM:LOGO:CONT?",
							state,
							trepItemStateArr,
							VI_ERROR_PARAMETER4);
	
		break;
		case RSESCI_TREP_LINE:
			
			if (rsesci_invalidViInt32Range (lineNumber, 1, 7) == VI_TRUE)
        		return VI_ERROR_PARAMETER3; 
			
			rsesci_getEnumWindow (instrSession,
				                  "HCOP:TREP:ITEM:HEAD:LINE%ld:CONT?",
								  lineNumber,
							      state,
							      trepItemStateArr,
							      VI_ERROR_PARAMETER4);
		break;
		case RSESCI_TREP_LTEXT:
			 
			if (rsesci_invalidViInt32Range (lineNumber, 1, 7) == VI_TRUE)
        		return VI_ERROR_PARAMETER3; 
			
			sprintf (buffer, "HCOP:TREP:ITEM:HEAD:LINE%ld:TEXT?", lineNumber);
	
			rsesci_getViStringQuoted (instrSession,
				                      buffer,
									  "'",
									  BUFFER_SIZE,
									  lineTextTitle,
									  VI_ERROR_PARAMETER5);
		break;
		case RSESCI_TREP_LTITLE:
			 
			if (rsesci_invalidViInt32Range (lineNumber, 1, 7) == VI_TRUE)
        		return VI_ERROR_PARAMETER3; 
			
			sprintf (buffer, "HCOP:TREP:ITEM:HEAD:LINE%ld:TITL?", lineNumber);
	
			rsesci_getViStringQuoted (instrSession,
				                      buffer,
									  "'",
									  BUFFER_SIZE,
									  lineTextTitle,
									  VI_ERROR_PARAMETER5);
		break;
		default:
			VI_ERROR_PARAMETER2;
		
	}
	
	return rsesci_status;    
}

/*****************************************************************************
 * Function: Set Test Report Header Line State
 * Purpose:  This function controls whether the line indicated in the numeric
 *           suffix of the report header is output.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetTestReportHeaderLineState (ViSession instrSession,
                                                       ViInt32 lineNumber,
                                                       ViInt32 state)
{
    CHECKMODEL(RSESU8 | RSESU26 | RSESU40);
    
    if (rsesci_invalidViInt32Range (lineNumber, 1, 7) == VI_TRUE)
        return VI_ERROR_PARAMETER2;

    return rsesci_setEnumWindow (instrSession, 
                                 "HCOP:TREP:ITEM:HEAD:LINE%ld:CONT",
                                 lineNumber,
                                 state,
                                 trepItemStateArr,
                                 VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Test Report Header Line State
 * Purpose:  This function returns whether the line indicated in the numeric
 *           suffix of the report header is output.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetTestReportHeaderLineState (ViSession instrSession,
                                                       ViInt32 lineNumber,
                                                       ViInt32 *state)
{
    CHECKMODEL(RSESU8 | RSESU26 | RSESU40);
    
    if (rsesci_invalidViInt32Range (lineNumber, 1, 7) == VI_TRUE)
        return VI_ERROR_PARAMETER2;

    return rsesci_getEnumWindow (instrSession, 
                                 "HCOP:TREP:ITEM:HEAD:LINE%ld:CONT?",
                                 lineNumber,
                                 state,
                                 trepItemStateArr,
                                 VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set Test Report Header Logo State
 * Purpose:  This function switches the output of the diagram on the test
 *           instrument screen either on or off.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetTestReportHeaderLogoState (ViSession instrSession,
                                                       ViInt32 state)
{
    CHECKMODEL(RSESU8 | RSESU26 | RSESU40);
    
    return rsesci_setEnum (instrSession, 
                         "HCOP:TREP:ITEM:LOGO:CONT",
                         state,
                         trepItemStateArr,
                         VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Test Report Header Logo State
 * Purpose:  This function returns the state of the output of the diagram on the test
 *           instrument screen.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetTestReportHeaderLogoState (ViSession instrSession,
                                                       ViInt32 *state)
{
    CHECKMODEL(RSESU8 | RSESU26 | RSESU40);
    
    return rsesci_getEnum (instrSession, 
                         "HCOP:TREP:ITEM:LOGO:CONT?",
                         state,
                         trepItemStateArr,
                         VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Test Report Header Line Title
 * Purpose:  This function defines a name for the line of the report header indicated
 *           in the numeric suffix. This name is also output in the test report.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetTestReportHeaderLineTitle (ViSession instrSession,
                                                       ViInt32 lineNumber,
                                                       ViString lineTitle)
{
    ViChar	buffer[BUFFER_SIZE] = "";
	
	CHECKMODEL(RSESU8 | RSESU26 | RSESU40);
    
    if (rsesci_invalidViInt32Range (lineNumber, 1, 7) == VI_TRUE)
        return VI_ERROR_PARAMETER2;
    
    sprintf (buffer, "HCOP:TREP:ITEM:HEAD:LINE%ld:TITL", lineNumber);
	
	return rsesci_setViStringQuoted (instrSession, 
                                     buffer,
                                     "'",
                                     lineTitle,
                                     VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Test Report Header Line Title
 * Purpose:  This function returns a name for the line of the report header indicated
 *           in the numeric suffix.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetTestReportHeaderLineTitle (ViSession instrSession,
                                                       ViInt32 lineNumber,
                                                       ViInt32 arraySize,
                                                       ViChar lineTitle[])
{
    ViChar	buffer[BUFFER_SIZE] = "";
	
	CHECKMODEL(RSESU8 | RSESU26 | RSESU40);
    
    if (rsesci_invalidViInt32Range (lineNumber, 1, 7) == VI_TRUE)
        return VI_ERROR_PARAMETER2;
    
    sprintf (buffer, "HCOP:TREP:ITEM:HEAD:LINE%ld:TITL?", lineNumber);
	
	return rsesci_getViStringQuoted (instrSession, 
                                     buffer,
                                     "'",
                                     arraySize,
                                     lineTitle,
                                     VI_ERROR_PARAMETER4);
}

/*****************************************************************************
 * Function: Set Test Report Header Line Text
 * Purpose:  This function defines the text used for the line of the report header
 *           indicated in the numeric suffix.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetTestReportHeaderLineText (ViSession instrSession,
                                                      ViInt32 lineNumber,
                                                      ViString lineText)
{
    ViChar	buffer[BUFFER_SIZE] = "";
	
	CHECKMODEL(RSESU8 | RSESU26 | RSESU40);
    
    if (rsesci_invalidViInt32Range (lineNumber, 1, 7) == VI_TRUE)
        return VI_ERROR_PARAMETER2;
    
    sprintf (buffer, "HCOP:TREP:ITEM:HEAD:LINE%ld:TEXT", lineNumber);
	
	return rsesci_setViStringQuoted (instrSession, 
                                     buffer,
                                     "'",
                                     lineText,
                                     VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Test Report Header Line Text
 * Purpose:  This function returns the text used for the line of the report header
 *           indicated in the numeric suffix.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetTestReportHeaderLineText (ViSession instrSession,
                                                      ViInt32 lineNumber,
                                                      ViInt32 arraySize,
                                                      ViChar lineText[])
{
    ViChar	buffer[BUFFER_SIZE] = "";
	
	CHECKMODEL(RSESU8 | RSESU26 | RSESU40);
    
    if (rsesci_invalidViInt32Range (lineNumber, 1, 7) == VI_TRUE)
        return VI_ERROR_PARAMETER2;
    
    sprintf (buffer, "HCOP:TREP:ITEM:HEAD:LINE%ld:TEXT?", lineNumber);
	
	return rsesci_getViStringQuoted (instrSession, 
                                     buffer,
									 "'",
                                     arraySize,
                                     lineText,
                                     VI_ERROR_PARAMETER4);
}

/*****************************************************************************
 * Function: Test Report Operations
 * Purpose:  This function configures the test report operations.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_TestReportOperations (ViSession instrSession,
                                                        ViInt32 operation,
                                                        ViString testReportName)
{
	ViChar		buffer[BUFFER_SIZE] = "";
	
	CHECKMODEL(RSESU8 | RSESU26 | RSESU40);
	
	sprintf (buffer, "HCOP:TREP:ITEM:TEMP:%s", trepOperationArr[operation]);
	
	return rsesci_setViStringQuoted (instrSession,
		                             buffer,
									 "'",
									 testReportName,
									 VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Load Logo
 * Purpose:  This function selects a graphics file which will appear as the 
 *           logo in the header of the test report.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_LoadLogo (ViSession instrSession,
                                   ViString directory, ViString fileName)
{
	ViStatus 	rsesci_status = VI_SUCCESS;
	
	CHECKMODEL(RSESU8 | RSESU26 | RSESU40);
	
	rsesci_status = rsesci_setViStringQuoted (instrSession,
		                                      "MMEM:LOGO:CDIR",
							          		  "'",
									          directory,
									          VI_ERROR_PARAMETER2);

	return rsesci_setViStringQuoted (instrSession,
		                             "MMEM:LOGO:NAME",
		  		          		     "'",
									 fileName,
									 VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Test Report Catalog
 * Purpose:  This function outputs a list showing the complete test report 
 *           configuration.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetTestReportCatalog (ViSession instrSession,
                                               ViInt32 *number_ofTestReports,
                                               ViInt32 bufferSize,
                                               ViChar testReportList[])
{
	ViStatus    rsesci_status = VI_SUCCESS;
    ViUInt32    retCnt = 0;
    ViChar      *buf=VI_NULL;
	
	CHECKMODEL(RSESU8 | RSESU26 | RSESU40);
	
	if (bufferSize < 0)
        return VI_ERROR_PARAMETER3;
    if ((bufferSize > 0) && (testReportList == VI_NULL))
        return VI_ERROR_PARAMETER4;
       
    CHECKERR(rsesci_clearBeforeRead (instrSession));
    rsesci_status = viPrintf (instrSession, "HCOP:TREP:ITEM:TEMP:CAT?\n");
    CHECKERR (rsesci_checkBeforeRead (instrSession));
    rsesci_status = rsesci_readDataUnknownLength(instrSession, &buf, &retCnt);
    rsesci_status = rsesci_ParseCatalog(buf, bufferSize, testReportList, number_ofTestReports);
    free (buf);
	
	return rsesci_status;    
}

/*****************************************************************************
 * Function: Obsolete Set Receiver Detector
 * Purpose:  This function activates the detectors for single point
 *           measurements in EMI receiver operating mode.
 *           Three detectors can be activated at the same time.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetReceiverDetector(
    ViSession   instrSession,
    ViInt32     window,
    ViBoolean   maxPeak,
    ViBoolean   minPeak,
    ViBoolean   rms,
    ViBoolean   average,
    ViBoolean   quasipeak,
    ViBoolean   cisprAverage
)
{
    ViChar      buffer[BUFFER_SIZE];
    ViInt32     count = 0;

    // CHANGE

    sprintf (buffer, "SENS%%ld:DET:REC ");

    if (maxPeak)
    {
        strcat (buffer, "POS");
        count++;
    }

    if (minPeak)
    {
        if (count)
            strcat (buffer, ",");

        strcat (buffer, "NEG");
        count++;
    }

    if (rms)
    {
        if (count)
            strcat (buffer, ",");

        strcat (buffer, "RMS");
        count++;
    }

    if (average && (count < 3))
    {
        if (count)
            strcat (buffer, ",");

        strcat (buffer, "AVER");
        count++;
    }

    if (quasipeak && (count < 3))
    {
        if (count)
            strcat (buffer, ",");

        strcat (buffer, "QPE");
        count++;
    }

    if (cisprAverage && (count < 3))
    {
        if (count)
            strcat (buffer, ",");

        strcat (buffer, "CAV");
        count++;
    }

    return rsesci_sendCmdWindow (instrSession,
                                 buffer,
                                 window);
}

/*****************************************************************************
 * Function: Obsolete Get Receiver Detector
 * Purpose:  This function queries the selected receiver mode detectors.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetReceiverDetector(
    ViSession   instrSession,
    ViInt32     window,
    ViBoolean*  maxPeak,
    ViBoolean*  minPeak,
    ViBoolean*  rms,
    ViBoolean*  average,
    ViBoolean*  quasipeak,
    ViBoolean*  cisprAverage
)
{
    ViStatus    rsesci_status;
    ViChar      buffer[BUFFER_SIZE];
    ViChar      *p2buf;
    ViInt32     index;

    CHECKERR(rsesci_getViStringWindow (instrSession,
                                       "SENS%ld:DET:REC?",
                                       window,
                                       buffer,
                                       BUFFER_SIZE,
                                       VI_ERROR_PARAMETER1));

    *maxPeak      = VI_FALSE;
    *minPeak      = VI_FALSE;
    *rms          = VI_FALSE;
    *average      = VI_FALSE;
    *quasipeak    = VI_FALSE;
    *cisprAverage = VI_FALSE;

    p2buf = strtok (buffer, ",");

    while (p2buf)
    {
        CHECKERR (rsesci_getIndexFromString(instrSession,
                                            p2buf,
                                            detectorTypeArr,
                                            &index));

        *maxPeak      = *maxPeak || (index == 0);
        *minPeak      = *minPeak || (index == 1);
        *rms          = *rms || (index == 2);
        *average      = *average || (index == 3);
        *quasipeak    = *quasipeak || (index == 4);
        *cisprAverage = *cisprAverage || (index == 5);

        p2buf = strtok(NULL, ",");
    }

    return rsesci_status;
}

/*****************************************************************************
 * Function: Set Harmonic Distortion State
 * Purpose:  This function sets harmonic distortion state.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetHDistState (ViSession instrSession,
                                        ViBoolean state)
{
	return rsesci_setViBoolean (instrSession, 
		                        "CALC:MARK:FUNC:HARM:STAT",
								state,
								VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Harmonic Distortion State
 * Purpose:  This function gets harmonic distortion state.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetHDistState (ViSession instrSession,
                                        ViBoolean *state)
{
	return rsesci_getViBoolean (instrSession, 
		                        "CALC:MARK:FUNC:HARM:STAT?",
								state,
								VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Set Number Of Harmonic Distortion
 * Purpose:  This function defines the number of harmonics of a carrier signal 
 *           to be measured.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetNumberOfHDist (ViSession instrSession,
                                           ViInt32 no_ofHarmonics)
{
	return rsesci_setViInt32 (instrSession, 
		                      "CALC:MARK:FUNC:HARM:NHAR",
							  no_ofHarmonics,
							  1, 26,
							  VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Get Number Of Harmonic Distortion
 * Purpose:  This function returns the number of harmonics of a carrier signal 
 *           to be measured.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetNumberOfHDist (ViSession instrSession,
                                           ViInt32 *no_ofHarmonics)
{
	return rsesci_getViInt32 (instrSession, 
		                      "CALC:MARK:FUNC:HARM:NHAR?",
							  no_ofHarmonics,
							  VI_ERROR_PARAMETER2);     
}

/*****************************************************************************
 * Function: Set Harmonic Distortion Bandwidth State
 * Purpose:  This function specifies whether the resolution bandwidth of the 
 *           2nd to the nth harmonic should be identical to the bandwidth of 
 *           the first harmonic (OFF) or to the next largest bandwidth 
 *           (corresponding to the harmonic) of the multiple of the bandwidth 
 *           of the first harmonic (ON).
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetHDistBandwidthState (ViSession instrSession,
                                                 ViBoolean harmonicRBWAuto)
{
	return rsesci_setViBoolean (instrSession, 
		                        "CALC:MARK:FUNC:HARM:BAND:AUTO",
								harmonicRBWAuto,
								VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Get Harmonic Distortion Bandwidth State
 * Purpose:  This function returns whether the resolution bandwidth of the 
 *           2nd to the nth harmonic should be identical to the bandwidth of 
 *           the first harmonic (OFF) or to the next largest bandwidth 
 *           (corresponding to the harmonic) of the multiple of the bandwidth 
 *           of the first harmonic (ON).
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetHDistBandwidthState (ViSession instrSession,
                                                 ViBoolean *harmonicRBWAuto)
{
 	return rsesci_getViBoolean (instrSession, 
		                        "CALC:MARK:FUNC:HARM:BAND:AUTO?",
								harmonicRBWAuto,
								VI_ERROR_PARAMETER2);   
}

/*****************************************************************************
 * Function: Adjust Harmonic Distortion Settings
 * Purpose:  This function optimizes the device settings depending on the mode 
 *           in which the harmonic measurement was started:
 *           If the harmonic measurement was started in the frequency domain
 *           (span > 0), the frequency and the level of the first harmonic
 *           are calculated, from which the measurement list is set up.
 *           If the measurement was started in the time domain (span = 0),
 *           the frequency of the first harmonic is not changed. The level,
 *           however, is not calculated. The function is independent of the
 *           marker selection.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_AdjustHDistSettings (ViSession instrSession)
{
	return rsesci_sendCmd (instrSession, "CALC:MARK:FUNC:HARM:PRES");    
}

/*****************************************************************************
 * Function: Get Harmonic Distortion
 * Purpose:  This function queries the results of the total harmonic distortion 
 *           (THD).
 *           
 *           A complete sweep with synchronization to sweep end must be
 *           performed between switching on the function and querying the
 *           measured value to obtain a valid query result. This is only
 *           possible in single sweep mode.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetHDist (ViSession instrSession,
                                   ViReal64 *harmonicDistortion_in_percent,
                                   ViReal64 *harmonicDistortion_in_dB)
{
	if (!harmonicDistortion_in_percent)
        return VI_ERROR_PARAMETER2;
    if (!harmonicDistortion_in_dB)
        return VI_ERROR_PARAMETER3;
    
    return viQueryf (instrSession, ":CALC:MARK:FUNC:HARM:DIST? TOT\n", "%le,%le", 
                harmonicDistortion_in_percent, harmonicDistortion_in_dB);
        
}

/*****************************************************************************
 * Function: Get Harmonic Distortion Harmonics List
 * Purpose:  This function reads out the list of harmonics. The first value 
 *           is the absolute power of the first harmonic in the unit set via 
 *           UNIT. The other values are relative to the carrier signal and are 
 *           output in dB.
 *
 *           A complete sweep with synchronization to sweep end must be
 *           performed between switching on the function and querying the
 *           measured value to obtain a valid query result. This is only
 *           possible in single sweep mode.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetHDistHarmonicsList (ViSession instrSession,
                                                ViReal64 harmonicsList[])
{
	if (harmonicsList == NULL)
        return VI_ERROR_PARAMETER2;
    
    return viQueryf (instrSession, "CALC:MARK:FUNC:HARM:LIST?\n", "%,255lf", harmonicsList);
        
}

/*****************************************************************************
 * Function: Get Harmonic Bandwidth Resolution
 * Purpose:  This function returns the resolution bandwidth values used by
             the harmonic measurment. A separate value for every harmonic
             frequency is automatically calculated.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetHDistBandwidthResolution (ViSession instrSession,
                                                         ViInt32 arraySize,
                                                         ViReal64 resolutionBandwidthValue[])
{
    ViStatus    rsesci_status = VI_SUCCESS;
    ViChar      cmd[BUFFER_SIZE];
    ViChar      *pbuffer        = VI_NULL;
    ViChar      *p2buf;
    ViUInt32    local_count     = 0;
    ViInt32     cnt             = 0;
   
    /* Create common temporary value array */

    GOCHECKERR (viPrintf(instrSession, "CALC:MARK:FUNC:HARM:BAND?\n", cmd));
    GOCHECKERR (rsesci_checkBeforeRead (instrSession));
    GOCHECKERR (rsesci_readDataUnknownLength(instrSession, &pbuffer, &local_count));

    p2buf = strtok(pbuffer, ",");

    while(p2buf && (cnt < arraySize))
        {
        resolutionBandwidthValue[cnt++] = atof(p2buf);
        p2buf = strtok(NULL, ",");
        };

    GOCHECKERR (rsesci_checkStatus (instrSession));

ERROR:

    free(pbuffer);

    return rsesci_status;   
}

/*****************************************************************************
 * Function: Set Spurious Emissions Mode
 * Purpose:  This function switches the spurious emissions measurement on
 *           or off.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetSEMode (ViSession instrSession, ViInt32 mode)
{
	CHECKMODEL(RSESU8 | RSESU26 | RSESU40);
	
	return rsesci_setEnum (instrSession,
		                   "SENS:SWE:MODE",
						   mode,
						   modeSEArr,
						   VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Spurious Emissions Mode
 * Purpose:  This function returns the state of the spurious emissions measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetSEMode (ViSession instrSession, ViInt32 *mode)
{
	CHECKMODEL(RSESU8 | RSESU26 | RSESU40);
	
	return rsesci_getEnum (instrSession,
		                   "SENS:SWE:MODE?",
						   mode,
						   modeSEArr,
						   VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Configure Spurious Emissions Sweep Range
 * Purpose:  This function configures the sweep range.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_ConfigureSESweepRange (ViSession instrSession,
                                               ViInt32 range,
                                               ViReal64 startFrequency,
                                               ViReal64 stopFrequency,
                                               ViInt32 filter,
                                               ViReal64 resolutionBandwidth,
                                               ViReal64 videoBandwidth,
                                               ViBoolean sweepTimeAuto,
                                               ViReal64 sweepTime,
                                               ViInt32 detector,
                                               ViReal64 refLevelValue,
                                               ViBoolean attenuatorAuto,
                                               ViReal64 manualAttenuatorValuedB,
                                               ViInt32 sweepPoints,
                                               ViBoolean stopAfterSweep,
                                               ViString transducerFactor)
{
	ViStatus 	rsesci_status = VI_SUCCESS;

    CHECKERR (rsesci_SetSESweepStart (instrSession, range, startFrequency));
    
    rsesci_status = rsesci_SetSESweepStop (instrSession, range, stopFrequency);
    REMAP (3,4);
    
    rsesci_status = rsesci_SetSESweepFilter (instrSession, range, filter);
    REMAP (3,5);
                                    
    rsesci_status = rsesci_SetSESweepRBW (instrSession, range, resolutionBandwidth);
    REMAP (3,6);

    rsesci_status = rsesci_SetSESweepVBW (instrSession, range, videoBandwidth);
    REMAP (3,7);

    rsesci_status = rsesci_SetSESweepTimeAuto (instrSession, range, sweepTimeAuto);
    REMAP (3,8);
    
    if (sweepTimeAuto == VI_FALSE)
    {
        rsesci_status = rsesci_SetSESweepTime(instrSession,range,sweepTime);
        REMAPINSTR (3,9);
    }                                                
    
    rsesci_status = rsesci_SetSESweepDetector (instrSession,range,detector);
    REMAPINSTR (3,10);
    
    rsesci_status = rsesci_SetSESweepRefLevel (instrSession,range,refLevelValue);
    REMAPINSTR (3,11);

    rsesci_status = rsesci_SetSESweepAttenuatorAuto (instrSession,range,attenuatorAuto);
    REMAPINSTR (3,12);
    
    if (attenuatorAuto == VI_FALSE)
    {
        rsesci_status = rsesci_SetSESweepAttenuatorValue (instrSession,range,manualAttenuatorValuedB);
       REMAPINSTR (3,13);
    }                                            

    rsesci_status = rsesci_SetSESweepPoints(instrSession,range,sweepPoints);
    REMAPINSTR (3,14);

    rsesci_status = rsesci_SetSESweepStopAfterSweep(instrSession,range,stopAfterSweep);
    REMAPINSTR (3,15);
    
    if (strlen (transducerFactor) > 0)
    {
	rsesci_status = rsesci_SetSESweepTransducerFactor (instrSession,range,transducerFactor);
        REMAPINSTR (3,16);
    }
        
    return rsesci_status;
}

/*****************************************************************************
 * Function: Set Spurious Emissions Preamplifier State
 * Purpose:  This function configures state of pre-amplifier for selected 
 *           sweep range.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetSEPreamplifierState (ViSession instrSession,
                                                  ViInt32 range,
                                                  ViBoolean preamplifier)
{
	ViChar	buffer[BUFFER_SIZE] = "";
	
	CHECKMODEL(RSESU8 | RSESU26 | RSESU40);
	
    if (rsesci_invalidViInt32Range (range, 1, 20) == VI_TRUE)
        return VI_ERROR_PARAMETER2;
	
	sprintf (buffer, "SENS:LIST:RANG%ld:INP:GAIN:STAT", range);
	
    return rsesci_setViBoolean (instrSession, 
                                buffer,
                                preamplifier,
                                VI_ERROR_PARAMETER3);
	
}

/*****************************************************************************
 * Function: Query Spurious Emissions Preamplifier State
 * Purpose:  This function returns the state of pre-amplifier for selected 
 *           sweep range.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetSEPreamplifierState (ViSession instrSession,
                                            ViInt32 range,
                                            ViBoolean *preamplifier)
{
    ViChar	buffer[BUFFER_SIZE] = "";
	
	CHECKMODEL(RSESU8 | RSESU26 | RSESU40);
	
    if (rsesci_invalidViInt32Range (range, 1, 20) == VI_TRUE)
        return VI_ERROR_PARAMETER2;
	
	sprintf (buffer, "SENS:LIST:RANG%ld:INP:GAIN:STAT?", range);
	
    return rsesci_getViBoolean (instrSession, 
                                buffer,
                                preamplifier,
                                VI_ERROR_PARAMETER3);    
}

/*****************************************************************************
 * Function: Spurious Emissions Delete Range
 * Purpose:  This function deletes a range.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SEDeleteRange (ViSession instrSession,
                                        ViInt32 range)
{
    ViChar	buffer[BUFFER_SIZE] = "";
	
	CHECKMODEL(RSESU8 | RSESU26 | RSESU40);
	
    if (rsesci_invalidViInt32Range (range, 1, 20) == VI_TRUE)
        return VI_ERROR_PARAMETER2;
	
	sprintf (buffer, "SENS:LIST:RANG%ld:DEL", range);
	
    return rsesci_sendCmd (instrSession, buffer);
}

/*****************************************************************************
 * Function: Set Spurious Emissions Sweep Start
 * Purpose:  This function configures start frequency of the sweep range.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetSESweepStart (ViSession instrSession,
                                          ViInt32 range, ViReal64 startFrequency)
{
    ViChar	buffer[BUFFER_SIZE] = "";
	
	CHECKMODEL(RSESU8 | RSESU26 | RSESU40);
	
    if (rsesci_invalidViInt32Range (range, 1, 20) == VI_TRUE)
        return VI_ERROR_PARAMETER2;
	
	sprintf (buffer, "SENS:LIST:RANG%ld:FREQ:STAR", range);
	
    return rsesci_setViReal64 (instrSession, 
	                           buffer,
                               startFrequency, 0.0, 0.0,
                               VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Spurious Emissions Sweep Start
 * Purpose:  This function queries start frequency of the sweep range.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetSESweepStart (ViSession instrSession,
                                          ViInt32 range,
                                          ViReal64 *startFrequency)
{
    ViChar	buffer[BUFFER_SIZE] = "";
	
	CHECKMODEL(RSESU8 | RSESU26 | RSESU40);
	
    if (rsesci_invalidViInt32Range (range, 1, 20) == VI_TRUE)
        return VI_ERROR_PARAMETER2;
	
	sprintf (buffer, "SENS:LIST:RANG%ld:FREQ:STAR?", range);
	
    return rsesci_getViReal64 (instrSession, 
                               buffer,
				               startFrequency,
				               VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set Spurious Emissions Sweep Stop
 * Purpose:  This function sets stop frequency of the sweep range.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetSESweepStop (ViSession instrSession,
                                         ViInt32 range, ViReal64 stopFrequency)
{
    ViChar	buffer[BUFFER_SIZE] = "";
	
	CHECKMODEL(RSESU8 | RSESU26 | RSESU40);
	
    if (rsesci_invalidViInt32Range (range, 1, 20) == VI_TRUE)
        return VI_ERROR_PARAMETER2;
	
	sprintf (buffer, "SENS:LIST:RANG%ld:FREQ:STOP", range);
	
    return rsesci_setViReal64 (instrSession, 
	                           buffer,
							   stopFrequency,
							   0, 0,
                               VI_ERROR_PARAMETER3);
}
    
/*****************************************************************************
 * Function: Get Spurious Emissions Sweep Stop
 * Purpose:  This function queries stop frequency of the sweep range.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetSESweepStop (ViSession instrSession,
                                         ViInt32 range, ViReal64 *stopFrequency)
{
    ViChar	buffer[BUFFER_SIZE] = "";
	
	CHECKMODEL(RSESU8 | RSESU26 | RSESU40);
	
    if (rsesci_invalidViInt32Range (range, 1, 20) == VI_TRUE)
        return VI_ERROR_PARAMETER2;
	
	sprintf (buffer, "SENS:LIST:RANG%ld:FREQ:STOP?", range);
	
    return rsesci_getViReal64 (instrSession, 
                               buffer,
                               stopFrequency,
                               VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set Spurious Emissions Sweep Filter
 * Purpose:  This function sets the filter in the spurious measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetSESweepFilter (ViSession instrSession,
                                           ViInt32 range, ViInt32 filter)
{
    ViChar	buffer[BUFFER_SIZE] = "";
	
	CHECKMODEL(RSESU8 | RSESU26 | RSESU40);
	
    if (rsesci_invalidViInt32Range (range, 1, 20) == VI_TRUE)
        return VI_ERROR_PARAMETER2;
	
	sprintf (buffer, "SENS:LIST:RANG%ld:FILT:TYPE", range);
	
    return rsesci_setEnum (instrSession,
		                   buffer,
						   filter,
						   filterListArr,
                           VI_ERROR_PARAMETER3);
}
									
/*****************************************************************************
 * Function: Get Spurious Emissions Sweep Filter
 * Purpose:  This function queries the filter in the spurious measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetSESweepFilter (ViSession instrSession,
                                           ViInt32 range, ViInt32 *filter)
{
    ViChar	buffer[BUFFER_SIZE] = "";
	
	CHECKMODEL(RSESU8 | RSESU26 | RSESU40);
	
    if (rsesci_invalidViInt32Range (range, 1, 20) == VI_TRUE)
        return VI_ERROR_PARAMETER2;
	
	sprintf (buffer, "SENS:LIST:RANG%ld:FILT:TYPE?", range);
	
    return rsesci_getEnum (instrSession,
                           buffer,
                           filter,
                           filterListArr,
                           VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set Spurious Emissions Sweep RBW
 * Purpose:  This function sets the the resolution bandwidth.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetSESweepRBW (ViSession instrSession,
                                        ViInt32 range,
                                        ViReal64 resolutionBandwidth)
{
    ViChar	buffer[BUFFER_SIZE] = "";
	
	CHECKMODEL(RSESU8 | RSESU26 | RSESU40);
	
    if (rsesci_invalidViInt32Range (range, 1, 20) == VI_TRUE)
        return VI_ERROR_PARAMETER2;
	
	sprintf (buffer, "SENS:LIST:RANG%ld:BAND:RES", range);
	
    return rsesci_setViReal64 (instrSession, 
	                           buffer,
							   resolutionBandwidth,
							   0, 0,
                               VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Spurious Emissions Sweep RBW
 * Purpose:  This function queries the the resolution bandwidth.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetSESweepRBW (ViSession instrSession,
                                        ViInt32 range,
                                        ViReal64 *resolutionBandwidth)
{
    ViChar	buffer[BUFFER_SIZE] = "";
	
	CHECKMODEL(RSESU8 | RSESU26 | RSESU40);
	
    if (rsesci_invalidViInt32Range (range, 1, 20) == VI_TRUE)
        return VI_ERROR_PARAMETER2;
	
	sprintf (buffer, "SENS:LIST:RANG%ld:BAND:RES?", range);
	
    return rsesci_getViReal64 (instrSession, 
                               buffer,
                               resolutionBandwidth,
                               VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set Spurious Emissions Sweep VBW
 * Purpose:  This function sets the the video bandwidth.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetSESweepVBW (ViSession instrSession,
                                        ViInt32 range, ViReal64 videoBandwidth)
{
    ViChar	buffer[BUFFER_SIZE] = "";
	
	CHECKMODEL(RSESU8 | RSESU26 | RSESU40);
	
    if (rsesci_invalidViInt32Range (range, 1, 20) == VI_TRUE)
        return VI_ERROR_PARAMETER2;
	
	sprintf (buffer, "SENS:LIST:RANG%ld:BAND:VID", range);
	
    return rsesci_setViReal64 (instrSession, 
	                           buffer,
							   videoBandwidth,
							   0, 0,
                               VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Spurious Emissions Sweep VBW
 * Purpose:  This function queries the the video bandwidth.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetSESweepVBW (ViSession instrSession,
                                        ViInt32 range, ViReal64 *videoBandwidth)
{
    ViChar	buffer[BUFFER_SIZE] = "";
	
	CHECKMODEL(RSESU8 | RSESU26 | RSESU40);
	
    if (rsesci_invalidViInt32Range (range, 1, 20) == VI_TRUE)
        return VI_ERROR_PARAMETER2;
	
	sprintf (buffer, "SENS:LIST:RANG%ld:BAND:VID?", range);
	
    return rsesci_getViReal64 (instrSession, 
                               buffer,
                               videoBandwidth,
                               VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set Spurious Emissions Sweep Time Auto
 * Purpose:  This function sets the automatic coupling of the sweep duration on the
 *           frequency span and the bandwidth settings in a spurious measurement
 *           range.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetSESweepTimeAuto (ViSession instrSession,
                                             ViInt32 range,
                                             ViBoolean sweepTimeAuto)
{
    ViChar	buffer[BUFFER_SIZE] = "";
	
	CHECKMODEL(RSESU8 | RSESU26 | RSESU40);
	
    if (rsesci_invalidViInt32Range (range, 1, 20) == VI_TRUE)
        return VI_ERROR_PARAMETER2;
	
	sprintf (buffer, "SENS:LIST:RANG%ld:SWE:TIME:AUTO", range);
	
    return rsesci_setViBoolean (instrSession, 
	                            buffer,
								sweepTimeAuto,
                                VI_ERROR_PARAMETER3);
}
	
/*****************************************************************************
 * Function: Get Spurious Emissions Sweep Time Auto
 * Purpose:  This function queries the state of automatic coupling of the sweep
 *           duration on the frequency span and the bandwidth settings in a spurious
 *           measurement range.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetSESweepTimeAuto (ViSession instrSession,
                                             ViInt32 range,
                                             ViBoolean *sweepTimeAuto)
{
    ViChar	buffer[BUFFER_SIZE] = "";
	
	CHECKMODEL(RSESU8 | RSESU26 | RSESU40);
	
    if (rsesci_invalidViInt32Range (range, 1, 20) == VI_TRUE)
        return VI_ERROR_PARAMETER2;
	
	sprintf (buffer, "SENS:LIST:RANG%ld:SWE:TIME:AUTO?", range);
	
    return rsesci_getViBoolean (instrSession, 
                                buffer,
                                sweepTimeAuto,
                                VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set Spurious Emissions Sweep Time
 * Purpose:  This function sets the sweep duration of a range in the spurious
 *           measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetSESweepTime (ViSession instrSession,
                                         ViInt32 range, ViReal64 sweepTime)
{
    ViChar	buffer[BUFFER_SIZE] = "";
	
	CHECKMODEL(RSESU8 | RSESU26 | RSESU40);
	
    if (rsesci_invalidViInt32Range (range, 1, 20) == VI_TRUE)
        return VI_ERROR_PARAMETER2;
	
	sprintf (buffer, "SENS:LIST:RANG%ld:SWE:TIME", range);
	
    return rsesci_setViReal64 (instrSession, 
		                       buffer,
							   sweepTime,
							   0.001, 16000.0,
                               VI_ERROR_PARAMETER3);
}                                                
	
/*****************************************************************************
 * Function: Get Spurious Emissions Sweep Time
 * Purpose:  This function queries the sweep duration of a range in the spurious
 *           measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetSESweepTime (ViSession instrSession,
                                         ViInt32 range, ViReal64 *sweepTime)
{
    ViChar	buffer[BUFFER_SIZE] = "";
	
	CHECKMODEL(RSESU8 | RSESU26 | RSESU40);
	
    if (rsesci_invalidViInt32Range (range, 1, 20) == VI_TRUE)
        return VI_ERROR_PARAMETER2;
	
	sprintf (buffer, "SENS:LIST:RANG%ld:SWE:TIME?", range);
	
    return rsesci_getViReal64 (instrSession, 
                               buffer,
                               sweepTime,
                               VI_ERROR_PARAMETER3);
}
	
/*****************************************************************************
 * Function: Set Spurious Emissions Sweep Detector
 * Purpose:  This function sets the detector in the spurious measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetSESweepDetector (ViSession instrSession,
                                             ViInt32 range, ViInt32 detector)
{
    ViChar	buffer[BUFFER_SIZE] = "";
	
	CHECKMODEL(RSESU8 | RSESU26 | RSESU40);
	
    if (rsesci_invalidViInt32Range (range, 1, 20) == VI_TRUE)
        return VI_ERROR_PARAMETER2;
	
	sprintf (buffer, "SENS:LIST:RANG%ld:DET", range);
	
    return rsesci_setEnum (instrSession,
                           buffer,
                           detector,
                           traceDetectorArr,
                           VI_ERROR_PARAMETER3);
}
	
/*****************************************************************************
 * Function: Get Spurious Emissions Sweep Detector
 * Purpose:  This function queries the detector in the spurious measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetSESweepDetector (ViSession instrSession,
                                             ViInt32 range, ViInt32 *detector)
{
    ViChar	buffer[BUFFER_SIZE] = "";
	
	CHECKMODEL(RSESU8 | RSESU26 | RSESU40);
	
    if (rsesci_invalidViInt32Range (range, 1, 20) == VI_TRUE)
        return VI_ERROR_PARAMETER2;
	
	sprintf (buffer, "SENS:LIST:RANG%ld:DET?", range);
	
    return rsesci_getEnum (instrSession,
                           buffer,
                           detector,
                           traceDetectorArr,
                           VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set Spurious Emissions Sweep Ref Level
 * Purpose:  This function specifies the reference level of a range in the spurious
 *           measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetSESweepRefLevel (ViSession instrSession,
                                             ViInt32 range,
                                             ViReal64 refLevelValue)
{
    ViChar	buffer[BUFFER_SIZE] = "";
	
	CHECKMODEL(RSESU8 | RSESU26 | RSESU40);
	
    if (rsesci_invalidViInt32Range (range, 1, 20) == VI_TRUE)
        return VI_ERROR_PARAMETER2;
	
	sprintf (buffer, "SENS:LIST:RANG%ld:RLEV", range);
	
    return rsesci_setViReal64 (instrSession, 
                               buffer,
                               refLevelValue,
                               -130.0, 30.0,
							   VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Spurious Emissions Sweep Ref Level
 * Purpose:  This function queries the reference level of a range in the spurious
 *           measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetSESweepRefLevel (ViSession instrSession,
                                             ViInt32 range,
                                             ViReal64 *refLevelValue)
{
    ViChar	buffer[BUFFER_SIZE] = "";
	
	CHECKMODEL(RSESU8 | RSESU26 | RSESU40);
	
    if (rsesci_invalidViInt32Range (range, 1, 20) == VI_TRUE)
        return VI_ERROR_PARAMETER2;
	
	sprintf (buffer, "SENS:LIST:RANG%ld:RLEV?", range);
	
    return rsesci_getViReal64 (instrSession, 
                               buffer,
                               refLevelValue,
                               VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set Spurious Emissions Sweep Attenuator Auto
 * Purpose:  This function sets the autoranging of a range in the spurious measurement
 *           on or off.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetSESweepAttenuatorAuto (ViSession instrSession,
                                                   ViInt32 range,
                                                   ViBoolean attenuatorAuto)
{
    ViChar	buffer[BUFFER_SIZE] = "";
	
	CHECKMODEL(RSESU8 | RSESU26 | RSESU40);
	
    if (rsesci_invalidViInt32Range (range, 1, 20) == VI_TRUE)
        return VI_ERROR_PARAMETER2;
	
	sprintf (buffer, "SENS:LIST:RANG%ld:INP:ATT:AUTO", range);
	
    return rsesci_setViBoolean (instrSession, 
                                buffer,
                                attenuatorAuto,
                                VI_ERROR_PARAMETER3);
}
    
/*****************************************************************************
 * Function: Get Spurious Emissions Sweep Attenuator Auto
 * Purpose:  This function queries the autoranging of a range in the spurious
 *           measurement on or off.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetSESweepAttenuatorAuto (ViSession instrSession,
                                                   ViInt32 range,
                                                   ViBoolean *attenuatorAuto)
{
    ViChar	buffer[BUFFER_SIZE] = "";
	
	CHECKMODEL(RSESU8 | RSESU26 | RSESU40);
	
    if (rsesci_invalidViInt32Range (range, 1, 20) == VI_TRUE)
        return VI_ERROR_PARAMETER2;
	
	sprintf (buffer, "SENS:LIST:RANG%ld:INP:ATT:AUTO?", range);
	
    return rsesci_getViBoolean (instrSession, 
	                            buffer,
				                attenuatorAuto,
                                VI_ERROR_PARAMETER3);
}
	
/*****************************************************************************
 * Function: Set Spurious Emissions Sweep Attenuator Value
 * Purpose:  This function sets the RF attenuation of a range.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetSESweepAttenuatorValue (ViSession instrSession,
                                                    ViInt32 range,
                                                    ViReal64 manualAttenuatorValuedB)
{
    ViChar	buffer[BUFFER_SIZE] = "";
	
	CHECKMODEL(RSESU8 | RSESU26 | RSESU40);
	
    if (rsesci_invalidViInt32Range (range, 1, 20) == VI_TRUE)
        return VI_ERROR_PARAMETER2;
	
	sprintf (buffer, "SENS:LIST:RANG%ld:INP:ATT", range);
	
    return rsesci_setViReal64 (instrSession, 
                               buffer,
                               manualAttenuatorValuedB,
                               0.0, 75.0,
                               VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Spurious Emissions Sweep Attenuator Value
 * Purpose:  This queries sets the RF attenuation of a range..
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetSESweepAttenuatorValue (ViSession instrSession,
                                                    ViInt32 range,
                                                    ViReal64 *manualAttenuatorValuedB)
{
    ViChar	buffer[BUFFER_SIZE] = "";
	
	CHECKMODEL(RSESU8 | RSESU26 | RSESU40);
	
    if (rsesci_invalidViInt32Range (range, 1, 20) == VI_TRUE)
        return VI_ERROR_PARAMETER2;
	
	sprintf (buffer, "SENS:LIST:RANG%ld:INP:ATT?", range);
	
    return rsesci_getViReal64 (instrSession, 
	                           buffer,
							   manualAttenuatorValuedB,
                               VI_ERROR_PARAMETER3);
}
    
/*****************************************************************************
 * Function: Set Spurious Emissions Sweep Points
 * Purpose:  This function sets the number of sweep points of a range in the spurious
 *           measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetSESweepPoints (ViSession instrSession,
                                           ViInt32 range, ViInt32 sweepPoints)
{
    ViChar	buffer[BUFFER_SIZE] = "";
	
	CHECKMODEL(RSESU8 | RSESU26 | RSESU40);
	
    if (rsesci_invalidViInt32Range (range, 1, 20) == VI_TRUE)
        return VI_ERROR_PARAMETER2;
	
	sprintf (buffer, "SENS:LIST:RANG%ld:POIN", range);
	
    return rsesci_setViInt32 (instrSession, 
                              buffer,
				              sweepPoints,
                              0, 0,
                              VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Spurious Emissions Sweep Points
 * Purpose:  This function quries the number of sweep points of a range in the spurious
 *           measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetSESweepPoints (ViSession instrSession,
                                           ViInt32 range, ViInt32 *sweepPoints)
{
    ViChar	buffer[BUFFER_SIZE] = "";
	
	CHECKMODEL(RSESU8 | RSESU26 | RSESU40);
	
    if (rsesci_invalidViInt32Range (range, 1, 20) == VI_TRUE)
        return VI_ERROR_PARAMETER2;
	
	sprintf (buffer, "SENS:LIST:RANG%ld:POIN?", range);
	
    return rsesci_getViInt32 (instrSession, 
                              buffer,
                              sweepPoints,
                              VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set Spurious Emissions Sweep Stop After Sweep
 * Purpose:  This function determines whether a sweep is stopped in the spurious
 *           measurement when a range is changed.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetSESweepStopAfterSweep (ViSession instrSession,
                                                  ViInt32 range,
                                                   ViBoolean stopAfterSweep)
{
	ViChar	buffer[BUFFER_SIZE] = "";
	
	CHECKMODEL(RSESU8 | RSESU26 | RSESU40);
	
    if (rsesci_invalidViInt32Range (range, 1, 20) == VI_TRUE)
        return VI_ERROR_PARAMETER2;
	
	sprintf (buffer, "SENS:LIST:RANG%ld:BREAK", range);
	
    return rsesci_setViBoolean (instrSession, 
                                buffer,
                                stopAfterSweep,
				                VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Spurious Emissions Sweep Stop After Sweep
 * Purpose:  This function queries whether a sweep is stopped in the spurious
 *           measurement when a range is changed.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetSESweepStopAfterSweep (ViSession instrSession,
                                            ViInt32 range,
                                                   ViBoolean *stopAfterSweep)
{
	ViChar	buffer[BUFFER_SIZE] = "";
	
	CHECKMODEL(RSESU8 | RSESU26 | RSESU40);
	
    if (rsesci_invalidViInt32Range (range, 1, 20) == VI_TRUE)
        return VI_ERROR_PARAMETER2;
	
	sprintf (buffer, "SENS:LIST:RANG%ld:BREAK?", range);
	
    return rsesci_getViBoolean (instrSession, 
                                buffer,
                                stopAfterSweep,
								VI_ERROR_PARAMETER3);    
}

/*****************************************************************************
 * Function: Set Spurious Emissions Sweep Transducer Factor
 * Purpose:  This function sets the transducer factor for a range in the spurious
 *           measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetSESweepTransducerFactor (ViSession instrSession,
                                                     ViInt32 range,
                                                     ViString transducerFactor)
{
	ViChar	buffer[BUFFER_SIZE] = "";
	
	CHECKMODEL(RSESU8 | RSESU26 | RSESU40);
	
    if (rsesci_invalidViInt32Range (range, 1, 20) == VI_TRUE)
        return VI_ERROR_PARAMETER2;
	
	sprintf (buffer, "SENS:LIST:RANG%ld:TRAN", range);
	
    return rsesci_setViStringQuoted (instrSession,
                                      buffer,
                                      "'",
                                      transducerFactor,
                                      VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Spurious Emissions Sweep Transducer Factor
 * Purpose:  This function queries the transducer factor for a range in the spurious
 *           measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetSESweepTransducerFactor (ViSession instrSession,
                                                     ViInt32 range,
                                                     ViInt32 arraySize,
                                                     ViChar transducerFactor[])
{
    ViChar	buffer[BUFFER_SIZE] = "";
	
	CHECKMODEL(RSESU8 | RSESU26 | RSESU40);
    
    if (rsesci_invalidViInt32Range (range, 1, 20) == VI_TRUE)
        return VI_ERROR_PARAMETER2;

    sprintf (buffer, "SENS:LIST:RANG%ld:TRAN?", range);
	
    return rsesci_getViStringQuoted (instrSession,
                                      buffer,
                                      "'",
                                      arraySize,
									  transducerFactor,
                                      VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set Spurious Emissions Sweep Limit
 * Purpose:  This function defines the limit lines for the spurious emission
 *           measurement list evaluation.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetSESweepLimit (ViSession instrSession,
                                          ViInt32 range, ViReal64 limitValue)
{
	CHECKMODEL(RSESU8 | RSESU26 | RSESU40);
    
    if (rsesci_invalidViInt32Range (range, 1, 20) == VI_TRUE)
        return VI_ERROR_PARAMETER2;

    return rsesci_setViReal64Window (instrSession
                             	   , "SENS:LIST:RANG%ld:LIM"
                                   , range
                                   , limitValue
								   , -200.0, 200.0
                                   , VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Spurious Emissions Sweep Limit
 * Purpose:  This function queries the limit lines for the spurious emission
 *           measurement list evaluation.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetSESweepLimit (ViSession instrSession,
                                          ViInt32 range, ViReal64 *limitValue)
{
	CHECKMODEL(RSESU8 | RSESU26 | RSESU40);
    
    if (rsesci_invalidViInt32Range (range, 1, 20) == VI_TRUE)
        return VI_ERROR_PARAMETER2;

    return rsesci_getViReal64Window (instrSession
                             	   , "SENS:LIST:RANG%ld:LIM?"
                                   , range
                                   , limitValue
                                   , VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set Spurious Emissions Sweep Limit State
 * Purpose:  This function activates/deactivates the limit checking for the
 *           spurious emission measurement list evaluation.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetSESweepLimitState (ViSession instrSession,
                                               ViBoolean limitState)
{
	CHECKMODEL(RSESU8 | RSESU26 | RSESU40);

	return rsesci_setViBoolean (instrSession
                              , "LIST:RANG1:LIM:STAT"
                              , limitState
				              , VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Spurious Emissions Sweep Limit State
 * Purpose:  This function queries state of the limit checking for the
 *           spurious emission measurement list evaluation.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetSESweepLimitState (ViSession instrSession,
                                               ViBoolean *limitState)
{
	CHECKMODEL(RSESU8 | RSESU26 | RSESU40);

	return rsesci_getViBoolean (instrSession
                              , "LIST:RANG1:LIM:STAT?"
                              , limitState
				              , VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Spurious Emissions Number Of List Ranges
 * Purpose:  This function returns the number of defined ranges of the spurious
 *           measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetSENumberOfListRanges (ViSession instrSession,
                                                  ViInt32 *numberOfRanges)
{
	CHECKMODEL(RSESU8 | RSESU26 | RSESU40);
	
	return rsesci_getViInt32 (instrSession, 
		                      "SENS:LIST:RANG:COUN?",
							  numberOfRanges,
							  VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Store Spurious Emissions to File
 * Purpose:  This function stores the peak list of selected trace in ASCII 
 *           format to a file.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_StoreSEToFile (ViSession instrSession,
                                        ViString fileName)
{
	CHECKMODEL(RSESU8 | RSESU26 | RSESU40);
	
	return rsesci_setViStringQuoted (instrSession,
                                     "MMEM:STOR:SPUR",
                                     "'",
                                     fileName,
                                     VI_ERROR_PARAMETER2);    
}

/// HIFN  This function stores the current spurious emission measurement 
/// HIFN  configuration to a file on the instrument hard disk.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR fileName/Specifies full path of the file.
ViStatus _VI_FUNC rsesci_SaveSEConfiguration(
	ViSession	instrSession,
	ViString	fileName
)
{
	CHECKMODEL(RSESU8 | RSESU26 | RSESU40);
	
	return rsesci_setViStringQuoted (instrSession,
                                     "SENS:LIST:STAN:SAVE",
                                     "'",
                                     fileName,
                                     VI_ERROR_PARAMETER2);    
}

/// HIFN  This function is used to load customized spurious emission measurement 
/// HIFN  configurations.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR fileName/Specifies full path of the file.
ViStatus _VI_FUNC rsesci_LoadSEConfiguration(
	ViSession	instrSession,
	ViString	fileName
)
{
 	CHECKMODEL(RSESU8 | RSESU26 | RSESU40);
	
	return rsesci_setViStringQuoted (instrSession,
                                     "SENS:LIST:PRES",
                                     "'",
                                     fileName,
                                     VI_ERROR_PARAMETER2);    
}


/// HIFN  This function deletes a file that contains a spurious emission 
/// HIFN  measurement configuration.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR fileName/Specifies full path of the file.
ViStatus _VI_FUNC rsesci_DeleteSEConfiguration(
	ViSession	instrSession,
	ViString	fileName
)
{
	CHECKMODEL(RSESU8 | RSESU26 | RSESU40);
	
	return rsesci_setViStringQuoted (instrSession,
                                     "SENS:LIST:STAN:DEL",
                                     "'",
                                     fileName,
                                     VI_ERROR_PARAMETER2);    
}


/// HIFN  This function queries the names of customized spurious emission 
/// HIFN  measurement configuration files currently stored on the hard disk.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR catalog/This indicator returns the names of customized spurious emission 
/// HIPAR catalog/measurement configuration files currently stored on the hard disk.
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the 
/// HIPAR bufferSize/'Catalog' parameter.  
ViStatus _VI_FUNC rsesci_SpuriousEmissionsConfigurationCatalog (ViSession instrSession,
                                                                ViInt32 bufferSize,
                                                                ViChar catalog[])
{
    return rsesci_getViString (instrSession,
                               "SENS:LIST:STAN:CAT?",
                               catalog,
                               bufferSize,
                               VI_ERROR_PARAMETER4);    
}

/*****************************************************************************
 * Function: Read Spurious Emissions Peak List
 * Purpose:  This function returns the spurious emissions peak list.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_ReadSEPeakList (ViSession instrSession,
                                         ViInt32  trace,
	                                     ViInt32 noOfValues,
                                         ViReal64 frequency[], ViReal64 level[],
                                         ViReal64 deltaLimit[],
                                         ViInt32 *returnedValues)
{
	ViStatus    rsesci_status = VI_SUCCESS;
    ViChar      cmd[BUFFER_SIZE];
    ViChar      *pbuffer        = VI_NULL;
    ViChar      *p2buf;
    ViUInt32    local_count     = 0;
    ViInt32     cnt             = 0,
                idx             = 0;
    ViReal64    *values;

    CHECKMODEL(RSESU8 | RSESU26 | RSESU40);
	
	if (rsesci_invalidViInt32Range (trace, 1, 2))
        return VI_ERROR_PARAMETER2;
    if (noOfValues < 1)
        return VI_ERROR_PARAMETER3;
    if (!frequency)
        return VI_ERROR_PARAMETER4;
    if (!level)
        return VI_ERROR_PARAMETER5;
    if (!deltaLimit)
        return VI_ERROR_PARAMETER6;
    if (!returnedValues)
        return VI_ERROR_PARAMETER7;

    /* Create common temporary value array */
    values = malloc (3 * noOfValues * sizeof (ViReal64));

    sprintf (cmd, ":FORM ASC;:TRAC%ld? SPUR", trace);

    GOCHECKERR(rsesci_clearBeforeRead (instrSession));
    GOCHECKERR (viPrintf(instrSession, "%s\n", cmd));
    GOCHECKERR (rsesci_checkBeforeRead (instrSession));
    GOCHECKERR (rsesci_readDataUnknownLength(instrSession, &pbuffer, &local_count));
    p2buf = pbuffer;

    p2buf = strtok(pbuffer, ",");

    while(p2buf && (cnt < 3*noOfValues))
        {
        values[cnt++] = atof(p2buf);
        p2buf = strtok(NULL, ",");
        };

    /* Distribute all measured values */
    cnt /= 3;
    for (idx = 0; idx < cnt; idx++)
        {
        frequency[idx]  = values[0 + idx * 3];
        level[idx]      = values[1 + idx * 3];
        deltaLimit[idx] = values[2 + idx * 3];
        }

    *returnedValues = cnt;

    GOCHECKERR (rsesci_checkStatus (instrSession));

ERROR:

    free(pbuffer);
    free(values);

    return rsesci_status;   
}

/*****************************************************************************
 * Function: Initiate Final Measurement Test
 * Purpose:  This function starts the final measurement based on the peak list.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_InitiateFinalMeasurementTest (ViSession instrSession,
                                                       ViInt32 window)
{
	CHECKMODEL(RSESU8 | RSESU26 | RSESU40);
	
	return rsesci_sendCmdWindow (instrSession, "INIT%ld:FME", window);    
}

/*****************************************************************************
 * Function: Initiate EMI Test
 * Purpose:  This function starts an automatic sequence consisting of the 
 *           preliminary measurement, determination of the peak list and 
 *           subsequent final measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_InitiateEMITest (ViSession instrSession,
                                          ViInt32 window)
{
	CHECKMODEL(RSESU8 | RSESU26 | RSESU40);
	window;
	
	return rsesci_sendCmd (instrSession, "INIT2:EMIT");
}

/*****************************************************************************
 * Function: Set Receiver Detector With CRMS
 * Purpose:  This function activates the detectors for single point
 *           measurements in EMI receiver operating mode.
 *           Three detectors can be activated at the same time.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetReceiverDetectorWithCRMS (ViSession instrSession,
                                                ViInt32 window,
                                                ViBoolean maxPeak,
                                                ViBoolean minPeak, 
												ViBoolean RMS,
                                                ViBoolean average,
                                                ViBoolean quasipeak,
                                                ViBoolean CISPRAverage,
                                                ViBoolean CISPRRMS)
{
	ViChar      buffer[BUFFER_SIZE];
    ViInt32     count = 0;

    sprintf (buffer, "SENS%%ld:DET:REC ");

    if (maxPeak)
    {
        strcat (buffer, "POS");
        count++;
    }

    if (minPeak)
    {
        if (count)
            strcat (buffer, ",");

        strcat (buffer, "NEG");
        count++;
    }

    if (RMS)
    {
        if (count)
            strcat (buffer, ",");

        strcat (buffer, "RMS");
        count++;
    }

    if (average && (count < 3))
    {
        if (count)
            strcat (buffer, ",");

        strcat (buffer, "AVER");
        count++;
    }

    if (quasipeak && (count < 3))
    {
        if (count)
            strcat (buffer, ",");

        strcat (buffer, "QPE");
        count++;
    }

    if (CISPRAverage && (count < 3))
    {
        if (count)
            strcat (buffer, ",");

        strcat (buffer, "CAV");
        count++;
    }
	
	if (CISPRRMS && (count < 3))
    {
        if (count)
            strcat (buffer, ",");

        strcat (buffer, "CRMS");
        count++;
    }

    return rsesci_sendCmdWindow (instrSession,
                                 buffer,
                                 window);    
}

/*****************************************************************************
 * Function: Get Receiver Detector With CRMS
 * Purpose:  This function queries the selected receiver mode detectors.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetReceiverDetectorWithCRMS (ViSession instrSession,
                                                ViInt32 window,
                                                ViBoolean *maxPeak,
                                                ViBoolean *minPeak,
                                                ViBoolean *RMS,
                                                ViBoolean *average,
                                                ViBoolean *quasipeak,
                                                ViBoolean *CISPRAverage,
                                                ViBoolean *CISPRRMS)
{
	ViStatus    rsesci_status;
    ViChar      buffer[BUFFER_SIZE];
    ViChar      *p2buf;
    ViInt32     index;

    CHECKERR(rsesci_getViStringWindow (instrSession,
                                       "SENS%ld:DET:REC?",
                                       window,
                                       buffer,
                                       BUFFER_SIZE,
                                       VI_ERROR_PARAMETER1));

    *maxPeak      = VI_FALSE;
    *minPeak      = VI_FALSE;
    *RMS          = VI_FALSE;
    *average      = VI_FALSE;
    *quasipeak    = VI_FALSE;
    *CISPRAverage = VI_FALSE;
	*CISPRRMS     = VI_FALSE;

    p2buf = strtok (buffer, ",");

    while (p2buf)
    {
        CHECKERR (rsesci_getIndexFromString(instrSession,
                                            p2buf,
                                            detectorTypeArr,
                                            &index));

        *maxPeak      = *maxPeak || (index == 0);
        *minPeak      = *minPeak || (index == 1);
        *RMS          = *RMS || (index == 2);
        *average      = *average || (index == 3);
        *quasipeak    = *quasipeak || (index == 4);
        *CISPRAverage = *CISPRAverage || (index == 5);
		*CISPRRMS     = *CISPRRMS || (index == 6);

        p2buf = strtok(NULL, ",");
    }

    return rsesci_status;    
}

/*****************************************************************************
 * Function: Set Final Receiver Detector
 * Purpose:  This function selects the detector for the final measurement (the
 *           detector used subsequent to the prescan).
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetFinalReceiverDetector(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32     trace,
    ViInt32     finalDetector
)
{
    ViChar  buffer[BUFFER_SIZE];

    if (rsesci_invalidViInt32Range (trace, 1, 3))
        return VI_ERROR_PARAMETER3;

    sprintf (buffer, ":SENS%%ld:DET%ld:FME", trace);

    return rsesci_setEnumWindow (instrSession,
                                 buffer,
                                 window,
                                 finalDetector,
                                 finalDetectorArr,
                                 VI_ERROR_PARAMETER4);
}

/*****************************************************************************
 * Function: Get Final Receiver Detector
 * Purpose:  This function queries selected detector for the final measurement
 *           (the detector used subsequent to the prescan).
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetFinalReceiverDetector(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32     trace,
    ViInt32*    finalDetector
)
{
    ViChar  buffer[BUFFER_SIZE];

    if (rsesci_invalidViInt32Range (trace, 1, 3))
        return VI_ERROR_PARAMETER3;

    sprintf (buffer, ":SENS%%ld:DET%ld:FME?", trace);

    return rsesci_getEnumWindow (instrSession,
                                 buffer,
                                 window,
                                 finalDetector,
                                 finalDetectorArr,
                                 VI_ERROR_PARAMETER4);
}

/*****************************************************************************
 * Function: Peak Search
 * Purpose:  This function generates a peak list from the scan results. For
 *           this purpose, a scan must be performed prior to the command. The
 *           numeric suffix under CALCulate<1|2> is irrelevant.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_PeakSearch(
    ViSession   instrSession
)
{
    return rsesci_sendCmd (instrSession,
                           "CALC:PEAK");
}

/*****************************************************************************
 * Function: Set Peak Search Auto
 * Purpose:  This function defines whether the peak list in the spurious
 *           measurement is calculated automatically after a measurement. For
 *           each range, exactly one peak value is calculated.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetPeakSearchAuto (ViSession instrSession,
                                            ViBoolean peakSearchAuto)
{
    return rsesci_setViBoolean (instrSession
		                      , "CALC:PEAK:AUTO"
							  , peakSearchAuto
							  , VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Peak Search Auto
 * Purpose:  This function returns whether the peak list in the spurious
 *           measurement is calculated automatically after a measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetPeakSearchAuto (ViSession instrSession,
                                            ViBoolean *peakSearchAuto)
{
    return rsesci_getViBoolean (instrSession
		                      , "CALC:PEAK:AUTO?"
							  , peakSearchAuto
							  , VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Peak Search Addition
 * Purpose:  This function adds a frequency value to the peaklist in receiver
 *           mode. Only frequencies in the currently displayed frequency
 *           range of the receiver scan are allowed.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetPeakSearchAddition (ViSession instrSession,
                                                ViInt32 window,
                                                ViReal64 frequencyAddition)
{
    CHECKWINDOW;

    return rsesci_setViReal64Window (instrSession
                               	   , "CALC%ld:PEAK:ADD"
                           		   , window
                                   , frequencyAddition
                                   , 0.0, 0.0
                                   , VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set Peak Search Method
 * Purpose:  This function defines the method that is used to determine the
 *           maxima of a scan. The numeric suffix in CALCULATE<1|2> is not
 *           significant.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetPeakSearchMethod(
    ViSession   instrSession,
    ViInt32 searchMethod
)
{
    return rsesci_setEnum (instrSession,
                           "CALC:PEAK:METH",
                           searchMethod,
                           searchMethodArr,
                           VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Peak Search Method
 * Purpose:  This function queries the method that is used to determine the
 *           maxima of a scan. The numeric suffix in CALCULATE<1|2> is not
 *           significant.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetPeakSearchMethod(
    ViSession   instrSession,
    ViInt32*    searchMethod
)
{
    return rsesci_getEnum (instrSession,
                           "CALC:PEAK:METH?",
                           searchMethod,
                           searchMethodArr,
                           VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Peak Search Margin
 * Purpose:  This function determines the margin (distance to the limit line)
 *           for generating the peak list. This parameter is not used unless a
 *           limit line has been assigned to the trace. The margin, which can
 *           be individually assigned to each limit line in the limit line
 *           menu, is not used when generating the peak list.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetPeakSearchMargin(
    ViSession   instrSession,
    ViReal64    margin
)
{
    return rsesci_setViReal64 (instrSession,
                               "CALC:PEAK:MARG",
                               margin,
                               0.0 , 0.0,
                               VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Peak Search Margin
 * Purpose:  This function queries the margin (distance to the limit line) for
 *           generating the peak list.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetPeakSearchMargin(
    ViSession   instrSession,
    ViReal64*   margin
)
{
    return rsesci_getViReal64 (instrSession,
                               "CALC:PEAK:MARG?",
                               margin,
                               VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Peak Search Subranges
 * Purpose:  This function determines the maximum number of subranges or peaks
 *           obtained when generating the peak list.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetPeakSearchSubranges(
    ViSession   instrSession,
    ViInt32 subranges
)
{
    return rsesci_setViInt32 (instrSession,
                              "CALC:PEAK:SUBR",
                              subranges,
                              1 , 500,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Peak Search Subranges
 * Purpose:  This function queries the maximum number of subranges or peaks
 *           obtained when generating the peak list.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetPeakSearchSubranges(
    ViSession   instrSession,
    ViInt32*    subranges
)
{
    return rsesci_getViInt32 (instrSession,
                              "CALC:PEAK:SUBR?",
                              subranges,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Peak Search List Display
 * Purpose:  This function switches on or off the indication of the peak list
 *           or the final measurement result on the trace.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetPeakSearchListDisplay(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32 trace,
    ViInt32 indication
)
{
    ViChar  buffer[BUFFER_SIZE];

    CHECKWINDOW;

    if (rsesci_invalidViInt32Range (trace, 1, 3))
        return VI_ERROR_PARAMETER3;

    sprintf (buffer, "DISP:WIND%ld:TRAC%ld:SYMB", window, trace);

    return rsesci_setEnum (instrSession,
                           buffer,
                           indication,
                           indicationArr,
                           VI_ERROR_PARAMETER4);
}

/*****************************************************************************
 * Function: Get Peak Search List Display
 * Purpose:  This function queries selected indication symbol of the peak list
 *           or the final measurement result on the trace.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetPeakSearchListDisplay(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32 trace,
    ViInt32*    indication
)
{
    ViChar  buffer[BUFFER_SIZE];

    CHECKWINDOW;

    if (rsesci_invalidViInt32Range (trace, 1, 3))
        return VI_ERROR_PARAMETER3;

    sprintf (buffer, "DISP:WIND%ld:TRAC%ld:SYMB?", window, trace);

    return rsesci_getEnum (instrSession,
                           buffer,
                           indication,
                           indicationArr,
                           VI_ERROR_PARAMETER4);
}

/*****************************************************************************
 * Function: Clear Peak Search List
 * Purpose:  This function clears the peaklist.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_ClearPeakSearchList (ViSession instrSession,
                                              ViInt32 window)
{
    CHECKWINDOW;
    return rsesci_sendCmdWindow (instrSession, "CALC%ld:PEAK:CLE", window);
}

/*****************************************************************************
 * Function: Set Threshold Scan
 * Purpose:  This function activates or deactivates the THRESHOLD SCAN
 *           measurement function.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetThresholdScan(
    ViSession   instrSession,
    ViInt32     window,
    ViBoolean   thresholdScan
)
{
    return rsesci_setViBooleanWindow (instrSession,
                                      "SENS%ld:FME:THR",
                                      window,
                                      thresholdScan,
                                      VI_ERROR_PARAMETER3);

}

/*****************************************************************************
 * Function: Get Threshold Scan
 * Purpose:  This function queries the state of the THRESHOLD SCAN measurement
 *           function.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetThresholdScan(
    ViSession   instrSession,
    ViInt32     window,
    ViBoolean*   thresholdScan
)
{
    return rsesci_getViBooleanWindow (instrSession,
                                      "SENS%ld:FME:THR?",
                                      window,
                                      thresholdScan,
                                      VI_ERROR_PARAMETER3);

}

/*****************************************************************************
 * Function: Set NB or BB Discrimination
 * Purpose:  This function enables decision threshold to be used by the
 *           analyzer to discriminate between broadband and narrowband
 *           interference.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetNBBBDiscrimination(
    ViSession   instrSession,
    ViInt32     window,
    ViBoolean   discriminationState
)
{
    return rsesci_setViBooleanWindow (instrSession,
                                      "SENS%ld:FME:NBBB",
                                      window,
                                      discriminationState,
                                      VI_ERROR_PARAMETER3);

}

/*****************************************************************************
 * Function: Get NB or BB Discrimination
 * Purpose:  This function returns the state of decision threshold to be used
 *           by the analyzer to discriminate between broadband and narrowband
 *           interference.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetNBBBDiscrimination(
    ViSession   instrSession,
    ViInt32     window,
    ViBoolean*  discriminationState
)
{
    return rsesci_getViBooleanWindow (instrSession,
                                      "SENS%ld:FME:NBBB?",
                                      window,
                                      discriminationState,
                                      VI_ERROR_PARAMETER3);

}

/*****************************************************************************
 * Function: Set NB or BB Discrimination Value
 * Purpose:  This function enters the decision threshold value to be used by
 *           the analyzer to discriminate between broadband and narrowband
 *           interference.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetNBBBDiscriminationValue(
    ViSession   instrSession,
    ViInt32     window,
    ViReal64    discriminationThreshold
)
{
    return rsesci_setViReal64Window (instrSession,
                                     "SENS%ld:FME:NBBB:LEV",
                                     window,
                                     discriminationThreshold,
                                     0.0 , 0.0,
                                     VI_ERROR_PARAMETER3);

}

/*****************************************************************************
 * Function: Get NB or BB Discrimination Value
 * Purpose:  This function returns the decision threshold value to be used by
 *           the analyzer to discriminate between broadband and narrowband
 *           interference.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetNBBBDiscriminationValue(
    ViSession   instrSession,
    ViInt32     window,
    ViReal64*   discriminationThreshold
)
{
    return rsesci_getViReal64Window (instrSession,
                                     "SENS%ld:FME:NBBB:LEV?",
                                     window,
                                     discriminationThreshold,
                                     VI_ERROR_PARAMETER3);

}

/*****************************************************************************
 * Function: Set Final Measurement Time
 * Purpose:  This function defines the measurement time in which the values
 *           indicated in the peak list (final measurement values) are
 *           re-checked.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetFinalMeasurementTime(
    ViSession   instrSession,
    ViInt32     window,
    ViReal64    measurementTime
)
{
    return rsesci_setViReal64Window (instrSession,
                                     "SENS%ld:FME:TIME",
                                     window,
                                     measurementTime,
                                     0.0 , 0.0,
                                     VI_ERROR_PARAMETER3);

}

/*****************************************************************************
 * Function: Get Final Measurement Time
 * Purpose:  This function queries the measurement time in which the values
 *           indicated in the peak list (final measurement values) are
 *           re-checked.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetFinalMeasurementTime(
    ViSession   instrSession,
    ViInt32     window,
    ViReal64*   measurementTime
)
{
    return rsesci_getViReal64Window (instrSession,
                                     "SENS%ld:FME:TIME?",
                                     window,
                                     measurementTime,
                                     VI_ERROR_PARAMETER3);

}

/*****************************************************************************
 * Function: Set Final Measurement Auto
 * Purpose:  This function switches between automatic and interactive final
 *           measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetFinalMeasurementAuto(
    ViSession   instrSession,
    ViInt32     window,
    ViBoolean   automaticMeasurement
)
{
    return rsesci_setViBooleanWindow (instrSession,
                                      "SENS%ld:FME:AUTO",
                                      window,
                                      automaticMeasurement,
                                      VI_ERROR_PARAMETER3);

}

/*****************************************************************************
 * Function: Get Final Measurement Auto
 * Purpose:  This function queries the state of the final measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetFinalMeasurementAuto(
    ViSession   instrSession,
    ViInt32     window,
    ViBoolean*  automaticMeasurement
)
{
    ViStatus    rsesci_status           = VI_SUCCESS;
    ViInt32     automaticMeasurementTmp = 0;

    rsesci_status = rsesci_getEnumWindow (instrSession,
                                 "SENS%ld:FME:AUTO?",
                                 window,
                                 &automaticMeasurementTmp,
                                 switchArr,
                                 VI_ERROR_PARAMETER3);

    if (automaticMeasurement)
        *automaticMeasurement = (ViBoolean) automaticMeasurementTmp;

    return rsesci_status;
}

/*****************************************************************************
 * Function: Set V-network Type
 * Purpose:  This function selects the V-network to be controlled via the user
 *           port.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetVnetworkType(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32     measurement,
    ViInt32     vnetwork
)
{
    ViChar  buffer[BUFFER_SIZE];

	switch (measurement)
    {
        case RSESCI_VNET_MEAS_PRESCAN:
            sprintf (buffer, "INP%%ld:LISN");
            break;

        case RSESCI_VNET_MEAS_FINAL:
            sprintf (buffer, ":SENS%%ld:FME:LISN");
            break;
    }
    return rsesci_setEnumWindow (instrSession,
                                 buffer,
                                 window,
                                 vnetwork,
                                 vnetTypeArr,
                                 VI_ERROR_PARAMETER4);
}

/*****************************************************************************
 * Function: Get V-network Type
 * Purpose:  This function queries selected V-network controlled via the user
 *           port.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetVnetworkType(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32 measurement,
    ViInt32*    vnetwork
)
{
    ViChar  buffer[BUFFER_SIZE];

    switch (measurement)
    {
        case RSESCI_VNET_MEAS_PRESCAN:
            sprintf (buffer, "INP%%ld:LISN?");
            break;

        case RSESCI_VNET_MEAS_FINAL:
            sprintf (buffer, ":SENS%%ld:FME:LISN?");
            break;
    }
    return rsesci_getEnumWindow (instrSession,
                                 buffer,
                                 window,
                                 vnetwork,
                                 vnetTypeArr,
                                 VI_ERROR_PARAMETER4);
}

/*****************************************************************************
 * Function: Set V-network Protection Earth
 * Purpose:  This function selects the setting of the protection earth of the
 *           V-network.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetVnetworkProtectionEarth(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32 measurement,
    ViInt32 protectiveEarth
)
{
    ViChar  buffer[BUFFER_SIZE];

    switch (measurement)
    {
        case RSESCI_VNET_MEAS_PRESCAN:
            sprintf (buffer, "INP%%ld:LISN:PEAR");
            break;

        case RSESCI_VNET_MEAS_FINAL:
            sprintf (buffer, ":SENS%%ld:FME:LISN:PEAR");
            break;
    }
    return rsesci_setEnumWindow (instrSession,
                                 buffer,
                                 window,
                                 protectiveEarth,
                                 earthArr,
                                 VI_ERROR_PARAMETER4);
}

/*****************************************************************************
 * Function: Get V-network Protection Earth
 * Purpose:  This function queries selected setting of the protection earth of
 *           the V-network.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetVnetworkProtectionEarth(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32 measurement,
    ViInt32*    protectiveEarth
)
{
    ViChar  buffer[BUFFER_SIZE];

    switch (measurement)
    {
        case RSESCI_VNET_MEAS_PRESCAN:
            sprintf (buffer, "INP%%ld:LISN:PEAR?");
            break;

        case RSESCI_VNET_MEAS_FINAL:
            sprintf (buffer, ":SENS%%ld:FME:LISN:PEAR?");
            break;
    }
    return rsesci_getEnumWindow (instrSession,
                                 buffer,
                                 window,
                                 protectiveEarth,
                                 earthArr,
                                 VI_ERROR_PARAMETER4);
}

/*****************************************************************************
 * Function: Set V-network Protection Earth Final Measurement
 * Purpose:  This function selects the ground settings on the V-network on 
 *           which level measurements are consecutively performed in the final
 *           measurement. This function is available only for certain
 *           V-networks (FMEasurement:LISN FOURphase | TWOPhase | ESH3Z5 |
 *           ESH2Z5).
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetVnetworkProtectionEarthFinalMeasurement (ViSession instrSession,
                                                                     ViInt32 window,
                                                                     ViInt32 numberOfProtectives,
                                                                     ViInt32 protectiveEarth[])
{
	ViStatus rsesci_status = VI_SUCCESS;
	ViChar	*pbuffer;
	ViChar	*p2buf;
	ViInt32 i = 0;
	
	CHECKMODEL(RSESU8 | RSESU26 | RSESU40);
	
	if (rsesci_invalidViInt32Range (numberOfProtectives, 1, 2))
        return VI_ERROR_PARAMETER3;
	
    pbuffer = (ViChar*) malloc (150 * numberOfProtectives + 30);

    p2buf = pbuffer + sprintf (pbuffer, "SENS%ld:FME:LISN:PEAR ", window);
    
	for (i = 0; i < numberOfProtectives; i++)
		p2buf += sprintf (p2buf, "%s,", earthArr[protectiveEarth[i]]);
	
	*--p2buf = '\0';
	
	rsesci_status = rsesci_sendCmd (instrSession, pbuffer);
	
	free (pbuffer);
	
	return rsesci_status;
}

/*****************************************************************************
 * Function: Get V-network Protection Earth Final Measurement
 * Purpose:  This function returns the ground settings on the V-network on 
 *           which level measurements are consecutively performed in the final
 *           measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetVnetworkProtectionEarthFinalMeasurement (ViSession instrSession,
                                                                     ViInt32 window,
                                                                     ViInt32 *numberOfProtectives,
                                                                     ViInt32 protectiveEarth[])
{
	ViStatus    rsesci_status;
    ViInt32     i = 0;
	ViInt32		value = 0;
    ViChar      buffer[BUFFER_SIZE];
    ViChar      *p2buf;

    CHECKMODEL(RSESU8 | RSESU26 | RSESU40); 
	
	CHECKERR (rsesci_getViStringWindow (instrSession,
                                        "SENS%ld:FME:LISN:PEAR?",
                                        window,
                                        buffer,
                                        BUFFER_SIZE,
                                        VI_ERROR_PARAMETER1));

    p2buf = strtok (buffer, ",");

    while (p2buf && (i < 100)) /* Max 100 elements */
    {
        CHECKERR (rsesci_getIndexFromString (instrSession,
                                         p2buf,
                                         earthArr,
                                         &value));
		
		protectiveEarth[i] = value;
        i++;
        p2buf = strtok (VI_NULL, ",");
    }
	
    *numberOfProtectives = i;

    return rsesci_status;
}

/*****************************************************************************
 * Function: Set V-network Phase
 * Purpose:  This function selects the phase of the V-network on which the RFI
 *           voltage is to be measured.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetVnetworkPhase(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32 measurement,
    ViInt32 phase
)
{
    ViChar  buffer[BUFFER_SIZE];

    switch (measurement)
    {
        case RSESCI_VNET_MEAS_PRESCAN:
            sprintf (buffer, "INP%%ld:LISN:PHAS");
            break;

        case RSESCI_VNET_MEAS_FINAL:
            sprintf (buffer, ":SENS%%ld:FME:LISN:PHAS");
            break;
    }
    return rsesci_setEnumWindow (instrSession,
                                 buffer,
                                 window,
                                 phase,
                                 phaseArr,
                                 VI_ERROR_PARAMETER4);
}

/*****************************************************************************
 * Function: Get V-network Phase
 * Purpose:  This function queries selected phase of the V-network on which
 *           the RFI voltage is to be measured.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetVnetworkPhase(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32 measurement,
    ViInt32*    phase
)
{
    ViChar  buffer[BUFFER_SIZE];

    switch (measurement)
    {
        case RSESCI_VNET_MEAS_PRESCAN:
            sprintf (buffer, "INP%%ld:LISN:PHAS?");
            break;

        case RSESCI_VNET_MEAS_FINAL:
            sprintf (buffer, ":SENS%%ld:FME:LISN:PHAS?");
            break;
    }
    return rsesci_getEnumWindow (instrSession,
                                 buffer,
                                 window,
                                 phase,
                                 phaseArr,
                                 VI_ERROR_PARAMETER4);
}

/*****************************************************************************
 * Function: Set V-network Phase Final Measurement
 * Purpose:  This function selects the phases of the V-network on which level 
 *           measurements are consecutively performed in the final measurement. 
 *           L2 and L3 are only available with four-line V-networks 
 *           (FMEasurement:LISN FOURphase | ENV4200).
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetVnetworkPhaseFinalMeasurement (ViSession instrSession,
                                                           ViInt32 window,
                                                           ViInt32 numberOfPhases,
                                                           ViInt32 phases[])
{
	ViStatus rsesci_status = VI_SUCCESS;
	ViChar	*pbuffer;
	ViChar	*p2buf;
	ViInt32 i = 0;
	
	CHECKMODEL(RSESU8 | RSESU26 | RSESU40);
	
    pbuffer = (ViChar*) malloc (150 * numberOfPhases + 30);

    p2buf = pbuffer + sprintf (pbuffer, "SENS%ld:FME:LISN:PHAS ", window);
    
	for (i = 0; i < numberOfPhases; i++)
		p2buf += sprintf (p2buf, "%s,", phaseArr[phases[i]]);
	
	*--p2buf = '\0';
	
	rsesci_status = rsesci_sendCmd (instrSession, pbuffer);
	
	free (pbuffer);
	
	return rsesci_status;    
}

/*****************************************************************************
 * Function: Get V-network Phase Final Measurement
 * Purpose:  This function returns the phases of the V-network on which level 
 *           measurements are consecutively performed in the final measurement. 
 *           L2 and L3 are only available with four-line V-networks 
 *           (FMEasurement:LISN FOURphase | ENV4200).
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetVnetworkPhaseFinalMeasurement (ViSession instrSession,
                                                           ViInt32 window,
                                                           ViInt32 *numberOfPhases,
                                                           ViInt32 phases[])
{
	ViStatus    rsesci_status;
    ViInt32     i = 0;
	ViInt32		value = 0;
    ViChar      buffer[BUFFER_SIZE];
    ViChar      *p2buf;

    CHECKMODEL(RSESU8 | RSESU26 | RSESU40); 
	
	CHECKERR (rsesci_getViStringWindow (instrSession,
                                        "SENS%ld:FME:LISN:PHAS?",
                                        window,
                                        buffer,
                                        BUFFER_SIZE,
                                        VI_ERROR_PARAMETER1));

    p2buf = strtok (buffer, ",");

    while (p2buf && (i < 100)) /* Max 100 elements */
    {
        CHECKERR (rsesci_getIndexFromString (instrSession,
                                         p2buf,
                                         phaseArr,
                                         &value));
		
		phases[i] = value;
        i++;
        p2buf = strtok (VI_NULL, ",");
    }
	
    *numberOfPhases = i;

    return rsesci_status;    
}

/*****************************************************************************
 * Function: Set V-network High Pass
 * Purpose:  This function selects the setting for the high pass on the V-network.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetVnetworkHighPass (ViSession instrSession,
                                              ViInt32 window,
                                              ViInt32 measurement,
                                              ViBoolean phase)
{
	ViStatus	rsesci_status = VI_SUCCESS;
	ViChar  buffer[BUFFER_SIZE];
	ViInt32 v_type = 0;
	
	CHECKMODEL(RSESU8 | RSESU26 | RSESU40);
	
	CHECKERR (rsesci_GetVnetworkType (instrSession, window,RSESCI_VNET_MEAS_PRESCAN, &v_type));
	
	if (v_type != RSESCI_VNET_ENV216)
		return rsesci_status;
	
    switch (measurement)
    {
        case RSESCI_VNET_MEAS_PRESCAN:
            sprintf (buffer, "INP%%ld:LISN:FILT:HPAS:STAT");
            break;

        case RSESCI_VNET_MEAS_FINAL:
            sprintf (buffer, ":SENS%%ld:FME:LISN:FILT:HPAS:STAT");
            break;
    }
    return rsesci_setViBooleanWindow (instrSession,
                                      buffer,
                                      window,
                                      phase,
                                      VI_ERROR_PARAMETER4);    
}

/*****************************************************************************
 * Function: Get V-network High Pass
 * Purpose:  This function returns the setting for the high pass on the V-network.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetVnetworkHighPass (ViSession instrSession,
                                              ViInt32 window,
                                              ViInt32 measurement,
                                              ViBoolean *phase)
{
	ViStatus	rsesci_status = VI_SUCCESS;
	ViChar  buffer[BUFFER_SIZE];
	ViInt32 v_type = 0;
	
	CHECKMODEL(RSESU8 | RSESU26 | RSESU40);
	
	CHECKERR (rsesci_GetVnetworkType (instrSession, window,RSESCI_VNET_MEAS_PRESCAN, &v_type));
	
	if (v_type != RSESCI_VNET_ENV216)
		return rsesci_status;
	
    switch (measurement)
    {
        case RSESCI_VNET_MEAS_PRESCAN:
            sprintf (buffer, "INP%%ld:LISN:FILT:HPAS:STAT?");
            break;

        case RSESCI_VNET_MEAS_FINAL:
            sprintf (buffer, ":SENS%%ld:FME:LISN:FILT:HPAS:STAT?");
            break;
    }
    return rsesci_getViBooleanWindow (instrSession,
                                      buffer,
                                      window,
                                      phase,
                                      VI_ERROR_PARAMETER4);    
}

/*****************************************************************************
 * Function: Set Demodulation Type
 * Purpose:  This function selects the type of analog demodulation.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetDemodulationType(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32 demodulation
)
{
    return rsesci_setEnumWindow (instrSession,
                                 "SENS%ld:DEM",
                                 window,
                                 demodulation,
                                 demodulationArr,
                                 VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Demodulation Type
 * Purpose:  This function queries selected type of analog demodulation.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetDemodulationType(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32*    demodulation
)
{
    return rsesci_getEnumWindow (instrSession,
                                 "SENS%ld:DEM?",
                                 window,
                                 demodulation,
                                 demodulationArr,
                                 VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set Squelch Receiver
 * Purpose:  This function enables the input of a level threshold below which 
 *           the audible AF is cut off. The squelch function is associated with 
 *           the internal trigger function (TRIGGER menu), which will be switched
 *           on automatically with the squelch. Squelch level and trigger
 *           level do have the same value.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetSquelchReceiver (ViSession instrSession,
                                             ViInt32 window,
                                             ViBoolean squelchState,
                                             ViReal64 squelchLevel)
{
	ViStatus 	rsesci_status = VI_SUCCESS;
	
	CHECKWINDOW;

    rsesci_status = rsesci_setViBooleanWindow (instrSession,
		                                       "SENS%ld:DEM:SQU:STAT",
										       window,
										       squelchState,
										       VI_ERROR_PARAMETER3);
	
	if (squelchState == VI_TRUE)
	{
		rsesci_status = rsesci_setViReal64Window (instrSession,
			                                      "SENS%ld:DEM:SQU:LEV",
											      window,
										          squelchLevel,
											      0.0, 100.0,
											      VI_ERROR_PARAMETER4);
	}
	
	return rsesci_status;    
}

/*****************************************************************************
 * Function: Get Squelch Receiver
 * Purpose:  This function returns the input of a level threshold below which 
 *           the audible AF is cut off.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetSquelchReceiver (ViSession instrSession,
                                             ViInt32 window,
                                             ViBoolean *squelchState,
                                             ViReal64 *squelchLevel)
{
	ViStatus 	rsesci_status = VI_SUCCESS;
	
	CHECKWINDOW;

    rsesci_status = rsesci_getViBooleanWindow (instrSession,
		                                       "SENS%ld:DEM:SQU:STAT?",
										       window,
										       squelchState,
										       VI_ERROR_PARAMETER3);
	
	rsesci_status = rsesci_getViReal64Window (instrSession,
		                                      "SENS%ld:DEM:SQU:LEV?",
										      window,
									          squelchLevel,
										      VI_ERROR_PARAMETER4);
	
	return rsesci_status;    
}

/*****************************************************************************
 * Function: Read Receiver Detector
 * Purpose:  This function is possible to use only as a query for single
             measurements in the receiver mode. The values of all activated
             detectors are transferred. For inactive detectors, the value is
             omitted at the position(s) in question.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_ReadReceiverDetector (
    ViSession   instrSession,
    ViReal64    *firstDetector,
    ViReal64    *secondDetector,
    ViReal64    *thirdDetector
)
{
    ViStatus    rsesci_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    ViUInt32    return_count;

    if (firstDetector) *firstDetector = 0.0;
    if (secondDetector) *secondDetector = 0.0;
    if (thirdDetector) *thirdDetector = 0.0;

    CHECKERR (rsesci_clearBeforeRead(instrSession));

    sprintf (buffer, "*CLS;:FORM ASC;:TRAC? SING\n");

    CHECKERR( viWrite (instrSession, (ViBuf) buffer, (ViUInt32) strlen (buffer), VI_NULL));
    CHECKERR( rsesci_checkBeforeRead (instrSession));
    CHECKERR( viRead (instrSession, (ViBuf) buffer, BUFFER_SIZE, &return_count));
    buffer[return_count] = '\0';

    if (sscanf (buffer, "%le%*1s%le%*1s%le%*1s",
                firstDetector,
                secondDetector,
                thirdDetector) < 1)
    {
        return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
    }

    CHECKERR( rsesci_checkStatus(instrSession));

    return rsesci_status;
}

/*****************************************************************************
 * Function: Read Receiver Detector Peak Hold
 * Purpose:  This function is possible to use only as a query for single
             measurements in the receiver mode. The peak hold values of all
             activated detectors are transferred. For inactive detectors, the
             value is omitted at the position(s) in question.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_ReadReceiverDetectorPeakHold (
    ViSession instrSession,
    ViReal64 _VI_FAR firstDetector[],
    ViReal64 _VI_FAR secondDetector[],
    ViReal64 _VI_FAR thirdDetector[]
)
{
    ViStatus    rsesci_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    ViUInt32    return_count;

    if (firstDetector)
        {
        firstDetector[0] = 0.0;
        firstDetector[1] = 0.0;
        }
    if (secondDetector)
        {
        secondDetector[0] = 0.0;
        secondDetector[1] = 0.0;
        }
    if (thirdDetector)
        {
        thirdDetector[0] = 0.0;
        thirdDetector[1] = 0.0;
        }

    CHECKERR (rsesci_clearBeforeRead(instrSession));

    sprintf (buffer, "*CLS;:FORM ASC;:TRAC? PHOL\n");

    CHECKERR( viWrite (instrSession, (ViBuf) buffer, (ViUInt32) strlen (buffer), VI_NULL));
    CHECKERR( rsesci_checkBeforeRead (instrSession));
    CHECKERR( viRead (instrSession, (ViBuf) buffer, BUFFER_SIZE, &return_count));
    buffer[return_count] = '\0';

    if (sscanf (buffer, "%le%*1s%le%*1s%le%*1s%le%*1s%le%*1s%le%*1s",
                &firstDetector[0], &firstDetector[1],
                &secondDetector[0], &secondDetector[1],
                &thirdDetector[0], &thirdDetector[1]) < 1)
        {
        return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        }

    CHECKERR( rsesci_checkStatus(instrSession));

    return rsesci_status;
}

/*****************************************************************************
 * Function: Read Scan Measurement
 * Purpose:  This function returns the result of the scan measurement. The
             number of transmitted measurement results depends on the scan
             settings.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_ReadScanMeasurement (
    ViSession   instrSession,
    ViInt32     arraySize,
    ViInt32     *traceStatus,
    ViInt32     *noofPoints,
    ViChar      _VI_FAR statusData[],
    ViReal64    _VI_FAR trace1[],
    ViReal64    _VI_FAR trace2[],
    ViReal64    _VI_FAR trace3[]
)
{
    ViStatus    rsesci_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    ViUInt32    return_count;

    ViInt32     lenLength   = 0,
                T1          = 0,
                T2          = 0,
                T3          = 0,
                T4          = 0,
                i           = 0,
                points      = 0;
    ViReal32    *temp;
    ViInt32     *p2data;
    ViChar      *exBuf;
    ViUInt16    remote_interface = VI_INTF_GPIB;

    if (arraySize < 1)
        return VI_ERROR_PARAMETER2;
    if (!traceStatus)
        return VI_ERROR_PARAMETER3;
    if (!noofPoints)
        return VI_ERROR_PARAMETER4;
    if (!statusData)
        return VI_ERROR_PARAMETER5;

    /* Initial settings. */
    *noofPoints = 0;
    statusData[0] = '\0';
    *traceStatus = 0;
    /* Temporary setings. */
    exBuf = buffer;

    CHECKERR( viGetAttribute (instrSession, VI_ATTR_INTF_TYPE, &remote_interface));

    CHECKERR (rsesci_clearBeforeRead(instrSession));

    sprintf (buffer, ":FORM REAL,32;*CLS;:TRAC? SCAN\n");
    /* Returns the result of the scan measurement

    Note:

    Structure of data:
    - 4 byte: trace status:
     bit 0 to 9 subscan
     bit 10: last block of subscan
     Bit 11: last block of last subscan of scan
     Bit 12: last of all blocks (for multiple scans after the last
    scan)
    - 4 bytes:number n of the transmitted measurement results of a
    trace
    - 4 byte:trace1 active (0/1)
    - 4 byte:trace2 active (0/1)
    - 4 byte:trace3 active (0/1)
    - 4 byte:reserved
    - n*4 bytes:measurement results of trace 1 if trace 1 is active
    - n*4 bytes:measurement results of trace 2 if trace 2 is active
    - n*4 bytes:measurement results of trace 3 if trace 3 is active
    - n*1 byte:status information per measurement result:
      bit 0: underrange trace1
      bit 1: underrange trace2
      bit 2: underrange trace3
      bit 4: overrange trace1 to trace4

    Where n is number of results. */

    switch (remote_interface)
        {
        case VI_INTF_GPIB: default: /* GPIB, RSIB Interface. */

            CHECKERR( viWrite (instrSession, (ViBuf) buffer, (ViUInt32) strlen (buffer), VI_NULL));
            if ((rsesci_status = rsesci_checkBeforeRead (instrSession)) < 0)
                {
                viPrintf (instrSession, "FORM ASCII;*CLS\n");
                return rsesci_status;
                }

            if ((rsesci_status = viRead (instrSession, (ViBuf) buffer, 2, &return_count)) < 0)
            {
                viPrintf (instrSession, "FORM ASCII\n");
                return rsesci_status;
            }
            buffer[return_count] = '\0';

            /* Get header info. */
            if (sscanf (buffer, "#%1d", &lenLength) != 1)
            {
                viPrintf (instrSession, "FORM ASCII");
                return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
            }

            if ((rsesci_status = viRead (instrSession, (ViBuf) buffer, lenLength, &return_count)) < 0)
            {
                viPrintf (instrSession, "FORM ASCII\n");
                return rsesci_status;
            }
            buffer[return_count] = '\0';

            if (sscanf (buffer, "%ld", noofPoints) != 1)
            {
                viPrintf (instrSession, "FORM ASCII\n");
                return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
            }

            /* Allocate memory space */
            if ((exBuf = malloc (*noofPoints + 1)) == NULL)
            {
                viPrintf (instrSession, "FORM ASCII\n");
                return VI_ERROR_ALLOC;
            }
            if ((rsesci_status = viRead (instrSession, (ViBuf) exBuf, *noofPoints + 1, &return_count)) < 0)
                {
                viPrintf (instrSession, "FORM ASCII\n");
                /* If not NULL pointer, then deallocate memory. */
                if (exBuf) free (exBuf);
                return rsesci_status;
                }

        break;
        case VI_INTF_ASRL: /* Serial Support. */

            CHECKERR( viWrite (instrSession, (ViBuf) buffer, (ViUInt32) strlen (buffer), VI_NULL< 0));
            if ((rsesci_status = rsesci_checkBeforeRead (instrSession)) < 0)
            {
                viPrintf (instrSession, "FORM ASCII;*CLS\n");
                return rsesci_status;
            }

            /* Switch to Off using of termination character. */
            if ((rsesci_status = viSetAttribute (instrSession, VI_ATTR_ASRL_END_IN, VI_ASRL_END_NONE)) < 0)
            {
                viPrintf (instrSession, "FORM ASCII\n");
                return rsesci_status;
            }
            
            if ((rsesci_status =  viSetAttribute (instrSession, VI_ATTR_TERMCHAR_EN, VI_FALSE)) < 0)
            {
                viPrintf (instrSession, "FORM ASCII\n");
                return rsesci_status;
            }

            /* Parse binary data header. */
            if ((rsesci_status = viRead (instrSession, (ViBuf) buffer, 2, VI_NULL)) < 0)
                {
                viSetAttribute (instrSession, VI_ATTR_ASRL_END_IN, VI_ASRL_END_TERMCHAR);
                viSetAttribute (instrSession, VI_ATTR_TERMCHAR_EN, VI_TRUE);
                viPrintf (instrSession, "FORM ASCII\n");
                return rsesci_status;
                }

            if (sscanf (buffer, "#%1d", &lenLength) != 1)
                {
                viSetAttribute (instrSession, VI_ATTR_ASRL_END_IN, VI_ASRL_END_TERMCHAR);
                viSetAttribute (instrSession, VI_ATTR_TERMCHAR_EN, VI_TRUE);
                viPrintf (instrSession, "FORM ASCII\n");
                return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
                }

            /* Get number of points. */
            if ((rsesci_status = viRead (instrSession, (ViBuf) buffer, lenLength, VI_NULL)) < 0)
                {
                viSetAttribute (instrSession, VI_ATTR_ASRL_END_IN, VI_ASRL_END_TERMCHAR);
                viSetAttribute (instrSession, VI_ATTR_TERMCHAR_EN, VI_TRUE);
                viPrintf (instrSession, "FORM ASCII\n");
                return rsesci_status;
                }

            if (sscanf (buffer, "%ld", noofPoints) != 1)
                {
                viSetAttribute (instrSession, VI_ATTR_ASRL_END_IN, VI_ASRL_END_TERMCHAR);
                viSetAttribute (instrSession, VI_ATTR_TERMCHAR_EN, VI_TRUE);
                viPrintf (instrSession, "FORM ASCII\n");
                return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
                }

            /* Allocate memory space */
            if ((exBuf = malloc (*noofPoints + 2)) == NULL)
            {
                viPrintf (instrSession, "FORM ASCII\n");
                return VI_ERROR_ALLOC;
            }
            /* Read exact number of bytes equal to number of points. */
            if ((rsesci_status = viRead (instrSession, (ViBuf) exBuf, *noofPoints + 2, &return_count)) < 0)
                {
                viSetAttribute (instrSession, VI_ATTR_ASRL_END_IN, VI_ASRL_END_TERMCHAR);
                viSetAttribute (instrSession, VI_ATTR_TERMCHAR_EN, VI_TRUE);
                viPrintf (instrSession, "FORM ASCII\n");
                /* If not NULL pointer, then deallocate memory. */
                if (exBuf) free (exBuf);
                return rsesci_status;
                }

            /* Switch to On using of termination character. */
            viSetAttribute (instrSession, VI_ATTR_ASRL_END_IN, VI_ASRL_END_TERMCHAR);
            viSetAttribute (instrSession, VI_ATTR_TERMCHAR_EN, VI_TRUE);

        break;
    }

    /* Get data info. */
    p2data = (ViInt32*)(exBuf);

    /* Binary data byte order (MOTOROLA or INTEL). */
    rsesci_SWAP_32((unsigned char*) exBuf, 2);

    *traceStatus = (ViInt32)*p2data++;
    *noofPoints = (ViInt32)*p2data++;
    T1 = (ViInt32)*p2data++;
    T2 = (ViInt32)*p2data++;
    T3 = (ViInt32)*p2data++;

    points = (arraySize > *noofPoints) ? *noofPoints : arraySize;

    /* Get trace data. */
    if (T1 || T2 || T3) {

        /* Trace 1 */
        if (T1)
        {
            /* Binary data byte order (MOTOROLA or INTEL). */
            rsesci_SWAP_32((unsigned char*) exBuf+24, *noofPoints);

            temp = (ViReal32*)(exBuf+24);

            for (i = 0; i < points; i++)
                *trace1++ = (ViReal64)*temp++;
        }
        else
            trace1 = memset (trace1, 0, 8*(points));
        /* Trace 2 */
        if (T2)
        {
            /* Binary data byte order (MOTOROLA or INTEL). */
            rsesci_SWAP_32((unsigned char*) exBuf+24+4*(*noofPoints)*(T1), *noofPoints);

            temp = (ViReal32*)(exBuf+24+4*(*noofPoints)*(T1));

            for (i = 0; i < points; i++)
                *trace2++ = (ViReal64)*temp++;
        }
        else
            trace2 = memset (trace2, 0, 8*(points));
        /* Trace 3 */
        if (T3)
        {
            /* Binary data byte order (MOTOROLA or INTEL). */
            rsesci_SWAP_32((unsigned char*) exBuf+24+4*(*noofPoints)*(T1+T2), *noofPoints);

            temp = (ViReal32*)(exBuf+24+4*(*noofPoints)*(T1+T2));

            for (i = 0; i < points; i++)
                *trace3++ = (ViReal64)*temp++;
        }
        else
            trace3 = memset (trace3, 0, 8*(points));
    }
    else
    {
        trace1 = memset (trace1, 0, 8*(points));
        trace2 = memset (trace2, 0, 8*(points));
        trace3 = memset (trace3, 0, 8*(points));
    }

    /* Remove header and remaining data - move pointer to status data. */
    statusData = memmove (statusData, exBuf+24+4*(*noofPoints)*(T1+T2+T3+T4), points);

    statusData[points] = '\0';

    if (exBuf) /* If not NULL pointer, then deallocate memory. */
        free (exBuf);

    if ((rsesci_status = viPrintf (instrSession, "FORM ASCII")) < 0)
        return rsesci_status;

    if ((rsesci_status = rsesci_checkStatus (instrSession)) < 0)
        return rsesci_status;

    return rsesci_status;
}

/*****************************************************************************
 * Function: Read Scan Measurement Status
 * Purpose:  This function returns the status information per measurement
             result during scan measurements.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_ReadScanMeasurementStatus (
    ViSession   instrSession,
    ViInt32     arraySize,
    ViInt32     *noofPoints,
    ViChar      _VI_FAR statusData[]
)
{
    ViStatus    rsesci_status = VI_SUCCESS;
    ViStatus    warning       = VI_SUCCESS;
    ViUInt16    remote_interface = VI_INTF_GPIB;
    ViChar      buffer[BUFFER_SIZE];
    ViUInt32    return_count = 0;
    ViInt32     nb_of_bytes;
    ViInt32     digits;
    ViChar*     pdynamic_data = NULL;
    ViChar*     pdata         = NULL;

    CHECKERR( viGetAttribute (instrSession, VI_ATTR_INTF_TYPE, &remote_interface));

    if (!statusData)
        return VI_ERROR_PARAMETER4;

    CHECKERR (rsesci_clearBeforeRead(instrSession));

    sprintf (buffer, ":FORM UINT,8;*CLS;:TRAC? STAT\n");

    if (remote_interface != VI_INTF_ASRL)
    {
        /*
         * GPIB, RSIB Interface.
         */
        CHECKERR( viWrite (instrSession, (ViBuf) buffer, (ViUInt32) strlen (buffer), VI_NULL));
        CHECKERR( rsesci_checkBeforeRead (instrSession));
        CHECKERR( rsesci_readDataUnknownLength(instrSession, &pdynamic_data, &return_count));

        /*
         * Set pointer to number of digits.
         * in dynamic buffer data have following format
         * #3128<Binary Data>
         */
        pdata = pdynamic_data + 1;
        digits = *pdata - '0';

        memcpy(buffer, pdynamic_data + 2, digits);
        *(buffer + digits) = '\0';
        nb_of_bytes = atoi(buffer);

        /*
         * Set pointer to data
         * we must skip header. first two characters in header is #x,
         * where x is number of digits.
         */
        pdata = pdynamic_data + 2 + digits;

    }
    else
    { /* Serial Interface. */
        CHECKERR( viWrite (instrSession, (ViBuf) buffer, (ViUInt32) strlen (buffer), VI_NULL));
        CHECKERR( rsesci_checkBeforeRead (instrSession));

        /* Switch to Off using of termination character. */
        CHECKERR( viSetAttribute (instrSession, VI_ATTR_ASRL_END_IN, VI_ASRL_END_NONE));
        CHECKERR( viSetAttribute (instrSession, VI_ATTR_TERMCHAR_EN, VI_FALSE));

        /* Parse binary data header. */
        if ((rsesci_status = viRead (instrSession, (ViBuf) buffer, 2, &return_count)) < 0)
            {
            viSetAttribute (instrSession, VI_ATTR_ASRL_END_IN, VI_ASRL_END_TERMCHAR);
            viSetAttribute (instrSession, VI_ATTR_TERMCHAR_EN, VI_TRUE);
            return rsesci_status;
            }

        if (sscanf (buffer, "#%1d", &digits) != 1)
            {
            viSetAttribute (instrSession, VI_ATTR_ASRL_END_IN, VI_ASRL_END_TERMCHAR);
            viSetAttribute (instrSession, VI_ATTR_TERMCHAR_EN, VI_TRUE);
            return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
            }

        /* Get number of points. */
        if ((rsesci_status = viRead (instrSession, (ViBuf) buffer, digits, &return_count)) < 0)
            {
            viSetAttribute (instrSession, VI_ATTR_ASRL_END_IN, VI_ASRL_END_TERMCHAR);
            viSetAttribute (instrSession, VI_ATTR_TERMCHAR_EN, VI_TRUE);
            return rsesci_status;
            }
        buffer[return_count] = '\0';

        if (sscanf (buffer, "%ld", &nb_of_bytes) != 1)
            {
            viSetAttribute (instrSession, VI_ATTR_ASRL_END_IN, VI_ASRL_END_TERMCHAR);
            viSetAttribute (instrSession, VI_ATTR_TERMCHAR_EN, VI_TRUE);
            return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
            }

        /* Read exact number of bytes equal to number of points. */
        pdynamic_data = (ViChar *)calloc (nb_of_bytes + 2, sizeof(ViChar));
        if (buffer == VI_NULL) return VI_ERROR_ALLOC;

        if ((rsesci_status = viRead (instrSession, (ViBuf) pdynamic_data, nb_of_bytes + 2, &return_count)) < 0)
            {
            free( pdynamic_data);
            viSetAttribute (instrSession, VI_ATTR_ASRL_END_IN, VI_ASRL_END_TERMCHAR);
            viSetAttribute (instrSession, VI_ATTR_TERMCHAR_EN, VI_TRUE);
            return rsesci_status;
            }
        /*
         * Set pointer to data
         */
        pdata = pdynamic_data;

        /* Switch to On using of termination character. */
        viSetAttribute (instrSession, VI_ATTR_ASRL_END_IN, VI_ASRL_END_TERMCHAR);
        viSetAttribute (instrSession, VI_ATTR_TERMCHAR_EN, VI_TRUE);
    }

    if (arraySize)
    {
        ViInt32 copy_bytes;

        if ( arraySize < 0 )
            copy_bytes = nb_of_bytes;
        else
        {
            copy_bytes = (arraySize < nb_of_bytes) ? arraySize : nb_of_bytes;
            warning    = (arraySize < nb_of_bytes) ? nb_of_bytes : VI_SUCCESS;
        }

        memcpy(statusData, pdata, copy_bytes);

        if (noofPoints) *noofPoints = copy_bytes;
    }
    else
        warning = nb_of_bytes;

    if (pdynamic_data)
        free(pdynamic_data);

    CHECKERR( rsesci_checkStatus (instrSession));

    return (rsesci_status = warning);
}

/*****************************************************************************
 * Function: Read Final Measurement
 * Purpose:  This function returns the results of the final scan measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_ReadFinalMeasurement (
    ViSession   instrSession,
    ViInt32     trace,
    ViInt32     arraySize,
    ViInt32     *number_ofResults,
    ViReal64    _VI_FAR frequency[],
    ViReal64    _VI_FAR level[],
    ViReal64    _VI_FAR deltaLimit[]
)
{
    ViStatus    rsesci_status = VI_SUCCESS;
    ViChar      cmd[BUFFER_SIZE];
    ViChar      *pbuffer        = VI_NULL;
    ViChar      *p2buf;
    ViUInt32    local_count     = 0;
    ViInt32     cnt             = 0,
                idx             = 0;
    ViReal64    *values;

    if (rsesci_invalidViInt32Range (trace, 1, 3))
        return VI_ERROR_PARAMETER2;
    if (arraySize < 1)
        return VI_ERROR_PARAMETER3;
    if (!number_ofResults)
        return VI_ERROR_PARAMETER4;
    if (!frequency)
        return VI_ERROR_PARAMETER5;
    if (!level)
        return VI_ERROR_PARAMETER6;
    if (!deltaLimit)
        return VI_ERROR_PARAMETER7;

    /* Create common temporary value array */
    values = malloc (3 * arraySize * sizeof (ViReal64));

    sprintf (cmd, ":FORM ASC;:TRAC? FINAL%ld", trace);

    GOCHECKERR(rsesci_clearBeforeRead (instrSession));
    GOCHECKERR (viPrintf(instrSession, "%s\n",cmd));
    GOCHECKERR (rsesci_checkBeforeRead (instrSession));
    GOCHECKERR (rsesci_readDataUnknownLength(instrSession, &pbuffer, &local_count));
    p2buf = pbuffer;

    p2buf = strtok(pbuffer, ",");

    while(p2buf && (cnt < arraySize))
        {
        values[cnt++] = atof(p2buf);
        p2buf = strtok(NULL, ",");
        };

    /* Distribute all measured values */
    cnt /= 3;
    for (idx = 0; idx < cnt; idx++)
        {
        frequency[idx]  = values[0 + idx * 3];
        level[idx]      = values[1 + idx * 3];
        deltaLimit[idx] = values[2 + idx * 3];
        }

    *number_ofResults = cnt;

    GOCHECKERR (rsesci_checkStatus (instrSession));

ERROR:

    free(pbuffer);
    free(values);

    return rsesci_status;
}

/*****************************************************************************
 * Function: Read Additional Measurement Data
 * Purpose:  This function returns data that represent the values in array.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_ReadAdditionalMeasurementData (ViSession instrSession,
                                                        ViInt32 window,
                                                        ViInt32 trace,
                                                        ViInt32 arrayLength,
                                                        ViInt32 *actualPoints,
                                                        ViReal64 values[])
{
	ViUInt32 retCnt = 0;
    ViChar buffer[BUFFER_SIZE];
    ViChar *exBuf;
    ViReal32 *temp;
    ViInt32 lenLength, i = 0;
    ViStatus rsesci_status = VI_SUCCESS;
    ViUInt16 intfc = VI_INTF_GPIB;
    rsesci_instrRange instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_INTF_TYPE, &intfc));

    if (rsesci_invalidViInt32Range (trace, RSESCI_TRAC_ABIT, RSESCI_TRAC_PWCD))
        return VI_ERROR_PARAMETER2;

    
    CHECKERR (rsesci_clearBeforeRead(instrSession));
    /* Interface specific operations */
    switch (intfc)
        {
        case VI_INTF_ASRL: /* Serial Interface. */
            sprintf (buffer, ":FORM ASCII;*CLS;:TRAC%ld? %s\n", window, addTraceNamesArr[trace]);

            CHECKERR(rsesci_clearBeforeRead (instrSession));
            CHECKERR (viWrite (instrSession, (ViBuf) buffer, (ViUInt32) strlen (buffer), &retCnt));
            CHECKERR (rsesci_checkBeforeRead (instrSession));
            CHECKERR (viScanf (instrSession, "%,#Lf", actualPoints, values));
        break;
        case VI_INTF_GPIB: default:  /* GPIB Interface. Default case */
            sprintf (buffer, ":FORM REAL,32;*CLS;:TRAC%ld? %s\n", window, addTraceNamesArr[trace]);

            if ((rsesci_status = viWrite (instrSession, (ViBuf) buffer, (ViUInt32) strlen (buffer),
                &retCnt)) < 0)
                return rsesci_status;
            if ((rsesci_status = rsesci_checkBeforeRead (instrSession)) < 0)
            {
                viPrintf (instrSession, "FORM ASCII");
                return rsesci_status;
            }

            if ((rsesci_status = viRead (instrSession, (ViBuf) buffer, 2, &retCnt)) < 0)
            {
                viPrintf (instrSession, "FORM ASCII");
                return rsesci_status;
            }

            if (sscanf (buffer, "#%1d", &lenLength) != 1) {
                viPrintf (instrSession, "FORM ASCII");
                rsesci_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
                return rsesci_status;
            }

            if ((rsesci_status = viRead (instrSession, (ViBuf) buffer, lenLength,
                &retCnt)) < 0)
            {
                viPrintf (instrSession, "FORM ASCII");
                return rsesci_status;
            }

            if (sscanf (buffer, "%ld", actualPoints) != 1) {
                viPrintf (instrSession, "FORM ASCII");
                rsesci_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
                return rsesci_status;
            }

            /* Allocate memory space */
            if ((exBuf = malloc (*actualPoints + 1)) == NULL)
            {
                viPrintf (instrSession, "FORM ASCII");
                return VI_ERROR_ALLOC;
            }
            if ((rsesci_status = viRead (instrSession, (ViBuf) exBuf, *actualPoints + 1,
                &retCnt)) < 0)
                {
                /* If not NULL pointer, then deallocate memory. */
                if (exBuf) free (exBuf);
                viPrintf (instrSession, "FORM ASCII");
                return rsesci_status;
                }
/******/
			if (arrayLength < *actualPoints) *actualPoints = arrayLength;
            /* Convert binary data into array of ViReal64 types. */
            *actualPoints = (ViInt32)(*actualPoints/4);
            /* Binary data byte order (MOTOROLA or INTEL). */
            rsesci_SWAP_32((unsigned char*) exBuf, *actualPoints);
            /* Pointer to data */
            temp = (ViReal32*)(exBuf);
            for (i = 0; i < *actualPoints; i++)
                *values++ = (ViReal64)*temp++;

            if (exBuf) /* If not NULL pointer, then deallocate memory. */
                free (exBuf);
            if ((rsesci_status = viPrintf (instrSession, "FORM ASCII")) < 0)
                return rsesci_status;

        break;
        }

    if ((rsesci_status = rsesci_checkStatus (instrSession)) < 0)
        return rsesci_status;

    return rsesci_status;    
}

/*****************************************************************************
 * Function: Get Current Results
 * Purpose:  This function returns the current X and Y results of the sweep. 
 *           During a sweep the last measured values are read out.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetCurrentResults (ViSession instrSession,
                                            ViInt32 window, ViReal64 *xValue,
                                            ViReal64 *yValue)
{
	CHECKWINDOW;
	
    if (!xValue)
        return VI_ERROR_PARAMETER3;
    if (!yValue)
        return VI_ERROR_PARAMETER4;
        
    return viQueryf (instrSession, "TRAC%ld:IMM:RES?\n", "%le,%le", window, xValue, yValue); 
        
}

/*****************************************************************************
 * Function: Get Current Y Results
 * Purpose:  This function returns the current Y results of the sweep. During 
 *           a sweep the last measured value is read out.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetCurrentYResult (ViSession instrSession,
                                            ViInt32 window, ViReal64 *level)
{
	CHECKWINDOW;
	
    return rsesci_getViReal64Window (instrSession, 
		                            "TRAC%ld:IMM:LEV?", 
									window,
									level,
									VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set Channel List
 * Purpose:  This function defines the points (channels) to be measured in
 *           the selected scan range by specifying the channel index
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetChannelList (ViSession instrSession,
                                         ViInt32 noOfValues, ViInt32 scanTable,
                                         ViInt32 channelList[])
{
    ViChar  buffer[BUFFER_SIZE] = "";
	ViInt32 i=0;

    if (noOfValues < 1)
        return VI_ERROR_PARAMETER1;
	
	if (rsesci_invalidViInt32Range (scanTable, 1, 10))
        return VI_ERROR_PARAMETER2;
	
	for (i=0; i < noOfValues; i++)
	{
		sprintf (buffer, "%s,%ld", buffer, channelList[i]);
	}
	buffer[0] = ' ';
	sprintf (buffer, "SENS:SCAN%ld:LIST:SEQU%s", scanTable, buffer);
    
    return rsesci_sendCmd (instrSession, buffer);
}

/*****************************************************************************
 * Function: Get Channel List
 * Purpose:  This function returns the points (channels) to be measured in
 *           the selected scan range by specifying the channel index.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetChannelList (ViSession instrSession,
                                         ViInt32 arraySize, ViInt32 scanTable,
                                         ViInt32 channelList[],
										 ViInt32 *returnedValues)
{
	ViChar     buffer[BUFFER_SIZE] = "";
	ViChar    *buf=VI_NULL;
	ViChar    *pbuffer = NULL;
	ViChar    *p2buf = NULL;
	ViUInt32   retCnt = 0;
	ViInt32    i=0;
	
	ViStatus   rsesci_status;
	
	if (arraySize < 1)
        return VI_ERROR_PARAMETER2;
	
	if (rsesci_invalidViInt32Range (scanTable, 1, 10))
        return VI_ERROR_PARAMETER3;
	
	sprintf (buffer, "SENS:SCAN%ld:LIST:SEQU?", scanTable);
	
	CHECKERR (rsesci_sendCmd (instrSession, "buffer"));
	
	CHECKERR (rsesci_readDataUnknownLength(instrSession, &buf, &retCnt));
	
	p2buf = strtok (pbuffer, ",\r\n");
	
	while(p2buf != NULL)
	{
		if (i < arraySize)
		{
			channelList[i] = atoi(p2buf);	
		}
		
		p2buf = strtok (NULL, ",\r\n");
		i++;
	}
	*returnedValues = i;
	
	free(buf);
	
    return rsesci_status;
}

/*****************************************************************************
 * Function: Set Channel List Scan
 * Purpose:  This function switches between a continuous scan and
 *           measurements according to the channel list for the selected
 *           range
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetChannelListScan (ViSession instrSession,
                                             ViInt32 scanTable, ViBoolean scan)
{
	ViChar     buffer[BUFFER_SIZE] = "";
	
	if (rsesci_invalidViInt32Range (scanTable, 1, 10))
        return VI_ERROR_PARAMETER2;
	
	sprintf (buffer, "SENS:SCAN%ld:LIST:STAT", scanTable);
	
	return rsesci_setViBoolean (instrSession, buffer, scan, VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Channel List Scan
 * Purpose:  This function return if a continuous scan or measurements
 *           according to the channel list for the selected range
 *           is selected.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetChannelListScan (ViSession instrSession,
                                             ViInt32 scanTable, ViBoolean *scan)
{
	ViChar     buffer[BUFFER_SIZE] = "";
	
	if (rsesci_invalidViInt32Range (scanTable, 1, 10))
        return VI_ERROR_PARAMETER2;
	
	sprintf (buffer, "SENS:SCAN%ld:LIST:STAT?", scanTable);
	
	return rsesci_getViBoolean (instrSession, buffer, scan, VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Channel List Store
 * Purpose:  This function adds the channel lists to the list of device
 *           settings to be stored/loaded.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetChannelListStore (ViSession instrSession,
                                              ViBoolean store)
{
	return rsesci_setViBoolean (instrSession, "MMEM:SEL:CLIS", store, VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Get Channel List Store
 * Purpose:  This function returns if the channel lists to the list of device
 *           settings should be stored/loaded.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetChannelListStore (ViSession instrSession,
                                              ViBoolean *store)
{
	return rsesci_getViBoolean (instrSession, "MMEM:SEL:CLIS?", store, VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Trace Single
 * Purpose:  This function switches between transmission of single measured
 *           values and continuous measurement with blockwise data
 *           transmission. The measurement is only performed on the set
 *           frequency and is started with INIT1.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetTraceSingle (ViSession instrSession,
                                         ViInt32 value)
{
	ViChar     buffer[BUFFER_SIZE] = "";
	
	if (value < 1)
        return VI_ERROR_PARAMETER2;
	
	sprintf(buffer, "TRAC:POIN SING,%ld", value);
	
	return rsesci_sendCmd(instrSession, buffer);
}

/*****************************************************************************
 * Function: Get Trace Single
 * Purpose:  This function returns the number of single measurement results
 *           that are transmitted by the query.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetTraceSingle (ViSession instrSession,
                                         ViInt32 *value)
{
	return rsesci_getViInt32(instrSession, "TRAC:POIN? SING", value, VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Trace Scan
 * Purpose:  This function switches the scan between measurement on all
 *           frequencies and the use of channel lists. The measurement is
 *           started with INIT2
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetTraceScan (ViSession instrSession, ViInt32 value)
{
	ViChar     buffer[BUFFER_SIZE] = "";
	
	if (value < 1)
        return VI_ERROR_PARAMETER2;
	
	sprintf(buffer, "TRAC:POIN SCAN,%ld", value);
	
	return rsesci_sendCmd(instrSession, buffer);
}

/*****************************************************************************
 * Function: Get Trace Scan
 * Purpose:  This function returns the number of scan measurement results
 *           that are transmitted by the query.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetTraceScan (ViSession instrSession,
                                       ViInt32 *value)
{
    return rsesci_getViInt32(instrSession, "TRAC:POIN? SCAN", value, VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Instrument Mode
 * Purpose:  This function switches between the instrument operating modes
 *           (Spectrum analysis, EMI receiver, FM demodulator and IF analyzer).
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetInstrumentMode(
    ViSession   instrSession,
    ViInt32 instrumentMode
)
{
	if (instrumentMode == RSESCI_INSTRUMENT_MODE_IFANALYZER)
		CHECKMODEL(RSESU8 | RSESU26 | RSESU40);
	
    return rsesci_setEnum (instrSession,
                             "INST",
                             instrumentMode,
                             instrModeArr,
                             VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Instrument Mode
 * Purpose:  This function returns the instrument operating mode (Spectrum
 *           analysis, EMI receiver, FM demodulator and IF analyzer).
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetInstrumentMode(
    ViSession   instrSession,
    ViInt32*    instrumentMode
)
{
    return rsesci_getEnum (instrSession,
                             "INST?",
                             instrumentMode,
                             instrModeArr,
                             VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Mixed Instrument Mode
 * Purpose:  This function sets the R&S ESCI in a combination of test receiver
 *           and analyzer mode.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetMixedInstrumentMode(
    ViSession   instrSession,
    ViBoolean   mixedMode,
    ViInt32 screens
)
{
    ViStatus    rsesci_status;

    CHECKMODEL(RSESCI3 | RSESCI7);
	
	CHECKERR (rsesci_setViBoolean (instrSession,
                                   "INST:CONF:MIX",
                                   mixedMode,
                                   VI_ERROR_PARAMETER2));

    if (mixedMode == VI_TRUE)
        {
        CHECKERR (rsesci_setEnum (instrSession,
                                  "INST:CONF:DISP",
                                  screens,
                                  instrModeArr,
                                  VI_ERROR_PARAMETER3));
        }

    return rsesci_status;
}

/*****************************************************************************
 * Function: Get Mixed Instrument Mode
 * Purpose:  This function returns the R&S ESCI combination of test receiver
 *           and analyzer mode.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetMixedInstrumentMode(
    ViSession   instrSession,
    ViBoolean*  mixedMode,
    ViInt32*    screens
)
{
    ViStatus    rsesci_status;

    CHECKMODEL(RSESCI3 | RSESCI7);
	
	CHECKERR (rsesci_getViBoolean (instrSession,
                                   "INST:CONF:MIX?",
                                   mixedMode,
                                   VI_ERROR_PARAMETER2));

    CHECKERR (rsesci_getEnum (instrSession,
                              "INST:CONF:DISP?",
                              screens,
                              instrModeArr,
                              VI_ERROR_PARAMETER3));

    return rsesci_status;
}

/*****************************************************************************
 * Function: Set Tracking Generator
 * Purpose:  This function switches the tracking generator on or off.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetTrackingGenerator(
    ViSession   instrSession,
    ViInt32     window,
    ViBoolean   trackingGenerator
)
{
    CHECKOPT (RSESCI_B9);

    return rsesci_setViBooleanWindow (instrSession,
                                      "OUTP%ld:STAT",
                                      window,
                                      trackingGenerator,
                                      VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Tracking Generator
 * Purpose:  This function queries the state of tracking generator.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetTrackingGenerator(
    ViSession   instrSession,
    ViInt32     window,
    ViBoolean*  trackingGenerator
)
{
    CHECKOPT (RSESCI_B9);

    return rsesci_getViBooleanWindow (instrSession,
                                      "OUTP%ld:STAT?",
                                      window,
                                      trackingGenerator,
                                      VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set Tracking Generator Source Power
 * Purpose:  This function defines the output level of the tracking generator
 *           in the current measurement window.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetTrackingGeneratorSourcePower(
    ViSession   instrSession,
    ViInt32     window,
    ViReal64    outputLevel
)
{
    CHECKOPT (RSESCI_B9);

    return rsesci_setViReal64Window (instrSession,
                                     "SOUR%ld:POW:LEV:IMM:AMPL",
                                     window,
                                     outputLevel,
                                     -100.0 , 5.0,
                                     VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Tracking Generator Source Power
 * Purpose:  This function queries the output level of the tracking generator
 *           in the current measurement window.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetTrackingGeneratorSourcePower(
    ViSession   instrSession,
    ViInt32     window,
    ViReal64*   outputLevel
)
{
    CHECKOPT (RSESCI_B9);

    return rsesci_getViReal64Window (instrSession,
                                     "SOUR%ld:POW:LEV:IMM:AMPL?",
                                     window,
                                     outputLevel,
                                     VI_ERROR_PARAMETER3);

}

/*****************************************************************************
 * Function: Set Tracking Generator Power Offset
 * Purpose:  This function defines a level offset for the tracking generator
 *           level. Thus, for example, attenuators or amplifiers at the output
 *           of the tracking generator can be taken into account for the
 *           setting.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetTrackingGeneratorPowerOffset(
    ViSession   instrSession,
    ViInt32     window,
    ViReal64    levelOffset
)
{
    CHECKOPT (RSESCI_B9);

    return rsesci_setViReal64Window (instrSession,
                                     "SOUR%ld:POW:LEV:IMM:OFFS",
                                     window,
                                     levelOffset,
                                     -200.0, 200.0,
                                     VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Tracking Generator Power Offset
 * Purpose:  This function queries a level offset for the tracking generator
 *           level.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetTrackingGeneratorPowerOffset(
    ViSession   instrSession,
    ViInt32     window,
    ViReal64*   levelOffset
)
{
    CHECKOPT (RSESCI_B9);

    return rsesci_getViReal64Window (instrSession,
                                     "SOUR%ld:POW:LEV:IMM:OFFS?",
                                     window,
                                     levelOffset,
                                     VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set Tracking Generator Frequency Offset
 * Purpose:  This function defines a frequency offset of the tracking
 *           generator for the indicated measurement window.
 *           Frequency-converting DUTs can be measured with this setting. It
 *           should be noted that the terms (start frequency - tracking
 *           frequency offset) and (stop frequency - tracking frequency offset)
 *           are both > 1 kHz or both < -1 kHz.
 *           External I/Q modulation is switched off, if active.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetTrackingGeneratorFrequencyOffset(
    ViSession   instrSession,
    ViInt32     window,
    ViReal64    frequencyOffset
)
{
    CHECKOPT (RSESCI_B9);

    return rsesci_setViReal64Window (instrSession,
                                     "SOUR%ld:FREQ:OFFS",
                                     window,
                                     frequencyOffset,
                                     -150.0E6 , 150.0E6,
                                     VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Tracking Generator Frequency Offset
 * Purpose:  This function queries a frequency offset of the tracking
 *           generator for the indicated measurement window.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetTrackingGeneratorFrequencyOffset(
    ViSession   instrSession,
    ViInt32     window,
    ViReal64*   frequencyOffset
)
{
    CHECKOPT (RSESCI_B9);

    return rsesci_getViReal64Window (instrSession,
                                     "SOUR%ld:FREQ:OFFS?",
                                     window,
                                     frequencyOffset,
                                     VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Configure Tracking Generator Power Sweep
 * Purpose:  This function configures the power sweep of the tracking generator.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_ConfigureTrackingGeneratorPowerSweep (ViSession instrSession,
                                                               ViInt32 window,
                                                               ViInt32 powerSweep,
                                                               ViReal64 powerStart,
                                                               ViReal64 powerStop)
{
	ViStatus	rsesci_status = VI_SUCCESS;
	
    CHECKERR (rsesci_SetTrackingGeneratorPowerSweepMode (instrSession, window, powerSweep));
    
    rsesci_status = rsesci_SetTrackingGeneratorPowerSweepStart (instrSession, window, powerStart);
    REMAP (3, 4);
    
    rsesci_status = rsesci_SetTrackingGeneratorPowerSweepStop (instrSession,window,powerStop);
    REMAP (3,5);
    
    return rsesci_status;
}

/*****************************************************************************
 * Function: Set Tracking Generator Power Sweep Mode
 * Purpose:  This function activates or deactivates the power sweep.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetTrackingGeneratorPowerSweepMode (ViSession instrSession,
                                                             ViInt32 window,
                                                             ViInt32 powerSweep)
{
	CHECKOPT (RSESCI_B9);
    CHECKWINDOW;
	
    return rsesci_setEnumWindow (instrSession,
		                 "SOUR%ld:POW:MODE",
				 window,
				 powerSweep,
				 freqModeArr,
				 VI_ERROR_PARAMETER3);
}
	
/*****************************************************************************
 * Function: Get Tracking Generator Power Sweep Mode
 * Purpose:  This function queries the power sweep mode.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetTrackingGeneratorPowerSweepMode (ViSession instrSession,
                                                             ViInt32 window,
                                                             ViInt32 *powerSweep)
{
    CHECKOPT (RSESCI_B9);
    CHECKWINDOW;
    
    return rsesci_getEnumWindow (instrSession,
                                 "SOUR%ld:POW:MODE?",
                                 window,
                                 powerSweep,
                                 freqModeArr,
                                 VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set Tracking Generator Power Sweep Start
 * Purpose:  This function sets the start power of the power sweep.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetTrackingGeneratorPowerSweepStart (ViSession instrSession,
                                                              ViInt32 window,
                                                              ViReal64 powerStart)
{
    CHECKOPT (RSESCI_B9);
    CHECKWINDOW;
    
    return  rsesci_setViReal64Window (instrSession,
		                      "SOUR%ld:POW:STAR",
				      window,
				      powerStart,
				      -30.0, 0.0,
				      VI_ERROR_PARAMETER4);
}
	
/*****************************************************************************
 * Function: Get Tracking Generator Power Sweep Start
 * Purpose:  This function queries the start power of the power sweep.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetTrackingGeneratorPowerSweepStart (ViSession instrSession,
                                                              ViInt32 window,
                                                              ViReal64 *powerStart)
{
    CHECKOPT (RSESCI_B9);
    CHECKWINDOW;
    
    return  rsesci_getViReal64Window (instrSession,
                                      "SOUR%ld:POW:STAR?",
                                      window,
                                      powerStart,
                                      VI_ERROR_PARAMETER4);
}

/*****************************************************************************
 * Function: Set Tracking Generator Power Sweep Stop
 * Purpose:  This function sets the stop power of the power sweep.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetTrackingGeneratorPowerSweepStop (ViSession instrSession,
                                                             ViInt32 window,
                                                             ViReal64 powerStop)
{
    CHECKOPT (RSESCI_B9);
    CHECKWINDOW;
    
    return  rsesci_setViReal64Window (instrSession,
		                      "SOUR%ld:POW:STOP",
				      window,
				      powerStop,
				      -30.0, 0.0,
                                      VI_ERROR_PARAMETER4);
}

/*****************************************************************************
 * Function: Get Tracking Generator Power Sweep Stop
 * Purpose:  This function queries the stop power of the power sweep.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetTrackingGeneratorPowerSweepStop (ViSession instrSession,
                                                           ViInt32 window,
                                                           ViReal64 *powerStop)
{
    CHECKOPT (RSESCI_B9);
    CHECKWINDOW;
	
    return  rsesci_getViReal64Window (instrSession,
                                      "SOUR%ld:POW:STOP?",
       				      window,
				      powerStop,
                                      VI_ERROR_PARAMETER4);
}

/*****************************************************************************
 * Function: Set Tracking Generator Measurement
 * Purpose:  This function selects the type of measurement with active
 *           tracking generator.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetTrackingGeneratorMeasurement(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32 measurementType
)
{
    CHECKOPT (RSESCI_B9 | RSESCI_B10);

    return rsesci_setEnumWindow (instrSession,
                                 "SENS%ld:CORR:METH",
                                 window,
                                 measurementType,
                                 measurementTypeArr,
                                 VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Tracking Generator Measurement
 * Purpose:  This function queries the type of measurement with active
 *           tracking generator.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetTrackingGeneratorMeasurement(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32*    measurementType
)
{
    CHECKOPT (RSESCI_B9 | RSESCI_B10);

    return rsesci_getEnumWindow (instrSession,
                                 "SENS%ld:CORR:METH?",
                                 window,
                                 measurementType,
                                 measurementTypeArr,
                                 VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set Tracking Generator Normalization
 * Purpose:  This function activates/deactivates the normalization of the
 *           measurement results in the selected window provided that the
 *           tracking generator is active. The function is available only after
 *           acquisition of a reference trace for the selected type of
 *           measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetTrackingGeneratorNormalization(
    ViSession   instrSession,
    ViInt32     window,
    ViBoolean   normalization
)
{
    CHECKOPT (RSESCI_B9 | RSESCI_B10);

    return rsesci_setViBooleanWindow (instrSession,
                                      "SENS%ld:CORR:STAT",
                                      window,
                                      normalization,
                                      VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Tracking Generator Normalization
 * Purpose:  This function returns if the normalization of the measurement
 *           results in the selected window provided of the tracking
 *           generator is active. The function is available only after
 *           acquisition of a reference trace for the selected type of
 *           measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetTrackingGeneratorNormalization(
    ViSession   instrSession,
    ViInt32     window,
    ViBoolean*  normalization
)
{
    CHECKOPT (RSESCI_B9 | RSESCI_B10);

    return rsesci_getViBooleanWindow (instrSession,
                                      "SENS%ld:CORR:STAT?",
                                      window,
                                      normalization,
                                      VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Tracking Generator Calibration
 * Purpose:  When the tracking generator is active, this function determines
 *           the type of result acquisition for the normalization reference
 *           measurement and starts the measurement selected.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_TrackingGeneratorCalibration(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32 acquisitionType
)
{
    CHECKOPT (RSESCI_B9 | RSESCI_B10);

    return rsesci_setEnumWindow (instrSession,
                                 "SENS%ld:CORR:COLL:ACQ",
                                 window,
                                 acquisitionType,
                                 acquisitionTypeArr,
                                 VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Tracking Generator Recall Settings
 * Purpose:  This function restores the instrument setting that was valid for
 *           the measurement of the reference data, provided that the tracking
 *           generator is active.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_TrackingGeneratorRecallSettings(
    ViSession   instrSession,
    ViInt32     window
)
{
    CHECKOPT (RSESCI_B9 | RSESCI_B10);

    return rsesci_sendCmdWindow (instrSession,
                                 "SENS%ld:CORR:REC",
                                 window);
}

/*****************************************************************************
 * Function: Tracking Generator Save As Transducer Factor
 * Purpose:  This function uses the normalized measurement data to
 *           generate a transducer factor with up to 625 points.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_TrackingGeneratorSaveAsTrdFactor (ViSession instrSession,
                                                           ViInt32 window,
                                                           ViString name)
{
	ViChar  buffer[BUFFER_SIZE];

    CHECKOPT (RSESCI_B9 | RSESCI_B10);
	CHECKMODEL(RSESU8 | RSESU26 | RSESU40);

    CHECKWINDOW;
	
	if (rsesci_invalidViString (name, 8))
        return VI_ERROR_PARAMETER3;
	
    sprintf (buffer, "SENS%ld:CORR:TRAN:GEN", window);

    return rsesci_setViStringQuoted (instrSession,
                                     buffer,
                                     "'",
                                     name,
                                     VI_ERROR_PARAMETER3);    
}

/*****************************************************************************
 * Function: Set External AM
 * Purpose:  This function switches on or off the external amplitude
 *           modulation of the tracking generator in the selected measurement
 *           window.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetExternalAM(
    ViSession   instrSession,
    ViInt32     window,
    ViBoolean   externalAM
)
{
    CHECKOPT (RSESCI_B9);

    return rsesci_setViBooleanWindow (instrSession,
                                      "SOUR%ld:AM:STAT",
                                      window,
                                      externalAM,
                                      VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get External AM
 * Purpose:  This function queries the state of the external amplitude
 *           modulation of the tracking generator in the selected measurement
 *           window.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetExternalAM(
    ViSession   instrSession,
    ViInt32     window,
    ViBoolean*  externalAM
)
{
    CHECKOPT (RSESCI_B9);

    return rsesci_getViBooleanWindow (instrSession,
                                      "SOUR%ld:AM:STAT?",
                                      window,
                                      externalAM,
                                      VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set External FM
 * Purpose:  This function switches on or off the external frequency
 *           modulation of the tracking generator in the selected measurement
 *           window.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetExternalFM(
    ViSession   instrSession,
    ViInt32     window,
    ViBoolean   externalFM
)
{
    CHECKOPT (RSESCI_B9);

    return rsesci_setViBooleanWindow (instrSession,
                                      "SOUR%ld:FM:STAT",
                                      window,
                                      externalFM,
                                      VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get External FM
 * Purpose:  This function queries the state of the external frequency
 *           modulation of the tracking generator in the selected measurement
 *           window.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetExternalFM(
    ViSession   instrSession,
    ViInt32     window,
    ViBoolean*  externalFM
)
{
    CHECKOPT (RSESCI_B9);

    return rsesci_getViBooleanWindow (instrSession,
                                      "SOUR%ld:FM:STAT?",
                                      window,
                                      externalFM,
                                      VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set External IQ Modulation
 * Purpose:  This function switches on or off the external I/Q modulation of
 *           the tracking generator in the selected measurement window.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetExternalIQModulation(
    ViSession   instrSession,
    ViInt32     window,
    ViBoolean   iqModulation
)
{
    CHECKOPT (RSESCI_B9);

    return rsesci_setViBooleanWindow (instrSession,
                                      "SOUR%ld:DM:STAT",
                                      window,
                                      iqModulation,
                                      VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get External IQ Modulation
 * Purpose:  This function queries the state of the external I/Q modulation of
 *           the tracking generator in the selected measurement window.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetExternalIQModulation(
    ViSession   instrSession,
    ViInt32     window,
    ViBoolean*  iqModulation
)
{
    CHECKOPT (RSESCI_B9);

    return rsesci_getViBooleanWindow (instrSession,
                                      "SOUR%ld:DM:STAT?",
                                      window,
                                      iqModulation,
                                      VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set External FM Deviation
 * Purpose:  This function defines the maximum frequency deviation at 1 V
 *           input voltage at the FM input of the tracking generator.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetExternalFMDeviation(
    ViSession   instrSession,
    ViInt32     window,
    ViReal64    fmDeviation
)
{
    CHECKOPT (RSESCI_B9);

    return rsesci_setViReal64Window (instrSession,
                                     "SOUR%ld:FM:DEV",
                                     window,
                                     fmDeviation,
                                     100.0,100.0E+6,
                                     VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get External FM Deviation
 * Purpose:  This function queries the maximum frequency deviation at 1 V
 *           input voltage at the FM input of the tracking generator.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetExternalFMDeviation(
    ViSession   instrSession,
    ViInt32     window,
    ViReal64*   fmDeviation
)
{
    CHECKOPT (RSESCI_B9);

    return rsesci_getViReal64Window (instrSession,
                                     "SOUR%ld:FM:DEV?",
                                     window,
                                     fmDeviation,
                                     VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set External Generator
 * Purpose:  This function activates or deactivates the external generator in
 *           the selected window.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetExternalGenerator(
    ViSession   instrSession,
    ViInt32     window,
    ViBoolean   externalGenerator
)
{
    CHECKOPT (RSESCI_B10);

    return rsesci_setViBooleanWindow (instrSession,
                                      "SOUR%ld:EXT",
                                      window,
                                      externalGenerator,
                                      VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get External Generator
 * Purpose:  This function queries the state of the external generator in the
 *           selected window.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetExternalGenerator(
    ViSession   instrSession,
    ViInt32     window,
    ViBoolean*  externalGenerator
)
{
    CHECKOPT (RSESCI_B10);

    return rsesci_getViBooleanWindow (instrSession,
                                      "SOUR%ld:EXT?",
                                      window,
                                      externalGenerator,
                                      VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set External Generator Selection
 * Purpose:  This function selects the external generator 1 or 2.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetExternalGeneratorSelection(
    ViSession   instrSession,
    ViInt32     generator,
    ViInt32     intfc,
    ViString    type,
    ViInt32     gpibAddress
)
{
    ViStatus    rsesci_status;
    ViChar      buffer[BUFFER_SIZE];

    CHECKOPT (RSESCI_B10);

    if (rsesci_invalidViInt32Range (generator, 1, 2))
        return VI_ERROR_PARAMETER2;

    sprintf (buffer, ":SYST:COMM:RDEV:GEN%ld:TYPE", generator);

    CHECKERR (rsesci_setViStringQuoted (instrSession,
                                        buffer,
                                        "'",
                                        type,
                                        VI_ERROR_PARAMETER4));
    
    CHECKERR (rsesci_setEnumWindow (instrSession,
                                    "SYST:COMM:RDEV:GEN%ld:LINK",
                                    generator,
                                    intfc,
                                    extLinkArr,
                                    VI_ERROR_PARAMETER3));

    CHECKERR (rsesci_setViInt32Window (instrSession,
                                       "SYST:COMM:GPIB:RDEV:GEN%ld:ADDR",
                                       generator,
                                       gpibAddress,
                                       0,30,
                                       VI_ERROR_PARAMETER5));

    return rsesci_status;
}

/*****************************************************************************
 * Function: Get External Generator Selection
 * Purpose:  This function returns the external generator parameters.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetExternalGeneratorSelection(
    ViSession   instrSession,
    ViInt32     generator,
    ViInt32*    intfc,
    ViChar      _VI_FAR type[],
    ViInt32*    gpibAddress
)
{
    ViStatus    rsesci_status;
    ViChar      buffer[BUFFER_SIZE];

    CHECKOPT (RSESCI_B10);

    if (rsesci_invalidViInt32Range (generator, 1, 2))
        return VI_ERROR_PARAMETER2;

    CHECKERR (rsesci_getEnumWindow (instrSession,
                                    "SYST:COMM:RDEV:GEN%ld:LINK?",
                                    generator,
                                    intfc,
                                    extLinkArr,
                                    VI_ERROR_PARAMETER3));

    CHECKERR (rsesci_getViInt32Window (instrSession,
                                       "SYST:COMM:GPIB:RDEV:GEN%ld:ADDR?",
                                       generator,
                                       gpibAddress,
                                       VI_ERROR_PARAMETER5));

    sprintf (buffer, "SYST:COMM:RDEV:GEN%ld:TYPE?", generator);

    return rsesci_getViString (instrSession,
                               buffer,
                               type,
                               BUFFER_SIZE,
                               VI_ERROR_PARAMETER4);
}

/*****************************************************************************
 * Function: Set External Generator Frequency Offset
 * Purpose:  This function defines the frequency offset of the selected
 *           generator 1 or 2 with reference to the receive frequency in the
 *           selected window.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetExternalGeneratorFrequencyOffset(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32 generator,
    ViReal64    offset
)
{
    ViChar  buffer[BUFFER_SIZE];

    CHECKOPT (RSESCI_B10);

    if (rsesci_invalidViInt32Range (generator, 1, 2))
        return VI_ERROR_PARAMETER3;

    sprintf (buffer, "SOUR%%ld:EXT%ld:FREQ:OFFS", generator);

    return rsesci_setViReal64Window (instrSession,
                                     buffer,
                                     window,
                                     offset,
                                     0.0, 0.0,
                                     VI_ERROR_PARAMETER4);
}

/*****************************************************************************
 * Function: Get External Generator Frequency Offset
 * Purpose:  This function queries the frequency offset of the selected
 *           generator 1 or 2 with reference to the receive frequency in the
 *           selected window.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetExternalGeneratorFrequencyOffset(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32 generator,
    ViReal64*   offset
)
{
    ViChar  buffer[BUFFER_SIZE];

    CHECKOPT (RSESCI_B10);

    if (rsesci_invalidViInt32Range (generator, 1, 2))
        return VI_ERROR_PARAMETER3;

    sprintf (buffer, "SOUR%%ld:EXT%ld:FREQ:OFFS?", generator);

    return rsesci_getViReal64Window (instrSession,
                                     buffer,
                                     window,
                                     offset,
                                     VI_ERROR_PARAMETER4);
}

/*****************************************************************************
 * Function: Set External Generator Multiplication Factor
 * Purpose:  This function defines the numerator and the denominator of the
 *           factor with which the analyzer frequency is multiplied in order to
 *           obtain the transmit frequency of the selected generator 1 or 2 in
 *           the selected window.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetExternalGeneratorMultiplicationFactor(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32 generator,
    ViInt32 numerator,
    ViInt32 denominator
)
{
    ViStatus    rsesci_status;
    ViChar      buffer[BUFFER_SIZE];

    CHECKOPT (RSESCI_B10);

    if (rsesci_invalidViInt32Range (generator, 1, 2))
        return VI_ERROR_PARAMETER3;

    sprintf (buffer, "SOUR%%ld:EXT%ld:FREQ:NUM", generator);
    CHECKERR (rsesci_setViInt32Window (instrSession,
                                       buffer,
                                       window,
                                       numerator,
                                       0,0,
                                       VI_ERROR_PARAMETER4));

    sprintf (buffer, "SOUR%%ld:EXT%ld:FREQ:DEN", generator);
    return rsesci_setViInt32Window (instrSession,
                                    buffer,
                                    window,
                                    denominator,
                                    0, 0,
                                    VI_ERROR_PARAMETER5);
}

/*****************************************************************************
 * Function: Get External Generator Multiplication Factor
 * Purpose:  This function queries the numerator and the denominator of the
 *           factor with which the analyzer frequency is multiplied in order to
 *           obtain the transmit frequency of the selected generator 1 or 2 in
 *           the selected window.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetExternalGeneratorMultiplicationFactor(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32 generator,
    ViInt32*    numerator,
    ViInt32*    denominator
)
{
    ViStatus    rsesci_status;
    ViChar      buffer[BUFFER_SIZE];

    CHECKOPT (RSESCI_B10);

    if (rsesci_invalidViInt32Range (generator, 1, 2))
        return VI_ERROR_PARAMETER3;

    sprintf (buffer, "SOUR%%ld:EXT%ld:FREQ:NUM?", generator);
    CHECKERR (rsesci_getViInt32Window (instrSession,
                                       buffer,
                                       window,
                                       numerator,
                                       VI_ERROR_PARAMETER4));

    sprintf (buffer, "SOUR%%ld:EXT%ld:FREQ:DEN?", generator);
    return rsesci_getViInt32Window (instrSession,
                                    buffer,
                                    window,
                                    denominator,
                                    VI_ERROR_PARAMETER5);
}

/*****************************************************************************
 * Function: Set External Generator Frequency Sweep
 * Purpose:  This function activates or deactivates the frequency sweep for
 *           generator 1 or 2 in the selected window.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetExternalGeneratorFrequencySweep(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32 generator,
    ViBoolean   sweep
)
{
    ViChar  buffer[BUFFER_SIZE];

    CHECKOPT (RSESCI_B10);

    if (rsesci_invalidViInt32Range (generator, 1, 2))
        return VI_ERROR_PARAMETER3;

    sprintf (buffer, "SOUR%%ld:EXT%ld:FREQ:SWE", generator);

    return rsesci_setViBooleanWindow (instrSession,
                                      buffer,
                                      window,
                                      sweep,
                                      VI_ERROR_PARAMETER4);
}

/*****************************************************************************
 * Function: Get External Generator Frequency Sweep
 * Purpose:  This function queries the state of the frequency sweep for
 *           generator 1 or 2 in the selected window.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetExternalGeneratorFrequencySweep(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32 generator,
    ViBoolean*  sweep
)
{
    ViChar  buffer[BUFFER_SIZE];

    CHECKOPT (RSESCI_B10);

    if (rsesci_invalidViInt32Range (generator, 1, 2))
        return VI_ERROR_PARAMETER3;

    sprintf (buffer, "SOUR%%ld:EXT%ld:FREQ:SWE?", generator);

    return rsesci_getViBooleanWindow (instrSession,
                                      buffer,
                                      window,
                                      sweep,
                                      VI_ERROR_PARAMETER4);
}

/*****************************************************************************
 * Function: Set External Generator Power
 * Purpose:  This function sets the output power of the selected generator 1
 *           or 2 in the selected window.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetExternalGeneratorPower(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32 generator,
    ViReal64    power
)
{
    ViChar  buffer[BUFFER_SIZE];

    CHECKOPT (RSESCI_B9 | RSESCI_B10);

    if (rsesci_invalidViInt32Range (generator, 1, 2))
        return VI_ERROR_PARAMETER3;

    sprintf (buffer, "SOUR%%ld:EXT%ld:POW", generator);

    return rsesci_setViReal64Window (instrSession,
                                     buffer,
                                     window,
                                     power,
                                     0.0, 0.0,
                                     VI_ERROR_PARAMETER4);
}

/*****************************************************************************
 * Function: Get External Generator Power
 * Purpose:  This function queries the output power of the selected generator
 *           1 or 2 in the selected window.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetExternalGeneratorPower(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32 generator,
    ViReal64*   power
)
{
    ViChar  buffer[BUFFER_SIZE];

    CHECKOPT (RSESCI_B9 | RSESCI_B10);

    if (rsesci_invalidViInt32Range (generator, 1, 2))
        return VI_ERROR_PARAMETER3;

    sprintf (buffer, "SOUR%%ld:EXT%ld:POW?", generator);

    return rsesci_getViReal64Window (instrSession,
                                     buffer,
                                     window,
                                     power,
                                     VI_ERROR_PARAMETER4);
}

/*****************************************************************************
 * Function: Set External Generator Reference Oscillator
 * Purpose:  This function switches between external and internal reference
 *           oscillator for the frequency processing of external generator 1
 *           and 2.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetExternalGeneratorReferenceOscillator(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32     generator,
    ViInt32     referenceOscillator
)
{
    ViChar  buffer[BUFFER_SIZE];

    CHECKOPT (RSESCI_B10);

    if (rsesci_invalidViInt32Range (generator, 1, 2))
        return VI_ERROR_PARAMETER3;

    sprintf (buffer, "SOUR%%ld:EXT%ld:ROSC", generator);

    return rsesci_setEnumWindow (instrSession,
                                 buffer,
                                 window,
                                 referenceOscillator,
                                 ccvsSignalArr,
                                 VI_ERROR_PARAMETER4);
}

/*****************************************************************************
 * Function: Get External Generator Reference Oscillator
 * Purpose:  This function queries selected reference oscillator for the
 *           frequency processing of external generator 1 and 2.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetExternalGeneratorReferenceOscillator(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32     generator,
    ViInt32*    referenceOscillator
)
{
    ViChar  buffer[BUFFER_SIZE];

    CHECKOPT (RSESCI_B10);

    if (rsesci_invalidViInt32Range (generator, 1, 2))
        return VI_ERROR_PARAMETER3;

    sprintf (buffer, "SOUR%%ld:EXT%ld:ROSC?", generator);

    return rsesci_getEnumWindow (instrSession,
                                 buffer,
                                 window,
                                 referenceOscillator,
                                 ccvsSignalArr,
                                 VI_ERROR_PARAMETER4);
}

/*****************************************************************************
 * Function: Set External Generator Reference Oscillator
 * Purpose:  This function sends a command string to the external generator
 *           connected via the GPIB interface of the Ext. Generator Control
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SendExternalGeneratorCommand (ViSession instrSession,
                                                       ViInt32 GPIBAddress,
                                                       ViString command)
{
    ViChar  buffer[BUFFER_SIZE];

    CHECKOPT (RSESCI_B10);

    if (rsesci_invalidViInt32Range (GPIBAddress, 0, 30))
        return VI_ERROR_PARAMETER2;

    sprintf (buffer, "SYST:COMM:GPIB:RDEV:COMM %ld,'%s'", GPIBAddress, command);

    return rsesci_sendCmd (instrSession, buffer);
}

/*****************************************************************************
 * Function: Query External Generator Reference Oscillator
 * Purpose:  This function sends query string to the external generator
 *           connected via the GPIB interface of the Ext. Generator Control
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_QueryExternalGeneratorCommand (ViSession instrSession,
                                                        ViInt32 GPIBAddress,
                                                        ViString command,
                                                        ViInt32 arraySize,
                                                        ViChar result[])
{
    ViChar  buffer[BUFFER_SIZE];

    CHECKOPT (RSESCI_B10);

    if (rsesci_invalidViInt32Range (GPIBAddress, 0, 30))
        return VI_ERROR_PARAMETER2;

    sprintf (buffer, "SYST:COMM:GPIB:RDEV:COMM? %ld,'%s'", GPIBAddress, command);

    return rsesci_getViString (instrSession
                             , buffer
                             , result
                             , arraySize
                             , VI_ERROR_PARAMETER5
                             );
}

/*****************************************************************************
 * Function: Set Analog Demodulator
 * Purpose:  This function activates the FM demodulator of the instrument. The
 *           instrument will be set to time domain measurement (span = 0) at
 *           the current center frequency. The detector will be set to SAMPle,
 *           the demodulator itself will be set up according to the parameters
 *           of command [SENSe:]ADEMod:SET.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetAnalogDemodulator(
    ViSession   instrSession,
    ViBoolean   analogDemodulator
)
{
    CHECKOPT (RSESCI_K7);

    return rsesci_setViBoolean (instrSession,
                                "ADEM",
                                analogDemodulator,
                                VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Analog Demodulator
 * Purpose:  This function returns if the FM demodulator of the instrument is
 *           activated or not.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetAnalogDemodulator(
    ViSession   instrSession,
    ViBoolean*  analogDemodulator
)
{
    CHECKOPT (RSESCI_K7);

    return rsesci_getViBoolean (instrSession,
                                "ADEM?",
                                analogDemodulator,
                                VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Analog Demodulator Parameters
 * Purpose:  This function configures the AM/FM demodulator of the instrument.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetAnalogDemodulatorParameters(
    ViSession   instrSession,
    ViReal64    sampleRate,
    ViInt32     recordLength,
    ViInt32     triggerSource,
    ViInt32     triggerSlope,
    ViInt32     offsetSamples,
    ViInt32     numberofMeas
)
{
    ViChar  buffer[BUFFER_SIZE];

    CHECKOPT (RSESCI_K7);

    if (rsesci_invalidViReal64Range (sampleRate, 15.625e3, 32.0e6))
        return VI_ERROR_PARAMETER2;
    if (rsesci_invalidViInt32Range (recordLength, 1, 130560))
        return VI_ERROR_PARAMETER3;
    if (rsesci_invalidViInt32Range (triggerSource, 0, 9))
        return VI_ERROR_PARAMETER4;
    if (rsesci_invalidViInt32Range (triggerSlope, 0, 1))
        return VI_ERROR_PARAMETER5;
    if (rsesci_invalidViInt32Range (offsetSamples, -65024, 130559))
        return VI_ERROR_PARAMETER6;
    if (rsesci_invalidViInt32Range (numberofMeas, 0, 32767))
        return VI_ERROR_PARAMETER7;

    sprintf (buffer, "ADEM:SET %.12lf,%ld,%s,%s,%ld,%ld",
             sampleRate,
             recordLength,
             triggersourceArr[triggerSource],
             polarityArr[triggerSlope],
             offsetSamples,
             numberofMeas);

    return rsesci_sendCmd (instrSession, buffer);
}

/*****************************************************************************
 * Function: Get Analog Demodulator Parameters
 * Purpose:  This function reads configuration of the AM/FM demodulator of
 *           the instrument.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetAnalogDemodulatorParameters (ViSession instrSession,
                                                         ViReal64 *sampleRate,
                                                         ViInt32 *recordLength,
                                                         ViInt32 *triggerSource,
                                                         ViInt32 *triggerSlope,
                                                         ViInt32 *offsetSamples,
                                                         ViInt32 *numberofMeas)
{
    ViChar      buffer[BUFFER_SIZE];
    ViStatus    rsesci_status = VI_SUCCESS;

    ViChar      triggerSourceTmp[32];
    ViChar      triggerSlopeTmp[32];
    
    CHECKOPT (RSESCI_K7);

    if (!sampleRate)
        return VI_ERROR_PARAMETER2;
    if (!recordLength)
        return VI_ERROR_PARAMETER3;
    if (!triggerSource)
        return VI_ERROR_PARAMETER4;
    if (!triggerSlope)
        return VI_ERROR_PARAMETER5;
    if (!offsetSamples)
        return VI_ERROR_PARAMETER6;
    if (!numberofMeas)
        return VI_ERROR_PARAMETER7;

    CHECKERR (rsesci_getViString (instrSession,
                                  "ADEM:SET?",
                                  buffer,
                                  BUFFER_SIZE,
                                  VI_ERROR_INSTR_INTERPRETING_RESPONSE));

    CHECKSCAN (sscanf (buffer, "%le,%ld,%[^,],%[^,],%ld,%ld",
                       sampleRate,
                       recordLength,
                       triggerSourceTmp,
                       triggerSlopeTmp,
                       offsetSamples,
                       numberofMeas));

    CHECKERR(rsesci_getIndexFromString (instrSession, triggerSourceTmp, triggersourceArr, triggerSource));
    CHECKERR(rsesci_getIndexFromString (instrSession, triggerSlopeTmp, polarityArr, triggerSlope));

    return rsesci_status;
}

/*****************************************************************************
 * Function: Set Analog Demodulator Result Display
 * Purpose:  This function selects the type of evaluation of the measured
 *           data. This corresponds to the selection of the Result Display in
 *           manual mode.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetAnalogDemodulatorResultDisplay(
    ViSession   instrSession,
    ViInt32     resultDisplay,
    ViInt32     trace
)
{
    ViChar  buffer[BUFFER_SIZE];

    CHECKOPT (RSESCI_K7);

    switch (resultDisplay)
        {
        case RSESCI_DEMOD_FM_SIG:   /* ('XTIM:FM[:TDOMain]') */
        case RSESCI_RFS_FM_SIG:     /* ('XTIM:SPECtrum') */
        case RSESCI_DEMOD_PM_SIG:   /* ('XTIM:PM[:TDOMain]') */
        case RSESCI_DEMOD_AM_SIG:   /* ('XTIM:AM') */
        case RSESCI_RFP_AM_SIG:     /* ('XTIM:RFPower[:TDOMain]') */
		case RSESCI_DEMOD_AM_SIG_REL:   /* ('XTIM:AM:REL') */

            sprintf (buffer, "CALC:FEED '%s'", resultDisplayArr[resultDisplay]);
            break;
        case RSESCI_FM_SUMMARY: /* ('XTIM:FMSummary<1 to 3>') */
        case RSESCI_AFS_FM_SIG: /* ('XTIM:FM:AFSPectrum<1 to 3>') */
        case RSESCI_PM_SUMMARY: /* ('XTIM:PMSummary<1 to 3>') */
        case RSESCI_AFS_PM_SIG: /* ('XTIM:PM:AFSPectrum<1 to 3>') */
        case RSESCI_AM_SUMMARY: /* ('XTIM:AMSummary<1 to 3>') */
        case RSESCI_AFS_AM_SIG: /* ('XTIM:RFPower:AFSPectrum<1 to 3>') */
		
            if (rsesci_invalidViInt32Range (trace, 1, 3))
                return VI_ERROR_PARAMETER3;

            sprintf (buffer, "CALC:FEED '%s%ld'", resultDisplayArr[resultDisplay], trace);
            break;
        case RSESCI_AM_SUMMARY_REL:   /* ('XTIM:AMSummary<1 to 3>:REL') */
		
			if (rsesci_invalidViInt32Range (trace, 1, 3))
                return VI_ERROR_PARAMETER3;

            sprintf (buffer, "CALC:FEED 'XTIM:AMS%ld:REL'", trace);
            break;
		case RSESCI_AFS_AM_SIG_REL:   /* ('XTIM:AM:REL:AFSPectrum<1 to 3>') */

			if (rsesci_invalidViInt32Range (trace, 1, 3))
                return VI_ERROR_PARAMETER3;

            sprintf (buffer, "CALC:FEED 'XTIM:AM:REL:AFSP%ld'", trace);
            break;
		default:
            return VI_ERROR_PARAMETER2;
        }

    return rsesci_sendCmd (instrSession, buffer);
}

/*****************************************************************************
 * Function: Set Analog Demodulation Measurement Time
 * Purpose:  This function defines the measurement time for analog
 *           demodulation.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetAnalogDemodulationMeasurementTime(
    ViSession   instrSession,
    ViReal64    measurementTime
)
{
    CHECKOPT (RSESCI_K7);

    return rsesci_setViReal64 (instrSession,
                               "ADEM:MTIM",
                               measurementTime,
                               0.0, 0.0,
                               VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Analog Demodulation Measurement Time
 * Purpose:  This function returns the measurement time for analog
 *           demodulation.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetAnalogDemodulationMeasurementTime(
    ViSession   instrSession,
    ViReal64*   measurementTime
)
{
    CHECKOPT (RSESCI_K7);

    return rsesci_getViReal64 (instrSession,
                               "ADEM:MTIM?",
                               measurementTime,
                               VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Analog Demodulation Sample Rate
 * Purpose:  This function returns the sample rate set up for the current
 *           analog demodulation measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetAnalogDemodulationSampleRate(
    ViSession   instrSession,
    ViReal64*   sampleRate
)
{
    CHECKOPT (RSESCI_K7);

    return rsesci_getViReal64 (instrSession,
                               "ADEM:SRAT?",
                               sampleRate,
                               VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Analog Demodulation Record Length
 * Purpose:  This function returns the record length set up for the current
 *           analog demodulation measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetAnalogDemodulationRecordLength(
    ViSession   instrSession,
    ViReal64*   recordLength
)
{
    CHECKOPT (RSESCI_K7);

    return rsesci_getViReal64 (instrSession,
                               "ADEM:RLEN?",
                               recordLength,
                               VI_ERROR_PARAMETER2);
}



/*****************************************************************************
 * Function: Set Analog Demodulation Phase Wrap
 * Purpose:  This function activates the phase wrapping to PI ... +PI 
 *           (-180 deg ... +180 deg).
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetAnalogDemodulationPhaseWrap (ViSession instrSession,
                                                         ViInt32 type)
{
    CHECKOPT (RSESCI_K7);
    
    return rsesci_setEnum (instrSession,
                           "CALC:FORM",
                           type,
                           phaseWrapTypeArr,
                           VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Analog Demodulation Phase Wrap
 * Purpose:  This function returns the phase wrapping to PI ... +PI 
 *           (-180 deg ... +180 deg).
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetAnalogDemodulationPhaseWrap (ViSession instrSession,
                                                         ViInt32 *type)
{
    CHECKOPT (RSESCI_K7);
    
    return rsesci_getEnum (instrSession,
                           "CALC:FORM?",
                           type,
                           phaseWrapTypeArr,
                           VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Set Analog Demodulation Zero Phase Ref Point
 * Purpose:  This function activates the input of the reference
 *           point, where the phase is set to 0 rad.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetAnalogDemodulationZeroPhaseRefPoint (ViSession instrSession,
                                                                 ViReal64 zeroPhaseRefPoint)
{
    CHECKOPT (RSESCI_K7);

    return rsesci_setViReal64 (instrSession,
                               "SENS:ADEM:PM:RPO:X",
                               zeroPhaseRefPoint,
                               0.0, 0.0,
                               VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Get Analog Demodulation Zero Phase Ref Point
 * Purpose:  This function returns the input of the reference
 *           point, where the phase is set to 0 rad.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetAnalogDemodulationZeroPhaseRefPoint (ViSession instrSession,
                                                                 ViReal64 *zeroPhaseRefPoint)
{
    CHECKOPT (RSESCI_K7);

    return rsesci_getViReal64 (instrSession,
                               "SENS:ADEM:PM:RPO:X?",
                               zeroPhaseRefPoint,  
                               VI_ERROR_PARAMETER2);     
}

/*****************************************************************************
 * Function: Configure Analog Demodulation Filter
 * Purpose:  This function selects demodulation bandwidth AF filters of FM 
 *           demodulation.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_ConfigureAnalogDemodulationFilter (ViSession instrSession,
                                                            ViInt32 filterType,
                                                            ViBoolean filterState,
                                                            ViReal64 filterFrequency,
                                                            ViReal64 AMFMDeemphasisValue)
{
	ViStatus	rsesci_status = VI_SUCCESS;
	
    CHECKERR (rsesci_SetAnalogDemodulationFilterState (instrSession, filterType, filterState));
    if (filterType == RSESCI_FM_FILTER_DEEMP)
    {
        rsesci_status = rsesci_SetAnalogDemodulationFilterDeemphasis(instrSession, AMFMDeemphasisValue);
        REMAP (2, 5);
    }
    else
    {
        rsesci_status = rsesci_SetAnalogDemodulationFilterFrequency(instrSession, filterType, filterFrequency);
        REMAP (3, 4);
    }
            
    return rsesci_status;
}

/*****************************************************************************
 * Function: Set Analog Demodulation Filter State
 * Purpose:  This function activates the selected filter.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetAnalogDemodulationFilterState (ViSession instrSession,
                                                           ViInt32 filterType,
                                                           ViBoolean filterState)
{
    ViChar      cmd[32];
    
	CHECKOPT (RSESCI_K7);
	if ((rsesci_checkModel (instrSession, RSESCI3 | RSESCI7 | RSESPI3 | RSESPI7) == VI_SUCCESS) && (filterType != RSESCI_FM_FILTER_LP_REL))
		CHECKOPT (RSESCI_B70);
		
    switch (filterType)
	{
		case RSESCI_FM_FILTER_HP:
            strcpy (cmd,"SENS:FILT:HPAS:STAT");
		break;	
		case RSESCI_FM_FILTER_LP:
		case RSESCI_FM_FILTER_LP_REL:
            strcpy (cmd,"SENS:FILT:LPAS:STAT");
		break;	
		case RSESCI_FM_FILTER_DEEMP:
			strcpy (cmd,"SENS:FILT:DEMP:STAT");
		break;	
		default:
			return VI_ERROR_PARAMETER2;
		
	}
	
    return rsesci_setViBoolean (instrSession, cmd, filterState, VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Analog Demodulation Filter State
 * Purpose:  This function queries the state of selected filter.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetAnalogDemodulationFilterState (ViSession instrSession,
                                                        ViInt32 filterType,
                                                           ViBoolean *filterState)
{
    ViChar      cmd[32];
	
	CHECKOPT (RSESCI_K7);
	if ((rsesci_checkModel (instrSession, RSESCI3 | RSESCI7 | RSESPI3 | RSESPI7) == VI_SUCCESS) && (filterType != RSESCI_FM_FILTER_LP_REL))
		CHECKOPT (RSESCI_B70);
		
    switch (filterType)
	{
		case RSESCI_FM_FILTER_HP:
            strcpy (cmd,"SENS:FILT:HPAS:STAT?");
		break;	
		case RSESCI_FM_FILTER_LP:
		case RSESCI_FM_FILTER_LP_REL:
            strcpy (cmd,"SENS:FILT:LPAS:STAT?");
		break;	
		case RSESCI_FM_FILTER_DEEMP:
			strcpy (cmd,"SENS:FILT:DEMP:STAT?");
		break;	
		default:
			return VI_ERROR_PARAMETER2;
		
	}
	
    return rsesci_getViBoolean (instrSession, cmd, filterState, VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set Analog Demodulation Filter Frequency
 * Purpose:  This function sets the filter frequency.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetAnalogDemodulationFilterFrequency (ViSession instrSession,
                                                               ViInt32 filterType,
                                                               ViReal64 filterFrequency)
{
    ViChar      cmd[32];
    ViReal64    min = 0.0, max = 0.0;
    
	CHECKOPT (RSESCI_K7);
	if ((rsesci_checkModel (instrSession, RSESCI3 | RSESCI7 | RSESPI3 | RSESPI7) == VI_SUCCESS) && (filterType != RSESCI_FM_FILTER_LP_REL))
		CHECKOPT (RSESCI_B70);
		
    switch (filterType)
    {
        case RSESCI_FM_FILTER_HP:
            min = 50.0;
            max = 300.0;
            strcpy (cmd,"SENS:FILT:HPAS:FREQ");
        break;  
        case RSESCI_FM_FILTER_LP:
            min = 3.0e3;
            max = 150.0e3;
            strcpy (cmd,"SENS:FILT:LPAS:FREQ");
        break;
        case RSESCI_FM_FILTER_LP_REL:
            min = 5.0;
            max = 25.0;
            strcpy (cmd,"SENS:FILT:LPAS:FREQ:REL");
        break;  
        default:
            return VI_ERROR_PARAMETER2;
        
    }
    
    return rsesci_setViReal64 (instrSession, cmd, filterFrequency, min, max, VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Analog Demodulation Filter Frequency
 * Purpose:  This function queries the filter frequency.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetAnalogDemodulationFilterFrequency (ViSession instrSession,
                                                               ViInt32 filterType,
                                                               ViReal64 *filterFrequency)
{
    ViChar      cmd[32];
    
	CHECKOPT (RSESCI_K7);
	if ((rsesci_checkModel (instrSession, RSESCI3 | RSESCI7 | RSESPI3 | RSESPI7) == VI_SUCCESS) && (filterType != RSESCI_FM_FILTER_LP_REL))
		CHECKOPT (RSESCI_B70);
		
    switch (filterType)
    {
        case RSESCI_FM_FILTER_HP:
            strcpy (cmd,"SENS:FILT:HPAS:FREQ?");
        break;  
        case RSESCI_FM_FILTER_LP:
            strcpy (cmd,"SENS:FILT:LPAS:FREQ?");
        break;
        case RSESCI_FM_FILTER_LP_REL:
            strcpy (cmd,"SENS:FILT:LPAS:FREQ:REL?");
        break;  
        default:
            return VI_ERROR_PARAMETER2;
        
    }
    
    return rsesci_getViReal64 (instrSession, cmd, filterFrequency, VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set Analog Demodulation Filter Deemphasis
 * Purpose:  This function sets the time constant of the de-emphasis for 
 *           analog demodulation.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetAnalogDemodulationFilterDeemphasis (ViSession instrSession,
                                                                ViReal64 AMFMDeemphasisValue)
{
    CHECKOPT (RSESCI_K7);
	if (rsesci_checkModel (instrSession, RSESCI3 | RSESCI7 | RSESPI3 | RSESPI7) == VI_SUCCESS)
		CHECKOPT (RSESCI_B70);
		
    return rsesci_setViReal64 (instrSession,
                               "SENS:FILT:DEMP:TCON",
                                AMFMDeemphasisValue,
                                25.0e-6, 750.0e-6,
                               VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Get Analog Demodulation Filter Deemphasis
 * Purpose:  This function queries the time constant of the de-emphasis for 
 *           analog demodulation.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetAnalogDemodulationFilterDeemphasis (ViSession instrSession,
                                                                ViReal64 *AMFMDeemphasisValue)
{
    CHECKOPT (RSESCI_K7);
	if (rsesci_checkModel (instrSession, RSESCI3 | RSESCI7 | RSESPI3 | RSESPI7) == VI_SUCCESS)
		CHECKOPT (RSESCI_B70);
		
    return rsesci_getViReal64 (instrSession,
                               "SENS:FILT:DEMP:TCON?",
                                AMFMDeemphasisValue,
                               VI_ERROR_PARAMETER2);     
}

/// HIFN  This function  activates/deactivates the unweighted CCIR filter.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR state/This control  activates/deactivates the unweighted CCIR filter.
ViStatus _VI_FUNC rsesci_SetAnalogDemodulationCCIRUnweightedFilterState(
	ViSession	instrSession,
	ViBoolean	state
)
{
	CHECKOPT (RSESCI_K7);
	if (rsesci_checkModel (instrSession, RSESCI3 | RSESCI7 | RSESPI3 | RSESPI7) == VI_SUCCESS)
		CHECKOPT (RSESCI_B70);
		
    return rsesci_setViBoolean (instrSession,
                               "SENS:FILT:CCIR:UNW:STAT",
                                state,
                                VI_ERROR_PARAMETER2);    
}


/// HIFN  This function returns the unweighted CCIR filter state.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR state/This control returns the unweighted CCIR filter state.
ViStatus _VI_FUNC rsesci_GetAnalogDemodulationCCIRUnweightedFilterState(
	ViSession	instrSession,
	ViBoolean*	state
)
{
	CHECKOPT (RSESCI_K7);
	if (rsesci_checkModel (instrSession, RSESCI3 | RSESCI7 | RSESPI3 | RSESPI7) == VI_SUCCESS)
		CHECKOPT (RSESCI_B70);
		
    return rsesci_getViBoolean (instrSession,
                               "SENS:FILT:CCIR:UNW:STAT?",
                                state,
                                VI_ERROR_PARAMETER2);    
}


/// HIFN  This function  activates/deactivates the weighted CCIR filter.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR state/This control  activates/deactivates the weighted CCIR filter.
ViStatus _VI_FUNC rsesci_SetAnalogDemodulationCCIRWeightedFilterState(
	ViSession	instrSession,
	ViBoolean	state
)
{
	CHECKOPT (RSESCI_K7);
	if (rsesci_checkModel (instrSession, RSESCI3 | RSESCI7 | RSESPI3 | RSESPI7) == VI_SUCCESS)
		CHECKOPT (RSESCI_B70);
		
    return rsesci_setViBoolean (instrSession,
                               "SENS:FILT:CCIR:WEIG:STAT",
                                state,
                                VI_ERROR_PARAMETER2);     
}


/// HIFN  This function returns the weighted CCIR filter state.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR state/This control returns the weighted CCIR filter state.
ViStatus _VI_FUNC rsesci_GetAnalogDemodulationCCIRWeightedFilterState(
	ViSession	instrSession,
	ViBoolean*	state
)
{
	CHECKOPT (RSESCI_K7);
	if (rsesci_checkModel (instrSession, RSESCI3 | RSESCI7 | RSESPI3 | RSESPI7) == VI_SUCCESS)
		CHECKOPT (RSESCI_B70);
		
    return rsesci_getViBoolean (instrSession,
                               "SENS:FILT:CCIR:WEIG:STAT?",
                                state,
                                VI_ERROR_PARAMETER2);    
}


/// HIFN  This function  activates/deactivates the "A" weighting filter
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR state/This control activates/deactivates the A weighting filter.
ViStatus _VI_FUNC rsesci_SetAnalogDemodulationAWeightedFilterState(
	ViSession	instrSession,
	ViBoolean	state
)
{
	CHECKOPT (RSESCI_K7);
	if (rsesci_checkModel (instrSession, RSESCI3 | RSESCI7 | RSESPI3 | RSESPI7) == VI_SUCCESS)
		CHECKOPT (RSESCI_B70);
		
    return rsesci_setViBoolean (instrSession,
                               "SENS:FILT:AWE:STAT",
                                state,
                                VI_ERROR_PARAMETER2);     
}


/// HIFN  This function returns the "A" weighting filter state.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR state/This control returns the "A" weighting filter state.
ViStatus _VI_FUNC rsesci_GetAnalogDemodulationAWeightedFilterState(
	ViSession	instrSession,
	ViBoolean*	state
)
{
	CHECKOPT (RSESCI_K7);
	if (rsesci_checkModel (instrSession, RSESCI3 | RSESCI7 | RSESPI3 | RSESPI7) == VI_SUCCESS)
		CHECKOPT (RSESCI_B70);
		
    return rsesci_getViBoolean (instrSession,
                               "SENS:FILT:AWE:STAT?",
                                state,
                                VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Set AF Center Frequency
 * Purpose:  This function sets the center frequency for result display AF
 *           spectrum.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetAFCenterFrequency(
    ViSession   instrSession,
    ViReal64    centerFrequency
)
{
    CHECKOPT (RSESCI_K7);

    return rsesci_setViReal64 (instrSession,
                               "ADEM:AF:CENT",
                               centerFrequency,
                               0.0, 0.0,
                               VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get AF Center Frequency
 * Purpose:  This function returns the center frequency for result display AF
 *           spectrum.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetAFCenterFrequency(
    ViSession   instrSession,
    ViReal64*   centerFrequency
)
{
    CHECKOPT (RSESCI_K7);

    return rsesci_getViReal64 (instrSession,
                               "ADEM:AF:CENT?",
                               centerFrequency,
                               VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set AF Start Frequency
 * Purpose:  This function sets the start frequency for result display AF
 *           spectrum.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetAFStartFrequency(
    ViSession   instrSession,
    ViReal64    startFrequency
)
{
    CHECKOPT (RSESCI_K7);

    return rsesci_setViReal64 (instrSession,
                               "ADEM:AF:STAR",
                               startFrequency,
                               0.0, 0.0,
                               VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get AF Start Frequency *
 * Purpose:  This function returns the start frequency for result display AF
 *           spectrum.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetAFStartFrequency(
    ViSession   instrSession,
    ViReal64*   startFrequency
)
{
    CHECKOPT (RSESCI_K7);

    return rsesci_getViReal64 (instrSession,
                               "ADEM:AF:STAR?",
                               startFrequency,
                               VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set AF Stop Frequency
 * Purpose:  This function sets the stop frequency for result display AF
 *           spectrum. The stop frequency is limited to half the measurement
 *           bandwidth of analog demodulation (SENS:ADEM:BAND).
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetAFStopFrequency(
    ViSession   instrSession,
    ViReal64    stopFrequency
)
{
    CHECKOPT (RSESCI_K7);

    return rsesci_setViReal64 (instrSession,
                               "ADEM:AF:STOP",
                               stopFrequency,
                               0.0, 0.0,
                               VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get AF Stop Frequency
 * Purpose:  This function returns the stop frequency for result display AF
 *           spectrum.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetAFStopFrequency(
    ViSession   instrSession,
    ViReal64*   stopFrequency
)
{
    CHECKOPT (RSESCI_K7);

    return rsesci_getViReal64 (instrSession,
                               "ADEM:AF:STOP?",
                               stopFrequency,
                               VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Automatic Signal Search
 * Purpose:  This function activates an automatic signal search..
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_AutomaticSignalSearch (ViSession instrSession)
{
    ViChar  buffer[BUFFER_SIZE];

	CHECKOPT (RSESCI_K7);

    return rsesci_waitOPC (instrSession, "SENS:FREQ:CW:AFC ONCE;*OPC?", buffer, 30000);
}

/*****************************************************************************
 * Function: Set AF Span
 * Purpose:  This function sets the span for result display AF spectrum. The
 *           span is limited to half the measurement bandwidth of analog
 *           demodulation.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetAFSpan(
    ViSession   instrSession,
    ViReal64    span
)
{
    CHECKOPT (RSESCI_K7);

    return rsesci_setViReal64 (instrSession,
                               "ADEM:AF:SPAN",
                               span,
                               0.0, 0.0,
                               VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get AF Span
 * Purpose:  This function returns the span for result display AF spectrum.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetAFSpan(
    ViSession   instrSession,
    ViReal64*   span
)
{
    CHECKOPT (RSESCI_K7);

    return rsesci_getViReal64 (instrSession,
                               "ADEM:AF:SPAN?",
                               span,
                               VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set AF Full Span
 * Purpose:  This function sets the maximum span for result display AF spectrum. 
 *           The maximum span corresponds to half the measurement bandwidth of
 *           analog demodulation (SENS:ADEM:BAND).
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetAFFullSpan (ViSession instrSession)
{
	CHECKOPT (RSESCI_K7);

    return rsesci_sendCmd (instrSession, "SENS:ADEM:AF:SPAN:FULL");    
}

/*****************************************************************************
 * Function: Set RF Frequency Span
 * Purpose:  This function sets the frequency range for result display of RF
 *           spectrum determined from FM demodulation data. The frequency range
 *           for result display is limited to the maximum span
 *           (SENS:ADEM:SPEC:SPAN:MAX) or to the measurement bandwidth of
 *           analog demodulation (SENS:ADEM:BAND).
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetRFFrequencySpan(
    ViSession   instrSession,
    ViReal64    span
)
{
    CHECKOPT (RSESCI_K7);

    return rsesci_setViReal64 (instrSession,
                               "ADEM:SPEC:SPAN:ZOOM",
                               span,
                               0.0, 0.0,
                               VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get RF Frequency Span
 * Purpose:  This function returns the frequency range for result display of
 *           RF spectrum determined from FM demodulation data.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetRFFrequencySpan(
    ViSession   instrSession,
    ViReal64*   span
)
{
    CHECKOPT (RSESCI_K7);

    return rsesci_getViReal64 (instrSession,
                               "ADEM:SPEC:SPAN:ZOOM?",
                               span,
                               VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Max RF Frequency Span
 * Purpose:  This function sets the maximum frequency range for displaying the
 *           RF spectrum that was determined from the FM demodulation data. The
 *           maximum span corresponds to the measurement bandwidth of analog
 *           demodulation (SENS:ADEM:BAND).
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetMaxRFFrequencySpan(
    ViSession   instrSession,
    ViReal64    span
)
{
    CHECKOPT (RSESCI_K7);

    return rsesci_setViReal64 (instrSession,
                               "ADEM:SPEC:SPAN:MAX",
                               span,
                               0.0, 0.0,
                               VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Max RF Frequency Span
 * Purpose:  This function returns the maximum frequency range for displaying
 *           the RF spectrum that was determined from the FM demodulation data.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetMaxRFFrequencySpan(
    ViSession   instrSession,
    ViReal64*   span
)
{
    CHECKOPT (RSESCI_K7);

    return rsesci_getViReal64 (instrSession,
                               "ADEM:SPEC:SPAN:MAX?",
                               span,
                               VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Analog Demodulation Bandwidth
 * Purpose:  This function defines the demodulation bandwidth used for analog
 *           demodulation. The required sampling rate is automatically set
 *           depending on the selected demodulation bandwidth. The available
 *           demodulation bandwidths are determined by the existing sampling
 *           rates.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetAnalogDemodulationBandwidth(
    ViSession   instrSession,
    ViReal64    demodulationBandwidth
)
{
    CHECKOPT (RSESCI_K7);

    return rsesci_setViReal64 (instrSession,
                               "ADEM:BWID:DEM",
                               demodulationBandwidth,
                               0.0, 0.0,
                               VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Analog Demodulation Bandwidth
 * Purpose:  This function returns the demodulation bandwidth used for analog
 *           demodulation.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetAnalogDemodulationBandwidth(
    ViSession   instrSession,
    ViReal64*   demodulationBandwidth
)
{
    CHECKOPT (RSESCI_K7);

    return rsesci_getViReal64 (instrSession,
                               "ADEM:BWID:DEM?",
                               demodulationBandwidth,
                               VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Analog Demodulation Resolution Bandwidth
 * Purpose:  This function sets the resolution bandwidth for spectrum
 *           represetation that was determined from the analog demodulation
 *           data.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetAnalogDemodulationResolutionBandwidth(
    ViSession   instrSession,
    ViReal64    resolutionBandwidth
)
{
    CHECKOPT (RSESCI_K7);

    return rsesci_setViReal64 (instrSession,
                               "ADEM:SPEC:BWID:RES",
                               resolutionBandwidth,
                               0.0, 0.0,
                               VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Analog Demodulation Resolution Bandwidth
 * Purpose:  This function returns the resolution bandwidth for spectrum
 *           represetation that was determined from the analog demodulation
 *           data.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetAnalogDemodulationResolutionBandwidth(
    ViSession   instrSession,
    ViReal64*   resolutionBandwidth
)
{
    CHECKOPT (RSESCI_K7);

    return rsesci_getViReal64 (instrSession,
                               "ADEM:SPEC:BWID:RES?",
                               resolutionBandwidth,
                               VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set AF Input Coupling
 * Purpose:  This function selects the coupling of the AF path of the
 *           analyzer.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetAFInputCoupling(
    ViSession   instrSession,
    ViInt32     afInputCoupling
)
{
    CHECKOPT (RSESCI_K7);

    return rsesci_setEnum (instrSession,
                           "ADEM:AF:COUP",
                           afInputCoupling,
                           couplingArr,
                           VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get AF Input Coupling
 * Purpose:  This function returns the coupling of the AF path of the
 *           analyzer.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetAFInputCoupling(
    ViSession   instrSession,
    ViInt32*    afInputCoupling
)
{
    CHECKOPT (RSESCI_K7);

    return rsesci_getEnum (instrSession,
                           "ADEM:AF:COUP?",
                           afInputCoupling,
                           couplingArr,
                           VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Analog Demodulation Angle Units
 * Purpose:  This function selects the default unit for angles.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetAnalogDemodulationAngleUnits(
    ViSession   instrSession,
    ViInt32     angleUnits
)
{
    CHECKOPT (RSESCI_K7);

    return rsesci_setEnum (instrSession,
                           "UNIT:ANGL",
                           angleUnits,
                           angleUnitsArr,
                           VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Analog Demodulation Angle Units
 * Purpose:  This function returns the default unit for angles.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetAnalogDemodulationAngleUnits(
    ViSession   instrSession,
    ViInt32*    angleUnits
)
{
    CHECKOPT (RSESCI_K7);

    return rsesci_getEnum (instrSession,
                           "UNIT:ANGL?",
                           angleUnits,
                           angleUnitsArr,
                           VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Analog Demodulation Zoom
 * Purpose:  This function enables or disables the zoom function for the
 *           FM-demodulator measurement data.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetAnalogDemodulationZoom(
    ViSession   instrSession,
    ViBoolean   zoom
)
{
    CHECKOPT (RSESCI_K7);

    return rsesci_setViBoolean (instrSession,
                                "ADEM:ZOOM:STAT",
                                zoom,
                                VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Analog Demodulation Zoom
 * Purpose:  This function returns if the zoom function for the FM-demodulator
 *           measurement data is enabled or disabled.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetAnalogDemodulationZoom(
    ViSession   instrSession,
    ViBoolean*  zoom
)
{
    CHECKOPT (RSESCI_K7);

    return rsesci_getViBoolean (instrSession,
                                "ADEM:ZOOM:STAT?",
                                zoom,
                                VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Analog Demodulation Zoom Start
 * Purpose:  This function selects the start time for the display of
 *           individual measured values of the FM demodulator. The maximum
 *           possible value depends on the measurement time, which is set in
 *           the instrument and can be queried with command
 *           [SENSe]:ADEMod:MTIMe?.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetAnalogDemodulationZoomStart(
    ViSession   instrSession,
    ViReal64    zoomStart
)
{
    CHECKOPT (RSESCI_K7);

    return rsesci_setViReal64 (instrSession,
                               "ADEM:ZOOM:STAR",
                               zoomStart,
                               0.0, 0.0,
                               VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Analog Demodulation Zoom Start
 * Purpose:  This function returns the start time for the display of
 *           individual measured values of the FM demodulator.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetAnalogDemodulationZoomStart(
    ViSession   instrSession,
    ViReal64*   zoomStart
)
{
    CHECKOPT (RSESCI_K7);

    return rsesci_getViReal64 (instrSession,
                               "ADEM:ZOOM:STAR?",
                               zoomStart,
                               VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get FM Modulation Measurement
 * Purpose:  This function queries the results of FM modulation measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetFMModulationMeasurement(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32     marker,
    ViInt32     trace,
    ViInt32     measurement,
    ViReal64*   result
)
{
    ViChar  buffer[BUFFER_SIZE];

    CHECKWINDOW;

    CHECKOPT (RSESCI_K7);

    if (rsesci_invalidViInt32Range (marker, 1, 4))
        return VI_ERROR_PARAMETER3;
    if (rsesci_invalidViInt32Range (trace, 1, 3))
        return VI_ERROR_PARAMETER4;
    if (rsesci_invalidViInt32Range (measurement, 0, 3))
        return VI_ERROR_PARAMETER5;

    sprintf (buffer, "CALC%ld:MARK%ld:FUNC:ADEM:FM:RES%ld? %s",
             window,
             marker,
             trace,
             modulationMeasurementArr[measurement]);

    return rsesci_getViReal64(instrSession,
                              buffer,
                              result,
                              VI_ERROR_PARAMETER6);
}

/*****************************************************************************
 * Function: Get PM Modulation Measurement
 * Purpose:  This function queries the results of PM modulation measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetPMModulationMeasurement(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32     marker,
    ViInt32     trace,
    ViInt32     measurement,
    ViReal64*   result
)
{
    ViChar  buffer[BUFFER_SIZE];

    CHECKWINDOW;

    CHECKOPT (RSESCI_K7);

    if (rsesci_invalidViInt32Range (marker, 1, 4))
        return VI_ERROR_PARAMETER3;
    if (rsesci_invalidViInt32Range (trace, 1, 3))
        return VI_ERROR_PARAMETER4;
    if (rsesci_invalidViInt32Range (measurement, 0, 3))
        return VI_ERROR_PARAMETER5;

    sprintf (buffer, "CALC%ld:MARK%ld:FUNC:ADEM:PM:RES%ld? %s",
             window,
             marker,
             trace,
             modulationMeasurementArr[measurement]);

    return rsesci_getViReal64(instrSession,
                              buffer,
                              result,
                              VI_ERROR_PARAMETER6);
}

/*****************************************************************************
 * Function: Get AM Modulation Measurement
 * Purpose:  This function queries the results of AM modulation measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetAMModulationMeasurement (ViSession instrSession,
                                                     ViInt32 window,
                                                     ViInt32 marker,
                                                     ViInt32 trace,
                                                     ViInt32 measurement,
                                                     ViReal64 *result)
{
	ViChar  buffer[BUFFER_SIZE];

    CHECKWINDOW;

    CHECKOPT (RSESCI_K7);

    if (rsesci_invalidViInt32Range (marker, 1, 4))
        return VI_ERROR_PARAMETER3;
    if (rsesci_invalidViInt32Range (trace, 1, 3))
        return VI_ERROR_PARAMETER4;
    if (rsesci_invalidViInt32Range (measurement, 0, 3))
        return VI_ERROR_PARAMETER5;

    sprintf (buffer, "CALC%ld:MARK%ld:FUNC:ADEM:AM:RES%ld? %s",
             window,
             marker,
             trace,
             modulationMeasurementArr[measurement]);

    return rsesci_getViReal64(instrSession,
                              buffer,
                              result,
                              VI_ERROR_PARAMETER6);    
}

/*****************************************************************************
 * Function: Get Analog Demodulation Audio Frequency
 * Purpose:  This function queries the audio frequency with analog
 *           demodulation.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetAnalogDemodulationAudioFrequency(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32     marker,
    ViInt32     trace,
    ViReal64*   result
)
{
    ViChar  buffer[BUFFER_SIZE];

    CHECKWINDOW;

    CHECKOPT (RSESCI_K7);

    if (rsesci_invalidViInt32Range (marker, 1, 4))
        return VI_ERROR_PARAMETER3;
    if (rsesci_invalidViInt32Range (trace, 1, 3))
        return VI_ERROR_PARAMETER4;

    sprintf (buffer, "CALC%ld:MARK%ld:FUNC:ADEM:AFR:RES%ld?",
             window,
             marker,
             trace);

    return rsesci_getViReal64(instrSession,
                              buffer,
                              result,
                              VI_ERROR_PARAMETER5);
}

/*****************************************************************************
 * Function: Get Analog Demodulation Frequency Error
 * Purpose:  This function queries the frequency error with FM demodulation.
 *           The offset thus determined differs from that calculated in the
 *           command [SENSe:]ADEMod:FM:OFFSet?, since, for determination of the
 *           frequency deviation, the modulation is removed by means of lowpass
 *           filtering, producing results that are different from those
 *           obtained by averaging with the SENSe:...command.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetAnalogDemodulationFrequencyError(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32     marker,
    ViInt32     trace,
    ViReal64*   result
)
{
    ViChar  buffer[BUFFER_SIZE];

    CHECKWINDOW;

    CHECKOPT (RSESCI_K7);

    if (rsesci_invalidViInt32Range (marker, 1, 4))
        return VI_ERROR_PARAMETER3;
    if (rsesci_invalidViInt32Range (trace, 1, 3))
        return VI_ERROR_PARAMETER4;

    sprintf (buffer, "CALC%ld:MARK%ld:FUNC:ADEM:FERR:RES%ld?",
             window,
             marker,
             trace);

    return rsesci_getViReal64(instrSession,
                              buffer,
                              result,
                              VI_ERROR_PARAMETER5);
}

/*****************************************************************************
 * Function: Get Analog Demodulation SINAD
 * Purpose:  This function queries the result of the SINAD measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetAnalogDemodulationSINAD(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32     marker,
    ViInt32     trace,
    ViReal64*   result
)
{
    ViChar  buffer[BUFFER_SIZE];

    CHECKWINDOW;

    CHECKOPT (RSESCI_K7);

    if (rsesci_invalidViInt32Range (marker, 1, 4))
        return VI_ERROR_PARAMETER3;
    if (rsesci_invalidViInt32Range (trace, 1, 3))
        return VI_ERROR_PARAMETER4;

    sprintf (buffer, "CALC%ld:MARK%ld:FUNC:ADEM:SIN:RES%ld?",
             window,
             marker,
             trace);

    return rsesci_getViReal64(instrSession,
                              buffer,
                              result,
                              VI_ERROR_PARAMETER5);
}

/*****************************************************************************
 * Function: Get Analog Demodulation THD
 * Purpose:  This function queries the result of the THD measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetAnalogDemodulationTHD(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32     marker,
    ViInt32     trace,
    ViReal64*   result
)
{
    ViChar  buffer[BUFFER_SIZE];

    CHECKWINDOW;

    CHECKOPT (RSESCI_K7);

    if (rsesci_invalidViInt32Range (marker, 1, 4))
        return VI_ERROR_PARAMETER3;
    if (rsesci_invalidViInt32Range (trace, 1, 3))
        return VI_ERROR_PARAMETER4;

    sprintf (buffer, "CALC%ld:MARK%ld:FUNC:ADEM:THD:RES%ld?",
             window,
             marker,
             trace);

    return rsesci_getViReal64(instrSession,
                              buffer,
                              result,
                              VI_ERROR_PARAMETER5);
}

/*****************************************************************************
 * Function: Get Analog Demodulation Carrier Power
 * Purpose:  This function queries the carrier power.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetAnalogDemodulationCarrierPower(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32     marker,
    ViInt32     trace,
    ViReal64*   result
)
{
    ViChar  buffer[BUFFER_SIZE];

    CHECKWINDOW;

    CHECKOPT (RSESCI_K7);

    if (rsesci_invalidViInt32Range (marker, 1, 4))
        return VI_ERROR_PARAMETER3;
    if (rsesci_invalidViInt32Range (trace, 1, 3))
        return VI_ERROR_PARAMETER4;

    sprintf (buffer, "CALC%ld:MARK%ld:FUNC:ADEM:CARR:RES%ld?",
             window,
             marker,
             trace);

    return rsesci_getViReal64(instrSession,
                              buffer,
                              result,
                              VI_ERROR_PARAMETER5);
}

/*****************************************************************************
 * Function: Set AF Spectrum Result Types
 * Purpose:  This function selects the simultaneously measured result types of
 *           the AF spectrum of the demodulated signal.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetAFSpectrumResultTypes(
    ViSession   instrSession,
    ViInt32     demodulationType,
    ViInt32     resultType1,
    ViInt32     resultType2,
    ViInt32     resultType3
)
{
    ViChar  buffer[BUFFER_SIZE];

    CHECKOPT (RSESCI_K7);

    if (rsesci_invalidViInt32Range (demodulationType, 0, 2))
        return VI_ERROR_PARAMETER2;
    if (rsesci_invalidViInt32Range (resultType1, 0, 5))
        return VI_ERROR_PARAMETER3;
    if (rsesci_invalidViInt32Range (resultType2, 0, 5))
        return VI_ERROR_PARAMETER4;
    if (rsesci_invalidViInt32Range (resultType3, 0, 5))
        return VI_ERROR_PARAMETER5;

    sprintf (buffer, "ADEM:%s:AFSP:TYPE %s,%s,%s",
        demodulationTypeArr[demodulationType],
        afspectrumResultTypeArr[resultType1],
        afspectrumResultTypeArr[resultType2],
        afspectrumResultTypeArr[resultType3]);

    return rsesci_sendCmd (instrSession, buffer);
}

/*****************************************************************************
 * Function: Get AF Spectrum Result Values
 * Purpose:  This function reads out measurement results of the AF spectrum of
 *           the demodulated signal with the specified result type.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetAFSpectrumResultValues(
    ViSession   instrSession,
    ViInt32     demodulationType,
    ViInt32     resultType,
    ViInt32     arraySize,
    ViInt32*    numberofResults,
    ViReal64    _VI_FAR results[]
)
{
    ViStatus    rsesci_status   = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];

    CHECKOPT (RSESCI_K7);

    if (rsesci_invalidViInt32Range (demodulationType, 0, 2))
        return VI_ERROR_PARAMETER2;
    if (rsesci_invalidViInt32Range (resultType, 1, 4))
        return VI_ERROR_PARAMETER3;
    if (arraySize < 0)
        return VI_ERROR_PARAMETER4;
    if (!numberofResults)
        return VI_ERROR_PARAMETER5;

    sprintf (buffer, "ADEM:%s:AFSP:RES? %s",
             demodulationTypeArr[demodulationType],
             afspectrumResultTypeArr[resultType]);

    rsesci_status = rsesci_getViReal64BinArray (instrSession,
                                                buffer,
                                                results,
                                                arraySize,
                                                numberofResults,
                                                VI_ERROR_PARAMETER6);

    return rsesci_status;
}

/*****************************************************************************
 * Function: Set RF Spectrum Result Types
 * Purpose:  This function selects the result types to be created in parallel
 *           by the RF Spectrum measurement with active analog demodulation.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetRFSpectrumResultTypes(
    ViSession   instrSession,
    ViInt32 resultType1,
    ViInt32 resultType2,
    ViInt32 resultType3
)
{
    ViChar  buffer[BUFFER_SIZE];

    CHECKOPT (RSESCI_K7);

    if (rsesci_invalidViInt32Range (resultType1, 0, 5))
        return VI_ERROR_PARAMETER2;
    if (rsesci_invalidViInt32Range (resultType2, 0, 5))
        return VI_ERROR_PARAMETER3;
    if (rsesci_invalidViInt32Range (resultType3, 0, 5))
        return VI_ERROR_PARAMETER4;

    sprintf (buffer, "ADEM:SPEC:TYPE %s,%s,%s",
        afspectrumResultTypeArr[resultType1],
        afspectrumResultTypeArr[resultType2],
        afspectrumResultTypeArr[resultType3]);

    return rsesci_sendCmd (instrSession, buffer);
}

/*****************************************************************************
 * Function: Get RF Spectrum Result Values
 * Purpose:  This function reads the result data obtained by RF Spectrum
 *           measurement depending on the type indicated as a parameter.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetRFSpectrumResultValues(
    ViSession   instrSession,
    ViInt32     resultType,
    ViInt32     arraySize,
    ViInt32*    numberofResults,
    ViReal64    _VI_FAR results[]
)
{
    ViStatus    rsesci_status   = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];

    CHECKOPT (RSESCI_K7);

    if (rsesci_invalidViInt32Range (resultType, 1, 4))
        return VI_ERROR_PARAMETER2;
    if (arraySize < 0)
        return VI_ERROR_PARAMETER3;
    if (!numberofResults)
        return VI_ERROR_PARAMETER4;

    sprintf (buffer, "ADEM:SPEC:RES? %s",
             afspectrumResultTypeArr[resultType]);

    rsesci_status = rsesci_getViReal64BinArray (instrSession,
                                                buffer,
                                                results,
                                                arraySize,
                                                numberofResults,
                                                VI_ERROR_PARAMETER5);

    return rsesci_status;
}

/*****************************************************************************
 * Function: Set Time Domain Result Types
 * Purpose:  This function selects the result types to be created in parallel
 *           by currently selected demodulation.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetTimeDomainResultTypes(
    ViSession   instrSession,
    ViInt32 demodulationType,
    ViInt32 resultType1,
    ViInt32 resultType2,
    ViInt32 resultType3
)
{
    ViChar  buffer[BUFFER_SIZE];

    CHECKOPT (RSESCI_K7);

    if (rsesci_invalidViInt32Range (demodulationType, 0, 3))
        return VI_ERROR_PARAMETER2;
    if (rsesci_invalidViInt32Range (resultType1, 0, 5))
        return VI_ERROR_PARAMETER3;
    if (rsesci_invalidViInt32Range (resultType2, 0, 5))
        return VI_ERROR_PARAMETER4;
    if (rsesci_invalidViInt32Range (resultType3, 0, 5))
        return VI_ERROR_PARAMETER5;

    sprintf (buffer, "ADEM:%s:TDOM:TYPE %s,%s,%s",
        demodulationTypeArr[demodulationType],
        afspectrumResultTypeArr[resultType1],
        afspectrumResultTypeArr[resultType2],
        afspectrumResultTypeArr[resultType3]);

    return rsesci_sendCmd (instrSession, buffer);
}

/*****************************************************************************
 * Function: Get Time Domain Result Values
 * Purpose:  This function reads the result data obtained by demodulation
 *           depending on the type indicated as a parameter.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetTimeDomainResultValues(
    ViSession   instrSession,
    ViInt32     demodulationType,
    ViInt32     resultType,
    ViInt32     arraySize,
    ViInt32*    numberofResults,
    ViReal64    _VI_FAR results[]
)
{
    ViStatus    rsesci_status   = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];

    CHECKOPT (RSESCI_K7);

    if (rsesci_invalidViInt32Range (demodulationType, 0, 3))
        return VI_ERROR_PARAMETER2;
    if (rsesci_invalidViInt32Range (resultType, 1, 4))
        return VI_ERROR_PARAMETER3;
    if (arraySize < 0)
        return VI_ERROR_PARAMETER4;
    if (!numberofResults)
        return VI_ERROR_PARAMETER5;

    sprintf (buffer, "ADEM:%s:TDOM:RES? %s",
             demodulationTypeArr[demodulationType],
             afspectrumResultTypeArr[resultType]);

    rsesci_status = rsesci_getViReal64BinArray (instrSession,
                                                buffer,
                                                results,
                                                arraySize,
                                                numberofResults,
                                                VI_ERROR_PARAMETER6);

    return rsesci_status;
}

/*****************************************************************************
 * Function: Get FM Offset Value
 * Purpose:  This function calculates the FM offset of the currently available
 *           measurement data set. If averaging has been activated before
 *           acquiring the data set (using [SENSe:]ADEMod:FM[:TYPE]), the
 *           averaged FM offset over several measurements can be obtained as
 *           well by setting <result type> = AVERage.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetFMOffsetValue(
    ViSession   instrSession,
    ViInt32     measurement,
    ViReal64*   result
)
{
    ViChar  buffer[BUFFER_SIZE];

    CHECKOPT (RSESCI_K7);

    if (rsesci_invalidViInt32Range (measurement, 0, 1))
        return VI_ERROR_PARAMETER2;

    sprintf (buffer, "ADEM:FM:OFFS? %s", measurementArr[measurement]);

    return rsesci_getViReal64(instrSession,
                              buffer,
                              result,
                              VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set Data Averaging
 * Purpose:  This function switches on or off the average calculation for the
 *           selected trace in the selected measurement window.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetDataAveraging(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32     trace,
    ViBoolean   dataAveraging
)
{
    ViChar  buffer[BUFFER_SIZE];

    CHECKOPT (RSESCI_K7);

    if (rsesci_invalidViInt32Range (trace, 1, 3))
        return VI_ERROR_PARAMETER3;

    sprintf (buffer, "SENS%%ld:AVER:STAT%ld", trace);

    return rsesci_setViBooleanWindow(instrSession,
                                     buffer,
                                     window,
                                     dataAveraging,
                                     VI_ERROR_PARAMETER4);
}

/*****************************************************************************
 * Function: Get Data Averaging
 * Purpose:  This function returns if the average calculation for the selected
 *           trace in the selected measurement window is activate dor not.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetDataAveraging(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32     trace,
    ViBoolean*  dataAveraging
)
{
    ViChar  buffer[BUFFER_SIZE];

    CHECKOPT (RSESCI_K7);

    if (rsesci_invalidViInt32Range (trace, 1, 3))
        return VI_ERROR_PARAMETER3;

    sprintf (buffer, "SENS%%ld:AVER:STAT%ld?", trace);

    return rsesci_getViBooleanWindow(instrSession,
                                     buffer,
                                     window,
                                     dataAveraging,
                                     VI_ERROR_PARAMETER4);
}

/*****************************************************************************
 * Function: Set Average Count
 * Purpose:  In receiver mode, this function defines the number of scan
 *           started with single scan. In analyzer mode, it defines the number
 *           of measurements which contribute to the average value. It should
 *           be noted that continuous averaging will be performed after the
 *           indicated number has been reached in continuous sweep mode.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetAverageCount(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32     averageCount
)
{
    return rsesci_setViInt32Window(instrSession,
                                   "SENS%ld:AVER:COUN",
                                   window,
                                   averageCount,
                                   0, 32767,
                                   VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Average Count
 * Purpose:  In receiver mode, this function returns the number of scan
 *           started with single scan. In analyzer mode, it returns the number
 *           of measurements which contribute to the average value.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetAverageCount(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32*    averageCount
)
{
    return rsesci_getViInt32Window(instrSession,
                                   "SENS%ld:AVER:COUN?",
                                   window,
                                   averageCount,
                                   VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set Average Type
 * Purpose:  This function selects the type of average function. If VIDeo is
 *           selected, the logarithmic power is averaged and, if LINear is
 *           selected, the power values are averaged before they are converted
 *           to logarithmic values.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetAverageType(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32     averageType
)
{
    return rsesci_setEnumWindow(instrSession,
                                "SENS%ld:AVER:TYPE",
                                window,
                                averageType,
                                averageTypeArr,
                                VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Average Type
 * Purpose:  This function returns the type of average function.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetAverageType(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32*    averageType
)
{
    return rsesci_getEnumWindow(instrSession,
                                "SENS%ld:AVER:TYPE?",
                                window,
                                averageType,
                                averageTypeArr,
                                VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set Limit Line Name
 * Purpose:  This function assigns a name to a limit line numbered 1 to 8. If
 *           it does not exist already, a limit line with this name is created.
 *           The command is independent of the measurement window.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetLimitLineName(
    ViSession   instrSession,
    ViInt32     limit,
    ViString    name
)
{
    ViChar  buffer[BUFFER_SIZE];

    if (rsesci_invalidViInt32Range (limit, 1, 8))
        return VI_ERROR_PARAMETER2;

    sprintf (buffer, "CALC:LIM%ld:NAME", limit);

    return rsesci_setViStringQuoted (instrSession,
                                     buffer,
                                     "'",
                                     name,
                                     VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Limit Line Name
 * Purpose:  This function queries the limit line name.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetLimitLineName(
    ViSession   instrSession,
    ViInt32     limit,
    ViChar      _VI_FAR name[]
)
{
    ViChar  buffer[BUFFER_SIZE];

    if (rsesci_invalidViInt32Range (limit, 1, 8))
        return VI_ERROR_PARAMETER2;

    sprintf (buffer, "CALC:LIM%ld:NAME?", limit);

    return rsesci_getViStringQuoted (instrSession,
                                     buffer,
                                     "'",
                                     8,
                                     name,
                                     VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set Limit Line Visible
 * Purpose:  This function switches on or off the indicated limit line in the
 *           selected measurement window.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetLimitLineVisible(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32     limit,
    ViInt32     limitType,
    ViBoolean   state
)
{
    ViChar  buffer[BUFFER_SIZE];

    if (rsesci_invalidViInt32Range (limit, 1, 8))
        return VI_ERROR_PARAMETER3;
    if (rsesci_invalidViInt32Range (limitType, 0, 1))
        return VI_ERROR_PARAMETER4;

    sprintf (buffer, "CALC%%ld:LIM%ld:%s:STAT", limit, limitTypeArr[limitType]);

    return rsesci_setViBooleanWindow(instrSession,
                                     buffer,
                                     window,
                                     state,
                                     VI_ERROR_PARAMETER5);
}

/*****************************************************************************
 * Function: Get Limit Line Visible
 * Purpose:  This function the state of the indicated limit line in the
 *           selected measurement window.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetLimitLineVisible(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32     limit,
    ViInt32     limitType,
    ViBoolean*  state
)
{
    ViChar  buffer[BUFFER_SIZE];

    if (rsesci_invalidViInt32Range (limit, 1, 8))
        return VI_ERROR_PARAMETER3;

    if (rsesci_invalidViInt32Range (limitType, 0, 1))
        return VI_ERROR_PARAMETER4;

    sprintf (buffer, "CALC%%ld:LIM%ld:%s:STAT?", limit, limitTypeArr[limitType]);

    return rsesci_getViBooleanWindow(instrSession,
                                     buffer,
                                     window,
                                     state,
                                     VI_ERROR_PARAMETER5);
}

/*****************************************************************************
 * Function: Set Limit Line Trace
 * Purpose:  This function assigns a limit line to a trace in the indicated
 *           measurement window.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetLimitLineTrace(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32     limit,
    ViInt32     trace
)
{
    ViChar  buffer[BUFFER_SIZE];

    if (rsesci_invalidViInt32Range (limit, 1, 8))
        return VI_ERROR_PARAMETER3;

    sprintf (buffer, "CALC%%ld:LIM%ld:TRAC", limit);

    return rsesci_setViInt32Window(instrSession,
                                   buffer,
                                   window,
                                   trace,
                                   1,3,
                                   VI_ERROR_PARAMETER4);
}

/*****************************************************************************
 * Function: Get Limit Line Trace
 * Purpose:  This function queries the trace associated with indicated limit
 *           line.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetLimitLineTrace(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32 limit,
    ViInt32*    trace
)
{
    ViChar  buffer[BUFFER_SIZE];

    if (rsesci_invalidViInt32Range (limit, 1, 8))
        return VI_ERROR_PARAMETER3;

    sprintf (buffer, "CALC%%ld:LIM%ld:TRAC?", limit);

    return rsesci_getViInt32Window(instrSession,
                                   buffer,
                                   window,
                                   trace,
                                   VI_ERROR_PARAMETER4);
}

/*****************************************************************************
 * Function: Copy Limit Line
 * Purpose:  This function copies one limit line onto another one. It is
 *           independent of the measurement window.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_CopyLimitLine(
    ViSession   instrSession,
    ViInt32 sourceLimit,
    ViInt32 destinationLimit
)
{
    ViChar  buffer[BUFFER_SIZE];

    if (rsesci_invalidViInt32Range (sourceLimit, 1, 8))
        return VI_ERROR_PARAMETER3;

    sprintf (buffer, "CALC:LIM%ld:COPY", sourceLimit);

    return rsesci_setViInt32(instrSession,
                             buffer,
                             destinationLimit,
                             1, 8,
                             VI_ERROR_PARAMETER4);
}

/*****************************************************************************
 * Function: Delete Limit Line
 * Purpose:  This function deletes the selected limit line. The command is
 *           independent of the measurement window.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_DeleteLimitLine(
    ViSession   instrSession,
    ViInt32 limit
)
{
    ViChar  buffer[BUFFER_SIZE];

    if (rsesci_invalidViInt32Range (limit, 1, 8))
        return VI_ERROR_PARAMETER2;

    sprintf (buffer, "CALC:LIM%ld:DEL", limit);

    return rsesci_sendCmd(instrSession,
                          buffer);
}

/*****************************************************************************
 * Function: Set Limit Line X Offset
 * Purpose:  This function defines an offset for the X axis value of the
 *           selected relative limit line in the frequency or time domain.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetLimitLineXOffset(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32 limit,
    ViReal64    offset
)
{
    ViChar  buffer[BUFFER_SIZE];

    if (rsesci_invalidViInt32Range (limit, 1, 8))
        return VI_ERROR_PARAMETER3;

    sprintf (buffer, "CALC%%ld:LIM%ld:CONT:OFFS", limit);

    return rsesci_setViReal64Window (instrSession,
                                     buffer,
                                     window,
                                     offset,
                                     0.0, 0.0,
                                     VI_ERROR_PARAMETER4);
}

/*****************************************************************************
 * Function: Get Limit Line X Offset
 * Purpose:  This function queries an offset for the X axis value of the
 *           selected relative limit line in the frequency or time domain.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetLimitLineXOffset(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32 limit,
    ViReal64*   offset
)
{
    ViChar  buffer[BUFFER_SIZE];

    if (rsesci_invalidViInt32Range (limit, 1, 8))
        return VI_ERROR_PARAMETER3;

    sprintf (buffer, "CALC%%ld:LIM%ld:CONT:OFFS?", limit);

    return rsesci_getViReal64Window (instrSession,
                                     buffer,
                                     window,
                                     offset,
                                     VI_ERROR_PARAMETER4);
}

/*****************************************************************************
 * Function: Set Limit Line Y Offset
 * Purpose:  This function defines an offset for the Y axis of the selected
 *           relative lower limit line. In contrast to CALC:LIM:LOW:SHIFt, the
 *           line is not shifted by modifying the individual Y values but by
 *           means of an additive offset.
 *           The offset is independent of the measurement window.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetLimitLineYOffset(
    ViSession   instrSession,
    ViInt32 limit,
    ViInt32 limitType,
    ViReal64    offset
)
{
    ViChar  buffer[BUFFER_SIZE];

    if (rsesci_invalidViInt32Range (limit, 1, 8))
        return VI_ERROR_PARAMETER3;

    if (rsesci_invalidViInt32Range (limitType, 0, 1))
        return VI_ERROR_PARAMETER4;

    sprintf (buffer, "CALC:LIM%ld:%s:OFFS", limit, limitTypeArr[limitType]);

    return rsesci_setViReal64(instrSession,
                              buffer,
                              offset,
                              0.0,0.0,
                              VI_ERROR_PARAMETER5);
}

/*****************************************************************************
 * Function: Get Limit Line Y Offset
 * Purpose:  This function queries an offset for the Y axis of the selected
 *           relative lower limit line.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetLimitLineYOffset(
    ViSession   instrSession,
    ViInt32     limit,
    ViInt32     limitType,
    ViReal64*   offset
)
{
    ViChar  buffer[BUFFER_SIZE];

    if (rsesci_invalidViInt32Range (limit, 1, 8))
        return VI_ERROR_PARAMETER3;

    if (rsesci_invalidViInt32Range (limitType, 0, 1))
        return VI_ERROR_PARAMETER4;

    sprintf (buffer, "CALC:LIM%ld:%s:OFFS?", limit, limitTypeArr[limitType]);

    return rsesci_getViReal64(instrSession,
                              buffer,
                              offset,
                              VI_ERROR_PARAMETER5);
}

/*****************************************************************************
 * Function: Get Active Limit Lines
 * Purpose:  This function queries the names of all active limit lines.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetActiveLimitLines (ViSession instrSession,
                                              ViInt32 arraySize,
                                              ViChar activeLimitLines[])
{
	return rsesci_getViString (instrSession,
		                       "CALC1:LIM1:ACT?",
							   activeLimitLines,
							   arraySize,
							   VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set Limit Line Check
 * Purpose:  This function switches on or off the limit check for the
 *           selected limit line in the selected measurement window. The result
 *           of the limit check can be queried with
 *           rsesci_GetLimitLineCheckResult().
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetLimitLineCheck(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32     limit,
    ViBoolean   limitCheck
)
{
    ViChar  buffer[BUFFER_SIZE];

    if (rsesci_invalidViInt32Range (limit, 1, 8))
        return VI_ERROR_PARAMETER3;

    sprintf (buffer, "CALC%%ld:LIM%ld:STAT", limit);

    return rsesci_setViBooleanWindow(instrSession,
                                     buffer,
                                     window,
                                     limitCheck,
                                     VI_ERROR_PARAMETER4);
}

/*****************************************************************************
 * Function: Get Limit Line Check
 * Purpose:  This function queries the state of the limit check for the
 *           selected limit line in the selected measurement window.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetLimitLineCheck(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32     limit,
    ViBoolean*  limitCheck
)
{
    ViChar  buffer[BUFFER_SIZE];

    if (rsesci_invalidViInt32Range (limit, 1, 8))
        return VI_ERROR_PARAMETER3;

    sprintf (buffer, "CALC%%ld:LIM%ld:STAT?", limit);

    return rsesci_getViBooleanWindow(instrSession,
                                     buffer,
                                     window,
                                     limitCheck,
                                     VI_ERROR_PARAMETER4);
}

/*****************************************************************************
 * Function: Get Limit Line Check Result
 * Purpose:  This function queries the result of the limit check of the limit
 *           line indicated in the selected measurement window. It should be
 *           noted that a complete sweep must have been performed for obtaining
 *           a valid result. A synchronization with *OPC, *OPC? or *WAI should
 *           therefore be provided.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetLimitLineCheckResult(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32     limit,
    ViInt32*    result
)
{
    ViChar  buffer[BUFFER_SIZE];

    if (rsesci_invalidViInt32Range (limit, 1, 8))
        return VI_ERROR_PARAMETER3;

    sprintf (buffer, "CALC%%ld:LIM%ld:FAIL?", limit);

    return rsesci_getViInt32Window(instrSession,
                                   buffer,
                                   window,
                                   result,
                                   VI_ERROR_PARAMETER4);
}

/*****************************************************************************
 * Function: Clear Limit Line Check Result
 * Purpose:  This function deletes the result of the current limit check for
 *           all limit lines in the selected measurement window.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_ClearLimitLineCheckResult(
    ViSession   instrSession,
    ViInt32     window
)
{
    return rsesci_sendCmdWindow (instrSession,
                                 "CALC%ld:LIM:CLE",
                                 window);
}

/*****************************************************************************
 * Function: Set Limit Line Domain
 * Purpose:  This function defines the frequency or time domain for the x axis
 *           values.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetLimitLineDomain(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32 limit,
    ViInt32 domain
)
{
    ViChar  buffer[BUFFER_SIZE];

    if (rsesci_invalidViInt32Range (limit, 1, 8))
        return VI_ERROR_PARAMETER3;

    sprintf (buffer, "CALC%%ld:LIM%ld:CONT:DOM", limit);

    return rsesci_setEnumWindow (instrSession,
                                 buffer,
                                 window,
                                 domain,
                                 limitDomainArr,
                                 VI_ERROR_PARAMETER4);
}

/*****************************************************************************
 * Function: Get Limit Line Domain
 * Purpose:  This function queries selected domain for the x axis values.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetLimitLineDomain(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32 limit,
    ViInt32*    domain
)
{
    ViChar  buffer[BUFFER_SIZE];

    if (rsesci_invalidViInt32Range (limit, 1, 8))
        return VI_ERROR_PARAMETER3;

    sprintf (buffer, "CALC%%ld:LIM%ld:CONT:DOM?", limit);

    return rsesci_getEnumWindow (instrSession,
                                 buffer,
                                 window,
                                 domain,
                                 limitDomainArr,
                                 VI_ERROR_PARAMETER4);
}

/*****************************************************************************
 * Function: Set Limit Line Interpolation
 * Purpose:  This function selects linear or logarithmic interpolation for the
 *           calculation of limit lines from frequency points.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetLimitLineInterpolation(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32 limit,
    ViInt32 interpolationAxis,
    ViInt32 interpolation
)
{
    ViChar  buffer[BUFFER_SIZE];

    if (rsesci_invalidViInt32Range (limit, 1, 8))
        return VI_ERROR_PARAMETER3;

    if (rsesci_invalidViInt32Range (interpolationAxis, 0, 2))
        return VI_ERROR_PARAMETER4;

    if (rsesci_invalidViInt32Range (interpolation, 0, 1))
        return VI_ERROR_PARAMETER5;

    sprintf (buffer, "CALC%%ld:LIM%ld:%s:SPAC", limit, limitTypeArr[interpolationAxis]);

    return rsesci_setEnumWindow(instrSession,
                                buffer,
                                window,
                                interpolation,
                                spacingArr,
                                VI_ERROR_PARAMETER5);
}

/*****************************************************************************
 * Function: Get Limit Line Interpolation
 * Purpose:  This function queries selected interpolation for the calculation
 *           of limit lines from frequency points.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetLimitLineInterpolation(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32 limit,
    ViInt32 interpolationAxis,
    ViInt32*    interpolation
)
{
    ViChar  buffer[BUFFER_SIZE];

    if (rsesci_invalidViInt32Range (limit, 1, 8))
        return VI_ERROR_PARAMETER3;

    if (rsesci_invalidViInt32Range (interpolationAxis, 0, 2))
        return VI_ERROR_PARAMETER4;

    sprintf (buffer, "CALC%%ld:LIM%ld:%s:SPAC?", limit, limitTypeArr[interpolationAxis]);

    return rsesci_getEnumWindow(instrSession,
                                buffer,
                                window,
                                interpolation,
                                spacingArr,
                                VI_ERROR_PARAMETER5);
}

/*****************************************************************************
 * Function: Set Limit Line Scaling
 * Purpose:  This function selects the relative or absolute scaling for
 *           selected axis of the selected limit line. The definition is
 *           independent of the measurement window.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetLimitLineScaling(
    ViSession   instrSession,
    ViInt32 limit,
    ViInt32 interpolationAxis,
    ViInt32 limitLineScaling
)
{
    ViChar  buffer[BUFFER_SIZE];

    if (rsesci_invalidViInt32Range (limit, 1, 8))
        return VI_ERROR_PARAMETER2;

    if (rsesci_invalidViInt32Range (interpolationAxis, 0, 2))
        return VI_ERROR_PARAMETER3;

    sprintf (buffer, "CALC:LIM%ld:%s:MODE", limit, limitTypeArr[interpolationAxis]);

    return rsesci_setEnum(instrSession,
                          buffer,
                          limitLineScaling,
                          absRelArr,
                          VI_ERROR_PARAMETER4);
}

/*****************************************************************************
 * Function: Get Limit Line Scaling
 * Purpose:  This function queries selected scaling for selected axis of the
 *           selected limit line. The function is independent of the
 *           measurement window.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetLimitLineScaling(
    ViSession   instrSession,
    ViInt32     limit,
    ViInt32     interpolationAxis,
    ViInt32*    limitLineScaling
)
{
    ViChar  buffer[BUFFER_SIZE];

    if (rsesci_invalidViInt32Range (limit, 1, 8))
        return VI_ERROR_PARAMETER2;

    if (rsesci_invalidViInt32Range (interpolationAxis, 0, 2))
        return VI_ERROR_PARAMETER3;

    sprintf (buffer, "CALC:LIM%ld:%s:MODE?", limit, limitTypeArr[interpolationAxis]);

    return rsesci_getEnum(instrSession,
                          buffer,
                          limitLineScaling,
                          absRelArr,
                          VI_ERROR_PARAMETER4);
}

/*****************************************************************************
 * Function: Set Limit Line Unit
 * Purpose:  This function defines the unit of the selected limit line. The
 *           definition is valid independently of the measurement window.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetLimitLineUnit(
    ViSession   instrSession,
    ViInt32     limit,
    ViInt32     levelAxisUnit
)
{
    ViChar  buffer[BUFFER_SIZE];

    if (rsesci_invalidViInt32Range (limit, 1, 8))
        return VI_ERROR_PARAMETER2;

	if ( (levelAxisUnit == RSESCI_UNIT_DBPT) || (levelAxisUnit == RSESCI_UNIT_DBUA_M) ||
		((levelAxisUnit >= RSESCI_UNIT_DEG) && (levelAxisUnit <= RSESCI_UNIT_UNITLESS)) )
		CHECKMODEL(RSESU8 | RSESU26 | RSESU40);
		
    sprintf (buffer, "CALC:LIM%ld:UNIT", limit);

    return rsesci_setEnum (instrSession,
                           buffer,
                           levelAxisUnit,
                           limitLineUnitArr,
                           VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Limit Line Unit
 * Purpose:  This function queries the unit of the selected limit line.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetLimitLineUnit(
    ViSession   instrSession,
    ViInt32     limit,
    ViInt32*    levelAxisUnit
)
{
    ViChar  buffer[BUFFER_SIZE];

    if (rsesci_invalidViInt32Range (limit, 1, 8))
        return VI_ERROR_PARAMETER2;

    sprintf (buffer, "CALC:LIM%ld:UNIT?", limit);

    return rsesci_getEnum (instrSession,
                           buffer,
                           levelAxisUnit,
                           limitLineUnitArr,
                           VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set Limit Line Margin
 * Purpose:  This function defines a margin to a lower limit line, at which
 *           out-of-limit values are signalled (if the limit check is active),
 *           but not handled as a violation of the limit value.
 *           The margin is independent of the measurement window.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetLimitLineMargin(
    ViSession   instrSession,
    ViInt32     limit,
    ViInt32     limitLineType,
    ViReal64    margin
)
{
    ViChar  buffer[BUFFER_SIZE];

    if (rsesci_invalidViInt32Range (limit, 1, 8))
        return VI_ERROR_PARAMETER2;

    if (rsesci_invalidViInt32Range (limitLineType, 0, 1))
        return VI_ERROR_PARAMETER3;

    sprintf (buffer, "CALC:LIM%ld:%s:MARG", limit, limitTypeArr[limitLineType]);

    return rsesci_setViReal64(instrSession,
                              buffer,
                              margin,
                              0.0,0.0,
                              VI_ERROR_PARAMETER4);
}

/*****************************************************************************
 * Function: Get Limit Line Margin
 * Purpose:  This function queries a margin to a lower limit line
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetLimitLineMargin(
    ViSession   instrSession,
    ViInt32     limit,
    ViInt32     limitLineType,
    ViReal64*   margin
)
{
    ViChar  buffer[BUFFER_SIZE];

    if (rsesci_invalidViInt32Range (limit, 1, 8))
        return VI_ERROR_PARAMETER2;

    if (rsesci_invalidViInt32Range (limitLineType, 0, 1))
        return VI_ERROR_PARAMETER3;

    sprintf (buffer, "CALC:LIM%ld:%s:MARG?", limit, limitTypeArr[limitLineType]);

    return rsesci_getViReal64(instrSession,
                              buffer,
                              margin,
                              VI_ERROR_PARAMETER4);
}

/*****************************************************************************
 * Function: Set Limit Line Threshold
 * Purpose:  This function defines an absolute threshold value for limit lines
 *           with relative Y axis scaling independently of the measurement
 *           window. The absolute threshold value is used in the limit check as
 *           soon as it exceeds the relative limit value. The unit must
 *           correspond to the unit selected with CALC:LIM:UNIT (except dB
 *           which is not allowed). If no unit is indicated, the unit defined
 *           with CALC:LIM:UNIT is automatically used (exception: dBm instead
 *           of dB).
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetLimitLineThreshold(
    ViSession   instrSession,
    ViInt32 limit,
    ViInt32 limitLineType,
    ViReal64    threshold
)
{
    ViChar  buffer[BUFFER_SIZE];

    if (rsesci_invalidViInt32Range (limit, 1, 8))
        return VI_ERROR_PARAMETER2;

    if (rsesci_invalidViInt32Range (limitLineType, 0, 1))
        return VI_ERROR_PARAMETER3;

    sprintf (buffer, "CALC:LIM%ld:%s:THR", limit, limitTypeArr[limitLineType]);

    return rsesci_setViReal64(instrSession,
                              buffer,
                              threshold,
                              0.0,0.0,
                              VI_ERROR_PARAMETER4);
}

/*****************************************************************************
 * Function: Get Limit Line Threshold
 * Purpose:  This function queries an absolute threshold value for limit lines
 *           with relative Y axis scaling.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetLimitLineThreshold(
    ViSession   instrSession,
    ViInt32 limit,
    ViInt32 limitLineType,
    ViReal64*   threshold
)
{
    ViChar  buffer[BUFFER_SIZE];

    if (rsesci_invalidViInt32Range (limit, 1, 8))
        return VI_ERROR_PARAMETER2;

    if (rsesci_invalidViInt32Range (limitLineType, 0, 1))
        return VI_ERROR_PARAMETER3;

    sprintf (buffer, "CALC:LIM%ld:%s:THR?", limit, limitTypeArr[limitLineType]);

    return rsesci_getViReal64(instrSession,
                              buffer,
                              threshold,
                              VI_ERROR_PARAMETER4);
}

/*****************************************************************************
 * Function: Set Limit Line Comment
 * Purpose:  This function defines a comment for the limit line selected.
 *           The comment is independent from the measurement window.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetLimitLineComment(
    ViSession   instrSession,
    ViInt32 limit,
    ViString    comment
)
{
    ViChar  buffer[BUFFER_SIZE];

    if (rsesci_invalidViInt32Range (limit, 1, 8))
        return VI_ERROR_PARAMETER3;

    sprintf (buffer, "CAlC:LIM%ld:COMM", limit);

    return rsesci_setViStringQuoted (instrSession,
                                     buffer,
                                     "'",
                                     comment,
                                     VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Limit Line Comment
 * Purpose:  This function queries a comment for the limit line selected.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetLimitLineComment(
    ViSession   instrSession,
    ViInt32     limit,
    ViChar      _VI_FAR comment[]
)
{
    ViChar  buffer[BUFFER_SIZE];

    if (rsesci_invalidViInt32Range (limit, 1, 8))
        return VI_ERROR_PARAMETER3;

    sprintf (buffer, "CAlC:LIM%ld:COMM?", limit);

    return rsesci_getViStringQuoted (instrSession,
                                     buffer,
                                     "'",
                                     40,
                                     comment,
                                     VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set Limit Line Values
 * Purpose:  This function defines the selected axis values of the upper or
 *           lower limit lines. The values are defined independently of the
 *           measurement window.
 *           The number of values for the X axis and for the corresponding
 *           Upper and/or Lower limit lines have to be identical. Otherwise
 *           default values are entered for missing values or not required
 *           values are deleted.
 *           In analyzer mode, the unit of values depends on the frequency or
 *           time domain of the X axis.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetLimitLineValues(
    ViSession   instrSession,
    ViInt32     limit,
    ViInt32     axis,
    ViInt32     arraySize,
    ViReal64    _VI_FAR data[]
)
{
    ViChar  buffer[BUFFER_SIZE];

    if (rsesci_invalidViInt32Range (limit, 1, 8))
        return VI_ERROR_PARAMETER2;

    if (rsesci_invalidViInt32Range (axis, 0, 2))
        return VI_ERROR_PARAMETER3;

    sprintf (buffer, "CALC:LIM%ld:%s", limit, limitTypeArr[axis]);

    return rsesci_setViReal64Array(instrSession,
                                   buffer,
                                   data,
                                   arraySize,
                                   VI_ERROR_PARAMETER5);
}

/*****************************************************************************
 * Function: Get Limit Line Values
 * Purpose:  This function returns values of the selected limit lines.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetLimitLineValues(
    ViSession   instrSession,
    ViInt32     limit,
    ViInt32     axis,
    ViInt32     arraySize,
    ViReal64    _VI_FAR data[]
)
{
    ViChar  buffer[BUFFER_SIZE];

    if (rsesci_invalidViInt32Range (limit, 1, 8))
        return VI_ERROR_PARAMETER2;

    if (rsesci_invalidViInt32Range (axis, 0, 2))
        return VI_ERROR_PARAMETER3;

    sprintf (buffer, "CALC:LIM%ld:%s?", limit, limitTypeArr[axis]);

    return rsesci_getViReal64Array(instrSession,
                                     buffer,
                                     data,
                                     arraySize,
                                     VI_ERROR_PARAMETER5);
}

/*****************************************************************************
 * Function: Limit Line Shift X
 * Purpose:  This function moves a limit line by the indicated value in x
 *           direction. In contrast to CALC:LIM:CONT:OFFS, the line is shifted
 *           by modifying the individual x values and not by means of an
 *           additive offset.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_LimitLineShiftX(
    ViSession   instrSession,
    ViInt32 limit,
    ViReal64    shift
)
{
    ViChar  buffer[BUFFER_SIZE];

    if (rsesci_invalidViInt32Range (limit, 1, 8))
        return VI_ERROR_PARAMETER2;

    sprintf (buffer, "CALC:LIM%ld:CONT:SHIF", limit);

    return rsesci_setViReal64 (instrSession,
                               buffer,
                               shift,
                               0.0, 0.0,
                               VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Limit Line Shift Y
 * Purpose:  This function shifts a limit line by the indicated value in Y
 *           direction. In contrast to CALC:LIM:LOW:OFFS, the line is shifted
 *           by modifying the individual Y values but not by means of an
 *           additive offset.
 *           The shift is independent of the measurement window.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_LimitLineShiftY(
    ViSession   instrSession,
    ViInt32 limit,
    ViInt32 axisType,
    ViReal64    shift
)
{
    ViChar  buffer[BUFFER_SIZE];

    if (rsesci_invalidViInt32Range (limit, 1, 8))
        return VI_ERROR_PARAMETER2;

    if (rsesci_invalidViInt32Range (axisType, 0, 1))
        return VI_ERROR_PARAMETER3;

    sprintf (buffer, "CALC:LIM%ld:%s:SHIF", limit, limitTypeArr[axisType]);

    return rsesci_setViReal64 (instrSession,
                               buffer,
                               shift,
                               0.0, 0.0,
                               VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Limit Line Catalog
 * Purpose:  This function queries all the names of the limit lines saved on 
 *           the hard disc.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetLimitLineCatalog (ViSession instrSession,
                                              ViInt32 window, ViInt32 limit,
                                              ViInt32 bufferSize,
                                              ViChar limitLineList[])
{
	ViChar	buffer[BUFFER_SIZE] = "";
	
	CHECKWINDOW;
	
	if (rsesci_invalidViInt32Range (limit, 1, 8))
		return VI_ERROR_PARAMETER3;
		
	sprintf (buffer, "CALC%ld:LIM%ld:CAT?", window, limit);
		
	return rsesci_getViString (instrSession,
		                       buffer,
							   limitLineList,
							   bufferSize,
							   VI_ERROR_PARAMETER4);
}

/*****************************************************************************
 * Function: Set Display Line
 * Purpose:  This function switches Display Line 1 or 2 (level lines) on or
 *           off.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetDisplayLine(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32 line,
    ViBoolean   state
)
{
    ViChar  buffer[BUFFER_SIZE];

    if (rsesci_invalidViInt32Range (line, 1, 2))
        return VI_ERROR_PARAMETER3;

    sprintf (buffer, "CALC%%ld:DLIN%ld:STAT", line);

    return rsesci_setViBooleanWindow(instrSession,
                                     buffer,
                                     window,
                                     state,
                                     VI_ERROR_PARAMETER4);
}

/*****************************************************************************
 * Function: Get Display Line
 * Purpose:  This function queries the state of selected display line.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetDisplayLine(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32 line,
    ViBoolean*  state
)
{
    ViChar  buffer[BUFFER_SIZE];

    if (rsesci_invalidViInt32Range (line, 1, 2))
        return VI_ERROR_PARAMETER3;

    sprintf (buffer, "CALC%%ld:DLIN%ld:STAT?", line);

    return rsesci_getViBooleanWindow(instrSession,
                                     buffer,
                                     window,
                                     state,
                                     VI_ERROR_PARAMETER4);
}

/*****************************************************************************
 * Function: Set Display Line Position
 * Purpose:  This function defines the position of Display Line 1 or 2. These
 *           lines enable the user to mark any levels in the diagram.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetDisplayLinePosition(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32 line,
    ViReal64    position
)
{
    ViChar  buffer[BUFFER_SIZE];

    if (rsesci_invalidViInt32Range (line, 1, 2))
        return VI_ERROR_PARAMETER3;

    sprintf (buffer, "CALC%%ld:DLIN%ld", line);

    return rsesci_setViReal64Window(instrSession,
                                    buffer,
                                    window,
                                    position,
                                    0.0,0.0,
                                    VI_ERROR_PARAMETER4);
}

/*****************************************************************************
 * Function: Get Display Line Position
 * Purpose:  This function queries the position of Display Line 1 or 2.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetDisplayLinePosition(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32 line,
    ViReal64*   position
)
{
    ViChar  buffer[BUFFER_SIZE];

    if (rsesci_invalidViInt32Range (line, 1, 2))
        return VI_ERROR_PARAMETER3;

    sprintf (buffer, "CALC%%ld:DLIN%ld?", line);

    return rsesci_getViReal64Window(instrSession,
                                    buffer,
                                    window,
                                    position,
                                    VI_ERROR_PARAMETER4);
}

/*****************************************************************************
 * Function: Set Frequency Line
 * Purpose:  This function switches the frequency line on or off.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetFrequencyLine(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32 line,
    ViBoolean   state
)
{
    ViChar  buffer[BUFFER_SIZE];

    if (rsesci_invalidViInt32Range (line, 1, 2))
        return VI_ERROR_PARAMETER3;

    sprintf (buffer, "CALC%%ld:FLIN%ld:STAT", line);

    return rsesci_setViBooleanWindow(instrSession,
                                     buffer,
                                     window,
                                     state,
                                     VI_ERROR_PARAMETER4);
}

/*****************************************************************************
 * Function: Get Frequency Line
 * Purpose:  This function queries the state of the frequency line
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetFrequencyLine(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32 line,
    ViBoolean*  state
)
{
    ViChar  buffer[BUFFER_SIZE];

    if (rsesci_invalidViInt32Range (line, 1, 2))
        return VI_ERROR_PARAMETER3;

    sprintf (buffer, "CALC%%ld:FLIN%ld:STAT?", line);

    return rsesci_getViBooleanWindow(instrSession,
                                     buffer,
                                     window,
                                     state,
                                     VI_ERROR_PARAMETER4);
}

/*****************************************************************************
 * Function: Set Frequency Line Position
 * Purpose:  This function defines the position of the frequency lines.
 *           The frequency lines mark the frequencies specified in the
 *           measurement window. Frequency lines are only available with SPAN >
 *           0.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetFrequencyLinePosition(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32 line,
    ViReal64    position
)
{
    ViChar  buffer[BUFFER_SIZE];

    if (rsesci_invalidViInt32Range (line, 1, 2))
        return VI_ERROR_PARAMETER3;

    sprintf (buffer, "CALC%%ld:FLIN%ld", line);

    return rsesci_setViReal64Window(instrSession,
                                    buffer,
                                    window,
                                    position,
                                    0.0,0.0,
                                    VI_ERROR_PARAMETER4);
}

/*****************************************************************************
 * Function: Get Frequency Line Position
 * Purpose:  This function queries the position of Frequency Line 1 or 2.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetFrequencyLinePosition(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32 line,
    ViReal64*   position
)
{
    ViChar  buffer[BUFFER_SIZE];

    if (rsesci_invalidViInt32Range (line, 1, 2))
        return VI_ERROR_PARAMETER3;

    sprintf (buffer, "CALC%%ld:FLIN%ld?", line);

    return rsesci_getViReal64Window(instrSession,
                                    buffer,
                                    window,
                                    position,
                                    VI_ERROR_PARAMETER4);
}

/*****************************************************************************
 * Function: Set Time Line
 * Purpose:  This function switches the time line on or off.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetTimeLine(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32 line,
    ViBoolean   state
)
{
    ViChar  buffer[BUFFER_SIZE];

    if (rsesci_invalidViInt32Range (line, 1, 2))
        return VI_ERROR_PARAMETER3;

    sprintf (buffer, "CALC%%ld:TLIN%ld:STAT", line);

    return rsesci_setViBooleanWindow(instrSession,
                                     buffer,
                                     window,
                                     state,
                                     VI_ERROR_PARAMETER4);
}

/*****************************************************************************
 * Function: Get Time Line
 * Purpose:  This function queries the state of time line.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetTimeLine(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32 line,
    ViBoolean*  state
)
{
    ViChar  buffer[BUFFER_SIZE];

    if (rsesci_invalidViInt32Range (line, 1, 2))
        return VI_ERROR_PARAMETER3;

    sprintf (buffer, "CALC%%ld:TLIN%ld:STAT?", line);

    return rsesci_getViBooleanWindow(instrSession,
                                     buffer,
                                     window,
                                     state,
                                     VI_ERROR_PARAMETER4);
}

/*****************************************************************************
 * Function: Set Time Line Position
 * Purpose:  This function defines the position of the time lines.
 *           The time lines mark the times specified in the measurement
 *           window. Time lines are only available with SPAN = 0.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetTimeLinePosition(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32 line,
    ViReal64    position
)
{
    ViChar  buffer[BUFFER_SIZE];

    if (rsesci_invalidViInt32Range (line, 1, 2))
        return VI_ERROR_PARAMETER3;

    sprintf (buffer, "CALC%%ld:TLIN%ld", line);

    return rsesci_setViReal64Window(instrSession,
                                    buffer,
                                    window,
                                    position,
                                    0.0,0.0,
                                    VI_ERROR_PARAMETER4);
}

/*****************************************************************************
 * Function: Get Time Line Position
 * Purpose:  This function queries the position of the time line.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetTimeLinePosition(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32 line,
    ViReal64*   position
)
{
    ViChar  buffer[BUFFER_SIZE];

    if (rsesci_invalidViInt32Range (line, 1, 2))
        return VI_ERROR_PARAMETER3;

    sprintf (buffer, "CALC%%ld:TLIN%ld?", line);

    return rsesci_getViReal64Window(instrSession,
                                    buffer,
                                    window,
                                    position,
                                    VI_ERROR_PARAMETER4);
}

/*****************************************************************************
 * Function: Set Active Window
 * Purpose:  This function selects the active measurement window. In FULL
 *           SCREEN mode, the measurements are only performed in the active
 *           measurement window. Measurements are therefore initiated in the
 *           active window and result queries (marker, trace data and other
 *           results) answered also in the active window. Initiating
 *           measurements and query? results in the inactive window yields an
 *           error message (execution error).
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetActiveWindow(
    ViSession   instrSession,
    ViInt32     window
)
{
    return rsesci_sendCmdWindow (instrSession,
                                 "DISP:WIND%ld:SEL",
                                 window);
}

/*****************************************************************************
 * Function: Get Active Window
 * Purpose:  This function returns the active measurement window.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetActiveWindow (ViSession instrSession,
                                          ViInt32 *window)
{
    return rsesci_getViInt32 (instrSession
                            , "DISP:WIND:ACT?"
                            , window
                            , VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Screen Mode
 * Purpose:  This function switches the measurement result display between
 *           FULL SCREEN and SPLIT SCREEN. The coupling of settings between
 *           screen A and screen B can be selected with the function
 *           rsesci_SetInstrumentCoupling().
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetScreenMode(
    ViSession   instrSession,
    ViInt32 mode
)
{
    return rsesci_setEnum (instrSession,
                           "DISP:FORM",
                           mode,
                           displayModeArr,
                           VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Screen Mode
 * Purpose:  This function returns screen mode.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetScreenMode(
    ViSession   instrSession,
    ViInt32*    mode
)
{
    return rsesci_getEnum (instrSession,
                           "DISP:FORM?",
                           mode,
                           displayModeArr,
                           VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Display Size
 * Purpose:  This function switches the measurement window for channel and
 *           adjacent-channel power measurements or for active FM demodulator
 *           (FS-K7) to full screen or half screen.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetDisplaySize(
    ViSession   instrSession,
    ViInt32     displaySize
)
{
    CHECKOPT (RSESCI_K7);

    return rsesci_setEnum(instrSession,
                          "DISP:SIZE",
                          displaySize,
                          displaySizeArr,
                          VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Display Size
 * Purpose:  This function returns if the measurement window for channel and
 *           adjacent-channel power measurements or for active FM demodulator
 *           (FS-K7) is set to full screen or half screen.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetDisplaySize(
    ViSession   instrSession,
    ViInt32*    displaySize
)
{
    CHECKOPT (RSESCI_K7);

    return rsesci_getEnum(instrSession,
                          "DISP:SIZE?",
                          displaySize,
                          displaySizeArr,
                          VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Display Update
 * Purpose:  This function switches on or off the update of all display
 *           elements during remote control.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetDisplayUpdate(
    ViSession   instrSession,
    ViBoolean   displayUpdate
)
{
    return rsesci_setViBoolean (instrSession,
                                "SYST:DISP:UPD",
                                displayUpdate,
                                VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Display Update
 * Purpose:  This function queries the state the update of all display
 *           elements during remote control.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetDisplayUpdate(
    ViSession   instrSession,
    ViBoolean*  displayUpdate
)
{
    return rsesci_getViBoolean (instrSession,
                                "SYST:DISP:UPD?",
                                displayUpdate,
                                VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Display Bargraph Maxhold
 * Purpose:  This function switches on the numeric indication of the maximum
 *           value of the bargraph.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetDisplayBargraphMaxhold(
    ViSession   instrSession,
    ViBoolean   bargraphMaxhold
)
{
    return rsesci_setViBoolean (instrSession,
                                ":DISP:BARG:PHOL",
                                bargraphMaxhold,
                                VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Display Bargraph Maxhold
 * Purpose:  This function returns the state of the numeric indication of
 *           maximum bargraph value.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetDisplayBargraphMaxhold(
    ViSession   instrSession,
    ViBoolean*  bargraphMaxhold
)
{
    return rsesci_getViBoolean (instrSession,
                                ":DISP:BARG:PHOL?",
                                bargraphMaxhold,
                                VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Display Bargraph Level
 * Purpose:  This function queries the minimum and maximum level of the
 *           bargraph.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetDisplayBargraphLevel(
    ViSession   instrSession,
    ViReal64*   minimumBargraphLevel,
    ViReal64*   maximumBargraphLevel
)
{
    ViStatus    rsesci_status;

    CHECKERR (rsesci_getViReal64 (instrSession,
                                  ":DISP:BARG:LEV:LOW?",
                                  minimumBargraphLevel,
                                  VI_ERROR_PARAMETER2));
    return rsesci_getViReal64 (instrSession,
                               ":DISP:BARG:LEV:UPP?",
                               maximumBargraphLevel,
                               VI_ERROR_PARAMETER3);

}

/*****************************************************************************
 * Function: Reset Display Bargraph Maxhold
 * Purpose:  This function resets the maxhold value of the numeric indication
 *           of the bargraph measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_ResetDisplayBargraphMaxhold(
    ViSession   instrSession
)
{
    return rsesci_sendCmd (instrSession, ":DISP:BARG:PHOL:RES");
}

/*****************************************************************************
 * Function: Set Parameter Coupling
 * Purpose:  This function selects the coupled parameters of receiver and
 *           analyzer mode.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetParameterCoupling(
    ViSession   instrSession,
    ViInt32 coupledParameters,
    ViBoolean   coupling
)
{
    ViChar  buffer[BUFFER_SIZE];

    if (rsesci_invalidViInt32Range (coupledParameters, 0, 7))
        return VI_ERROR_PARAMETER2;

    sprintf (buffer, "INST:COUP:%s", couplingTypeArr[coupledParameters]);

    return rsesci_setEnum (instrSession,
                           buffer,
                           (ViInt32) coupling,
                           couplingStateArr,
                           VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Parameter Coupling
 * Purpose:  This function returns whether is selected coupled parameters of
 *           receiver and analyzer mode switched on or off.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetParameterCoupling(
    ViSession   instrSession,
    ViInt32 coupledParameters,
    ViBoolean*  coupling
)
{
    ViStatus    rsesci_status;
    ViChar      buffer[BUFFER_SIZE];
    ViInt32     coup_local;

    if (rsesci_invalidViInt32Range (coupledParameters, 0, 7))
        return VI_ERROR_PARAMETER2;

    sprintf (buffer, "INST:COUP:%s?", couplingTypeArr[coupledParameters]);

    CHECKERR(rsesci_getEnum (instrSession,
                             buffer,
                             &coup_local,
                             couplingStateArr,
                             VI_ERROR_PARAMETER3));

    *coupling = (coup_local == 1);

    return rsesci_status;
}

/*****************************************************************************
 * Function: Set Instrument Coupling
 * Purpose:  This function selects the parameter coupling between the two
 *           measurement windows screen A and B.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetInstrumentCoupling(
    ViSession   instrSession,
    ViInt32 coupling
)
{
    return rsesci_setEnum (instrSession,
                           "INST:COUP",
                           coupling,
                           instrCouplingArr,
                           VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set X Axis Display Spacing
 * Purpose:  This function toggles between linear and logarithmic display in
 *           receiver mode.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetXAxisDisplaySpacing (ViSession instrSession,
                                                 ViInt32 window,
                                                 ViInt32 displaySpacing)
{
    CHECKWINDOW;
    
    return rsesci_setEnumWindow (instrSession
                               , "DISP:WIND%ld:TRACE1:X:SPAC"
                               , window
                               , displaySpacing
                               , modeArr
                               , VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get X Axis Display Spacing
 * Purpose:  This function queries setting of display spacing in receiver
 *           mode.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetXAxisDisplaySpacing (ViSession instrSession,
                                                 ViInt32 window,
                                                 ViInt32 *displaySpacing)
{
    CHECKWINDOW;
    
    return rsesci_getEnumWindow (instrSession
                               , "DISP:WIND%ld:TRACE1:X:SPAC?"
                               , window
                               , displaySpacing
                               , modeArr
                               , VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set X Axis Display Zoom
 * Purpose:  This function switches the zoom on or off.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetXAxisDisplayZoom (ViSession instrSession,
                                              ViInt32 window,
                                              ViBoolean zoomState)
{
    CHECKWINDOW;
    
    return rsesci_setViBooleanWindow (instrSession
                                    , "DISP:WIND%ld:TRAC:X:ZOOM"
                                    , window
                                    , zoomState
                                    , VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set Screen Title
 * Purpose:  This function defines a comment (max. 20 characters) which can be
 *           displayed on the screen in the selected measurement window.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetScreenTitle(
    ViSession   instrSession,
    ViBoolean   screenTitle,
    ViString    commentString
)
{
    ViChar buffer[BUFFER_SIZE];

    if (rsesci_invalidViString (commentString, 20))
        return VI_ERROR_PARAMETER3;

    sprintf (buffer, ":DISP:TEXT:DATA '%s';:DISP:TEXT:STAT", commentString);

    return rsesci_setViBoolean (instrSession,
                                buffer,
                                screenTitle,
                                VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Display Elements
 * Purpose:  This function controls the apperance of display elements
 *           (annotation, company logo, display date + time).
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetDisplayElements(
    ViSession   instrSession,
    ViBoolean   annotation,
    ViBoolean   logo,
    ViBoolean   timeDate
)
{
    ViStatus    rsesci_status = VI_SUCCESS;

    CHECKERR (rsesci_setViBoolean (instrSession,
                                   "DISP:ANN:FREQ",
                                   annotation,
                                   VI_ERROR_PARAMETER2));

    CHECKERR (rsesci_setViBoolean (instrSession,
                                   "DISP:LOGO",
                                   logo,
                                   VI_ERROR_PARAMETER3));

    CHECKERR (rsesci_setViBoolean (instrSession,
                                   "DISP:TIME",
                                   timeDate,
                                   VI_ERROR_PARAMETER4));

    return rsesci_status;
}

/*****************************************************************************
 * Function: Set Display User Defined Colors
 * Purpose:  This function defines the color table of the instrument.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetDisplayUserDefinedColors(
    ViSession   instrSession,
    ViInt32     colorMap,
    ViInt32     predefinedColors,
    ViReal64    tint,
    ViReal64    saturation,
    ViReal64    brightness
)
{
    ViChar  buffer[BUFFER_SIZE];

    if (rsesci_invalidViInt32Range (colorMap, 1, 34))
        return VI_ERROR_PARAMETER2;
    if (rsesci_invalidViInt32Range (predefinedColors, -1, 15))
        return VI_ERROR_PARAMETER3;

    if (predefinedColors == RSESCI_USER_DEFINED)
        {
        if (rsesci_invalidViReal64Range (tint, 0.0, 100.0))
            return VI_ERROR_PARAMETER4;
        if (rsesci_invalidViReal64Range (saturation, 0.0, 100.0))
            return VI_ERROR_PARAMETER5;
        if (rsesci_invalidViReal64Range (brightness, 0.0, 100.0))
            return VI_ERROR_PARAMETER6;

        sprintf (buffer, "DISP:CMAP%ld:HSL %lf,%lf,%lf",
                 colorMap,
                 tint/100,
                 saturation/100,
                 brightness/100);
        }
    else
        sprintf (buffer, "DISP:CMAP%ld:PDEF %s",
                 colorMap, predefinedColorsArr[predefinedColors]);

    return rsesci_sendCmd (instrSession, buffer);
}

/*****************************************************************************
 * Function: Set Display Default Colors
 * Purpose:  This function resets the screen colors of all display items to
 *           their default settings.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetDisplayDefaultColors(
    ViSession   instrSession,
    ViInt32     defaultColors
)
{
    ViChar  buffer[BUFFER_SIZE];

    if (rsesci_invalidViInt32Range (defaultColors, 1, 2))
        return VI_ERROR_PARAMETER2;

    sprintf (buffer, "DISP:CMAP:DEF%ld", defaultColors);

    return rsesci_sendCmd (instrSession, buffer);
}

/*****************************************************************************
 * Function: Set Display Power Saving
 * Purpose:  This function switches on or off the power-save mode of the
 *           display. With the power-save mode activated the display including
 *           backlight is completely switched off after the elapse of the
 *           response time.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetDisplayPowerSaving(
    ViSession   instrSession,
    ViBoolean   powerSaving,
    ViInt32     holdoffTime
)
{
    ViStatus rsesci_status = VI_SUCCESS;

    CHECKERR (rsesci_setViBoolean (instrSession,
                                "DISP:PSAV:STAT",
                                powerSaving,
                                VI_ERROR_PARAMETER2));

    return rsesci_setViInt32 (instrSession,
                              "DISP:PSAV:HOLD",
                              holdoffTime,
                              1, 60,
                              VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set External Reference Oscillator
 * Purpose:  This function controls selection of the reference oscillator.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetExternalReferenceOscillator(
    ViSession   instrSession,
    ViInt32 referenceOscillator
)
{
    return rsesci_setEnum (instrSession,
                           "ROSC:SOUR",
                           referenceOscillator,
                           ccvsSignalArr,
                           VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get External Reference Oscillator
 * Purpose:  This function returns selection of the reference oscillator
 *           source.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetExternalReferenceOscillator(
    ViSession   instrSession,
    ViInt32*    referenceOscillator
)
{
    return rsesci_getEnum (instrSession,
                           "ROSC:SOUR?",
                           referenceOscillator,
                           ccvsSignalArr,
                           VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set External Reference Oscillator Frequency
 * Purpose:  This function informs the instrument on the frequency of the external
 *           reference oscillator. This frequency is used as starting point
 *           for the synchronization of the internal reference frequencies.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetExternalReferenceOscillatorFrequency (ViSession instrSession,
                                                                  ViReal64 externalFrequency)
{
	CHECKMODEL(RSESU8 | RSESU26 | RSESU40);
	
	return rsesci_setViReal64 (instrSession,
		                       "SENS:ROSC:EXT:FREQ",
							   externalFrequency,
							   1.0e6, 20.0e6,
							   VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get External Reference Oscillator Frequency
 * Purpose:  This function returns the frequency of the external reference 
 *           oscillator. This frequency is used as starting point for the 
 *           synchronization of the internal reference frequencies.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetExternalReferenceOscillatorFrequency (ViSession instrSession,
                                                                  ViReal64 *externalFrequency)
{
	CHECKMODEL(RSESU8 | RSESU26 | RSESU40);
	
	return rsesci_getViReal64 (instrSession,
		                       "SENS:ROSC:EXT:FREQ?",
							   externalFrequency,
							   VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Set External Noise Source
 * Purpose:  This function switches the 28-V supply of the noise source at the
 *           rear panel on or off.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetExternalNoiseSource(
    ViSession   instrSession,
    ViBoolean   noiseSource
)
{
    return rsesci_setViBoolean (instrSession,
                                ":DIAG:SERV:NSO",
                                noiseSource,
                                VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get External Noise Source
 * Purpose:  This function returns if the 28-V supply of the noise source at
 *           the rear panel is switched on or off.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetExternalNoiseSource(
    ViSession   instrSession,
    ViBoolean*  noiseSource
)
{
    return rsesci_getViBoolean (instrSession,
                                ":DIAG:SERV:NSO?",
                                noiseSource,
                                VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Preselector
 * Purpose:  In analyzer mode, this function switches the preselection on or
 *           off.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetPreselector(
    ViSession   instrSession,
    ViBoolean   preselector
)
{
    return rsesci_setViBoolean (instrSession,
                                ":INP:PRES",
                                preselector,
                                VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Preselector
 * Purpose:  This function returns if in analyzer mode the preselection is
 *           switched on or off.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetPreselector(
    ViSession   instrSession,
    ViBoolean*  preselector
)
{
    return rsesci_getViBoolean (instrSession,
                                ":INP:PRES?",
                                preselector,
                                VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Switchover Frequency Highpass
 * Purpose:  This function configures switchover frequency of the 150 kHz
 * 			 highpass in the instruments preselector.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetSwitchoverFrequencyHighpass (
	ViSession instrSession,
	ViInt32   band
)
{
    return rsesci_setEnum (instrSession,
                           "INP1:PRES:CUT",
                           band,
                           bandArr,
                           VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Get Switchover Frequency Highpass 
 * Purpose:  This function returns switchover frequency of the 150 kHz
 * 			 highpass in the instruments preselector.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetSwitchoverFrequencyHighpass (
	ViSession  instrSession,
	ViInt32   *band
)
{
	return rsesci_getEnum (instrSession,
						   "INP1:PRES:CUT?",
						   band,
						   bandArr,
						   VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Transducer Factor
 * Purpose:  This function switches the selected transducer factor on or off.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetTransducerFactor(
    ViSession   instrSession,
    ViInt32     window,
    ViBoolean   transducerFactor
)
{
    return rsesci_setViBooleanWindow (instrSession,
                                      ":SENS%ld:CORR:TRAN",
                                      window,
                                      transducerFactor,
                                      VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Transducer Factor
 * Purpose:  This function returns if the selected transducer factor is on or
 *           off.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetTransducerFactor(
    ViSession   instrSession,
    ViInt32     window,
    ViBoolean*  transducerFactor
)
{
    return rsesci_getViBooleanWindow (instrSession,
                                      ":SENS%ld:CORR:TRAN?",
                                      window,
                                      transducerFactor,
                                      VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set Transducer Factor Name
 * Purpose:  This function selects the transducer factor designated by <name>.
 *           If <name> does not exist yet, a new transducer factor is created.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetTransducerFactorName(
    ViSession   instrSession,
    ViInt32     window,
    ViString    name
)
{
    ViChar  buffer[BUFFER_SIZE];

    sprintf (buffer, "SENS%%ld:CORR:TRAN:SEL '%s'", name);

    return rsesci_sendCmdWindow (instrSession,
                                 buffer,
                                 window);
}

/*****************************************************************************
 * Function: Get Transducer Factor Name
 * Purpose:  This function returns the name of the currently active transducer
 *           factor.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetTransducerFactorName(
    ViSession   instrSession,
    ViInt32     window,
    ViChar      _VI_FAR name[]
)
{
    ViChar      buffer[BUFFER_SIZE];

    CHECKWINDOW;

    sprintf (buffer, "SENS%ld:CORR:TRAN:SEL?", window);

    return rsesci_getViStringQuoted (instrSession,
                                     buffer,
                                     "'",
                                     8,
                                     name,
                                     VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Transducer Factor Catalog
 * Purpose:  This function reads out the names of all transducer factors stored 
 *           on the harddisk.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetTransducerFactorCatalog (ViSession instrSession,
                                                     ViInt32 window,
                                                     ViInt32 *number_ofTransducerFactors,
                                                     ViInt32 bufferSize,
                                                     ViChar transducerFactorsList[])
{
	ViStatus    rsesci_status = VI_SUCCESS;
    ViUInt32    retCnt = 0;
    ViChar      *buf=VI_NULL;
	
	if (bufferSize < 0)
        return VI_ERROR_PARAMETER4;
    if ((bufferSize > 0) && (transducerFactorsList == VI_NULL))
        return VI_ERROR_PARAMETER5;
       
    CHECKERR(rsesci_clearBeforeRead (instrSession));
    CHECKERR (viPrintf (instrSession, "SENS%ld:CORR:TRAN:CAT?\n", window));
    CHECKERR (rsesci_checkBeforeRead (instrSession));
    rsesci_status = rsesci_readDataUnknownLength(instrSession, &buf, &retCnt);
    rsesci_status = rsesci_ParseCatalog(buf, bufferSize, transducerFactorsList, number_ofTransducerFactors);
    
    free (buf);
	
	return rsesci_status;
}

/*****************************************************************************
 * Function: Get Active Transducer Factor
 * Purpose:  This function queries the active transducer factor. If no 
 *           transducer factor is active, an empty string is returned.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetActiveTransducerFactor (ViSession instrSession,
                                                    ViInt32 window,
                                                    ViInt32 bufferSize,
                                                    ViChar activeTransducerFactor[])
{
    CHECKWINDOW;
    
    return rsesci_getViStringWindow (instrSession
                                   , "SENS%ld:CORR:TRAN:ACT?"
                                   , window
                                   , activeTransducerFactor
                                   , bufferSize
                                   , VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Delete Transducer Factor
 * Purpose:  This function deletes the selected transducer factor.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_DeleteTransducerFactor(
    ViSession   instrSession,
    ViInt32     window,
    ViString    name
)
{
    ViStatus    rsesci_status;

    CHECKERR (rsesci_SetTransducerFactorName (instrSession,
                                              window,
                                              name));
    return rsesci_sendCmdWindow (instrSession,
                                 "SENS:CORR:TRAN:DEL",
                                 window);
}

/*****************************************************************************
 * Function: Set Transducer Factor View
 * Purpose:  This function switches on the display of the active transducer
 *           factor or set.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetTransducerFactorView(
    ViSession   instrSession,
    ViInt32     window,
    ViBoolean   transducerFactorView
)
{
    return rsesci_setViBooleanWindow (instrSession,
                                      ":SENS%ld:CORR:TRAN:VIEW",
                                      window,
                                      transducerFactorView,
                                      VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Transducer Factor View
 * Purpose:  This function returns if the display of the active transducer
 *           factor or set is on or off.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetTransducerFactorView(
    ViSession   instrSession,
    ViInt32     window,
    ViBoolean*  transducerFactorView
)
{
    return rsesci_getViBooleanWindow (instrSession,
                                      ":SENS%ld:CORR:TRAN:VIEW?",
                                      window,
                                      transducerFactorView,
                                      VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set Transducer Factor Auto Adjust
 * Purpose:  This function switches on the automatic adjustment of the
 *           reference level to the selected transducer factor.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetTransducerFactorAutoAdjust(
    ViSession   instrSession,
    ViInt32     window,
    ViBoolean   autoAdjust
)
{
    return rsesci_setViBooleanWindow (instrSession,
                                      ":SENS%ld:CORR:TRAN:ADJ:RLEV",
                                      window,
                                      autoAdjust,
                                      VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Transducer Factor Auto Adjust
 * Purpose:  This function returns if the automatic adjustment of the
 *           reference level to the selected transducer factor is enabled or
 *           disabled.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetTransducerFactorAutoAdjust(
    ViSession   instrSession,
    ViInt32     window,
    ViBoolean*  autoAdjust
)
{
    return rsesci_getViBooleanWindow (instrSession,
                                      ":SENS%ld:CORR:TRAN:ADJ:RLEV?",
                                      window,
                                      autoAdjust,
                                      VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set Transducer Set
 * Purpose:  This function switches the selected transducer set on or off.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetTransducerSet(
    ViSession   instrSession,
    ViInt32     window,
    ViBoolean   transducerSet
)
{
    return rsesci_setViBooleanWindow (instrSession,
                                      ":SENS%ld:CORR:TSET",
                                      window,
                                      transducerSet,
                                      VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Transducer Set
 * Purpose:  This function returns if the selected transducer set is switched
 *           on or off.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetTransducerSet(
    ViSession   instrSession,
    ViInt32     window,
    ViBoolean*  transducerSet
)
{
    return rsesci_getViBooleanWindow (instrSession,
                                      ":SENS%ld:CORR:TSET?",
                                      window,
                                      transducerSet,
                                      VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set Transducer Set Name
 * Purpose:  This function selected the transducer set designated by <name>.
 *           If <name> does not exist yet, a new set is created.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetTransducerSetName(
    ViSession   instrSession,
    ViInt32     window,
    ViString    name
)
{
    ViChar  buffer[BUFFER_SIZE];

    sprintf (buffer, "SENS%%ld:CORR:TSET:SEL '%s'", name);

    return rsesci_sendCmdWindow (instrSession,
                                 buffer,
                                 window);
}

/*****************************************************************************
 * Function: Get Transducer Set Name
 * Purpose:  This function returns the name of the currently active transducer
 *           set.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetTransducerSetName(
    ViSession   instrSession,
    ViInt32     window,
    ViChar      _VI_FAR name[]
)
{
    ViChar      buffer[BUFFER_SIZE];

    CHECKWINDOW;

    sprintf (buffer, "SENS%ld:CORR:TSET:SEL?", window);

    return rsesci_getViStringQuoted (instrSession,
                                     buffer,
                                     "'",
                                     8,
                                     name,
                                     VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Transducer Set Catalog
 * Purpose:  This function reads out the names of all transducer sets stored
 *           on the harddisk.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetTransducerSetCatalog (ViSession instrSession,
                                                  ViInt32 window,
                                                  ViInt32 *number_ofTransducerSets,
                                                  ViInt32 bufferSize,
                                                  ViChar transducerSetsList[])
{
	ViStatus    rsesci_status = VI_SUCCESS;
    ViUInt32    retCnt = 0;
    ViChar      *buf=VI_NULL;
	
	if (bufferSize < 0)
        return VI_ERROR_PARAMETER4;
    if ((bufferSize > 0) && (transducerSetsList == VI_NULL))
        return VI_ERROR_PARAMETER5;
       
    CHECKERR(rsesci_clearBeforeRead (instrSession));
    CHECKERR(viPrintf (instrSession, "SENS%ld:CORR:TSET:CAT?\n", window));
    CHECKERR (rsesci_checkBeforeRead (instrSession));
    rsesci_status = rsesci_readDataUnknownLength(instrSession, &buf, &retCnt);
    rsesci_status = rsesci_ParseCatalog(buf, bufferSize, transducerSetsList, number_ofTransducerSets);

    free (buf);
	
	return rsesci_status;
}

/*****************************************************************************
 * Function: Delete Transducer Set
 * Purpose:  This function deletes the selected transducer set.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_DeleteTransducerSet(
    ViSession   instrSession,
    ViInt32     window,
    ViString    name
)
{
    ViStatus    rsesci_status;

    CHECKERR (rsesci_SetTransducerSetName (instrSession,
                                           window,
                                           name));
    return rsesci_sendCmdWindow (instrSession,
                                 "SENS:CORR:TSET:DEL",
                                 window);
}

/*****************************************************************************
 * Function: Set Transducer Factor Unit
 * Purpose:  This function defines the unit of the transducer factor selected.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetTransducerFactorUnit(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32 transducerFactorUnit
)
{
    ViChar  buffer[BUFFER_SIZE];

    CHECKWINDOW;

    if (rsesci_invalidViInt32Range (transducerFactorUnit, 4, 11) && (transducerFactorUnit != 0))
        return VI_ERROR_PARAMETER2;

    sprintf (buffer, "SENS%ld:CORR:TRAN:UNIT", window);

    return rsesci_setViStringQuoted (instrSession,
                                     buffer,
                                     "'",
                                     unitArr[transducerFactorUnit],
                                     VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Transducer Factor Unit
 * Purpose:  This function returns the unit of the transducer factor selected.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetTransducerFactorUnit(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32*    transducerFactorUnit
)
{
    ViStatus    rsesci_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE],
                tmpBuf[BUFFER_SIZE];

    CHECKWINDOW;

    if (!transducerFactorUnit)
        return VI_ERROR_PARAMETER3;

    sprintf (buffer, "SENS%ld:CORR:TRAN:UNIT?", window);

    CHECKERR (rsesci_getViStringQuoted (instrSession,
                                        buffer,
                                        "'",
                                        BUFFER_SIZE,
                                        tmpBuf,
                                        VI_ERROR_INSTR_INTERPRETING_RESPONSE));

    return rsesci_getIndexFromString (instrSession, tmpBuf, unitArr, transducerFactorUnit);
}

/*****************************************************************************
 * Function: Set Transducer Factor Interpolation
 * Purpose:  This function defines whether the frequency scaling of the
 *           transducer factor is linear or logarithmic.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetTransducerFactorInterpolation(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32 transducerFactorInterpolation
)
{
    if (rsesci_invalidViInt32Range (transducerFactorInterpolation, 0, 1))
        return VI_ERROR_PARAMETER2;

    return rsesci_setEnumWindow (instrSession,
                                 "SENS%ld:CORR:TRAN:SCAL",
                                 window,
                                 transducerFactorInterpolation,
                                 spacingArr,
                                 VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Transducer Factor Interpolation
 * Purpose:  This function returns whether the frequency scaling of the
 *           transducer factor is linear or logarithmic.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetTransducerFactorInterpolation(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32*    transducerFactorInterpolation
)
{
    return rsesci_getEnumWindow (instrSession,
                                 "SENS%ld:CORR:TRAN:SCAL?",
                                 window,
                                 transducerFactorInterpolation,
                                 spacingArr,
                                 VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set Transducer Factor Comment
 * Purpose:  This function defines the comment for the selected transducer
 *           factor.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetTransducerFactorComment(
    ViSession   instrSession,
    ViInt32     window,
    ViString    comment
)
{
    ViChar  buffer[BUFFER_SIZE];

    CHECKWINDOW;

    sprintf (buffer, "SENS%ld:CORR:TRAN:COMM", window);

    return rsesci_setViStringQuoted (instrSession,
                                     buffer,
                                     "'",
                                     comment,
                                     VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Transducer Factor Comment
 * Purpose:  This function returns the comment for the selected transducer
 *           factor.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetTransducerFactorComment(
    ViSession   instrSession,
    ViInt32     window,
    ViChar      _VI_FAR comment[]
)
{
    ViChar  buffer[BUFFER_SIZE];

    CHECKWINDOW;

    sprintf (buffer, "SENS%ld:CORR:TRAN:COMM?", window);

    return rsesci_getViStringQuoted (instrSession,
                                     buffer,
                                     "'",
                                     8,
                                     comment,
                                     VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set Transducer Factor Values
 * Purpose:  This function defines the reference values of the transducer
 *           factor selected. These values are entered as a sequence of
 *           frequency/level pairs. The frequencies must be sent in ascending
 *           order.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetTransducerFactorValues(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32     arraySize,
    ViReal64    _VI_FAR frequencyValues[],
    ViReal64    _VI_FAR levelValues[]
)
{
    ViChar      *pbuffer;
    ViChar      *p2buf;
    ViInt32     i;
    ViStatus    rsesci_status;

    if (!frequencyValues)
        return VI_ERROR_PARAMETER4;
    if (!levelValues)
        return VI_ERROR_PARAMETER5;

    pbuffer = (ViChar*) malloc (arraySize * 20 * 2 + 40);

    p2buf = pbuffer + sprintf (pbuffer, "SENS%%ld:CORR:TRAN:DATA ");

    for (i = 0; i < arraySize; i++)
        p2buf += sprintf (p2buf, "%.12lf,%lf,", frequencyValues[i], levelValues[i]);

    *--p2buf = '\0';

    rsesci_status = rsesci_sendCmdWindow (instrSession,
                                          pbuffer,
                                          window);

    free (pbuffer);

    return rsesci_status;
}

/*****************************************************************************
 * Function: Get Transducer Factor Values
 * Purpose:  This function returns the reference values of the transducer
 *           factor selected. These values are entered as a sequence of
 *           frequency/level pairs.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetTransducerFactorValues(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32*    arraySize,
    ViReal64    _VI_FAR frequencyValues[],
    ViReal64    _VI_FAR levelValues[]
)
{
    ViChar      buffer[BUFFER_SIZE];
    ViInt32     idx             = 0,
                count           = 0,
                maxArraySize    = 2 * 50;
    ViReal64    results[2 * 50 + 1],
                *p2results;
    ViStatus    rsesci_status;

    CHECKWINDOW;

    if (!arraySize)
        return VI_ERROR_PARAMETER3;
    if (!frequencyValues)
        return VI_ERROR_PARAMETER4;
    if (!levelValues)
        return VI_ERROR_PARAMETER5;

    sprintf (buffer, "SENS%ld:CORR:TRAN:DATA?", window);

    rsesci_status = rsesci_getViReal64BinArray (instrSession,
                                                buffer,
                                                results,
                                                maxArraySize,
                                                &count,
                                                VI_ERROR_INSTR_INTERPRETING_RESPONSE);

    p2results = results;
    *arraySize = count / 2;

    for (idx = 0; idx < *arraySize; idx++)
        {
        frequencyValues[idx] = *p2results++;
        levelValues[idx] = *p2results++;
        }

    return rsesci_status;
}

/*****************************************************************************
 * Function: Set Transducer Set Unit
 * Purpose:  This function defines the unit of the selected transducer sets.
 *           When assigning transducer factors to the set, only factors which
 *           are compatible to the selected unit, i. e. factors with the same
 *           unit or the unit dB, are allowed.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetTransducerSetUnit(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32     transducerSetUnit
)
{
    ViChar  buffer[BUFFER_SIZE];

    CHECKWINDOW;

    if (rsesci_invalidViInt32Range (transducerSetUnit, 4, 11) && (transducerSetUnit != 0))
        return VI_ERROR_PARAMETER2;

    sprintf (buffer, "SENS%ld:CORR:TSET:UNIT", window);

    return rsesci_setViStringQuoted (instrSession,
                                     buffer,
                                     "'",
                                     unitArr[transducerSetUnit],
                                     VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Transducer Set Unit
 * Purpose:  This function returns the unit of the selected transducer sets.
 *           When assigning transducer factors to the set, only factors which
 *           are compatible to the selected unit, i. e. factors with the same
 *           unit or the unit dB, are allowed.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetTransducerSetUnit(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32*    transducerSetUnit
)
{
    ViStatus    rsesci_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE],
                tmpBuf[BUFFER_SIZE];

    CHECKWINDOW;

    if (!transducerSetUnit)
        return VI_ERROR_PARAMETER3;

    sprintf (buffer, "SENS%ld:CORR:TSET:UNIT?", window);

    CHECKERR (rsesci_getViStringQuoted (instrSession,
                                        buffer,
                                        "'",
                                        BUFFER_SIZE,
                                        tmpBuf,
                                        VI_ERROR_INSTR_INTERPRETING_RESPONSE));

    return rsesci_getIndexFromString (instrSession, tmpBuf, unitArr, transducerSetUnit);
}

/*****************************************************************************
 * Function: Set Transducer Set Break
 * Purpose:  This function defines if the sweep is to be stopped on changeover
 *           from range to another.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetTransducerSetBreak(
    ViSession   instrSession,
    ViInt32     window,
    ViBoolean   transducerSetBreak
)
{
    return rsesci_setViBooleanWindow (instrSession,
                                      ":SENS%ld:CORR:TSET:BRE",
                                      window,
                                      transducerSetBreak,
                                      VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Transducer Set Break
 * Purpose:  This function returns if the sweep is to be stopped on changeover
 *           from range to another.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetTransducerSetBreak(
    ViSession   instrSession,
    ViInt32     window,
    ViBoolean*  transducerSetBreak
)
{
    return rsesci_getViBooleanWindow (instrSession,
                                      ":SENS%ld:CORR:TSET:BRE?",
                                      window,
                                      transducerSetBreak,
                                      VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set Transducer Set Comment
 * Purpose:  This function defines the comment for the selected transducer
 *           set.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetTransducerSetComment(
    ViSession   instrSession,
    ViInt32     window,
    ViString    comment
)
{
    ViChar  buffer[BUFFER_SIZE];

    CHECKWINDOW;

    sprintf (buffer, "SENS%ld:CORR:TSET:COMM", window);

    return rsesci_setViStringQuoted (instrSession,
                                     buffer,
                                     "'",
                                     comment,
                                     VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Transducer Set Comment
 * Purpose:  This function returns the comment for the selected transducer
 *           set.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetTransducerSetComment(
    ViSession   instrSession,
    ViInt32     window,
    ViChar      _VI_FAR comment[]
)
{
    ViChar  buffer[BUFFER_SIZE];

    CHECKWINDOW;

    sprintf (buffer, "SENS%ld:CORR:TSET:COMM?", window);

    return rsesci_getViStringQuoted (instrSession,
                                     buffer,
                                     "'",
                                     8,
                                     comment,
                                     VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Set Transducer Set Values
 * Purpose:  This function defines a partial range of the selected transducer
 *           set. The partial range is determined by its start and stop
 *           frequencies plus a list of names of the assigned transducer
 *           factors. The ranges 1 to 10 must be sent in ascending order.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetTransducerSetValues(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32     range,
    ViReal64    startFrequency,
    ViReal64    stopFrequency,
    ViString    factors
)
{
    ViChar  buffer[BUFFER_SIZE],
            tmpBuffer[BUFFER_SIZE],
            *p2buf,
            *p2tmpBuf;

    if (rsesci_invalidViInt32Range (range, 1, 10))
        return VI_ERROR_PARAMETER3;
    if (!factors)
        return VI_ERROR_PARAMETER6;

    strcpy (buffer, factors);
    p2buf = strtok (buffer, ",");
    p2tmpBuf = tmpBuffer;

    while(p2buf)
        {
        p2tmpBuf += sprintf (p2tmpBuf, "'%s',", p2buf);
        p2buf = strtok (NULL, ",");
        }

    p2tmpBuf--;
    *p2tmpBuf = '\0';
    
    sprintf (buffer, "SENS%%ld:CORR:TSET:RANG%ld %.12lf,%.12lf,%s", range,
                                                              startFrequency,
                                                              stopFrequency,
                                                              tmpBuffer);

    return rsesci_sendCmdWindow (instrSession,
                                 buffer,
                                 window);
}

/*****************************************************************************
 * Function: Get Transducer Set Values
 * Purpose:  This function returns a partial range of the selected transducer
 *           set. The partial range is determined by its start and stop
 *           frequencies plus a list of names of the assigned transducer
 *           factors.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetTransducerSetValues(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32     range,
    ViReal64*   startFrequency,
    ViReal64*   stopFrequency,
    ViChar      _VI_FAR factors[]
)
{
    ViStatus    rsesci_status;
    ViChar      buffer[BUFFER_SIZE];
    ViChar      read_buf[BUFFER_SIZE];

    if (rsesci_invalidViInt32Range (range, 1, 10))
        return VI_ERROR_PARAMETER3;
    if (!startFrequency)
        return VI_ERROR_PARAMETER4;
    if (!stopFrequency)
        return VI_ERROR_PARAMETER5;
    if (!factors)
        return VI_ERROR_PARAMETER6;

    sprintf (buffer, "SENS%%ld:CORR:TSET:RANG%ld?", range);

    CHECKERR (rsesci_getViStringWindow (instrSession,
                                        buffer,
                                        window,
                                        read_buf,
                                        BUFFER_SIZE,
                                        VI_ERROR_INSTR_INTERPRETING_RESPONSE));

    if (sscanf (read_buf,"%le,%le,%[^\r\n]", startFrequency, stopFrequency, factors) != 3)
        return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

    return rsesci_status;
}

/*****************************************************************************
 * Function: Set GPIB Address
 * Purpose:  This function changes the IEC/IEEE-bus address of the unit.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetGPIBAddress(
    ViSession   instrSession,
    ViInt32 gpibAddress
)
{
    return rsesci_setViInt32 (instrSession,
                              "SYST:COMM:GPIB:ADDR",
                              gpibAddress,
                              0,30,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get GPIB Address
 * Purpose:  This function returns the IEC/IEEE-bus address of the unit.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetGPIBAddress(
    ViSession   instrSession,
    ViInt32*    gpibAddress
)
{
    return rsesci_getViInt32 (instrSession,
                              "SYST:COMM:GPIB:ADDR?",
                              gpibAddress,
                              VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Serial Interface Configuration
 * Purpose:  This function sets the serial interface parameters.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetSerialInterfaceConfiguration(
    ViSession   instrSession,
    ViInt32 baudRate,
    ViInt32 dataBits,
    ViInt32 stopBits,
    ViBoolean   hardwareHandshake,
    ViBoolean   softwareHandshake,
    ViInt32 parity
)
{
    ViStatus    rsesci_status;
    ViInt32     i = 0;

    while (baudArr[i] != VI_NULL)
    {
        if (baudArr [i] == baudRate) break;
        i++;
    }

    if (baudArr[i] == VI_NULL)
        return VI_ERROR_PARAMETER2;

    CHECKERR (rsesci_setViInt32 (instrSession,
                                 "SYST:COMM:SER:BAUD",
                                 baudRate,
                                 0, 0,
                                 VI_ERROR_PARAMETER2));

    CHECKERR (rsesci_setViInt32 (instrSession,
                                 "SYST:COMM:SER:BITS",
                                 dataBits,
                                 7, 8,
                                 VI_ERROR_PARAMETER3));

    CHECKERR (rsesci_setViInt32 (instrSession,
                                "SYST:COMM:SER:SBIT",
                                 stopBits,
                                 1, 2,
                                 VI_ERROR_PARAMETER4));

    CHECKERR (rsesci_setEnum (instrSession,
                              "SYST:COMM:SER:CONT:DTR",
                              hardwareHandshake,
                              serialHWHandshakeArr,
                              VI_ERROR_PARAMETER5));

    CHECKERR (rsesci_setEnum (instrSession,
                              "SYST:COMM:SER:CONT:RTS",
                              hardwareHandshake,
                              serialHWHandshakeArr,
                              VI_ERROR_PARAMETER5));

    CHECKERR (rsesci_setEnum (instrSession,
                              "SYST:COMM:SER:PACE",
                              softwareHandshake,
                              serialSWHandshakeArr,
                              VI_ERROR_PARAMETER6));

    CHECKERR (rsesci_setEnum (instrSession,
                              "SYST:COMM:SER:PAR",
                              parity,
                              parityArr,
                              VI_ERROR_PARAMETER7));

    return rsesci_status;
}

/*****************************************************************************
 * Function: Get Serial Interface Configuration
 * Purpose:  This function returns the settings of serial interface
 *           parameters.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetSerialInterfaceConfiguration(
    ViSession   instrSession,
    ViInt32*    baudRate,
    ViInt32*    dataBits,
    ViInt32*    stopBits,
    ViBoolean*  hardwareHandshake,
    ViBoolean*  softwareHandshake,
    ViInt32*    parity
)
{
    ViStatus    rsesci_status;
    ViInt32     i;

    CHECKERR (rsesci_getViInt32 (instrSession,
                                 "SYST:COMM:SER:BAUD?",
                                 baudRate,
                                 VI_ERROR_PARAMETER2));

    CHECKERR (rsesci_getViInt32 (instrSession,
                                 "SYST:COMM:SER:BITS?",
                                 dataBits,
                                 VI_ERROR_PARAMETER3));

    CHECKERR (rsesci_getViInt32 (instrSession,
                                "SYST:COMM:SER:SBIT?",
                                 stopBits,
                                 VI_ERROR_PARAMETER4));

    CHECKERR (rsesci_getEnum (instrSession,
                              "SYST:COMM:SER:CONT:DTR?",
                              &i,
                              serialHWHandshakeArr,
                              VI_ERROR_PARAMETER5));
    *hardwareHandshake = (i == 1);

    CHECKERR (rsesci_getEnum (instrSession,
                              "SYST:COMM:SER:PACE?",
                              &i,
                              serialSWHandshakeArr,
                              VI_ERROR_PARAMETER6));
    *softwareHandshake = (i == 1);

    CHECKERR (rsesci_getEnum (instrSession,
                              "SYST:COMM:SER:PAR?",
                              parity,
                              parityArr,
                              VI_ERROR_PARAMETER7));

    return rsesci_status;
}

/*****************************************************************************
 * Function: Set System Time
 * Purpose:  This function sets the internal clock. The sequence of entry is
 *           hour, minute, second.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetSystemTime(
    ViSession   instrSession,
    ViInt32 hour,
    ViInt32 minute,
    ViInt32 second
)
{
    ViChar  buffer[BUFFER_SIZE];

    if (rsesci_invalidViInt32Range (hour, 0, 23))
        return VI_ERROR_PARAMETER2;
    if (rsesci_invalidViInt32Range (minute, 0, 59))
        return VI_ERROR_PARAMETER3;
    if (rsesci_invalidViInt32Range (second, 0, 59))
        return VI_ERROR_PARAMETER4;

    sprintf (buffer,"SYST:TIME %ld,%ld,%ld", hour, minute, second);

    return rsesci_sendCmd (instrSession, buffer);
}

/*****************************************************************************
 * Function: Get System Time
 * Purpose:  This function returns the internal clock. The sequence of entry
 *           is hour, minute, second.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetSystemTime(
    ViSession   instrSession,
    ViInt32*    hour,
    ViInt32*    minute,
    ViInt32*    second
)
{
    ViStatus    rsesci_status;
    ViChar      buffer[BUFFER_SIZE];

    if (!hour)
        return VI_ERROR_PARAMETER2;
    if (!minute)
        return VI_ERROR_PARAMETER3;
    if (!second)
        return VI_ERROR_PARAMETER4;

    CHECKERR (rsesci_getViString (instrSession,
                                  "SYST:TIME?",
                                  buffer,
                                  BUFFER_SIZE,
                                  VI_ERROR_PARAMETER1));

    sscanf (buffer, "%ld,%ld,%ld", hour, minute, second);

    return rsesci_status;
}

/*****************************************************************************
 * Function: Set System Date
 * Purpose:  This function is used to enter the date for the internal
 *           calendar. The sequence of entry is year, month, day.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetSystemDate(
    ViSession   instrSession,
    ViInt32 day,
    ViInt32 month,
    ViInt32 year
)
{
    ViChar  buffer[BUFFER_SIZE];

    if (rsesci_invalidViInt32Range (day, 1, 31))
        return VI_ERROR_PARAMETER2;
    if (rsesci_invalidViInt32Range (month, 1, 12))
        return VI_ERROR_PARAMETER3;
    if (rsesci_invalidViInt32Range (year, 1980, 2099))
        return VI_ERROR_PARAMETER4;

    sprintf (buffer,"SYST:DATE %ld,%ld,%ld", year, month, day);

    return rsesci_sendCmd (instrSession, buffer);
}

/*****************************************************************************
 * Function: Get System Date
 * Purpose:  This function returns the date for the internal calendar.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetSystemDate(
    ViSession   instrSession,
    ViInt32*    day,
    ViInt32*    month,
    ViInt32*    year
)
{
    ViStatus    rsesci_status;
    ViChar      buffer[BUFFER_SIZE];

    if (!day)
        return VI_ERROR_PARAMETER2;
    if (!month)
        return VI_ERROR_PARAMETER3;
    if (!year)
        return VI_ERROR_PARAMETER4;

    CHECKERR (rsesci_getViString (instrSession,
                                  "SYST:DATE?",
                                  buffer,
                                  BUFFER_SIZE,
                                  VI_ERROR_PARAMETER1));

    sscanf (buffer, "%ld,%ld,%ld", year, month, day);

    return rsesci_status;
}

/*****************************************************************************
 * Function: Set Speaker Volume
 * Purpose:  This function sets the volume of the built-in loudspeaker for
 *           demodulated signals.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetSpeakerVolume(
    ViSession   instrSession,
    ViReal64    speakerVolume
)
{
    return rsesci_setViReal64 (instrSession,
                               "SYST:SPE:VOL",
                               speakerVolume / 100,
                               0.0, 1.0,
                               VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Speaker Volume
 * Purpose:  This function returns the volume of the built-in loudspeaker for
 *           demodulated signals.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetSpeakerVolume(
    ViSession   instrSession,
    ViReal64*   speakerVolume
)
{
    ViStatus    rsesci_status;

    rsesci_status = rsesci_getViReal64 (instrSession,
                                        "SYST:SPE:VOL?",
                                        speakerVolume,
                                        VI_ERROR_PARAMETER2);

    if (speakerVolume)
        *speakerVolume = *speakerVolume * 100;

    return rsesci_status;
}

/*****************************************************************************
 * Function: Set Soft Frontpanel
 * Purpose:  This function activates or deactivates the display of the front
 *           panel keys on the screen.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetSoftFrontpanel(
    ViSession   instrSession,
    ViBoolean   softFrontpanel
)
{
    return rsesci_setViBoolean (instrSession,
                                "SYST:DISP:FPAN",
                                softFrontpanel,
                                VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Soft Frontpanel
 * Purpose:  This function returns if the display of the front panel keys on
 *           the screen is activated or deactivated.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetSoftFrontpanel(
    ViSession   instrSession,
    ViBoolean*  softFrontpanel
)
{
    return rsesci_getViBoolean (instrSession,
                                "SYST:DISP:FPAN?",
                                softFrontpanel,
                                VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set User Port
 * Purpose:  This function selects whether user port is used as output or
 *           input.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetUserPort(
    ViSession   instrSession,
    ViInt32 userPort
)
{
    ViChar  buffer[BUFFER_SIZE];

    if (userPort == 0)
        strcpy (buffer, "INP:UPOR:STAT 1");
    else
        strcpy (buffer, "OUTP:UPOR:STAT 1");

    return rsesci_sendCmd (instrSession, buffer);
}

/*****************************************************************************
 * Function: Get User Port
 * Purpose:  This function returns whether user port is used as output or
 *           input.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetUserPort(
    ViSession   instrSession,
    ViInt32*    userPort
)
{
    ViStatus    rsesci_status;
    ViBoolean   state;

    CHECKERR (rsesci_getViBoolean (instrSession,
                                   "INP:UPOR:STAT?",
                                   &state,
                                   VI_ERROR_PARAMETER2));

    *userPort = (state == VI_FALSE);

    return rsesci_status;
}

/*****************************************************************************
 * Function: Read User Port Value
 * Purpose:  This function queries the control lines of the user ports.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_ReadUserPortValue(
    ViSession   instrSession,
    ViBoolean   _VI_FAR userPortValue[]
)
{
    ViStatus    rsesci_status;
    ViChar      buffer[BUFFER_SIZE];

    CHECKERR (rsesci_getViString(instrSession,
                                 "INP:UPOR?",
                                 buffer,
                                 BUFFER_SIZE,
                                 VI_ERROR_PARAMETER1));

    sscanf (buffer, "#B%1hu%1hu%1hu%1hu%1hu%1hu%1hu%1hu", userPortValue + 7,
                                                          userPortValue + 6,
                                                          userPortValue + 5,
                                                          userPortValue + 4,
                                                          userPortValue + 3,
                                                          userPortValue + 2,
                                                          userPortValue + 1,
                                                          userPortValue);
    return rsesci_status;
}

/*****************************************************************************
 * Function: Write User Port Value
 * Purpose:  This function sets the control lines of the user ports. In manual
 *           operation, the control lines are represented by softkeys PORT 0 to
 *           7.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_WriteUserPortValue(
    ViSession   instrSession,
    ViBoolean   _VI_FAR userPortValue[]
)
{
    ViChar  buffer[BUFFER_SIZE];
    ViChar  *p2buf;
    ViInt32 i;

    p2buf = buffer + sprintf (buffer, "OUTP:UPOR #B");

    for (i = 0; i < 8; i++)
        p2buf += sprintf (p2buf, "%ld", userPortValue[i] != VI_FALSE);

    return rsesci_sendCmd (instrSession, buffer);
}

/*****************************************************************************
 * Function: Set Service Source
 * Purpose:  This function toggles between the RF input on the front panel and
 *           the internal 128-MHz reference signal in the selected measurement
 *           window. The level of the 128-MHz signals can be also selected.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetServiceSource(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32 serviceSource,
    ViReal64    calibrationSourceLevel
)
{
    ViStatus    rsesci_status;

    CHECKERR (rsesci_setEnumWindow (instrSession,
                                    "DIAG%ld:SERV:INP",
                                    window,
                                    serviceSource,
                                    serviceSourceArr,
                                    VI_ERROR_PARAMETER3));

    if (serviceSource == RSESCI_INT_CALIB_SOURCE)
        {
        CHECKERR (rsesci_setViReal64Window (instrSession,
                                            "DIAG%ld:SERV:CSO",
                                            window,
                                            calibrationSourceLevel,
                                            -30.0, 0.0,
                                            VI_ERROR_PARAMETER4));
        }

    return rsesci_status;
}

/*****************************************************************************
 * Function: Get Service Source
 * Purpose:  This function returns whether the RF input on the front panel or
 *           the internal 128-MHz reference signal in the selected measurement
 *           window is used. The level of the 128-MHz signals is also queried.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetServiceSource(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32*    serviceSource,
    ViReal64*   calibrationSourceLevel
)
{
    ViStatus    rsesci_status;

    CHECKERR (rsesci_getEnumWindow (instrSession,
                                    "DIAG%ld:SERV:INP?",
                                    window,
                                    serviceSource,
                                    serviceSourceArr,
                                    VI_ERROR_PARAMETER3));

    CHECKERR (rsesci_getViReal64Window (instrSession,
                                        "DIAG%ld:SERV:CSO?",
                                        window,
                                        calibrationSourceLevel,
                                        VI_ERROR_PARAMETER4));

    return rsesci_status;
}

/*****************************************************************************
 * Function: Set Service Pulse Generator
 * Purpose:  This function toggles the calibration signal in the selected
 *           measurement window between pulsed and non-pulsed. The selection
 *           takes effect only if the RF input has been set to the internal
 *           reference signal using DIAG:SERV:INP CAL.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetServicePulseGenerator(
    ViSession   instrSession,
    ViInt32     window,
    ViBoolean   pulseGenerator,
    ViReal64    pulseRate
)
{
    ViStatus    rsesci_status;
    ViReal64    prat_local = pulseRate;
    ViInt16     index;

    rsesci_coerceToNearest (pulseRateArr, &prat_local, &index);

    CHECKERR (rsesci_setViBooleanWindow (instrSession,
                                         "DIAG%ld:SERV:INP:PULS",
                                         window,
                                         pulseGenerator,
                                         VI_ERROR_PARAMETER3));

    if (pulseGenerator == VI_TRUE)
        {
        rsesci_status = rsesci_setViReal64Window (instrSession,
                                                  "DIAG%ld:SERV:INP:PULS:PRAT",
                                                  window,
                                                  pulseRate,
                                                  0.0, 0.0,
                                                  VI_ERROR_PARAMETER4);
        }

    return rsesci_status;
}

/*****************************************************************************
 * Function: Get Service Pulse Generator
 * Purpose:  This function returns if the calibration signal in the selected
 *           measurement window is pulsed or non-pulsed.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetServicePulseGenerator(
    ViSession   instrSession,
    ViInt32     window,
    ViBoolean*  pulseGenerator,
    ViReal64*   pulseRate
)
{
    ViStatus    rsesci_status       = VI_SUCCESS;
    ViInt32     pulseGeneratorTmp   = 0;

    CHECKERR (rsesci_getEnumWindow (instrSession,
                                    "DIAG%ld:SERV:INP:PULS?",
                                    window,
                                    &pulseGeneratorTmp,
                                    switchArr,
                                    VI_ERROR_PARAMETER3));

    if (pulseGenerator)
        *pulseGenerator = (ViBoolean) pulseGeneratorTmp;

    if (pulseGeneratorTmp == VI_TRUE)
        {
        rsesci_status = rsesci_getViReal64Window (instrSession,
                                                  "DIAG%ld:SERV:INP:PULS:PRAT?",
                                                  window,
                                                  pulseRate,
                                                  VI_ERROR_PARAMETER4);
        }

    return rsesci_status;
}

/*****************************************************************************
 * Function: Set Calibration Signal Destination
 * Purpose:  This function configures rectangle calibration signal. This
 *           function takes effect only if the RF input has been set to the
 *           internal reference signal.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_ServiceConfigureRectangleCalibrationSignal (ViSession instrSession,
                                                                     ViInt32 window,
                                                                     ViBoolean state,
                                                                     ViReal64 frequency)
{
    ViStatus rsesci_status = VI_SUCCESS;
    CHECKERR (rsesci_SetServiceRectangleState (instrSession, window, state));
    
    rsesci_status = rsesci_SetServiceRectangleRate (instrSession, window, frequency);
    REMAP (3, 4);
    
    return rsesci_status;
}

/*****************************************************************************
 * Function: Set IF Shift
 * Purpose:  This function configures the shifting of the 1st IF.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetIFShift (ViSession instrSession, ViInt32 IFShift)
{
    return rsesci_setEnum (instrSession
                         , "SWE:IF:SHIF"
                         , IFShift
                         , ifShiftArr
                         , VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get IF Shift
 * Purpose:  This function queries the shifting of the 1st IF.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetIFShift (ViSession instrSession,
                                     ViInt32 *IFShift)
{
    return rsesci_getEnum (instrSession
                         , "SWE:IF:SHIF?"
                         , IFShift
                         , ifShiftArr
                         , VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Calibration Signal Destination
 * Purpose:  This function switches the calibration signal to the I or Q path.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetCalibrationSignalDestination (ViSession instrSession,
                                                          ViInt32 signalDestination)
{
	return rsesci_setEnum (instrSession,
		                   "DIAG:SERV:IQ:CAL:DEST",
						   signalDestination,
						   signalDestinationArr,
						   VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Calibration Signal Destination
 * Purpose:  This function returns the calibration signal to the I or Q path.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetCalibrationSignalDestination (ViSession instrSession,
                                                          ViInt32 *signalDestination)
{
	return rsesci_getEnum (instrSession,
		                   "DIAG:SERV:IQ:CAL:DEST?",
						   signalDestination,
						   signalDestinationArr,
						   VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Set Calibration Signal Voltage
 * Purpose:  This function selects the voltage for the DC Cal signal in Volt.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetCalibrationSignalVoltage (ViSession instrSession,
                                                      ViInt32 signalVoltage)
{
 	return rsesci_setEnum (instrSession,
		                   "DIAG:SERV:IQ:CAL:DC",
						   signalVoltage,
						   signalVoltageArr,
						   VI_ERROR_PARAMETER2);   
}

/*****************************************************************************
 * Function: Get Calibration Signal Voltage
 * Purpose:  This function returns the voltage for the DC Cal signal in Volt.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetCalibrationSignalVoltage (ViSession instrSession,
                                                      ViInt32 *signalVoltage)
{
	return rsesci_getEnum (instrSession,
		                   "DIAG:SERV:IQ:CAL:DC?",
						   signalVoltage,
						   signalVoltageArr,
						   VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Set Calibration Signal Pulse Frequency
 * Purpose:  This function sets the frequency of the Pulse Cal signal.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetCalibrationSignalPulseFrequency (ViSession instrSession,
                                                             ViInt32 pulseFrequency)
{
	return rsesci_setEnum (instrSession,
		                   "DIAG:SERV:IQ:CAL:PULS:PRAT",
						   pulseFrequency,
						   pulseFrequencyArr,
						   VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Get Calibration Signal Pulse Frequency
 * Purpose:  This function gets the frequency of the Pulse Cal signal.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetCalibrationSignalPulseFrequency (ViSession instrSession,
                                                             ViInt32 *pulseFrequency)
{
	return rsesci_getEnum (instrSession,
		                   "DIAG:SERV:IQ:CAL:PULS:PRAT?",
						   pulseFrequency,
						   pulseFrequencyArr,
						   VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Set Calibration Signal Baseband Source
 * Purpose:  This function selects the baseband signal source.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetCalibrationSignalBasebandSource (ViSession instrSession,
                                                             ViInt32 basebandSource)
{
	return rsesci_setEnum (instrSession,
		                   "DIAG:SERV:IQ:INP",
						   basebandSource,
						   basebandSourceArr,
						   VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Get Calibration Signal Baseband Source
 * Purpose:  This function returns the baseband signal source.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetCalibrationSignalBasebandSource (ViSession instrSession,
                                                             ViInt32 *basebandSource)
{
	return rsesci_getEnum (instrSession,
		                   "DIAG:SERV:IQ:INP?",
						   basebandSource,
						   basebandSourceArr,
						   VI_ERROR_PARAMETER2);    
}

/*****************************************************************************
 * Function: Set Service Rectangle State
 * Purpose:  This function enables or disables rectangle calibration signal.
 *           This function takes effect only if the RF input has been set to
 *           the internal reference signal.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetServiceRectangleState (ViSession instrSession,
                                                   ViInt32 window,
                                                   ViBoolean state)
{
    CHECKWINDOW;
    
    return rsesci_setViBooleanWindow (instrSession
                                    , "DIAG%ld:SERV:INP:RECT"
                                    , window
                                    , state
                                    , VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Service Rectangle State
 * Purpose:  This function queries the state of rectangle calibration signal.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetServiceRectangleState (ViSession instrSession,
                                                   ViInt32 window,
                                                   ViBoolean *state)
{
    ViStatus rsesci_status = VI_SUCCESS;
    ViInt32  i = 0;
    
    CHECKWINDOW;
    
    CHECKERR (rsesci_getEnumWindow (instrSession
                                  , "DIAG%ld:SERV:INP:RECT?"
                                  , window
                                  , &i
                                  , switchArr
                                  , VI_ERROR_PARAMETER3));
    *state = (i != 0);
    return rsesci_status;
}

/*****************************************************************************
 * Function: Set Service Rectangle Rate
 * Purpose:  This function sets rectangle calibration signal frequency.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetServiceRectangleRate (ViSession instrSession,
                                                  ViInt32 window,
                                                  ViReal64 frequency)
{
    CHECKWINDOW;
    
    return rsesci_setViReal64Window (instrSession
                                   , "DIAG%ld:SERV:INP:RECT:PRAT"
                                   , window
                                   , frequency
                                   , 5000.0, 500000.0
                                   , VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Get Service Rectangle Rate
 * Purpose:  This function queries rectangle calibration signal frequency.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetServiceRectangleRate (ViSession instrSession,
                                                  ViInt32 window,
                                                  ViReal64 *frequency)
{
    CHECKWINDOW;
    
    return rsesci_getViReal64Window (instrSession
                                   , "DIAG%ld:SERV:INP:RECT:PRAT?"
                                   , window
                                   , frequency
                                   , VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Firmware Update
 * Purpose:  This function starts a firmware update using the data set in the
 *           selected directory. Beforehand, the update files have to be stored
 *           in the following subdirectories using command MMEM:DATA:
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_FirmwareUpdate(
    ViSession   instrSession,
    ViString    path
)
{
    return rsesci_setViStringQuoted (instrSession,
                                     "SYST:FIRM:UPD",
                                     "'",
                                     path,
                                     VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Compatibility Mode
 * Purpose:  This function determines if the settings of the receiver after
 *           reset are compatible to the settings of the R&S FSP (selection
 *           FSP). If Off selected, the receiver specific settings are active
 *           after a reset.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_CompatibilityMode (ViSession instrSession,
                                            ViInt32 compatibilityMode)
{
    return rsesci_setEnum (instrSession,
                           "SYST:PRES:COMP",
                           compatibilityMode,
                           compatibilityModeArr,
                           VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Data Set Save
 * Purpose:  This function stores the current device settings in a series of
 *           files which have the indicated file name, but different
 *           extensions. The file name includes indication of the path and may
 *           also include the drive name. The path name complies with DOS
 *           conventions.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_DataSetSave(
    ViSession   instrSession,
    ViString    fileName,
    ViString    comment
)
{
    ViStatus    rsesci_status;
    ViChar      buffer[BUFFER_SIZE];

    CHECKERR (rsesci_setViStringQuoted (instrSession,
                                        "MMEM:COMM",
                                        "'",
                                        comment,
                                        VI_ERROR_PARAMETER3));
    sprintf (buffer, "MMEM:STOR:STAT 1,'%s'", fileName);

    return rsesci_sendCmd (instrSession, buffer);
}

/*****************************************************************************
 * Function: Data Set Recall
 * Purpose:  This function loads device settings from files.The contents of
 *           the file are loaded and set as the new device state. The device
 *           automatically recognizes the files that are required for the whole
 *           setup from the list of file extensions:
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_DataSetRecall(
    ViSession   instrSession,
    ViString    fileName
)
{
    ViChar      buffer[BUFFER_SIZE];

    sprintf (buffer, "MMEM:LOAD:STAT 1,'%s'", fileName);

    return rsesci_sendCmd (instrSession, buffer);
}

/*****************************************************************************
 * Function: Data Set Recall Auto
 * Purpose:  This function defines which device setting is automatically
 *           loaded after the device is switched on. The contents of the file
 *           are read after switching on the device and used to define the new
 *           device state.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_DataSetRecallAuto(
    ViSession   instrSession,
    ViString    fileName
)
{
    ViChar      buffer[BUFFER_SIZE];

    sprintf (buffer, "MMEM:LOAD:AUTO 1,'%s'", fileName);

    return rsesci_sendCmd (instrSession, buffer);
}

/*****************************************************************************
 * Function: Data Set Items
 * Purpose:  This function sets the list of device settings to be
 *           stored/loaded.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_DataSetItems(
    ViSession   instrSession,
    ViInt32 dataSetItems,
    ViBoolean   selectedItem
)
{
    ViStatus    rsesci_status;
    ViChar      buffer[BUFFER_SIZE];

    if (rsesci_invalidViInt32Range (dataSetItems, 0, 10))
        return VI_ERROR_PARAMETER2;
	
	if (dataSetItems == RSESCI_DATA_SET_HCOPY)
		CHECKMODEL(RSESU8 | RSESU26 | RSESU40);

    sprintf (buffer, "MMEM:SEL:%s", dataSetArr[dataSetItems]);

    if (dataSetItems < RSESCI_DATA_SET_CURRENT)
        rsesci_status = rsesci_sendCmd (instrSession, buffer);
    else
        rsesci_status = rsesci_setViBoolean (instrSession,
                                             buffer,
                                             selectedItem,
                                             VI_ERROR_PARAMETER3);

    return rsesci_status;
}

/*****************************************************************************
 * Function: Data Set Catalog
 * Purpose:  This function reads the indicated directory. According to DOS
 *           convention, wild card characters can be entered in order to query
 *           e.g. a list of all files of a certain type.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_DataSetCatalog(
    ViSession   instrSession,
    ViString    directory,
    ViChar      _VI_FAR catalog[],
    ViInt32     bufferSize
)
{
    ViChar      buffer[BUFFER_SIZE];

    sprintf (buffer, "MMEM:CAT? '%s'", directory);

    return rsesci_getViString (instrSession,
                               buffer,
                               catalog,
                               bufferSize,
                               VI_ERROR_PARAMETER4);
}

/*****************************************************************************
 * Function: Data Set Catalog Long
 * Purpose:  This function queries the directories and files in the given path.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_DataSetCatalogLong (ViSession instrSession,
                                             ViString directory,
                                             ViChar catalog[],
                                             ViInt32 bufferSize)
{
	ViChar      buffer[BUFFER_SIZE];

    sprintf (buffer, "MMEM:CAT:LONG? '%s'", directory);

    return rsesci_getViString (instrSession,
                               buffer,
                               catalog,
                               bufferSize,
                               VI_ERROR_PARAMETER4);    
}

/*****************************************************************************
 * Function: Data Set Clear
 * Purpose:  This function deletes the instrument setting selected by file
 *           names ('path' parameter) or deletes all device settings in the
 *           current directory ('path' parameter). All associated files on the
 *           mass memory storage are cleared. A list of the extensions used is
 *           included under MMEMory:LOAD:STATe.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_DataSetClear(
    ViSession   instrSession,
    ViInt32 clear,
    ViString    path
)
{
    ViStatus    rsesci_status;
    ViChar      buffer[BUFFER_SIZE];

    if (clear == RSESCI_CLEAR_FILE)
    {
        sprintf (buffer, "MMEM:CLE:STAT 1,'%s'", path);
        CHECKERR (rsesci_sendCmd (instrSession, buffer));
    }
    else
    {
        sprintf (buffer, "MMEM:CDIR '%s'", path);
        CHECKERR (rsesci_sendCmd (instrSession, buffer));
        CHECKERR (rsesci_sendCmd (instrSession, "MMEM:CLE:ALL"));
    }

    return rsesci_status;
}

/*****************************************************************************
 * Function: File Manager
 * Purpose:  This function provides operations which allows to do basic file
 *           operations on the storage media of the instrument. It consists
 *           from the following operations:
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_FileManager(
    ViSession   instrSession,
    ViInt32     operationtobePerformed,
    ViString    source,
    ViString    destination
)
{
    ViStatus    rsesci_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];

    switch (operationtobePerformed)
        {
        case RSESCI_FILE_MAN_CDRIVE:
            rsesci_status = rsesci_setViStringQuoted (instrSession, "MMEM:MSIS", "'", source, VI_ERROR_PARAMETER3);
            break;
        case RSESCI_FILE_MAN_CDIR:
            rsesci_status = rsesci_setViStringQuoted (instrSession, "MMEM:CDIR", "'", source, VI_ERROR_PARAMETER3);
            break;
        case RSESCI_FILE_MAN_MDIR:
            rsesci_status = rsesci_setViStringQuoted (instrSession, "MMEM:MDIR", "'", source, VI_ERROR_PARAMETER3);
            break;
        case RSESCI_FILE_MAN_RDIR:
            rsesci_status = rsesci_setViStringQuoted (instrSession, "MMEM:RDIR", "'", source, VI_ERROR_PARAMETER3);
            break;
        case RSESCI_FILE_MAN_COPY:
            sprintf (buffer,":MMEM:COPY '%s','%s'", source, destination);
            rsesci_status = rsesci_sendCmd (instrSession, buffer);
            break;
        case RSESCI_FILE_MAN_MOVE:
            sprintf (buffer,":MMEM:MOVE '%s','%s'", source, destination);
            rsesci_status = rsesci_sendCmd (instrSession, buffer);
            break;
        case RSESCI_FILE_MAN_DELETE:
            rsesci_status = rsesci_setViStringQuoted (instrSession, "MMEM:DEL", "'", source, VI_ERROR_PARAMETER3);
            break;
        case RSESCI_FILE_MAN_FORMAT:
            rsesci_status = rsesci_sendCmd (instrSession, ":MMEM:INIT 'A:'");
            break;
        default:
            return VI_ERROR_PARAMETER2;
        }

    return rsesci_status;
}

/*****************************************************************************
 * Function: Store Peak List
 * Purpose:  This function stores the content of the marker peak list in
 *           ASCII format to the specified file.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_StorePeakList (ViSession instrSession,
                                        ViString fileName)
{
    return rsesci_setViStringQuoted (instrSession, "MMEM:STOR:PEAK", "'", fileName, VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Store Markers
 * Purpose:  This function saves the data of all active markers to the
 *           specified file.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_StoreMarkers (ViSession instrSession,
                                       ViInt32 window, ViString fileName)
{
    ViChar      buffer[BUFFER_SIZE];
    
    CHECKWINDOW;
    
    sprintf (buffer, "MMEM:STOR%ld:MARK", window);
    return rsesci_setViStringQuoted (instrSession, buffer, "'", fileName, VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Read To File From Instrument
 * Purpose:  This function is used to read data from the instrument and write
 *           it to a user specified file on the host computer.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_readToFile(
    ViSession   instrSession,
    ViString    source,
    ViString    destination
)
{
    ViUInt32    retCnt = 0;
    ViChar      buffer[BUFFER_SIZE];
    ViChar      *exBuf;
    ViInt32     lenLength;
    ViUInt32     count = 0;
    ViStatus    rsesci_status = VI_SUCCESS;
    ViUInt16    intfc = VI_INTF_GPIB;
    FILE        *targetFile;

    exBuf = 0;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_INTF_TYPE, &intfc));

    CHECKERR (rsesci_clearBeforeRead(instrSession));

    sprintf (buffer, "*CLS;:MMEM:DATA? '%s'\n", source);

    if ((rsesci_status = viWrite (instrSession, (ViBuf) buffer, (ViUInt32) strlen (buffer),
        &retCnt)) < 0)
        return rsesci_status;
    if ((rsesci_status = rsesci_checkBeforeRead (instrSession)) < 0)
        return rsesci_status;

    /* Interface specific operations */
    switch (intfc)
        {
        case VI_INTF_ASRL: /* Serial Interface. */
            /* Switch to Off using of termination character. */
            CHECKERR( viSetAttribute (instrSession, VI_ATTR_ASRL_END_IN, VI_ASRL_END_NONE));
            CHECKERR( viSetAttribute (instrSession, VI_ATTR_TERMCHAR_EN, VI_FALSE));

            /* Parse binary data header. */
            if ((rsesci_status = viRead (instrSession, (ViBuf) buffer, 2, &retCnt)) < 0)
                {
                viSetAttribute (instrSession, VI_ATTR_ASRL_END_IN, VI_ASRL_END_TERMCHAR);
                viSetAttribute (instrSession, VI_ATTR_TERMCHAR_EN, VI_TRUE);
                return rsesci_status;
                }
            buffer[retCnt] = '\0';
            if (sscanf (buffer, "#%1d", &lenLength) != 1)
                {
                viSetAttribute (instrSession, VI_ATTR_ASRL_END_IN, VI_ASRL_END_TERMCHAR);
                viSetAttribute (instrSession, VI_ATTR_TERMCHAR_EN, VI_TRUE);
                return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
                }

            /* Get number of points. */
            if ((rsesci_status = viRead (instrSession, (ViBuf) buffer, lenLength, &retCnt)) < 0)
                {
                viSetAttribute (instrSession, VI_ATTR_ASRL_END_IN, VI_ASRL_END_TERMCHAR);
                viSetAttribute (instrSession, VI_ATTR_TERMCHAR_EN, VI_TRUE);
                return rsesci_status;
                }

            buffer[retCnt] = '\0';
            if (sscanf (buffer, "%ld", &count) != 1)
                {
                viSetAttribute (instrSession, VI_ATTR_ASRL_END_IN, VI_ASRL_END_TERMCHAR);
                viSetAttribute (instrSession, VI_ATTR_TERMCHAR_EN, VI_TRUE);
                return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
                }

            /* Allocate memory space */
            if ((exBuf = malloc (count + 2)) == NULL)
                return VI_ERROR_ALLOC;
            /* Read exact number of bytes equal to number of points. */
            if ((rsesci_status = viRead (instrSession, (ViBuf) exBuf, count + 2, &retCnt)) < 0)
                {
                viSetAttribute (instrSession, VI_ATTR_ASRL_END_IN, VI_ASRL_END_TERMCHAR);
                viSetAttribute (instrSession, VI_ATTR_TERMCHAR_EN, VI_TRUE);
                /* If not NULL pointer, then deallocate memory. */
                if (exBuf) free (exBuf);
                return rsesci_status;
                }
            viSetAttribute (instrSession, VI_ATTR_ASRL_END_IN, VI_ASRL_END_TERMCHAR);
            viSetAttribute (instrSession, VI_ATTR_TERMCHAR_EN, VI_TRUE);
            break;

        case VI_INTF_GPIB: default:  /* GPIB Interface. Default case */

            if ((rsesci_status = viRead (instrSession, (ViBuf) buffer, 2, &retCnt)) < 0)
                return rsesci_status;
            if (sscanf (buffer, "#%1d", &lenLength) != 1) {
                rsesci_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
                return rsesci_status;
            }

            if ((rsesci_status = viRead (instrSession, (ViBuf) buffer, lenLength,
                &retCnt)) < 0)
                return rsesci_status;

            buffer [lenLength]='\0';

            if (sscanf (buffer, "%ld", &count) != 1) {
                rsesci_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
                return rsesci_status;
            }

            /* Allocate memory space */
            if ((exBuf = malloc (count + 1)) == NULL)
                return VI_ERROR_ALLOC;

            if (count > 0)
                {
                if ((rsesci_status = viRead (instrSession, (ViBuf) exBuf, count + 1,
                    &retCnt)) < 0)
                    {
                    /* If not NULL pointer, then deallocate memory. */
                    if (exBuf) free (exBuf);
                    return rsesci_status;
                    }
                }

            break;
        }

    /* Store data from instrument to file on host computer */
    if ((targetFile = fopen (destination, "wb")) == VI_NULL)
        {
        if (exBuf)
            free (exBuf);
        return VI_ERROR_INSTR_FILE_OPEN;
        }

    if (count != fwrite (exBuf, sizeof (ViByte), (size_t)(count), targetFile))
        {
        fclose (targetFile);
        if (exBuf)
            free (exBuf);
        return VI_ERROR_INSTR_FILE_WRITE;
        }

    fclose (targetFile);

    if (exBuf) /* If not NULL pointer, then deallocate memory. */
        free (exBuf);

    if ((rsesci_status = rsesci_checkStatus (instrSession)) < 0)
        return rsesci_status;

    return rsesci_status;
}

/*****************************************************************************
 * Function: Write From File To Instrument
 * Purpose:  This function is used to read data from the host computer and
 *           write it to a user specified file in the instrument.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_writeFromFile(
    ViSession   instrSession,
    ViString    source,
    ViString    destination
)
{
    ViStatus    rsesci_status       = VI_SUCCESS;
    ViUInt16    remote_interface    = VI_INTF_GPIB,
                endOut              = VI_ASRL_END_NONE;
    ViInt32     digits              = 0;
    ViUInt32    fileSize            = 0;
    ViUInt32    bytesWrite          = 0;
    ViChar      *exBuf;
    ViChar      *p2buf;
    FILE        *sourceFile;

    /* Open source file from host controller */
    if ((sourceFile = fopen (source, "rb")) == VI_NULL)
        return VI_ERROR_INSTR_FILE_OPEN;

    /* Get size of file */
    fseek (sourceFile, 0, SEEK_END);
    fileSize = ftell (sourceFile);
    fseek (sourceFile, 0, SEEK_SET);

    /* Allocate memory space (add 64 bytes for command header) */
    if ((exBuf = malloc (fileSize + 64)) == NULL)
        return VI_ERROR_ALLOC;

    /* Create command header */
    if (fileSize > 0)
        digits = ((int)floor(log10(fileSize))) + 1;

    p2buf = exBuf;
    bytesWrite = sprintf (p2buf, ":MMEM:DATA '%s',#%ld%ld", destination, digits, fileSize);
    p2buf += bytesWrite;

    if (fileSize != fread (p2buf, sizeof (ViByte), (size_t)fileSize, sourceFile))
        {
        free (exBuf);
        return VI_ERROR_INSTR_FILE_OPEN;
        }

    bytesWrite += fileSize;
    fclose (sourceFile);

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_INTF_TYPE, &remote_interface));

    /* Interface specific operations */
    switch (remote_interface)
        {
        case VI_INTF_ASRL: /* Serial Interface. */
            CHECKERR (viGetAttribute (instrSession, VI_ATTR_ASRL_END_OUT, &endOut));
            /* Switch to Off using of termination character. */
            if ((rsesci_status = viSetAttribute (instrSession, VI_ATTR_ASRL_END_OUT, VI_ASRL_END_NONE)) < 0)
                {
                /* If not NULL pointer, then deallocate memory. */
                if (exBuf) free (exBuf);
                return rsesci_status;
                }
            if ((rsesci_status =  viSetAttribute (instrSession, VI_ATTR_TERMCHAR_EN, VI_FALSE)) < 0)
                {
                /* If not NULL pointer, then deallocate memory. */
                if (exBuf) free (exBuf);
                return rsesci_status;
                }

            /* Write binary data into instrument. */
            if ((rsesci_status = viWrite (instrSession, (ViBuf) exBuf, bytesWrite, VI_NULL)) < 0)
                {
                viSetAttribute (instrSession, VI_ATTR_ASRL_END_OUT, endOut);
                viSetAttribute (instrSession, VI_ATTR_TERMCHAR_EN, VI_TRUE);
                /* If not NULL pointer, then deallocate memory. */
                if (exBuf) free (exBuf);
                return rsesci_status;
                }

            /* Switch to On using of termination character. */
            viSetAttribute (instrSession, VI_ATTR_ASRL_END_OUT, endOut);
            viSetAttribute (instrSession, VI_ATTR_TERMCHAR_EN, VI_TRUE);

            /* Terminate data input. */
            if ((rsesci_status = viPrintf (instrSession, "%s", "\n")) < 0)
                {
                /* If not NULL pointer, then deallocate memory. */
                if (exBuf) free (exBuf);
                return rsesci_status;
                }

            break;

        case VI_INTF_GPIB: default: /* GPIB Interface. */

            if ((rsesci_status = viWrite (instrSession,
                (ViBuf) ":SYST:COMM:GPIB:SELF:RTER EOI\n", 30, VI_NULL)) < 0)
                {
                /* If not NULL pointer, then deallocate memory. */
                free (exBuf);
                return rsesci_status;
                }
            if ((rsesci_status = viWrite (instrSession, (ViBuf) exBuf, bytesWrite, VI_NULL)) < 0)
                {
                /* If not NULL pointer, then deallocate memory. */
                free (exBuf);
                return rsesci_status;
                }
            if ((rsesci_status = viWrite (instrSession,
                (ViBuf) ":SYST:COMM:GPIB:SELF:RTER LFEOI\n", 32, VI_NULL)) < 0)
                {
                /* If not NULL pointer, then deallocate memory. */
                free (exBuf);
                return rsesci_status;
                }

            break;
        }

    if (exBuf) /* If not NULL pointer, then deallocate memory. */
        free (exBuf);

    CHECKERR( rsesci_checkStatus (instrSession));

    return rsesci_status;
}

/*****************************************************************************
 * Function: Store Trace Data
 * Purpose:  This function stores the selected trace (1 to 3) in the
 *           measurement window (screen A or B) in a file with ASCII format.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_StoreTraceData(
    ViSession   instrSession,
    ViInt32 trace,
    ViString    fileName
)
{
    ViChar      buffer[BUFFER_SIZE];

    if (rsesci_invalidViInt32Range (trace, 1, 3))
        return VI_ERROR_PARAMETER2;

    sprintf (buffer, "MMEM:STOR:TRAC %ld,'%s'", trace, fileName);

    return rsesci_sendCmd (instrSession, buffer);
}

/*****************************************************************************
 * Function: Store Final Measurement Data
 * Purpose:  This function stores the final measurement data in an ASCII file.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_StoreFinalMeasurementData(
    ViSession   instrSession,
    ViString    fileName
)
{
    return rsesci_setViStringQuoted (instrSession,
                                     "FORM ASC;:MMEM:STOR:FIN",
                                     "'",
                                     fileName,
                                     VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Set Decimal Point Notation
 * Purpose:  This function defines which decimal separator (decimal point or
 *           comma) is to be used for outputting measurement data to the file
 *           in ASCII format. Different languages of evaluation programs (e.g.
 *           MS-Excel) can thus be supported.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SetDecimalPointNotation(
    ViSession   instrSession,
    ViInt32     decimalSeparator
)
{
    return rsesci_setEnum (instrSession,
                           "FORM:DEXP:DSEP",
                           decimalSeparator,
                           decSeparatorArr,
                           VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Get Decimal Point Notation
 * Purpose:  This function returns which decimal separator (decimal point or
 *           comma) is to be used for outputting measurement data to the file
 *           in ASCII format.
 *           FORMat:DEXPort:DSEParator?
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_GetDecimalPointNotation(
    ViSession   instrSession,
    ViInt32*    decimalSeparator
)
{
    return rsesci_getEnum (instrSession,
                           "FORM:DEXP:DSEP?",
                           decimalSeparator,
                           decSeparatorArr,
                           VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Select Printer
 * Purpose:  This function selects one of the printers installed under
 *           Windows. Prior to use this function get list of available printers
 *           using function Get Printer List.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_SelectPrinter(
    ViSession   instrSession,
    ViInt32     deviceNumber,
    ViString    printerName
)
{
    ViChar buffer[BUFFER_SIZE];

    if (rsesci_invalidViInt32Range (deviceNumber, 1, 2))
        return VI_ERROR_PARAMETER2;
    if (rsesci_invalidViString (printerName, 128))
        return VI_ERROR_PARAMETER3;

    sprintf (buffer, "SYST:COMM:PRIN:SEL%ld", deviceNumber);

    return rsesci_setViStringQuoted (instrSession,
                                     buffer,
                                     "'",
                                     printerName,
                                     VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Query Printer List
 * Purpose:  This function queries the list of printers under Windows.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_QueryPrinterList(
    ViSession   instrSession,
    ViChar      _VI_FAR printerList[]
)
{
    ViUInt32    retCnt = 0;
    ViChar      buffer[BUFFER_SIZE];
    ViChar*     plist;
    ViStatus    rsesci_status = VI_SUCCESS;

    CHECKERR (rsesci_clearBeforeRead(instrSession));

    CHECKERR(viWrite (instrSession, (ViBuf) "*CLS;:SYST:COMM:PRIN:ENUM:FIRS?\n", 32, &retCnt));
    CHECKERR(rsesci_checkBeforeRead (instrSession));
    CHECKERR(viRead (instrSession, (ViBuf) buffer, BUFFER_SIZE, &retCnt));

    *printerList = '\0';
    if (sscanf (buffer, "'%[^']'", buffer) == 1)
    {
        plist = printerList;
        /* Get list of printers. */
        do
        {
            plist += sprintf (plist, "%s,", buffer);
            CHECKERR(viWrite (instrSession, (ViBuf) "*CLS;:SYST:COMM:PRIN:ENUM:NEXT?\n", 32, &retCnt));
            CHECKERR(rsesci_checkBeforeRead (instrSession));
            CHECKERR(viRead (instrSession, (ViBuf) buffer, BUFFER_SIZE, &retCnt));
        } while (sscanf (buffer, "'%[^']'", buffer) == 1);
        /* Remove remaining comma character */
        *--plist = '\0';
    }

    CHECKERR(rsesci_checkStatus (instrSession));

    return rsesci_status;
}

/*****************************************************************************
 * Function: Abort Hardcopy
 * Purpose:  This function aborts a running hardcopy output.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_AbortHardcopy(
    ViSession   instrSession
)
{
    return rsesci_sendCmd (instrSession, "HCOP:ABOR");
}

/*****************************************************************************
 * Function: Initiate Hardcopy
 * Purpose:  This function starts a hardcopy output. Device number parameter
 *           selects which printer configuration (1 or 2) is to be used for the
 *           hardcopy output.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_InitiateHardcopy(
    ViSession   instrSession,
    ViInt32 deviceNumber
)
{
    ViChar buffer[BUFFER_SIZE];

    sprintf (buffer, "HCOP:IMM%ld", deviceNumber);

    return rsesci_sendCmd (instrSession, buffer);
}

/*****************************************************************************
 * Function: Hardcopy Device Setup
 * Purpose:  This function selects the interface and configuration for the
 *           hardcopy operation. Destination and data format is also
 *           configured.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_HardcopyDeviceSetup(
    ViSession   instrSession,
    ViInt32     deviceNumber,
    ViInt32     destination,
    ViInt32     dataFormat,
    ViInt32     pageOrientation,
    ViString    fileName
)
{
    ViStatus    rsesci_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];

    if (rsesci_invalidViInt32Range (deviceNumber, 1, 2))
        return VI_ERROR_PARAMETER2;
    if (rsesci_invalidViInt32Range (destination, 0, 2))
        return VI_ERROR_PARAMETER3;
    if (rsesci_invalidViInt32Range (dataFormat, 0, 3))
        return VI_ERROR_PARAMETER4;
    if (rsesci_invalidViInt32Range (pageOrientation, 0, 1))
        return VI_ERROR_PARAMETER5;
    if (!strlen(fileName))
        return VI_ERROR_PARAMETER6;

    switch (destination)
        {
        case RSESCI_HARDCOPY_DISK:
            sprintf (buffer, ":HCOP:DEST%ld '%s';:MMEM:NAME '%s';:HCOP:DEV:LANG%ld %s",
                     deviceNumber,
                     destinationArr[destination],
                     fileName,
                     deviceNumber,
                     dataFormatArr[dataFormat]);
            break;
        case RSESCI_HARDCOPY_PRINTER:
            sprintf (buffer, ":HCOP:DEST%ld '%s';:HCOP:DEV:LANG%ld %s;:HCOP:PAGE:ORI%ld %s",
                     deviceNumber,
                     destinationArr[destination],
                     deviceNumber,
                     dataFormatArr[dataFormat],
                     deviceNumber,
                     pageOrientationArr[pageOrientation]);
            break;
        case RSESCI_HARDCOPY_CLIPBOARD:
            sprintf (buffer, ":HCOP:DEST%ld '%s';:HCOP:DEV:LANG%ld %s",
                     deviceNumber,
                     destinationArr[destination],
                     deviceNumber,
                     dataFormatArr[dataFormat]);
            break;
        default:
            return VI_ERROR_PARAMETER3;
        }

    CHECKERR (rsesci_sendCmd (instrSession, buffer));

    return rsesci_status;
}

/*****************************************************************************
 * Function: Hardcopy Comment
 * Purpose:  Defines the comment text for measurement window 1 or 2 for
 *           printout.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_HardcopyComment(
    ViSession   instrSession,
    ViInt32     window,
    ViString    comment
)
{
    ViChar  buffer[BUFFER_SIZE];

    CHECKWINDOW;

    sprintf (buffer, "HCOP:ITEM:WIND%ld:TEXT", window);

    return rsesci_setViStringQuoted (instrSession,
                                     buffer,
                                     "'",
                                     comment,
                                     VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Hardcopy Items
 * Purpose:  This hardcopy output is always provided with comments, title,
 *           time and date. As an alternative to the whole screen, only traces
 *           (commands 'HCOPy:ITEM:WINDow:TRACe: STATe ON') or tables (command
 *           'HCOPy:ITEM:WINDow:TABLe:STATe ON') can be output.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_HardcopyItems(
    ViSession   instrSession,
    ViInt32     window,
    ViInt32     hardcopyItems
)
{
    ViStatus rsesci_status = VI_SUCCESS;

    switch (hardcopyItems)
        {
        case RSESCI_HARDCOPY_ALL:
            return rsesci_sendCmd (instrSession, "HCOP:ITEM:ALL");
        case RSESCI_HARDCOPY_TABLE:
            CHECKERR (rsesci_setViBooleanWindow (instrSession,
                                                 "HCOP:ITEM:WIND%ld:TABL:STAT",
                                                 window,
                                                 VI_TRUE,
                                                 VI_ERROR_PARAMETER2));
            CHECKERR (rsesci_setViBooleanWindow (instrSession,
                                                 "HCOP:ITEM:WIND%ld:TRAC:STAT",
                                                 window,
                                                 VI_FALSE,
                                                 VI_ERROR_PARAMETER2));
            break;
        case RSESCI_HARDCOPY_TRACE:
            CHECKERR (rsesci_setViBooleanWindow (instrSession,
                                                 "HCOP:ITEM:WIND%ld:TABL:STAT",
                                                 window,
                                                 VI_FALSE,
                                                 VI_ERROR_PARAMETER2));
            CHECKERR (rsesci_setViBooleanWindow (instrSession,
                                                 "HCOP:ITEM:WIND%ld:TRAC:STAT",
                                                 window,
                                                 VI_TRUE,
                                                 VI_ERROR_PARAMETER2));
            break;
        default:
            return VI_ERROR_PARAMETER3;
        }

    return rsesci_status;
}

/*****************************************************************************
 * Function: Hardcopy Color
 * Purpose:  This function switches over from colour output to black-and-white
 *           output. All colour-highlighted areas are printed in white and all
 *           colour lines in black. This improves the contrast on the printout.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_HardcopyColor(
    ViSession   instrSession,
    ViBoolean   color
)
{
    return rsesci_setViBoolean (instrSession,
                                "HCOP:DEV:COL",
                                color,
                                VI_ERROR_PARAMETER2);
}

/*****************************************************************************
 * Function: Hardcopy User Defined Colors
 * Purpose:  This function allows selection of colors for the printouts of the
 *           screen contents.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_HardcopyUserDefinedColors(
    ViSession   instrSession,
    ViInt32     colorMap,
    ViInt32     predefinedColors,
    ViReal64    tint,
    ViReal64    saturation,
    ViReal64    brightness
)
{
    ViChar  buffer[BUFFER_SIZE];

    if (rsesci_invalidViInt32Range (colorMap, 1, 34))
        return VI_ERROR_PARAMETER2;
    if (rsesci_invalidViInt32Range (predefinedColors, -1, 15))
        return VI_ERROR_PARAMETER3;

    if (predefinedColors == RSESCI_USER_DEFINED)
        {
        if (rsesci_invalidViReal64Range (tint, 0.0, 100.0))
            return VI_ERROR_PARAMETER4;
        if (rsesci_invalidViReal64Range (saturation, 0.0, 100.0))
            return VI_ERROR_PARAMETER5;
        if (rsesci_invalidViReal64Range (brightness, 0.0, 100.0))
            return VI_ERROR_PARAMETER6;

        sprintf (buffer, "HCOP:CMAP%ld:HSL %lf,%lf,%lf",
                 colorMap,
                 tint/100,
                 saturation/100,
                 brightness/100);
        }
    else
        sprintf (buffer, "HCOP:CMAP%ld:PDEF %s",
                 colorMap, predefinedColorsArr[predefinedColors]);

    return rsesci_sendCmd (instrSession, buffer);
}

/*****************************************************************************
 * Function: Hardcopy Default Colors
 * Purpose:  This function resets the colors for a hardcopy to the selected
 *           default settings.
 *****************************************************************************/
ViStatus _VI_FUNC rsesci_HardcopyDefaultColors(
    ViSession   instrSession,
    ViInt32     defaultColors
)
{
    ViChar  buffer[BUFFER_SIZE];

    if (rsesci_invalidViInt32Range (defaultColors, 0, 2))
        return VI_ERROR_PARAMETER2;

    sprintf (buffer, "HCOP:CMAP:DEF%ld", defaultColors + 1);

    return rsesci_sendCmd (instrSession, buffer);
}

/*===========================================================================*/
/* Function: Initiate Hardcopy To File                                       */
/* Purpose:  This function starts stores a hardcopy output for selected      */
/*           device to file on local disk.                                   */
/*===========================================================================*/
ViStatus _VI_FUNC rsesci_InitiateHardcopyToFile (ViSession io,
                                         ViInt32 deviceNumber,
                                         ViString destination,
                                         ViInt32 dataFormat,
                                         ViInt32 pageOrientation)
{
    ViStatus    rsesci_status;                                                  
    
    CHECKERR (rsesci_FileManager (io, 0, "D:", ""));
    CHECKERR (rsesci_FileManager (io, 1, "\\user\\data", ""));  
    CHECKERR (rsesci_HardcopyDeviceSetup (io, deviceNumber, 0, dataFormat, pageOrientation,"_drvplot.hcp"));
    CHECKERR (rsesci_InitiateHardcopy (io, deviceNumber));
    CHECKERR (rsesci_readToFile (io, "_drvplot.hcp", destination));
    CHECKERR (rsesci_FileManager (io, 6, "_drvplot.hcp", ""));
    
    return rsesci_status;
}

/*===========================================================================*/
/* Function: Set Status Register
/* Purpose:  This Function sets the Enable, NTransition, and PTransition bits
/*           of the Main, Power, Limit, Left Margin, and Sync status
/*           questionable registers.
/*===========================================================================*/
ViStatus _VI_FUNC rsesci_setStatusRegister (ViSession instrSession,
                                            ViInt32 registerOperation,
                                            ViInt32 questionableRegister,
                                            ViInt32 enable, ViInt32 PTransition,
                                            ViInt32 NTransition)
{
    ViChar      buffer [BUFFER_SIZE];

    if (rsesci_invalidViInt32Range (registerOperation, 0, 2))
        return VI_ERROR_PARAMETER2;

    switch (registerOperation) {
        case 0:
            strcpy (buffer,":STAT:PRES");
        break;
        case 1:
            if (rsesci_invalidViInt32Range (questionableRegister, 0, 9))
                return VI_ERROR_PARAMETER3;
            if (rsesci_invalidViInt32Range (enable, 0, 65535))
                return VI_ERROR_PARAMETER4;
            if (rsesci_invalidViInt32Range (PTransition, 0, 65535))
                return VI_ERROR_PARAMETER5;
            if (rsesci_invalidViInt32Range (NTransition, 0, 65535))
                return VI_ERROR_PARAMETER6;

            sprintf (buffer,":STAT:QUES%s:ENAB %ld;PTR %ld;NTR %ld",
                            selStatusRegArr[questionableRegister], enable, PTransition,
                            NTransition);
        break;
        case 2:
            if (rsesci_invalidViInt32Range (enable, 0, 65535))
                return VI_ERROR_PARAMETER4;
            if (rsesci_invalidViInt32Range (PTransition, 0, 65535))
                return VI_ERROR_PARAMETER5;
            if (rsesci_invalidViInt32Range (NTransition, 0, 65535))
                return VI_ERROR_PARAMETER6;

            sprintf (buffer, ":STAT:OPER:ENAB %ld;PTR %ld;NTR %ld\n",
                     enable, PTransition, NTransition);
        break;
    }

    return rsesci_sendCmd (instrSession, buffer);
}

/*===========================================================================*/
/* Function: Get Status Register
/* Purpose:  This function contains the commands for the querying status
/*           reporting system.
/*===========================================================================*/
ViStatus _VI_FUNC rsesci_getStatusRegister (ViSession instrSession,
                                            ViInt32 statusRegistersQuery,
                                            ViInt32 *registerValue)
{
    ViChar buffer [BUFFER_SIZE];

    if (rsesci_invalidViInt32Range (statusRegistersQuery, 0, 22))
        return VI_ERROR_PARAMETER2;

    sprintf (buffer, ":STAT:%s?", statusRegArr[statusRegistersQuery]);

    return rsesci_getViInt32 (instrSession,
                              buffer,
                              registerValue,
                              VI_ERROR_PARAMETER3);
}

/*===========================================================================*/
/* Function: State Checking                                                  */
/* Purpose:  This function switches ON/OFF state checking of the instrument  */
/*           (reading of the Standard Event Register and checking it for     */
/*           error). Nearly all driver function are using state checking.    */
/*           Switch this function to ON when debug your application. For     */
/*           better bus throughput and instruments performance switch it OFF.*/
/*===========================================================================*/
ViStatus _VI_FUNC rsesci_errorCheckState (ViSession instrSession,
                                          ViBoolean state)
{
    ViStatus    rsesci_status = VI_SUCCESS;
    rsesci_instrRange instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    instrPtr -> errorChecking = state;

    return rsesci_status;
}

/*===========================================================================*/
/* Function: Write To Instrument                                             */
/* Purpose:  This function writes a command string to the instrument.        */
/*===========================================================================*/
ViStatus _VI_FUNC rsesci_writeInstrData (ViSession instrSession, ViString writeBuffer)
{
    ViStatus rsesci_status = VI_SUCCESS;

    CHECKERR (viPrintf (instrSession, "%s\n", writeBuffer));

    return rsesci_status;
}

/*===========================================================================*/
/* Function: Read Instrument Buffer                                          */
/* Purpose:  This function reads the output buffer of the instrument.        */
/*===========================================================================*/
ViStatus _VI_FUNC rsesci_readInstrData (ViSession instrSession,
                                        ViInt32 numberBytesToRead,
                                        ViChar _VI_FAR readBuffer[],
                                        ViPInt32 numBytesRead)
{
    ViStatus    rsesci_status = VI_SUCCESS;
    ViUInt32    tmpCnt;

    if (numBytesRead)
        *numBytesRead = 0L;

    CHECKERR (viRead (instrSession, (ViBuf)readBuffer, numberBytesToRead, &tmpCnt));

    if (numBytesRead)
        *numBytesRead = (ViInt32)tmpCnt;

    return rsesci_status;
}

/*===========================================================================*/
/* Function: Reset                                                           */
/* Purpose:  This function resets the instrument.  If the reset function     */
/*           is not supported by the instrument, this function returns       */
/*           the warning VI_WARN_NSUP_RESET.                                 */
/*===========================================================================*/
ViStatus _VI_FUNC rsesci_reset (ViSession instrSession)
{
    ViStatus rsesci_status = VI_SUCCESS;

    /*  Initialize the instrument to a known state.  */
    CHECKERR (viPrintf (instrSession, "*RST\n"));
    CHECKERR (rsesci_defaultInstrSetup (instrSession));

    return rsesci_status;
}

/*===========================================================================*/
/* Function: Self-Test                                                       */
/* Purpose:  This function executes the instrument self-test and returns     */
/*           the result. If the self test function is not supported by the   */
/*           instrument, this function returns the warning                   */
/*           VI_WARN_NSUP_SELF_TEST.                                         */
/*===========================================================================*/
ViStatus _VI_FUNC rsesci_self_test (ViSession instrSession,
                                    ViPInt16 testResult,
                                    ViChar _VI_FAR testMessage[])
{
    ViStatus rsesci_status = VI_SUCCESS;

    CHECKERR(rsesci_clearBeforeRead (instrSession));
    CHECKERR (viPrintf (instrSession, "*TST?\n"));
    CHECKERR (rsesci_checkBeforeRead (instrSession));
    CHECKERR (viScanf (instrSession, "%hd", testResult));
    
    sprintf (testMessage, "%s", (*testResult == 0) ? "Passed" : "Failed");
    
    return rsesci_status;
}

/*===========================================================================*/
/* Function: Error Query                                                     */
/* Purpose:  This function queries the instrument error queue, and returns   */
/*           the result. If the error query function is not supported by the */
/*           instrument, this function returns the warning                   */
/*           VI_WARN_NSUP_ERROR_QUERY.                                       */
/*===========================================================================*/
ViStatus _VI_FUNC rsesci_error_query (ViSession instrSession,
                                      ViPInt32 errorCode,
                                      ViChar _VI_FAR errorMessage[])
{
    ViStatus rsesci_status = VI_SUCCESS;

    CHECKERR (viPrintf (instrSession, ":SYST:ERR?\n"));
    CHECKERR (viScanf (instrSession, "%ld,\"%[^\"]", errorCode, errorMessage));

    return rsesci_status;
}

/*===========================================================================*/
/* Function: Error Message                                                   */
/* Purpose:  This function translates the error return value from the        */
/*           instrument driver into a user-readable string.                  */
/*===========================================================================*/
ViStatus _VI_FUNC rsesci_error_message (ViSession instrSession,
                                        ViStatus statusCode,
                                        ViChar _VI_FAR errMessage[])
{
    ViStatus rsesci_status = VI_SUCCESS;
    ViInt16 i;

    static rsesci_tStringValPair statusDescArray[] = {
        {VI_WARN_NSUP_ID_QUERY,     "WARNING: ID Query not supported"},
        {VI_WARN_NSUP_RESET,        "WARNING: Reset not supported"},
        {VI_WARN_NSUP_SELF_TEST,    "WARNING: Self-test not supported"},
        {VI_WARN_NSUP_ERROR_QUERY,  "WARNING: Error Query not supported"},
        {VI_WARN_NSUP_REV_QUERY,    "WARNING: Revision Query not supported"},
        {VI_ERROR_PARAMETER1,   "ERROR: Parameter 1 out of range"},
        {VI_ERROR_PARAMETER2,   "ERROR: Parameter 2 out of range"},
        {VI_ERROR_PARAMETER3,   "ERROR: Parameter 3 out of range"},
        {VI_ERROR_PARAMETER4,   "ERROR: Parameter 4 out of range"},
        {VI_ERROR_PARAMETER5,   "ERROR: Parameter 5 out of range"},
        {VI_ERROR_PARAMETER6,   "ERROR: Parameter 6 out of range"},
        {VI_ERROR_PARAMETER7,   "ERROR: Parameter 7 out of range"},
        {VI_ERROR_PARAMETER8,   "ERROR: Parameter 8 out of range"},
        {VI_ERROR_FAIL_ID_QUERY,"ERROR: Identification query failed"},
        {VI_ERROR_INV_RESPONSE, "ERROR: Interpreting instrument response"},
        {VI_ERROR_INSTR_FILE_OPEN,    "ERROR: Opening the specified file"},
        {VI_ERROR_INSTR_FILE_WRITE,   "ERROR: Writing to the specified file"},
        {VI_ERROR_INSTR_INTERPRETING_RESPONSE, "ERROR: Interpreting the instrument's response"},
        {VI_ERROR_INSTR_PARAMETER9 ,  "ERROR: Parameter 9 out of range"},
        {VI_ERROR_INSTR_PARAMETER10,  "ERROR: Parameter 10 out of range"},
        {VI_ERROR_INSTR_PARAMETER11,  "ERROR: Parameter 11 out of range"},
        {VI_ERROR_INSTR_PARAMETER12,  "ERROR: Parameter 12 out of range"},
        {VI_ERROR_INSTR_PARAMETER13,  "ERROR: Parameter 13 out of range"},
        {VI_ERROR_INSTR_PARAMETER14,  "ERROR: Parameter 14 out of range"},
        {VI_ERROR_INSTR_PARAMETER15,  "ERROR: Parameter 15 out of range"},
		{VI_ERROR_INSTR_PARAMETER16,  "ERROR: Parameter 16 out of range"},

        {RSESCI_ERROR_INSTRUMENT_ERROR,         "ERROR: Instrument status error"},
        {RSESCI_ERROR_INVALID_CONFIGURATION,    "ERROR: Instrument configuration error"},
        {RSESCI_ERROR_INSTRUMENT_OPTION,        "ERROR: Option not installed"},
        {RSESCI_ERROR_INSTRUMENT_DATA_NOT_AVAILABLE, "ERROR: Data are not available."},
        {RSESCI_ERROR_SETTINGS_CONFLICT,        "ERROR: Instrument settings conflict."},
        {RSESCI_WARNING_SMALL_BUFFER,           "WARNING: Buffer too small"},
        {RSESCI_ERROR_INSTRUMENT_NSUP_MODEL,    "ERROR: Required Instrument model is not available."},
        {RSESCI_ERROR_INSTRUMENT_IQ_OVERLOAD,   "ERROR: Overload. Instrument does not return I/Q values."},

        {VI_NULL, VI_NULL}
    };

    rsesci_status = viStatusDesc (instrSession, statusCode, errMessage);
    if (rsesci_status == VI_WARN_UNKNOWN_STATUS)
        {
        for (i = 0; statusDescArray[i].stringName; i++)
            {
            if (statusDescArray[i].stringVal == statusCode)
                {
                sprintf (errMessage, "%s", statusDescArray[i].stringName);
                return (VI_SUCCESS);
                }
            }
        sprintf (errMessage, "Unknown Error 0x%X", statusCode);
        return (VI_WARN_UNKNOWN_STATUS);
        }

    rsesci_status = VI_SUCCESS;
    return rsesci_status;
}

/*===========================================================================*/
/* Function: Revision Query                                                  */
/* Purpose:  This function returns the driver and instrument revisions.      */
/*           If the revision query function is not supported by the          */
/*           instrument, this function returns the warning                   */
/*           VI_WARN_NSUP_REV_QUERY.                                         */
/*===========================================================================*/
ViStatus _VI_FUNC rsesci_revision_query (ViSession instrSession,
                    ViChar _VI_FAR driverRev[], ViChar _VI_FAR instrRev[])
{
    ViStatus rsesci_status = VI_SUCCESS;

    CHECKERR(rsesci_clearBeforeRead (instrSession));
    CHECKERR (viPrintf (instrSession, "*IDN?\n"));
    CHECKERR (rsesci_checkBeforeRead (instrSession));
    CHECKERR (viScanf (instrSession, "%*[^,],%*[^,],%*[^,],%[^\r\n]", instrRev));

    sprintf (driverRev, "%s", RSESCI_REVISION);

    return rsesci_status;
}

/*===========================================================================*/
/* Function: Close                                                           */
/* Purpose:  This function closes the instrument.                            */
/*===========================================================================*/
ViStatus _VI_FUNC rsesci_close (ViSession instrSession)
{
    rsesci_instrRange instrPtr;
    ViSession rmSession;
    ViUInt16 intfc = VI_INTF_GPIB;
    ViStatus rsesci_status = VI_SUCCESS;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_RM_SESSION, &rmSession));
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_INTF_TYPE, &intfc));

    /* Interface specific operations */
    switch (intfc)
        {
        case VI_INTF_ASRL: /* Serial Port. Go to local mode. */
            CHECKERR (viWrite (instrSession, (ViBuf) "@LOC\n", 5, VI_NULL));
            break;

/* 
#ifdef VI_INTF_TCPIP

        case VI_INTF_TCPIP: RSIB Interface. Go to local mode. 
            CHECKERR (viSetAttribute (instrSession, RS_ATTR_RSIB_SRE, 0));
            break;

#endif
*/			
        }

    if (instrPtr != NULL)
        free (instrPtr);

    rsesci_status = viClose (instrSession);
    viClose (rmSession);

    return rsesci_status;
}

/*****************************************************************************/
/*= UTILITY ROUTINES (Non-Exportable Functions) =============================*/
/*****************************************************************************/

/*===========================================================================*/
/* Function: Delay
/* Purpose : With a call to rsesci_delay, the current program is suspended
/*           from execution for the number of seconds specified by the
/*           argument seconds. rsesci_delay is accurate to a millisecond.
/*           This function is platform independent.
/*===========================================================================*/
void rsesci_delay (ViReal64 numberOfSeconds)
{
  clock_t StartTime = clock();

  while ((((double)(clock() - StartTime))/CLOCKS_PER_SEC) < (numberOfSeconds));

  return;
}

/*===========================================================================*/
/* Function: Array Item to Const                                             */
/* Purpose:  Search for item in array and return index. Last element of array*/
/*           must be NULL pointer.                                           */
/*===========================================================================*/
ViStatus rsesci_itemToConst (ViChar **array, ViString buffer, ViPInt32 itemNo)
{
    for ( *itemNo = 0; array[*itemNo]; (*itemNo)++ )
        if (!strcmp (buffer, array[*itemNo]))
            break;

    return (!array[*itemNo]) ? VI_ERROR_INSTR_INTERPRETING_RESPONSE : VI_SUCCESS;
}

/*===========================================================================*/
/* Function: Boolean Value Out Of Range - ViBoolean                          */
/* Purpose:  This function checks a Boolean to see if it is equal to VI_TRUE */
/*           or VI_FALSE. If the value is out of range, the return value is  */
/*           VI_TRUE, otherwise the return value is VI_FALSE.                */
/*===========================================================================*/
ViBoolean rsesci_invalidViBooleanRange (ViBoolean val)
{
    return (ViBoolean)((val != VI_FALSE && val != VI_TRUE) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Long Signed Integer Value Out Of Range - ViInt32                */
/* Purpose:  This function checks a long signed integer value to see if it   */
/*           lies between a minimum and maximum value.  If the value is out  */
/*           of range, the return value is VI_TRUE, otherwise the return     */
/*           value is VI_FALSE.                                              */
/*===========================================================================*/
ViBoolean rsesci_invalidViInt32Range (ViInt32 val, ViInt32 min, ViInt32 max)
{
    return (ViBoolean)((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Long Unsigned Integer Value Out Of Range - ViUInt32             */
/* Purpose:  This function checks a long signed integer value to see if it   */
/*           lies between a minimum and maximum value.  If the value is out  */
/*           of range, the return value is VI_TRUE, otherwise the return     */
/*           value is VI_FALSE.                                              */
/*===========================================================================*/
ViBoolean rsesci_invalidViUInt32Range (ViUInt32 val, ViUInt32 min, ViUInt32 max)
{
    return (ViBoolean)((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Real (Double) Value Out Of Range - ViReal64                     */
/* Purpose:  This function checks a real (double) value to see if it lies    */
/*           between a minimum and maximum value.  If the value is out of    */
/*           range, the return value is VI_TRUE, otherwise the return value  */
/*           is VI_FALSE.                                                    */
/*===========================================================================*/
ViBoolean rsesci_invalidViReal64Range (ViReal64 val, ViReal64 min, ViReal64 max)
{
    return (ViBoolean)((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Check String                                                    */
/* Purpose:  This function checks a string value to see if it is             */
/*           of a defined length and initialized.                            */
/*===========================================================================*/
ViBoolean rsesci_invalidViString (ViString val, ViUInt32 maxLength)
{
    if (val)
        return (ViBoolean)((strlen (val) > maxLength) ? VI_TRUE : VI_FALSE);

    return VI_TRUE;
}

/*===========================================================================*/
/* Function: Wait for OPC                                                    */
/* Purpose:  This function sends a command to the instrument and waits until */
/*           the instrument response is available (queries MAV in STB).      */
/*===========================================================================*/
ViStatus rsesci_waitOPC (ViSession instrSession, ViPString command,
                ViPString instrResp, ViInt32 timeout)
{
    ViStatus rsesci_status = VI_SUCCESS;
    ViUInt16 response = 0;
    ViInt32 counter = 0;
    ViInt32 oldTimeout = 0;
    ViUInt16 intfc = VI_INTF_GPIB;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_INTF_TYPE, &intfc));

    /* Interface specific operations */
    switch (intfc)
        {
        case VI_INTF_ASRL: /* Serial interface */
            CHECKERR (viGetAttribute (instrSession, VI_ATTR_TMO_VALUE, &oldTimeout));
            CHECKERR (viSetAttribute (instrSession, VI_ATTR_TMO_VALUE, timeout));
             /* "\n" is appended in the passed buffer, do not include it here */
            if ((rsesci_status = viQueryf (instrSession, "%s", "%[^\r\n]", command, instrResp)) < 0)
                {
                viSetAttribute (instrSession, VI_ATTR_TMO_VALUE, oldTimeout);
                return rsesci_status;
                }
            CHECKERR (viSetAttribute (instrSession, VI_ATTR_TMO_VALUE, oldTimeout));
        break;
        default: /* Default case */

            /* "\n" is appended in the passed buffer, do not include it here */
            CHECKERR (viPrintf (instrSession, "%s", command));

            while (!(response & 0x10)) /* (queries MAV in STB) */
                {

                CHECKERR (viReadSTB (instrSession, &response));

                rsesci_delay (0.005);
                counter+= 5;
                if (counter >= timeout)
                    return VI_ERROR_TMO;
                }

            CHECKERR (viScanf (instrSession, "%[^\r\n]", instrResp));
        break;
        }

    return rsesci_status;
}

/*===========================================================================*/
/* Function: Initialize Clean Up                                             */
/* Purpose:  This function is used only by the rsesci_init function.  When   */
/*           an error is detected this function is called to close the       */
/*           open resource manager and instrument object sessions and to     */
/*           set the instrSession that is returned from rsesci_init to       */
/*           VI_NULL.                                                        */
/*===========================================================================*/
ViStatus rsesci_initCleanUp (ViSession openRMSession,
                             ViPSession openInstrSession,
                             ViStatus currentStatus)
{
    rsesci_instrRange instrPtr;

    if (viGetAttribute (*openInstrSession, VI_ATTR_USER_DATA, &instrPtr) >= 0)
        if (instrPtr != NULL)
            free (instrPtr);

    viClose (*openInstrSession);
    viClose (openRMSession);
    *openInstrSession = VI_NULL;

    return currentStatus;
}

/*****************************************************************************/
/*----------- INSERT INSTRUMENT-DEPENDENT UTILITY ROUTINES HERE -------------*/
/*****************************************************************************/

/*===========================================================================*/
/* Function: Read Data With Unknown length                                   */
/* Purpose:  This function allocates buffer and reads data from the          */
/*           instrument                                                      */
/* WARNING:  You must deallocate buffer, if any pointer has been returned.   */
/*           This function does not work correctly, when you read bin data   */
/*           via serial interface                                            */
/*===========================================================================*/
ViStatus rsesci_readDataUnknownLength(ViSession instrSession,
                                     ViChar **outBuffer,
                                     ViUInt32 *count)
{
    ViStatus rsesci_status = VI_SUCCESS;
    ViUInt32 actSize, retCnt = 0;
    ViChar   *buffer = VI_NULL, *p2buffer;

    *outBuffer = VI_NULL;
    if (count) *count = 0;

    /* Allocate Buffer */
    actSize = BUFFER_SIZE;
    buffer = (ViChar *)calloc(actSize, sizeof(ViChar));
    if (buffer == VI_NULL) return VI_ERROR_ALLOC;

    p2buffer = buffer;
    do
        {
        if ((rsesci_status = viRead (instrSession, (ViPBuf)p2buffer, BUFFER_SIZE, &retCnt)) < 0)
            {
            free(buffer);
            *outBuffer = VI_NULL;
            return rsesci_status;
            }
        if (retCnt == BUFFER_SIZE) /* Buffer was small, reallocate it */
        {
            actSize += BUFFER_SIZE;
            buffer = (ViChar *)realloc(buffer, actSize);
            if (buffer == VI_NULL) return VI_ERROR_ALLOC;
            p2buffer = buffer + (actSize - BUFFER_SIZE); /* Set pointer to end of data in rallocated buffer */
        }
        else
        {
            *(p2buffer+retCnt) = '\0'; /* For sure set end of string at the end of data */
        }
       } while (rsesci_status == VI_SUCCESS_MAX_CNT); /* if buffer was small, continue with next read */

    *outBuffer = buffer;
    if (count) *count = (actSize - BUFFER_SIZE) + retCnt;

    return rsesci_status;
}

/*===========================================================================*/
/* Function: Send Command                                                    */
/* Purpose:  This function sends command to the instrument with error check. */
/*===========================================================================*/
ViStatus rsesci_sendCmd (ViSession instrSession, ViString cmd)
{
    ViStatus    rsesci_status    = VI_SUCCESS;
    ViUInt32     length          = 0;
    ViChar      *pbuffer;

    length = (ViUInt32) strlen (cmd);
    pbuffer = (ViChar*) malloc (length + 1);
    memcpy (pbuffer, cmd, length);
    pbuffer[length] = '\n';

    GOCHECKERR (viWrite (instrSession, (ViPBuf) cmd, length + 1, VI_NULL));
    GOCHECKERR (rsesci_checkStatus (instrSession));

ERROR:

    free(pbuffer);

    return rsesci_status;
}

/*===========================================================================*/
/* Function: Send Command With Window                                          */
/* Purpose:  This function sends command to the instrument with error check. */
/*===========================================================================*/
ViStatus rsesci_sendCmdWindow (ViSession instrSession, ViString cmd, ViInt32     window)
{
    ViStatus    rsesci_status    = VI_SUCCESS;
    ViUInt32     length          = 0;
    ViChar      *pbuffer;

    length = (ViUInt32) strlen (cmd);
    pbuffer = (ViChar*) malloc (length + 2);
    memcpy (pbuffer, cmd, length);
    pbuffer[length] = '\n';
    pbuffer[length + 1] = '\0';

    GOCHECKERR (viPrintf (instrSession, pbuffer, window));
    GOCHECKERR (rsesci_checkStatus (instrSession));

ERROR:

    free(pbuffer);

    return rsesci_status;
}

/*===========================================================================*/
/* Function: Set ViInt32                                                     */
/* Purpose:  This function sets command with integer parameter.              */
/*===========================================================================*/
ViStatus rsesci_setViInt32 (ViSession instrSession,
                           ViString cmd,
                           ViInt32 value,
                           ViInt32 min,
                           ViInt32 max,
                           ViStatus parameterErrorCode)
{
    ViChar      wrt_buf[BUFFER_SIZE];

    if ((max - min) > 0)
        if (rsesci_invalidViInt32Range (value, min, max))
            return parameterErrorCode;

    sprintf (wrt_buf, "%s %ld", cmd, value);

    return rsesci_sendCmd (instrSession, wrt_buf);
}

/*===========================================================================*/
/* Function: Get ViInt32                                                     */
/* Purpose:  This function gets command with integer parameter.              */
/*===========================================================================*/
ViStatus rsesci_getViInt32 (ViSession instrSession,
                           ViString cmd,
                           ViInt32 *value,
                           ViStatus parameterErrorCode)
{
    ViStatus    rsesci_status = VI_SUCCESS;

    if (value == VI_NULL)
        return parameterErrorCode;

    CHECKERR(rsesci_clearBeforeRead (instrSession));
    CHECKERR( viPrintf(instrSession, "%s\n", cmd));
    CHECKERR (rsesci_checkBeforeRead (instrSession));
    CHECKERR( viScanf(instrSession, "%ld", value));
    CHECKERR( rsesci_checkStatus (instrSession));

    return rsesci_status;
}

/*===========================================================================*/
/* Function: Set ViInt32 With Window                                         */
/* Purpose:  This function sets command with integer parameter.              */
/*===========================================================================*/
ViStatus rsesci_setViInt32Window (ViSession instrSession,
                           ViString cmd,
                           ViInt32     window,
                           ViInt32 value,
                           ViInt32 min,
                           ViInt32 max,
                           ViStatus parameterErrorCode)
{
    ViChar      wrt_buf[BUFFER_SIZE];

    CHECKWINDOW;

    sprintf (wrt_buf, cmd, window);

    return rsesci_setViInt32(instrSession,
                             wrt_buf,
                             value,
                             min,
                             max,
                             parameterErrorCode);
}

/*===========================================================================*/
/* Function: Get ViInt32 With Window                                         */
/* Purpose:  This function gets command with integer parameter.              */
/*===========================================================================*/
ViStatus rsesci_getViInt32Window (ViSession instrSession,
                           ViString cmd,
                           ViInt32     window,
                           ViInt32 *value,
                           ViStatus parameterErrorCode)
{
    ViChar      wrt_buf[BUFFER_SIZE]="";

    CHECKWINDOW;

    sprintf(wrt_buf, cmd, window);

    return rsesci_getViInt32(instrSession,
                            wrt_buf,
                            value,
                            parameterErrorCode);
}

/*===========================================================================*/
/* Function: Set ViUInt32                                                    */
/* Purpose:  This function sets command with unsigned integer parameter.     */
/*===========================================================================*/
ViStatus rsesci_setViUInt32 (ViSession instrSession,
                           ViString cmd,
                           ViUInt32 value,
                           ViUInt32 min,
                           ViUInt32 max,
                           ViStatus parameterErrorCode)
{
    ViChar      wrt_buf[BUFFER_SIZE];

    if ((max - min) > 0)
        if (rsesci_invalidViUInt32Range (value, min, max))
            return parameterErrorCode;

    sprintf (wrt_buf, "%s %u", cmd, value);

    return rsesci_sendCmd (instrSession, wrt_buf);
}

/*===========================================================================*/
/* Function: Get ViUInt32                                                    */
/* Purpose:  This function gets command with unsigned integer parameter.     */
/*===========================================================================*/
ViStatus rsesci_getViUInt32 (ViSession instrSession,
                           ViString cmd,
                           ViUInt32 *value,
                           ViStatus parameterErrorCode)
{
    ViStatus    rsesci_status = VI_SUCCESS;

    if (value == VI_NULL)
        return parameterErrorCode;

    CHECKERR(rsesci_clearBeforeRead (instrSession));
    CHECKERR( viPrintf(instrSession, "%s\n", cmd));
    CHECKERR (rsesci_checkBeforeRead (instrSession));
    CHECKERR( viScanf(instrSession, "%u", value));
    CHECKERR( rsesci_checkStatus (instrSession));

    return rsesci_status;
}

/*===========================================================================*/
/* Function: Set ViUInt32 With Window                                        */
/* Purpose:  This function sets command with unsigned integer parameter.     */
/*===========================================================================*/
ViStatus rsesci_setViUInt32Window (ViSession instrSession,
                           ViString cmd,
                           ViInt32     window,
                           ViUInt32 value,
                           ViUInt32 min,
                           ViUInt32 max,
                           ViStatus parameterErrorCode)
{
    ViChar      wrt_buf[BUFFER_SIZE];

    CHECKWINDOW;

    sprintf (wrt_buf, cmd, window);

    return rsesci_setViUInt32(instrSession,
                             wrt_buf,
                             value,
                             min,
                             max,
                             parameterErrorCode);
}

/*===========================================================================*/
/* Function: Get ViUInt32 With Window                                        */
/* Purpose:  This function gets command with unsigned integer parameter.     */
/*===========================================================================*/
ViStatus rsesci_getViUInt32Window (ViSession instrSession,
                           ViString cmd,
                           ViInt32     window,
                           ViUInt32 *value,
                           ViStatus parameterErrorCode)
{
    ViChar      wrt_buf[BUFFER_SIZE]="";

    CHECKWINDOW;

    sprintf(wrt_buf, cmd, window);

    return rsesci_getViUInt32(instrSession,
                            wrt_buf,
                            value,
                            parameterErrorCode);
}

/*===========================================================================*/
/* Function: Set ViBoolean                                                   */
/* Purpose:  This function sets command with integer parameter.              */
/*===========================================================================*/
ViStatus rsesci_setViBoolean (ViSession instrSession,
                           ViString cmd,
                           ViBoolean value,
                           ViStatus parameterErrorCode)
{
    if (rsesci_invalidViBooleanRange (value))
        return parameterErrorCode;

    return rsesci_setViString (instrSession,
                              cmd,
                              value ? "ON" : "OFF",
                              0);
}

/*===========================================================================*/
/* Function: Get ViBoolean                                                   */
/* Purpose:  This function gets command with integer parameter.              */
/*===========================================================================*/
ViStatus rsesci_getViBoolean (ViSession instrSession,
                           ViString cmd,
                           ViBoolean *value,
                           ViStatus parameterErrorCode)
{
    ViStatus    rsesci_status;
    ViInt32     bool_val;

    if (value == NULL)
        return parameterErrorCode;

    CHECKERR (rsesci_getViInt32(instrSession,
                               cmd,
                               &bool_val,
                               parameterErrorCode));
    *value = (ViBoolean) bool_val;

    return rsesci_status;
}

/*===========================================================================*/
/* Function: Set ViBoolean With Window                                       */
/* Purpose:  This function sets command with integer parameter.              */
/*===========================================================================*/
ViStatus rsesci_setViBooleanWindow (ViSession instrSession,
                           ViString cmd,
                           ViInt32     window,
                           ViBoolean value,
                           ViStatus parameterErrorCode)
{
    ViChar  wrt_buf[BUFFER_SIZE];
    ViChar* p2buf;

    CHECKWINDOW;
    if (rsesci_invalidViBooleanRange (value))
        return parameterErrorCode;

    p2buf = wrt_buf + sprintf(wrt_buf, cmd, window);
    sprintf (p2buf, " %s", value ? "ON" : "OFF");

    return rsesci_sendCmd (instrSession, wrt_buf);
}

/*===========================================================================*/
/* Function: Get ViBoolean With Window                                       */
/* Purpose:  This function gets command with integer parameter.              */
/*===========================================================================*/
ViStatus rsesci_getViBooleanWindow (ViSession   instrSession,
                                    ViString    cmd,
                                    ViInt32     window,
                                    ViBoolean   *value,
                                    ViStatus    parameterErrorCode)
{
    ViStatus    rsesci_status;
    ViInt32     bool_val;

    if (value == NULL)
        return parameterErrorCode;

    CHECKERR (rsesci_getViInt32Window(instrSession,
                                   cmd,
                                   window,
                                   &bool_val,
                                   parameterErrorCode));
    *value = (ViBoolean) bool_val;

    return rsesci_status;
}

/*===========================================================================*/
/* Function: Set ViReal64                                                    */
/* Purpose:  This function sets command with double parameter.               */
/*===========================================================================*/
ViStatus rsesci_setViReal64 (ViSession instrSession,
                            ViString cmd,
                            ViReal64 value,
                            ViReal64 min,
                            ViReal64 max,
                            ViStatus parameterErrorCode)
{
    ViChar      buffer[BUFFER_SIZE];

    /* do range checking only if there's difference between max and min */
    if ((max - min) > 1.0e-14)
        if (rsesci_invalidViReal64Range (value, min, max))
            return parameterErrorCode;

    sprintf(buffer,"%s %.12Lf", cmd, value);

    return rsesci_sendCmd(instrSession, buffer);
}

/*===========================================================================*/
/* Function: Get ViReal64                                                    */
/* Purpose:  This function gets command with double parameter.               */
/*===========================================================================*/
ViStatus rsesci_getViReal64 (ViSession instrSession,
                            ViString cmd,
                            ViReal64 *value,
                            ViStatus parameterErrorCode)
{
    ViStatus    rsesci_status = VI_SUCCESS;

    if (value == VI_NULL)
        return parameterErrorCode;

    CHECKERR(rsesci_clearBeforeRead (instrSession));
    CHECKERR(viPrintf (instrSession, "*CLS;%s\n", cmd));
    CHECKERR(rsesci_checkBeforeRead (instrSession));
    CHECKERR(viScanf (instrSession, "%le", value));
    CHECKERR(rsesci_checkStatus (instrSession));

    return rsesci_status;
}

/*===========================================================================*/
/* Function: Set ViReal64 With Window                                        */
/* Purpose:  This function sets command with double parameter.               */
/*===========================================================================*/
ViStatus rsesci_setViReal64Window (ViSession instrSession,
                            ViString cmd,
                            ViInt32  window,
                            ViReal64 value,
                            ViReal64 min,
                            ViReal64 max,
                            ViStatus parameterErrorCode)
{
    ViChar      wrt_buf[BUFFER_SIZE];

    CHECKWINDOW;

    sprintf (wrt_buf, cmd, window);

    return rsesci_setViReal64(instrSession,
                             wrt_buf,
                             value,
                             min,
                             max,
                             parameterErrorCode);
}

/*===========================================================================*/
/* Function: Get ViReal64 With Window                                        */
/* Purpose:  This function gets command with double parameter.               */
/*===========================================================================*/
ViStatus rsesci_getViReal64Window (ViSession instrSession,
                            ViString cmd,
                            ViInt32     window,
                            ViReal64 *value,
                            ViStatus parameterErrorCode)
{
    ViChar      wrt_buf[BUFFER_SIZE];

    CHECKWINDOW;

    sprintf(wrt_buf, cmd, window);

    return rsesci_getViReal64(instrSession,
                             wrt_buf,
                             value,
                             parameterErrorCode);
}

/*===========================================================================*/
/* Function: Set String                                                      */
/* Purpose:  This function sets command with string parameter.               */
/*===========================================================================*/
ViStatus rsesci_setViString (ViSession instrSession,
                            ViString cmd,
                            ViString value,
                            ViStatus parameterErrorValue)
{
    ViStatus    rsesci_status = VI_SUCCESS;
    ViChar*     wrt_buf;
    ViChar*     p2buf;

    if (value == VI_NULL)
        return parameterErrorValue;

    wrt_buf = (ViChar*) malloc (strlen(cmd) + strlen(value) + 4 );
    p2buf = wrt_buf;
    p2buf += sprintf(wrt_buf, "%s %s\n", cmd, value);

    GOCHECKERR(viWrite (instrSession, (ViBuf)wrt_buf, (ViUInt32)(p2buf - wrt_buf), VI_NULL));
    GOCHECKERR(rsesci_checkStatus(instrSession));

ERROR:

    free(wrt_buf);

    return rsesci_status;
}

/*===========================================================================*/
/* Function: Get String                                                      */
/* Purpose:  This function gets command with string parameter.               */
/*===========================================================================*/
ViStatus rsesci_getViString (ViSession instrSession,
                            ViString cmd,
                            ViString value,
                            ViInt32 buffer_size,
                            ViStatus parameterErrorCode)
{
    ViStatus    rsesci_status    = VI_SUCCESS;
    ViChar      *pbuffer,
                *p2buffer;
    ViUInt32    length         = 0;
    ViUInt32     count          = 0;

    length = (ViUInt32) strlen (cmd);
    pbuffer = (ViChar*) malloc (length + 1);
    memcpy (pbuffer, cmd, length);
    pbuffer[length] = '\n';

    if (!value)
        return parameterErrorCode;

    GOCHECKERR(rsesci_clearBeforeRead (instrSession));
    GOCHECKERR (viWrite (instrSession, (ViPBuf)pbuffer, length + 1, VI_NULL));

    free (pbuffer);

    pbuffer = VI_NULL;

    GOCHECKERR (rsesci_checkBeforeRead (instrSession));
    GOCHECKERR (rsesci_readDataUnknownLength (instrSession, &pbuffer, &count));
    GOCHECKERR (rsesci_status = rsesci_checkStatus (instrSession));

    if ((ViInt32) count > buffer_size)
        {
        rsesci_status = RSESCI_WARNING_SMALL_BUFFER;
        count = buffer_size;
        memcpy (value, pbuffer, count);
        value[count] = '\0';
        }
    else
        if ((p2buffer = strpbrk (pbuffer, "\r\n")) != VI_NULL)
            {
            rsesci_status = VI_SUCCESS;
            memcpy (value, pbuffer, p2buffer - pbuffer);
            value[p2buffer - pbuffer] = '\0';
            }
         else
            {
            strcpy (value, pbuffer);
            }

ERROR:

    free (pbuffer);

    return rsesci_status;
}

/*===========================================================================*/
/* Function: Set Quoted String                                               */
/* Purpose:  This function sets command with quoted string parameter.        */
/*===========================================================================*/
ViStatus rsesci_setViStringQuoted (ViSession instrSession,
                            ViString cmd,
                            ViString quotes,
                            ViString value,
                            ViStatus parameterErrorValue)
{
    ViStatus    rsesci_status = VI_SUCCESS;
    ViChar*     wrt_buf;
    ViChar*     p2buf;

    if (value == VI_NULL)
        return parameterErrorValue;

    wrt_buf = (ViChar*)malloc(strlen(cmd) + strlen(value) + 2*strlen(quotes) + 4);
    p2buf = wrt_buf;
    p2buf += sprintf(wrt_buf,"%s %s%s%s\n", cmd, quotes, value, quotes);

    GOCHECKERR (viWrite (instrSession, (ViBuf)wrt_buf, (ViUInt32) (p2buf - wrt_buf), VI_NULL));
    GOCHECKERR (rsesci_checkStatus(instrSession));

ERROR:

    free (wrt_buf);

    return rsesci_status;
}

/*===========================================================================*/
/* Function: Get Quoted String                                               */
/* Purpose:  This function gets command with quoted string parameter.        */
/*===========================================================================*/
ViStatus rsesci_getViStringQuoted (ViSession instrSession,
                                  ViString cmd,
                                  ViString quotes,
                                  ViInt32  arraySize,
                                  ViChar   value[],
                                  ViStatus parameterErrorValue)
{
    ViStatus    rsesci_status = VI_SUCCESS;
    ViChar*     rd_buf;
    ViChar      fmt_string[BUFFER_SIZE];

    if (value == VI_NULL)
        return parameterErrorValue;

    rd_buf = (ViChar*)malloc(arraySize + 2*strlen(quotes) + 4);
    sprintf(fmt_string,"%s%%[^%s]", quotes, quotes);
    GOCHECKERR(rsesci_clearBeforeRead (instrSession));

    {
        ViUInt32     length      = 0;
        ViChar      *pbuffer;

        length = (ViUInt32) strlen (cmd);
        pbuffer = (ViChar*) malloc (length + 1);
        memcpy (pbuffer, cmd, length);
        pbuffer[length] = '\n';

        rsesci_status = viWrite (instrSession, (ViPBuf) cmd, length + 1, VI_NULL);

        free(pbuffer);
    }

    GOCHECKERR (rsesci_status);
    GOCHECKERR (rsesci_checkBeforeRead (instrSession));
    GOCHECKERR (viScanf (instrSession, fmt_string, value));
    GOCHECKERR (rsesci_checkStatus(instrSession));

ERROR:

    free (rd_buf);

    return rsesci_status;
}

/*===========================================================================*/
/* Function: Set String With Channel                                         */
/* Purpose:  This function sets command with string parameter.               */
/*===========================================================================*/
ViStatus rsesci_setViStringWindow (ViSession instrSession,
                            ViString cmd,
                            ViInt32     window,
                            ViString value,
                            ViStatus parameterErrorValue)
{
    ViStatus    rsesci_status = VI_SUCCESS;
    ViChar*     wrt_buf;
    ViChar*     p2buf;

    if (value == VI_NULL)
        return parameterErrorValue;

    wrt_buf = (ViChar*) malloc (strlen(cmd) + strlen(value) + 11);
    p2buf = wrt_buf;
    p2buf += sprintf(wrt_buf, cmd, window);
    p2buf += sprintf(p2buf," %s\n", value);

    rsesci_status = viWrite (instrSession, (ViBuf)wrt_buf, (ViUInt32) (p2buf - wrt_buf), VI_NULL);
    free(wrt_buf);

    CHECKERR(rsesci_status);
    CHECKERR(rsesci_checkStatus(instrSession));

    return rsesci_status;
}

/*===========================================================================*/
/* Function: Get String with Window                                          */
/* Purpose:  This function gets command with string parameter.               */
/*===========================================================================*/
ViStatus rsesci_getViStringWindow (ViSession instrSession,
                                ViString cmd,
                                ViInt32     window,
                                ViString value,
                                ViInt32 buffer_size,
                                ViStatus parameterErrorCode)
{
    ViChar      wrt_buf[BUFFER_SIZE];

    sprintf(wrt_buf, cmd, window);

    return rsesci_getViString(instrSession,
                             wrt_buf,
                             value,
                             buffer_size,
                             parameterErrorCode);
}

/*===========================================================================*/
/* Function: Set Enum                                                        */
/* Purpose:  This function sets command with enumerated parameter.           */
/*===========================================================================*/
ViStatus rsesci_setEnum (ViSession instrSession,
                        ViString cmd,
                        ViInt32 value,
                        ViString list[],
                        ViStatus parameterErrorCode)
{
    ViStatus rsesci_status = VI_SUCCESS;

    CHECKERR (rsesci_checkIndexInList (instrSession, value, list, parameterErrorCode));

    return rsesci_setViString (instrSession, cmd, list[value], parameterErrorCode);
}

/*===========================================================================*/
/* Function: Get Enum                                                        */
/* Purpose:  This function sets command with enumerated parameter.           */
/*===========================================================================*/
ViStatus rsesci_getEnum (ViSession instrSession,
                        ViString cmd,
                        ViInt32 *value,
                        ViString list[],
                        ViStatus parameterErrorCode)
{
    ViStatus    rsesci_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];

    CHECKERR( rsesci_getViString(instrSession, cmd, buffer, BUFFER_SIZE, parameterErrorCode));
    CHECKERR( rsesci_getIndexFromString (instrSession, buffer, list, value));

    return rsesci_status;
}

/*===========================================================================*/
/* Function: Set Enum With Window                                              */
/* Purpose:  This function sets command with enumerated parameter.           */
/*===========================================================================*/
ViStatus rsesci_setEnumWindow (ViSession instrSession,
                        ViString cmd,
                        ViInt32     window,
                        ViInt32 value,
                        ViString list[],
                        ViStatus parameterErrorCode)
{
    ViStatus rsesci_status = VI_SUCCESS;

    CHECKERR (rsesci_checkIndexInList (instrSession, value, list, parameterErrorCode));

    return rsesci_setViStringWindow (instrSession, cmd, window, list[value], parameterErrorCode);
}

/*===========================================================================*/
/* Function: Get Enum Window                                                   */
/* Purpose:  This function sets command with enumerated parameter.           */
/*===========================================================================*/
ViStatus rsesci_getEnumWindow (ViSession instrSession,
                        ViString cmd,
                        ViInt32     window,
                        ViInt32 *value,
                        ViString list[],
                        ViStatus parameterErrorCode)
{
    ViStatus    rsesci_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];

    CHECKERR(rsesci_getViStringWindow(instrSession, cmd, window, buffer, BUFFER_SIZE, parameterErrorCode));
    CHECKERR(rsesci_getIndexFromString (instrSession, buffer, list, value));

    return rsesci_status;

}

/*===========================================================================*/
/* Function: Check Index List                                                */
/* Purpose:  This function checks if the passed index is valid for given list*/
/*===========================================================================*/
ViStatus rsesci_checkIndexInList (ViSession instrSession,
                                 ViInt32 index,
                                 ViString list[],
                                 ViStatus statusIfError)

{
    ViInt32     i=0;
	instrSession;

    if (index < 0)
        return statusIfError;

    while (list[i++] != VI_NULL);
    if (index >= i)
        return statusIfError;

    return VI_SUCCESS;
}

/*===========================================================================*/
/* Function: Get Index From String                                           */
/* Purpose:  This function returns index for string.                         */
/*===========================================================================*/
ViStatus rsesci_getIndexFromString (ViSession instrSession,
                                   ViString string,
                                   ViString *list,
                                   ViInt32 *index)
{
    ViStatus    rsesci_status = VI_SUCCESS;
    ViUInt32    i,
                len;

	instrSession;
    for (i = 0; list[i]; i++)
        if ((len = (ViUInt32) strlen (string)) == (ViUInt32) strlen (list[i])) {
            if (strncmp(string, list[i], strlen (string)) == 0) break;
        }

    *index = i;

    return (list[i]) ? rsesci_status : VI_ERROR_INSTR_INTERPRETING_RESPONSE;
}

/*===========================================================================*/
/* Function: Default Instrument Setup                                        */
/* Purpose:  This function sends a default setup to the instrument.  This    */
/*           function is called by the rsesci_reset operation and by the     */
/*           rsesci_init function if the reset option has not been           */
/*           selected.  This function is useful for configuring any          */
/*           instrument settings that are required by the rest of the        */
/*           instrument driver functions such as turning headers ON or OFF   */
/*           or using the long or short form for commands, queries, and data.*/
/*===========================================================================*/
ViStatus rsesci_defaultInstrSetup (ViSession instrSession)
{
    ViStatus            rsesci_status = VI_SUCCESS;
    ViChar              buffer[BUFFER_SIZE];
    ViInt32             idx = 0;
    rsesci_instrRange   instrPtr;

    /* Determine if the structure has been initialized for the current VISA  */
    /* Session and malloc if it has not.                                     */
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    /*
        Following two ViInt32 variables are used to store information about
        currently connected instrument and its options. Each bit represents
        one instrument model or option.
    */
    instrPtr -> instrOption = 0x0;
    instrPtr -> instrModel = 0x0;
    /* --- */
    instrPtr -> errorChecking = VI_TRUE;
    strcpy (instrPtr -> instrDriverRevision, RSESCI_REVISION);

    /*--- Recognize Instrument options (use *OPT? command) --- */

    CHECKERR (viQueryf (instrSession, "*OPT?\n", "%[^\r\n]", buffer));

    for ( idx = 0; optionsList[idx]; idx++ )
        if (strstr (buffer, optionsList[idx]))
            instrPtr -> instrOption |= (1 << idx);

    /*--- Recognize Instrument model (use *IDN? command) --- */

    CHECKERR (viQueryf (instrSession, "*IDN?\n", "%*[^,],%[^,],%*[\r\n]", buffer));

    for ( idx = 0; instrList[idx]; idx++ )
        if (!strncmp (buffer, instrList[idx], strlen(instrList[idx])))
            instrPtr -> instrModel |= (1 << idx);

    /* Default Instrument Setup String - Data Format */
    /* If not under development CHANGE to ":SYST:DISP:UPD OFF" */

//    CHECKERR (viPrintf (instrSession, "*CLS;SYST:PRES:COMP OFF;:FORM REAL,32;*ESE 60;:SYST:DISP:UPD OFF\n"));
    CHECKERR (viPrintf (instrSession, "*CLS;*ESE 60;:SYST:DISP:UPD OFF\n"));

    return rsesci_status;
}

/*=========================================================================*/
/* Function: Check Status                                                  */
/* Purpose:  This function reads Event Status Register to check for        */
/*           possible errors.                                              */
/*=========================================================================*/
ViStatus rsesci_checkStatus (ViSession instrSession)
{
    ViStatus            rsesci_status = VI_SUCCESS;
    rsesci_instrRange   instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    /* Error Checking Enabled? */
    if (instrPtr -> errorChecking)
        {
        ViInt32 esrByte = 0;

        /*--------------- Read the Standard Event Register ------------------*/

        CHECKERR (viQueryf (instrSession, "*ESR?\n", "%ld", &esrByte));

        /*---------------- Check if any error or message bit was asserted ---*/

        rsesci_status = (esrByte & RSESCI_IEEE_ERROR_BITS) ? RSESCI_ERROR_INSTRUMENT_ERROR : VI_SUCCESS;
        }

    return rsesci_status;
}

/*===========================================================================*/
/* Function: Clear STB Before Read                                           */
/* Purpose:  This is a utility function which clears error and message queue */
/*           before starting a query                                         */
/*===========================================================================*/
ViStatus rsesci_clearBeforeRead (ViSession instrSession)
{
    ViStatus rsesci_status = VI_SUCCESS;
    ViUInt16 response = 0;
    ViChar  garbage[10];

    CHECKERR (viReadSTB (instrSession, &response));
    
    while (response & 0x14)
    {
        if (response & 0x4)
        {
            ViInt32 error;
            CHECKERR(viQueryf(instrSession, "*ESR?\n", "%ld", &error));
            CHECKERR(viQueryf(instrSession, ":SYST:ERR?\n", "%*[^\r\n]"));
        }
        
        if (response & 0x10)
            viRead (instrSession, (ViBuf) garbage, 10, VI_NULL);

        CHECKERR (viReadSTB (instrSession, &response));
    
    }
    
    return rsesci_status;
}

/*===========================================================================*/
/* Function: Check STB Before Read                                           */
/* Purpose:  This is a utility function which checks Error/Event Queue bit of*/
/*           status byte. Call this function before read result of query     */
/*           command. When error occurs, this function returns error code    */
/*           RSESCI_ERROR_INSTRUMENT_DATA_NOT_AVAILABLE. Prior to calling    */
/*           this function is necessary to clear STB.                        */
/*===========================================================================*/
ViStatus rsesci_checkBeforeRead (ViSession instrSession)
{
    ViStatus rsesci_status = VI_SUCCESS;
    ViUInt16 response = 0;
    ViInt32 counter = 0, timeout = 0;
    ViUInt16 intfc = VI_INTF_GPIB;
    rsesci_instrRange instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_INTF_TYPE, &intfc));

    if (instrPtr -> errorChecking) /* Is Error Checking Enabled? */
        {
        /* Interface specific operations */
        switch (intfc)
            {
            case VI_INTF_ASRL: /* Serial interface */

                /* Not supported */

            break;
            default: /* Default case */

                CHECKERR (viGetAttribute (instrSession, VI_ATTR_TMO_VALUE, &timeout));

                /* Check STB for errors. */
                do {
                    CHECKERR (rsesci_status = viReadSTB (instrSession, &response));

                    rsesci_delay (0.005);
                    counter += 5;
                    if (counter >= timeout)
                        return VI_ERROR_TMO;
                /* Wait while STB is non zero value. Only Error/Event Queue
                   and MAV bits are taken into consideration. */
                } while (!(response & 0x14));

                if (response & 0x4)
                {
                    ViInt32 error;
                    CHECKERR(viQueryf(instrSession, "*ESR?\n", "%ld", &error));
                    if (error == 4)
                        CHECKERR(viQueryf(instrSession, ":SYST:ERR?\n", "%*[^\r\n]"));
                }
                /* (check Error/Event Queue bit in STB) */
                return (response & 0x4) ?  RSESCI_ERROR_INSTRUMENT_DATA_NOT_AVAILABLE : rsesci_status;
            }
        }

    return rsesci_status;
}

/*===========================================================================*/
/* Function: Swap 32bit Integer                                              */
/* Purpose:  Byte order for binary transfers depends on MOTOROLA or INTEL    */
/*           formats.                                                        */
/*===========================================================================*/
void rsesci_SWAP_32(unsigned char *data, long points)
{
    /* MOTOROLA (byte order 0123) or INTEL (byte order 3210). */
    unsigned int    word = 0x1122;
    unsigned char   *p2word = (unsigned char*)&word;

    if (*p2word == 0x11) /* MOTOROLA format */
    {
       unsigned char    src[4];
       long             *p2data = (long*)data;
       int              idx;

       for (idx = 0; idx < points; idx++, p2data++)
          {
              *((long *)src) = *(p2data);
              ((unsigned char *)(p2data))[0] =  ((unsigned char*)(src))[3];
              ((unsigned char *)(p2data))[1] =  ((unsigned char*)(src))[2];
              ((unsigned char *)(p2data))[2] =  ((unsigned char*)(src))[1];
              ((unsigned char *)(p2data))[3] =  ((unsigned char*)(src))[0];
          }
    }
}

/*===========================================================================*/
/* Function: Get ViReal64 Array                                              */
/*===========================================================================*/
ViStatus rsesci_getViReal64Array (ViSession instrSession,
                            ViString cmd,
                            ViReal64 *value,
                            ViInt32  count,
                            ViStatus parameterErrorCode)
{
    ViStatus    rsesci_status = VI_SUCCESS;
    ViChar*     pbuffer = VI_NULL;
    ViChar*     p2buf;
    ViUInt32    local_count;
    ViInt32     i = 0;

    if (value == VI_NULL)
        return parameterErrorCode;

    GOCHECKERR(rsesci_clearBeforeRead (instrSession));
    GOCHECKERR (viPrintf(instrSession, "%s\n",cmd));
    GOCHECKERR (rsesci_checkBeforeRead (instrSession));
    GOCHECKERR (rsesci_readDataUnknownLength(instrSession, &pbuffer, &local_count));
    p2buf = pbuffer;

    p2buf = strtok(pbuffer, ",");

    while(p2buf && (i < count))
        {
        value[i++] = atof(p2buf);
        p2buf = strtok(NULL, ",");
        };

    GOCHECKERR (rsesci_checkStatus (instrSession));

ERROR:

    free(pbuffer);

    return rsesci_status;
}

/*===========================================================================*/
/* Function: Set ViReal64 Array                                              */
/*===========================================================================*/
ViStatus rsesci_setViReal64Array (ViSession instrSession,
                            ViString cmd,
                            ViReal64 *value,
                            ViInt32  count,
                            ViStatus parameterErrorCode)
{
    ViStatus    rsesci_status = VI_SUCCESS;
    ViChar*     pbuffer = VI_NULL;
    ViChar*     p2buf;
    ViInt32     i = 0;

    if (value == VI_NULL)
        return parameterErrorCode;

    p2buf = pbuffer = (ViChar*) malloc (strlen (cmd) + 30 * count + 10);

    strcpy (pbuffer, cmd);
    strcat (pbuffer, " ");
    p2buf += strlen (pbuffer);

    for (i = 0; i < count; i++)
        p2buf += sprintf (p2buf, "%.12Lf,", value[i]);

    *--p2buf = '\0';

    rsesci_status = viPrintf(instrSession, "%s\n", pbuffer);

    free(pbuffer);

    CHECKERR (rsesci_status);
    CHECKERR (rsesci_checkStatus (instrSession));

    return rsesci_status;
}

/*===========================================================================*/
/* Function: Set ViReal64 Array                                              */
/*===========================================================================*/
ViStatus rsesci_setViReal64ArrayWindow (ViSession instrSession,
                            ViString cmd,
                            ViInt32     window,
                            ViReal64 *value,
                            ViInt32  count,
                            ViStatus parameterErrorCode)
{
    ViStatus    rsesci_status = VI_SUCCESS;
    ViChar*     pbuffer = VI_NULL;
    ViChar*     p2buf;
    ViInt32     i = 0;

    if (value == VI_NULL)
        return parameterErrorCode;

    p2buf = pbuffer = (ViChar*) malloc (strlen (cmd) + 20 * count + 10);

    p2buf += sprintf (pbuffer, cmd, window);
    strcat (pbuffer, " ");
    p2buf++;

    for (i = 0; i < count; i++)
        p2buf += sprintf (p2buf, "%.12Lf,", value[i]);

    *--p2buf = '\0';

    rsesci_status = viPrintf(instrSession, "%s\n", pbuffer);

    free(pbuffer);

    CHECKERR (rsesci_status);
    CHECKERR (rsesci_checkStatus (instrSession));

    return rsesci_status;
}

/*===========================================================================*/
/* Function: Get ViReal64 Array Window                                         */
/*===========================================================================*/
ViStatus rsesci_getViReal64ArrayWindow (ViSession instrSession,
                            ViString cmd,
                            ViInt32     window,
                            ViReal64 *value,
                            ViInt32  count,
                            ViStatus parameterErrorCode)
{
    ViStatus    rsesci_status;
    ViChar      *pbuffer;

    pbuffer = (ViChar*) malloc (strlen (cmd) + 10);
    sprintf (pbuffer, cmd, window);

    rsesci_status = rsesci_getViReal64Array (instrSession,
                                           pbuffer,
                                           value,
                                           count,
                                           parameterErrorCode);

    free (pbuffer);

    return rsesci_status;
}

/*****************************************************************************
 * Function: Get ViReal64 Binary Array
 *****************************************************************************/
ViStatus rsesci_getViReal64BinArray (ViSession instrSession,
                            ViString    cmd,
                            ViReal64    *value,
                            ViInt32     maxCount,
                            ViInt32     *retCount,
                            ViStatus    parameterErrorCode)
{
    ViStatus    rsesci_status   = VI_SUCCESS;
    ViUInt16    intfc       = VI_INTF_GPIB;
    ViUInt32    retCnt          = 0;
    ViChar      *exBuf          = 0;
    ViReal32    *temp           = 0;
    ViInt32     lenLength       = 0,
                i               = 0,
                count           = 0;
    ViChar      buffer[BUFFER_SIZE];

    if (value == VI_NULL)
        return parameterErrorCode;

    CHECKERR( viGetAttribute (instrSession, VI_ATTR_INTF_TYPE, &intfc));

    CHECKERR (rsesci_clearBeforeRead(instrSession));

    CHECKERR( viPrintf (instrSession, ":FORM REAL,32;*CLS;%s\n", cmd));
    CHECKERR( rsesci_checkBeforeRead (instrSession));

    /* Interface specific operations */
    switch (intfc)
        {
        case VI_INTF_ASRL: /* Serial Interface. */
            /* Switch to Off using of termination character. */
            CHECKERR( viSetAttribute (instrSession, VI_ATTR_ASRL_END_IN, VI_ASRL_END_NONE));
            CHECKERR( viSetAttribute (instrSession, VI_ATTR_TERMCHAR_EN, VI_FALSE));

            /* Parse binary data header. */
            if ((rsesci_status = viRead (instrSession, (ViBuf) buffer, 2, &retCnt)) < 0)
                {
                viSetAttribute (instrSession, VI_ATTR_ASRL_END_IN, VI_ASRL_END_TERMCHAR);
                viSetAttribute (instrSession, VI_ATTR_TERMCHAR_EN, VI_TRUE);
                viPrintf (instrSession,"FORM ASCII");
                return rsesci_status;
                }
            buffer[retCnt] = '\0';
            if (sscanf (buffer, "#%1d", &lenLength) != 1)
                {
                viSetAttribute (instrSession, VI_ATTR_ASRL_END_IN, VI_ASRL_END_TERMCHAR);
                viSetAttribute (instrSession, VI_ATTR_TERMCHAR_EN, VI_TRUE);
                viPrintf (instrSession,"FORM ASCII");
                return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
                }

            /* Get number of points. */
            if ((rsesci_status = viRead (instrSession, (ViBuf) buffer, lenLength, &retCnt)) < 0)
                {
                viSetAttribute (instrSession, VI_ATTR_ASRL_END_IN, VI_ASRL_END_TERMCHAR);
                viSetAttribute (instrSession, VI_ATTR_TERMCHAR_EN, VI_TRUE);
                viPrintf (instrSession,"FORM ASCII");
                return rsesci_status;
                }

            buffer[retCnt] = '\0';
            if (sscanf (buffer, "%ld", &count) != 1)
                {
                viSetAttribute (instrSession, VI_ATTR_ASRL_END_IN, VI_ASRL_END_TERMCHAR);
                viSetAttribute (instrSession, VI_ATTR_TERMCHAR_EN, VI_TRUE);
                viPrintf (instrSession,"FORM ASCII");
                return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
                }

            /* Allocate memory space */
            if ((exBuf = malloc (count + 2)) == NULL)
                {
                viPrintf (instrSession,"FORM ASCII");
                return VI_ERROR_ALLOC;
                }
            /* Read exact number of bytes equal to number of points. */
            if ((rsesci_status = viRead (instrSession, (ViBuf) exBuf, count + 2, &retCnt)) < 0)
                {
                viSetAttribute (instrSession, VI_ATTR_ASRL_END_IN, VI_ASRL_END_TERMCHAR);
                viSetAttribute (instrSession, VI_ATTR_TERMCHAR_EN, VI_TRUE);
                viPrintf (instrSession,"FORM ASCII");
                /* If not NULL pointer, then deallocate memory. */
                if (exBuf) free (exBuf);
                return rsesci_status;
                }
            viSetAttribute (instrSession, VI_ATTR_ASRL_END_IN, VI_ASRL_END_TERMCHAR);
            viSetAttribute (instrSession, VI_ATTR_TERMCHAR_EN, VI_TRUE);
        break;
        case VI_INTF_GPIB: default: /* GPIB Interface. Default case */
            CHECKERR( viRead (instrSession, (ViBuf) buffer, 2, &retCnt));
            buffer[retCnt] = '\0';

            if (sscanf (buffer, "#%1d", &lenLength) != 1)
            {
                viPrintf (instrSession,"FORM ASCII");
                return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
            }

            CHECKERR( viRead (instrSession, (ViBuf) buffer, lenLength, &retCnt));
            buffer[retCnt] = '\0';

            if (sscanf (buffer, "%ld", &count) != 1)
            {
                viPrintf (instrSession,"FORM ASCII");
                return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
            }

          /* Allocate memory space */
          if ((exBuf = malloc (count + 4)) == NULL)
          {
              viPrintf (instrSession,"FORM ASCII");
              return VI_ERROR_ALLOC;
          }

          if ((rsesci_status = viRead (instrSession, (ViBuf) exBuf, count + 4, &retCnt)) < 0)
              {
                viPrintf (instrSession,"FORM ASCII");
              /* If not NULL pointer, then deallocate memory. */
              if (exBuf) free (exBuf);
              return rsesci_status;
              }
        break;
        }

    count /= 4; /* number samples */
    /* Binary data byte order (MOTOROLA or INTEL). */
    if (count > 0)
        rsesci_SWAP_32((unsigned char*) exBuf, count);

    /* Convert binary data into array of ViReal64 types. */
    temp = (ViReal32*)(exBuf);
    *retCount = (count > maxCount) ? maxCount : count;

    for (i = 0; i < *retCount; i++)
        *value++ = (ViReal64)*temp++;

    if (exBuf) /* If not NULL pointer, then deallocate memory. */
        free (exBuf);

    CHECKERR (viPrintf (instrSession,"FORM ASCII"));
    CHECKERR (rsesci_checkStatus (instrSession));

    return rsesci_status;
}

/*===========================================================================*/
/* Function: Coerce to nearest                                               */
/* Purpose:  This function is used to coerce entered value to the nearest    */
/*           value from the discreete table.                                 */
/*===========================================================================*/
void rsesci_coerceToNearest (void *table, ViPReal64 value, ViPInt16 index)
{
    ViReal64 *p2table = table;

    *index = 0;

    while ((*value > *p2table) && (fabs (*p2table) > NUMERIC_PRECISION))
        {
        *(++p2table);
        (*index)++;
        }

    if ((*value > NUMERIC_PRECISION) && (*index > 0))
        {
        if (fabs ((*value - *(--p2table)) / 2) < fabs ((*value - *(++p2table)) / 2))
            {
            *value = *(--p2table);
            (*index)--;
            }
        else
            {
            *value = *p2table;
            }
        }
    else
        {
        *value = *p2table;
        *index = 0;
        }

    return;
}

/*===========================================================================*/
/* Function: Check Option                                                    */
/* Purpose : This function try to find the option in option list.            */
/*===========================================================================*/
ViStatus rsesci_checkOption (ViSession instrSession, ViInt32 option)
{
    ViStatus rsesci_status = VI_SUCCESS;
    rsesci_instrRange instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    rsesci_status = (instrPtr -> instrOption & (option)) ? VI_SUCCESS : (instrPtr -> errorChecking) ? RSESCI_ERROR_INSTRUMENT_OPTION : VI_SUCCESS;

    return rsesci_status;
}

/*===========================================================================*/
/* Function: Check Model                                                     */
/* Purpose : This function tries to find the model in model list.            */
/*===========================================================================*/
ViStatus rsesci_checkModel (ViSession instrSession, ViInt32 model)
{
    ViStatus rsesci_status = VI_SUCCESS;
    rsesci_instrRange instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    rsesci_status = (instrPtr -> instrModel & (model)) ? VI_SUCCESS : (instrPtr -> errorChecking) ? RSESCI_ERROR_INSTRUMENT_NSUP_MODEL : VI_SUCCESS;

    return rsesci_status;
}

/*===========================================================================*/
/* Function: Parse Catalog List                                              */
/* Purpose:  Parse Catalog List returned from instrument and removes from it */
/*           all unneeded parameters from this list.                         */
/*===========================================================================*/
ViStatus rsesci_ParseCatalog(ViChar *buffer, ViInt32 size, ViChar *catalogList, ViInt32 *nbValues)
{
    ViStatus    error = VI_SUCCESS; 
    ViChar*     pbuf;
    ViInt32     num;
    
    if (size != 0)
    {
        memcpy(catalogList, buffer, ( (ViUInt32) size > strlen(buffer)) ? strlen(buffer) : size);
    }
    pbuf = strtok(buffer, ",");
    num=0;
    while (pbuf) 
    {
        num++;
        pbuf = strtok(NULL, ",");        
    } 
    if (nbValues) *nbValues = num;
    
    return error;
}

/*****************************************************************************/
/*=== END INSTRUMENT DRIVER SOURCE CODE =====================================*/
/*****************************************************************************/
